@@ -3,18 +3,18 @@ require 'ffi'
 iter = 100000
 str = "test"
 
-module JLibC
+module LibC
   extend FFI::Library
   attach_function :strlen, [ :string ], :int
 end
 
-if JLibC.strlen("test") != 4
+if LibC.strlen("test") != 4
   raise ArgumentError, "jruby FFI.strlen returned incorrect value"
 end
 
 puts "Benchmark jruby FFI api strlen(3) performance, #{iter}x"
 10.times {
   puts Benchmark.measure {
-    iter.times { JLibC.strlen(str) }
+    iter.times { LibC.strlen(str) }
   }
 }
