@@ -76,7 +76,6 @@ puts "Measure Fixnum#to_s, String being constructed"
   }
 }
 
-
 puts "Measure Integer.valueOf, overloaded call with a primitive"
 5.times {
   puts Benchmark.measure {
@@ -88,3 +87,23 @@ puts "Measure Integer.valueOf, overloaded call with a primitive"
   }
 }
     
+puts "Measure ArrayList<Object>.get, non-coercible type entering Ruby"
+5.times {
+  puts Benchmark.measure {
+    list = java.util.ArrayList.new
+    list.add(java.lang.Object.new)
+    1000000.times {
+      list.get(0)
+    }
+  }
+}
+    
+puts "Measure java.lang.Object.new"
+5.times {
+  puts Benchmark.measure {
+    1000000.times {
+      Object.new
+    }
+  }
+}
+    
