@@ -1,7 +1,16 @@
 require 'benchmark'
 
 def bench_def_method(bm)
-  bm.report("1m x def a; 1 + 1; end") do
+  bm.report("100k def a; 1 + 1; end") do
+    Class.new do
+      x = 0
+      while x < 100_000
+        def a; 1 + 1; end
+        x += 1
+      end
+    end
+  end
+  bm.report("100k toplevel def a; 1 + 1; end") do
     x = 0
     while x < 100_000
       def a; 1 + 1; end
