@@ -2,8 +2,8 @@
 #--
 #   matrix.rb - 
 #       $Release Version: 1.0$
-#       $Revision: 8696 $
-#       $Date: 2009-01-10 15:17:58 -0600 (Sat, 10 Jan 2009) $
+#       $Revision: 1.11 $
+#       $Date: 1999/10/06 11:01:53 $
 #       Original Version from Smalltalk-80 version
 #          on July 23, 1985 at 8:37:17 am
 #       by Keiju ISHITSUKA
@@ -53,7 +53,6 @@ end
 # * <tt> Matrix.columns(columns)        </tt>
 # * <tt> Matrix.diagonal(*values)       </tt>
 # * <tt> Matrix.scalar(n, value)        </tt>
-# * <tt> Matrix.scalar(n, value)        </tt>
 # * <tt> Matrix.identity(n)             </tt>
 # * <tt> Matrix.unit(n)                 </tt>
 # * <tt> Matrix.I(n)                    </tt>
@@ -105,7 +104,7 @@ end
 # * <tt> #inspect                       </tt>
 #
 class Matrix
-  @RCS_ID='-$Id: matrix.rb 8696 2009-01-10 21:17:58Z headius $-'
+  @RCS_ID='-$Id: matrix.rb,v 1.11 1999/10/06 11:01:53 keiju Exp keiju $-'
   
 #  extend Exception2MessageMapper
   include ExceptionForMatrix
@@ -125,11 +124,12 @@ class Matrix
   
   #
   # Creates a matrix where +rows+ is an array of arrays, each of which is a row
-  # to the matrix.  If the optional argument +copy+ is false, use the given
+  # of the matrix.  If the optional argument +copy+ is false, use the given
   # arrays as the internal structure of the matrix without copying.
   #   Matrix.rows([[25, 93], [-1, 66]])
   #      =>  25 93
   #          -1 66
+  #
   def Matrix.rows(rows, copy = true)
     new(:init_rows, rows, copy)
   end
@@ -140,12 +140,9 @@ class Matrix
   #      =>  25 -1
   #          93 66
   #
-  #
   def Matrix.columns(columns)
-    rows = (0 .. columns[0].size - 1).collect {
-      |i|
-      (0 .. columns.size - 1).collect {
-        |j|
+    rows = (0 ... columns[0].size).collect {|i|
+      (0 ... columns.size).collect {|j|
         columns[j][i]
       }
     }
@@ -161,8 +158,7 @@ class Matrix
   #
   def Matrix.diagonal(*values)
     size = values.size
-    rows = (0 .. size  - 1).collect {
-      |j|
+    rows = (0 ... size).collect {|j|
       row = Array.new(size).fill(0, 0, size)
       row[j] = values[j]
       row
@@ -287,11 +283,9 @@ class Matrix
   # Returns row vector number +i+ of the matrix as a Vector (starting at 0 like
   # an array).  When a block is given, the elements of that vector are iterated.
   #
-  def row(i) # :yield: e
+  def row(i, &block) # :yield: e
     if block_given?
-      for e in @rows[i]
-        yield e
-      end
+      @rows[i].each(&block)
     else
       Vector.elements(@rows[i])
     end
@@ -304,13 +298,11 @@ class Matrix
   #
   def column(j) # :yield: e
     if block_given?
-      0.upto(row_size - 1) do
-        |i|
+      row_size.times do |i|
         yield @rows[i][j]
       end
     else
-      col = (0 .. row_size - 1).collect {
-        |i|
+      col = (0 ... row_size).collect {|i|
         @rows[i][j]
       }
       Vector.elements(col, false)
@@ -324,8 +316,8 @@ class Matrix
   #     => 1  4
   #        9 16
   #
-  def collect # :yield: e
-    rows = @rows.collect{|row| row.collect{|e| yield e}}
+  def collect(&block) # :yield: e
+    rows = @rows.collect{|row| row.collect(&block)}
     Matrix.rows(rows, false)
   end
   alias map collect
@@ -349,10 +341,7 @@ class Matrix
       size_col = param[1].end - from_col
       size_col += 1 unless param[1].exclude_end?
     when 4
-      from_row = param[0]
-      size_row = param[1]
-      from_col = param[2]
-      size_col = param[3]
+      from_row, size_row, from_col, size_col = param
     else
       Matrix.Raise ArgumentError, param.inspect
     end
@@ -402,17 +391,20 @@ class Matrix
     
     other.compare_by_row_vectors(@rows)
   end
-  alias eql? ==
+  def eql?(other)
+    return false unless Matrix === other
+    
+    other.compare_by_row_vectors(@rows, :eql?)
+  end
   
   #
   # Not really intended for general consumption.
   #
-  def compare_by_row_vectors(rows)
+  def compare_by_row_vectors(rows, comparison = :==)
     return false unless @rows.size == rows.size
-    
-    0.upto(@rows.size - 1) do
-      |i|
-      return false unless @rows[i] == rows[i]
+
+    @rows.size.times do |i|
+      return false unless @rows[i].send(comparison, rows[i])
     end
     true
   end
@@ -429,13 +421,7 @@ class Matrix
   # Returns a hash-code for the matrix.
   #
   def hash
-    value = 0
-    for row in @rows
-      for e in row
-        value ^= e.hash
-      end
-    end
-    return value
+    @rows.hash
   end
   
   #--
@@ -465,17 +451,12 @@ class Matrix
       return r.column(0)
     when Matrix
       Matrix.Raise ErrDimensionMismatch if column_size != m.row_size
-    
-      rows = (0 .. row_size - 1).collect {
-        |i|
-        (0 .. m.column_size - 1).collect {
-          |j|
-          vij = 0
-          0.upto(column_size - 1) do
-            |k|
-            vij += self[i, k] * m[k, j]
+
+      rows = (0 ... row_size).collect {|i|
+        (0 ... m.column_size).collect {|j|
+          (0 ... column_size).inject(0) do |vij, k|
+            vij + self[i, k] * m[k, j]
           end
-          vij
         }
       }
       return Matrix.rows(rows, false)
@@ -504,11 +485,9 @@ class Matrix
     end
     
     Matrix.Raise ErrDimensionMismatch unless row_size == m.row_size and column_size == m.column_size
-    
-    rows = (0 .. row_size - 1).collect {
-      |i|
-      (0 .. column_size - 1).collect {
-        |j|
+
+    rows = (0 ... row_size).collect {|i|
+      (0 ... column_size).collect {|j|
         self[i, j] + m[i, j]
       }
     }
@@ -534,11 +513,9 @@ class Matrix
     end
     
     Matrix.Raise ErrDimensionMismatch unless row_size == m.row_size and column_size == m.column_size
-    
-    rows = (0 .. row_size - 1).collect {
-      |i|
-      (0 .. column_size - 1).collect {
-        |j|
+
+    rows = (0 ... row_size).collect {|i|
+      (0 ... column_size).collect {|j|
         self[i, j] - m[i, j]
       }
     }
@@ -566,7 +543,7 @@ class Matrix
       return self * other.inverse
     else
       x, y = other.coerce(self)
-      rerurn x / y
+      return x / y
     end
   end
 
@@ -586,41 +563,43 @@ class Matrix
   # Not for public consumption?
   #
   def inverse_from(src)
-    size = row_size - 1
+    size = row_size
     a = src.to_a
-    
-    for k in 0..size
-      if (akk = a[k][k]) == 0
-        i = k
-        begin
-          Matrix.Raise ErrNotRegular if (i += 1) > size
-        end while a[i][k] == 0
+
+    size.times do |k|
+      i = k
+      akk = a[k][k].abs
+      (k+1 ... size).each do |j|
+        v = a[j][k].abs
+        if v > akk
+          i = j
+          akk = v
+        end
+      end
+      Matrix.Raise ErrNotRegular if akk == 0
+      if i != k
         a[i], a[k] = a[k], a[i]
         @rows[i], @rows[k] = @rows[k], @rows[i]
-        akk = a[k][k]
       end
-      
-      for i in 0 .. size
+      akk = a[k][k]
+
+      size.times do |i|
         next if i == k
         q = a[i][k] / akk
         a[i][k] = 0
-        
-        (k + 1).upto(size) do   
-          |j|
+
+        (k + 1 ... size).each do |j|
           a[i][j] -= a[k][j] * q
         end
-        0.upto(size) do
-          |j|
+        size.times do |j|
           @rows[i][j] -= @rows[k][j] * q
         end
       end
-      
-      (k + 1).upto(size) do
-        |j|
+
+      (k + 1 ... size).each do |j|
         a[k][j] /= akk
       end
-      0.upto(size) do
-        |j|
+      size.times do |j|
         @rows[k][j] /= akk
       end
     end
@@ -674,36 +653,34 @@ class Matrix
   #
   def determinant
     return 0 unless square?
-    
-    size = row_size - 1
+
+    size = row_size
     a = to_a
     
     det = 1
-    k = 0
-    begin 
+    size.times do |k|
       if (akk = a[k][k]) == 0
-        i = k
-        begin
-          return 0 if (i += 1) > size
-        end while a[i][k] == 0
+        i = (k+1 ... size).find {|i|
+          a[i][k] != 0
+        }
+        return 0 if i.nil?
         a[i], a[k] = a[k], a[i]
         akk = a[k][k]
         det *= -1
       end
-      (k + 1).upto(size) do
-        |i|
+
+      (k + 1 ... size).each do |i|
         q = a[i][k] / akk
-        (k + 1).upto(size) do
-          |j|
+        (k + 1 ... size).each do |j|
           a[i][j] -= a[k][j] * q
         end
       end
       det *= akk
-    end while (k += 1) <= size
+    end
     det
   end
   alias det determinant
-        
+
   #
   # Returns the rank of the matrix.  Beware that using Float values, with their
   # usual lack of precision, can affect the value returned by this method.  Use
@@ -722,50 +699,34 @@ class Matrix
       a_row_size = row_size
     end
     rank = 0
-    k = 0
-    begin
+    a_column_size.times do |k|
       if (akk = a[k][k]) == 0
-        i = k
-        exists = true
-        begin
-          if (i += 1) > a_column_size - 1
-            exists = false
-            break
-          end
-        end while a[i][k] == 0
-        if exists
+        i = (k+1 ... a_row_size).find {|i|
+          a[i][k] != 0
+        }
+        if i
           a[i], a[k] = a[k], a[i]
           akk = a[k][k]
         else
-          i = k
-          exists = true
-          begin
-            if (i += 1) > a_row_size - 1
-              exists = false
-              break
-            end
-          end while a[k][i] == 0
-          if exists
-            k.upto(a_column_size - 1) do
-              |j|
-              a[j][k], a[j][i] = a[j][i], a[j][k]
-            end
-            akk = a[k][k]
-          else
-            next
+          i = (k+1 ... a_column_size).find {|i|
+            a[k][i] != 0
+          }
+          next if i.nil?
+          (k ... a_column_size).each do |j|
+            a[j][k], a[j][i] = a[j][i], a[j][k]
           end
+          akk = a[k][k]
         end
       end
-      (k + 1).upto(a_row_size - 1) do
-        |i|
+
+      (k + 1 ... a_row_size).each do |i|
         q = a[i][k] / akk
-        (k + 1).upto(a_column_size - 1) do
-          |j|
+        (k + 1... a_column_size).each do |j|
           a[i][j] -= a[k][j] * q
         end
       end
       rank += 1
-    end while (k += 1) <= a_column_size - 1
+    end
     return rank
   end
 
@@ -775,12 +736,9 @@ class Matrix
   #     => 16
   #
   def trace
-    tr = 0
-    0.upto(column_size - 1) do
-      |i|
-      tr += @rows[i][i]
+    (0...column_size).inject(0) do |tr, i|
+      tr + @rows[i][i]
     end
-    tr
   end
   alias tr trace
   
@@ -819,29 +777,25 @@ class Matrix
   # Returns an array of the row vectors of the matrix.  See Vector.
   #
   def row_vectors
-    rows = (0 .. row_size - 1).collect {
-      |i|
+    (0 ... row_size).collect {|i|
       row(i)
     }
-    rows
   end
   
   #
   # Returns an array of the column vectors of the matrix.  See Vector.
   #
   def column_vectors
-    columns = (0 .. column_size - 1).collect {
-      |i|
+    (0 ... column_size).collect {|i|
       column(i)
     }
-    columns
   end
   
   #
   # Returns an array of arrays that describe the rows of the matrix.
   #
   def to_a
-    @rows.collect{|row| row.collect{|e| e}}
+    @rows.collect{|row| row.dup}
   end
   
   #--
@@ -922,7 +876,7 @@ class Matrix
       when Vector
         Scalar.Raise WrongArgType, other.class, "Numeric or Scalar or Matrix"
       when Matrix
-        self * _M.inverse
+        self * other.inverse
       else
         x, y = other.coerce(self)
         x / y
@@ -1051,8 +1005,7 @@ class Vector
   #
   def each2(v) # :yield: e1, e2
     Vector.Raise ErrDimensionMismatch if size != v.size
-    0.upto(size - 1) do
-      |i|
+    size.times do |i|
       yield @elements[i], v[i]
     end
   end
@@ -1063,8 +1016,7 @@ class Vector
   #
   def collect2(v) # :yield: e1, e2
     Vector.Raise ErrDimensionMismatch if size != v.size
-    (0 .. size - 1).collect do
-      |i|
+    (0 ... size).collect do |i|
       yield @elements[i], v[i]
     end
   end
@@ -1081,13 +1033,17 @@ class Vector
     
     other.compare_by(@elements)
   end
-  alias eqn? ==
+  def eql?(other)
+    return false unless Vector === other
+    
+    other.compare_by(@elements, :eql?)
+  end
   
   #
   # For internal use.
   #
-  def compare_by(elements)
-    @elements == elements
+  def compare_by(elements, comparison = :==)
+    @elements.send(comparison, elements)
   end
   
   #
@@ -1186,11 +1142,8 @@ class Vector
   #
   # Like Array#collect.
   #
-  def collect # :yield: e
-    els = @elements.collect {
-      |v|
-      yield v
-    }
+  def collect(&block) # :yield: e
+    els = @elements.collect(&block)
     Vector.elements(els, false)
   end
   alias map collect
@@ -1198,11 +1151,8 @@ class Vector
   #
   # Like Vector#collect2, but returns a Vector instead of an Array.
   #
-  def map2(v) # :yield: e1, e2
-    els = collect2(v) {
-      |v1, v2|
-      yield v1, v2
-    }
+  def map2(v, &block) # :yield: e1, e2
+    els = collect2(v, &block)
     Vector.elements(els, false)
   end
   
@@ -1211,11 +1161,7 @@ class Vector
   #   Vector[5,8,2].r => 9.643650761
   #
   def r
-    v = 0
-    for e in @elements
-      v += e*e
-    end
-    return Math.sqrt(v)
+    Math.sqrt(@elements.inject(0) {|v, e| v + e*e})
   end
   
   #--
@@ -1267,6 +1213,5 @@ class Vector
   end
 end
 
-
 # Documentation comments:
 #  - Matrix#coerce and Vector#coerce need to be documented
