@@ -4,7 +4,7 @@ enabled=false
 
 # JRuby does not support mkmf yet, so we fail hard here with a useful message
 if !enabled
-  warn "WARNING: JRuby does not support native extensions or the `mkmf' library.\n         Check wiki.jruby.org for alternatives."
+  warn "WARNING: JRuby does not support native extensions or the `mkmf' library.\n         Check http://kenai.com/projects/jruby/pages/Home for alternatives."
 else
 
 # We're missing this in our rbconfig
@@ -42,7 +42,7 @@ CXX_EXT = %w[cc cxx cpp]
 if /mswin|bccwin|mingw|msdosdjgpp|human|os2/ !~ CONFIG['build_os']
   CXX_EXT.concat(%w[C])
 end
-SRC_EXT = %w[c m] << CXX_EXT
+SRC_EXT = %w[c m].concat(CXX_EXT)
 $static = $config_h = nil
 $default_static = $static
 
