@@ -1,28 +1,30 @@
 #
 #   help.rb - helper using ri
-#   	$Release Version: 0.9.5$
-#   	$Revision: 2062 $
-#   	$Date: 2006-06-10 14:14:15 -0500 (Sat, 10 Jun 2006) $
+#   	$Release Version: 0.9.6$
+#   	$Revision: 24227 $
 #
 # --
 #
-#   
+#
 #
 
-require 'rdoc/ri/ri_driver'
+require 'rdoc/ri/driver'
+require 'rdoc/ri/util'
+
+require "irb/cmd/nop.rb"
 
 module IRB
   module ExtendCommand
-    module Help
+    class Help<Nop
       begin
-        @ri = RiDriver.new
+        Ri = RDoc::RI::Driver.new
       rescue SystemExit
       else
-        def self.execute(context, *names)
+        def execute(*names)
           names.each do |name|
             begin
-              @ri.get_info_for(name.to_s)
-            rescue RiError
+              Ri.get_info_for(name.to_s)
+            rescue RDoc::RI::Error
               puts $!.message
             end
           end
