@@ -1,9 +1,9 @@
 # scanf for Ruby
 #
-# $Revision: 21682 $
-# $Id: scanf.rb 21682 2009-01-20 03:23:46Z shyouhei $
-# $Author: shyouhei $
-# $Date: 2009-01-20 12:23:46 +0900 (Tue, 20 Jan 2009) $
+# $Release Version: 1.1.2 $
+# $Revision: 22784 $
+# $Id: scanf.rb 22784 2009-03-06 03:56:38Z nobu $
+# $Author: nobu $
 #
 # A product of the Austin Ruby Codefest (Austin, Texas, August 2002)
 
@@ -63,7 +63,7 @@ to the beginning of the format string, and yields a new array of
 conversions to the block every time the format string is matched
 (including partial matches, but not including complete failures).  The
 actual return value of scanf when called with a block is an array
-containing the results of all the executions of the block. 
+containing the results of all the executions of the block.
 
    str = "123 abc 456 def 789 ghi"
    str.scanf("%d%s") { |num,str| [ num * 2, str.upcase ] }
@@ -100,7 +100,7 @@ and <tt>tests/scanftests.rb</tt> for examples.)
 [u]
   Same as d.
 
-[i] 
+[i]
   Matches an optionally signed integer. The integer is read in base
   16 if it begins with `0x' or `0X', in base 8 if it begins with `0',
   and in base 10 other- wise. Only characters that correspond to the
@@ -280,7 +280,7 @@ Project contributors:: Nolan Darilek, Jason Johnston
 
 Thanks to Hal Fulton for hosting the Codefest.
 
-Thanks to Matz for suggestions about the class design.  
+Thanks to Matz for suggestions about the class design.
 
 Thanks to Gavin Sinclair for some feedback on the documentation.
 
@@ -325,14 +325,14 @@ module Scanf
     end
 
     def count_space?
-      /(?:\A|\S)%\*?\d*c|\[/.match(@spec_string)
+      /(?:\A|\S)%\*?\d*c|%\d*\[/.match(@spec_string)
     end
 
     def initialize(str)
       @spec_string = str
       h = '[A-Fa-f0-9]'
 
-      @re_string, @handler = 
+      @re_string, @handler =
         case @spec_string
 
           # %[[:...:]]
@@ -466,11 +466,11 @@ module Scanf
     end
 
     def letter
-      /%\*?\d*([a-z\[])/.match(@spec_string).to_a[1]
+      @spec_string[/%\*?\d*([a-z\[])/, 1]
     end
 
     def width
-      w = /%\*?(\d+)/.match(@spec_string).to_a[1]
+      w = @spec_string[/%\*?(\d+)/, 1]
       w && w.to_i
     end
 
@@ -482,7 +482,7 @@ module Scanf
 
       return width_left || cc_no_width
     end
-    
+
   end
 
   class FormatString
@@ -541,7 +541,8 @@ module Scanf
       @string_left = str
       @matched_count = 0
 
-      @specs.each_with_index do |spec,@i|
+      @specs.each_with_index do |spec,i|
+        @i=i
         @last_spec_tried = spec
         @last_match_tried = spec.match(@string_left)
         break unless @last_match_tried
@@ -671,10 +672,10 @@ class String
     if b
       block_scanf(fstr,&b)
     else
-      fs = 
+      fs =
         if fstr.is_a? Scanf::FormatString
-          fstr 
-        else 
+          fstr
+        else
           Scanf::FormatString.new(fstr)
         end
       fs.match(self)
