@@ -1,4 +1,4 @@
-require 'java'
+require File.dirname(__FILE__) + "/../spec_helper"
 
 describe "A Java object's java_method method" do
   before :each do
@@ -10,6 +10,11 @@ describe "A Java object's java_method method" do
     m = @list.java_method :toString
     m.call.should == "[]"
   end
+
+  it "works with name plus empty array for no-arg methods" do
+    m = @list.java_method :toString, []
+    m.call.should == "[]"
+  end
   
   it "works with a signature" do
     m = @list.java_method :add, [Java::int, java.lang.Object]
@@ -19,7 +24,9 @@ describe "A Java object's java_method method" do
   
   it "produces Method for static methods against an instance" do
     m = @integer.java_method :valueOf, [Java::int]
-    m.call(1).should == @integer
+    # JRUBY-4107
+    #m.call(1).should == @integer
+    m.call(1).should == 1
   end
   
   it "produces UnboundMethod for instance methods against a class" do
@@ -30,7 +37,9 @@ describe "A Java object's java_method method" do
   
   it "produces Method for static methods against a class" do
     m = java.lang.Integer.java_method :valueOf, [Java::int]
-    m.call(1).should == @integer
+    # JRUBY-4107
+    #m.call(1).should == @integer
+    m.call(1).should == 1
   end
 
   it "raises NameError if the method can't be found" do
@@ -50,4 +59,14 @@ describe "A Java object's java_method method" do
       java.lang.Integer.java_method :valueOf, [Java::long]
     end.should raise_error(NameError)
   end
-end
\ No newline at end of file
+
+  it "calls static methods" do
+    lambda {
+      import 'java_integration.fixtures.PackageStaticMethod'
+
+      method = PackageStaticMethod.java_class.declared_method_smart :thePackageScopeMethod
+      method.accessible = true
+      method.invoke Java.ruby_to_java(nil)
+    }.should_not raise_error
+  end
+end
