@@ -351,12 +351,10 @@ describe "Java String, primitive, and object-typed interface methods" do
     vri_handler.receiveShort(obj).should == obj
     vri.result.should == obj
     vri.result.class.should == Fixnum
-    
-    pending "char appears to be getting signed/unsigned-garbled" do
-      vri_handler.receiveChar(obj).should == obj
-      vri.result.should == obj
-      vri.result.class.should == Fixnum
-    end
+
+    vri_handler.receiveChar(obj).should == obj
+    vri.result.should == obj
+    vri.result.class.should == Fixnum
     
     vri_handler.receiveInt(obj).should == obj
     vri.result.should == obj
@@ -392,6 +390,63 @@ describe "Java String, primitive, and object-typed interface methods" do
   end
 end
 
+describe "Java primitive-typed interface methods" do
+  it "should coerce nil to zero-magnitude primitives" do
+    impl = Class.new {
+      attr_accessor :result
+      include ValueReceivingInterface
+
+      def receive_primitive(obj)
+        self.result = obj
+        nil
+      end
+
+      %w[Byte Short Char Int Long Float Double Null True False].each do |type|
+        alias_method "receive#{type}".intern, :receive_primitive
+      end
+    }
+    
+    vri = impl.new
+    vri_handler = ValueReceivingInterfaceHandler.new(vri);
+
+    vri_handler.receiveByte(nil).should == 0
+    vri.result.should == 0
+    vri.result.class.should == Fixnum
+
+    vri_handler.receiveShort(nil).should == 0
+    vri.result.should == 0
+    vri.result.class.should == Fixnum
+
+    vri_handler.receiveChar(nil).should == 0
+    vri.result.should == 0
+    vri.result.class.should == Fixnum
+
+    vri_handler.receiveInt(nil).should == 0
+    vri.result.should == 0
+    vri.result.class.should == Fixnum
+
+    vri_handler.receiveLong(nil).should == 0
+    vri.result.should == 0
+    vri.result.class.should == Fixnum
+
+    vri_handler.receiveFloat(nil).should == 0.0
+    vri.result.should == 0.0
+    vri.result.class.should == Float
+
+    vri_handler.receiveDouble(nil).should == 0.0
+    vri.result.should == 0.0
+    vri.result.class.should == Float
+
+    vri_handler.receiveTrue(nil).should == false
+    vri.result.should == false
+    vri.result.class.should == FalseClass
+
+    vri_handler.receiveFalse(nil).should == false
+    vri.result.should == false
+    vri.result.class.should == FalseClass
+  end
+end
+
 describe "Java primitive-box-typed interface methods" do
   it "should coerce to Ruby types for the implementer" do
     impl = Class.new {
@@ -424,11 +479,9 @@ describe "Java primitive-box-typed interface methods" do
     vri.result.should == obj
     vri.result.class.should == Fixnum
 
-    pending "char appears to be getting signed/unsigned-garbled" do
-      vri_handler.receiveCharObj(obj).should == obj
-      vri.result.should == obj
-      vri.result.class.should == Fixnum
-    end
+    vri_handler.receiveCharObj(obj).should == obj
+    vri.result.should == obj
+    vri.result.class.should == Fixnum
 
     vri_handler.receiveIntObj(obj).should == obj
     vri.result.should == obj
@@ -454,6 +507,63 @@ describe "Java primitive-box-typed interface methods" do
     vri.result.should == false
     vri.result.class.should == FalseClass
   end
+
+  it "should coerce to null" do
+    impl = Class.new {
+      attr_accessor :result
+      include ValueReceivingInterface
+
+      def receive_primitive_box(obj)
+        self.result = obj
+        nil
+      end
+
+      %w[Byte Short Char Int Long Float Double True False].each do |type|
+        alias_method :"receive#{type}Obj", :receive_primitive_box
+      end
+    }
+
+    vri = impl.new
+    vri_handler = ValueReceivingInterfaceHandler.new(vri);
+
+    obj = 1
+
+    vri_handler.receiveByteObj(nil).should == nil
+    vri.result.should == nil
+    vri.result.class.should == NilClass
+
+    vri_handler.receiveShortObj(nil).should == nil
+    vri.result.should == nil
+    vri.result.class.should == NilClass
+
+    vri_handler.receiveCharObj(nil).should == nil
+    vri.result.should == nil
+    vri.result.class.should == NilClass
+
+    vri_handler.receiveIntObj(nil).should == nil
+    vri.result.should == nil
+    vri.result.class.should == NilClass
+
+    vri_handler.receiveLongObj(nil).should == nil
+    vri.result.should == nil
+    vri.result.class.should == NilClass
+
+    vri_handler.receiveFloatObj(nil).should == nil
+    vri.result.should == nil
+    vri.result.class.should == NilClass
+
+    vri_handler.receiveDoubleObj(nil).should == nil
+    vri.result.should == nil
+    vri.result.class.should == NilClass
+
+    vri_handler.receiveTrueObj(nil).should == nil
+    vri.result.should == nil
+    vri.result.class.should == NilClass
+
+    vri_handler.receiveFalseObj(nil).should == nil
+    vri.result.should == nil
+    vri.result.class.should == NilClass
+  end
 end
 
 describe "Java types with package or private constructors" do
@@ -468,3 +578,215 @@ describe "Java types with protected constructors" do
     lambda { ProtectedConstructor.new }.should raise_error(TypeError)
   end
 end
+
+describe "Fixnum\#to_java" do
+  it "should coerce to java.lang.Long by default" do
+    long = 123.to_java
+    long.class.should == java.lang.Long
+  end
+
+  it "should allow coercing to other primitive types using symbolic names" do
+    byte = 123.to_java :byte
+    short = 123.to_java :short
+    char = 123.to_java :char
+    int = 123.to_java :int
+    long = 123.to_java :long
+    float = 123.to_java :float
+    double = 123.to_java :double
+
+    byte.class.should == java.lang.Byte
+    short.class.should == java.lang.Short
+    char.class.should == java.lang.Character
+    int.class.should == java.lang.Integer
+    long.class.should == java.lang.Long
+    float.class.should == java.lang.Float
+    double.class.should == java.lang.Double
+  end
+  
+  it "coerces to java.lang.Long when asked to coerce to java.lang.Object" do
+    obj = 123.to_java java.lang.Object
+    obj2 = 123.to_java :object
+
+    obj.class.should == java.lang.Long
+    obj2.class.should == java.lang.Long
+  end
+
+  it "should allow coercing to other primitive types using boxed classes" do
+    byte = 123.to_java java.lang.Byte
+    short = 123.to_java java.lang.Short
+    char = 123.to_java java.lang.Character
+    int = 123.to_java java.lang.Integer
+    long = 123.to_java java.lang.Long
+    float = 123.to_java java.lang.Float
+    double = 123.to_java java.lang.Double
+
+    byte.class.should == java.lang.Byte
+    short.class.should == java.lang.Short
+    char.class.should == java.lang.Character
+    int.class.should == java.lang.Integer
+    long.class.should == java.lang.Long
+    float.class.should == java.lang.Float
+    double.class.should == java.lang.Double
+  end
+
+  it "should allow coercing to other primitive types using boxed classes" do
+    byte = 123.to_java Java::byte
+    short = 123.to_java Java::short
+    char = 123.to_java Java::char
+    int = 123.to_java Java::int
+    long = 123.to_java Java::long
+    float = 123.to_java Java::float
+    double = 123.to_java Java::double
+
+    byte.class.should == java.lang.Byte
+    short.class.should == java.lang.Short
+    char.class.should == java.lang.Character
+    int.class.should == java.lang.Integer
+    long.class.should == java.lang.Long
+    float.class.should == java.lang.Float
+    double.class.should == java.lang.Double
+  end
+end
+
+describe "String\#to_java" do
+  it "coerces to java.lang.String by default" do
+    str = "123".to_java
+    str.class.should == java.lang.String
+  end
+
+  describe "when passed java.lang.String" do
+    it "coerces to java.lang.String" do
+      cs = "123".to_java java.lang.String
+
+      cs.class.should == java.lang.String
+    end
+  end
+
+  describe "when passed java.lang.CharSequence" do
+    it "coerces to java.lang.String" do
+      cs = "123".to_java java.lang.CharSequence
+
+      cs.class.should == java.lang.String
+    end
+  end
+
+  describe "when passed java.lang.Object" do
+    it "coerces to java.lang.String" do
+      cs = "123".to_java java.lang.Object
+
+      cs.class.should == java.lang.String
+    end
+  end
+
+  describe "when passed void (java.lang.Void.TYPE)" do
+    it "coerces to null" do
+      cs = "123".to_java Java::java.lang.Void::TYPE
+
+      cs.class.should == NilClass
+    end
+  end
+end
+
+describe "Class\#to_java" do
+  describe "when passed java.lang.Class.class" do
+    cls = java.lang.Class
+    it "coerces core classes to their Java class object" do
+      # TODO: add all core, native types here
+      [Object, Array, String, Hash, File, IO].each do |rubycls|
+        rubycls.to_java(cls).should == eval("cls.forName('org.jruby.Ruby#{rubycls}')")
+      end
+    end
+
+    it "provides nearest reified class for unreified user classes" do
+      rubycls = Class.new
+      rubycls.to_java(cls).should == cls.forName('org.jruby.RubyObject');
+    end
+
+    it "returns reified class for reified used classes" do
+      rubycls = Class.new
+      rubycls.become_java!
+      rubycls.to_java(cls).should == JRuby.reference(rubycls).reified_class
+    end
+
+    it "converts Java proxy classes to their JavaClass/java.lang.Class equivalent" do
+      java.util.ArrayList.to_java(cls).should == java.util.ArrayList.java_class
+    end
+  end
+
+  describe "when passed java.lang.Object.class" do
+    cls = java.lang.Object
+    it "coerces core classes to their Ruby class object" do
+      # TODO: add all core, native types here
+      [Object, Array, String, Hash, File, IO].each do |rubycls|
+        rubycls.to_java(cls).should == rubycls
+      end
+    end
+
+    it "coerces user classes to their Ruby class object" do
+      rubycls = Class.new
+      rubycls.to_java(cls).should == rubycls;
+    end
+
+    it "converts Java proxy classes to their proxy class (Ruby class) equivalent" do
+      java.util.ArrayList.to_java(cls).should == java.util.ArrayList
+    end
+  end
+end
+
+describe "Time\"to_java" do
+  describe "when passed java.util.Date" do
+    it "coerces to java.util.Date" do
+      t = Time.now
+      d = t.to_java(java.util.Date)
+      d.class.should == java.util.Date
+    end
+  end
+
+  describe "when passed java.util.Calendar" do
+    it "coerces to java.util.Calendar" do
+      t = Time.now
+      d = t.to_java(java.util.Calendar)
+      d.class.should < java.util.Calendar
+    end
+  end
+
+  describe "when passed java.sql.Date" do
+    it "coerces to java.sql.Date" do
+      t = Time.now
+      d = t.to_java(java.sql.Date)
+      d.class.should == java.sql.Date
+    end
+  end
+
+  describe "when passed java.sql.Time" do
+    it "coerces to java.sql.Time" do
+      t = Time.now
+      d = t.to_java(java.sql.Time)
+      d.class.should == java.sql.Time
+    end
+  end
+
+  describe "when passed java.sql.Timestamp" do
+    it "coerces to java.sql.Timestamp" do
+      t = Time.now
+      d = t.to_java(java.sql.Timestamp)
+      d.class.should == java.sql.Timestamp
+    end
+  end
+
+  describe "when passed org.joda.time.DateTime" do
+    it "coerces to org.joda.time.DateTime" do
+      t = Time.now
+      d = t.to_java(org.joda.time.DateTime)
+      d.class.should == org.joda.time.DateTime
+    end
+  end
+
+  describe "when passed java.lang.Object" do
+    it "coerces to java.util.Date" do
+      t = Time.now
+      d = t.to_java(java.lang.Object)
+      d.class.should == java.util.Date
+    end
+  end
+end
\ No newline at end of file
