@@ -42,7 +42,10 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.PrintStream;
+import java.io.File;
 
+import java.io.FileInputStream;
+import java.io.FileNotFoundException;
 import org.jruby.exceptions.MainExitException;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.exceptions.ThreadKill;
@@ -91,20 +94,35 @@ public class Main {
         Main main = new Main();
         
         try {
-            int status = main.run(args);
-            if (status != 0) {
-                System.exit(status);
+            Status status = main.run(args);
+            if (status.isExit()) {
+                System.exit(status.getStatus());
+            }
+        } catch (RaiseException rj) {
+            RubyException raisedException = rj.getException();
+            Ruby runtime = raisedException.getRuntime();
+            if (runtime.getSystemExit().isInstance(raisedException)) {
+                IRubyObject status = raisedException.callMethod(runtime.getCurrentContext(), "status");
+
+                if (status != null && !status.isNil()) {
+                    System.exit(RubyNumeric.fix2int(status));
+                }
+            } else {
+                rj.printStackTrace(System.err);
+                System.exit(1);
             }
-        } catch (RaiseException re) {
-            throw re;
         } catch (Throwable t) {
             // print out as a nice Ruby backtrace
             System.err.println(ThreadContext.createRawBacktraceStringFromThrowable(t));
+            while ((t = t.getCause()) != null) {
+                System.err.println("Caused by:");
+                System.err.println(ThreadContext.createRawBacktraceStringFromThrowable(t));
+            }
             System.exit(1);
         }
     }
 
-    public int run(String[] args) {
+    public Status run(String[] args) {
         try {
             config.processArguments(args);
             return run();
@@ -115,7 +133,7 @@ public class Main {
                     printUsage();
                 }
             }
-            return mee.getStatus();
+            return new Status(mee.getStatus());
         } catch (OutOfMemoryError oome) {
             // produce a nicer error since Rubyists aren't used to seeing this
             System.gc();
@@ -125,16 +143,16 @@ public class Main {
             if (memoryMax != null) {
                 message = " of " + memoryMax;
             }
-            System.err.println("Error: Your application used more memory than the safety cap" + message + ".");
-            System.err.println("Specify -J-Xmx####m to increase it (#### = cap size in MB).");
+            config.getError().println("Error: Your application used more memory than the safety cap" + message + ".");
+            config.getError().println("Specify -J-Xmx####m to increase it (#### = cap size in MB).");
             
             if (config.getVerbose()) {
-                System.err.println("Exception trace follows:");
+                config.getError().println("Exception trace follows:");
                 oome.printStackTrace();
             } else {
-                System.err.println("Specify -w for full OutOfMemoryError stack trace");
+                config.getError().println("Specify -w for full OutOfMemoryError stack trace");
             }
-            return 1;
+            return new Status(1);
         } catch (StackOverflowError soe) {
             // produce a nicer error since Rubyists aren't used to seeing this
             System.gc();
@@ -144,33 +162,33 @@ public class Main {
             if (stackMax != null) {
                 message = " of " + stackMax;
             }
-            System.err.println("Error: Your application used more stack memory than the safety cap" + message + ".");
-            System.err.println("Specify -J-Xss####k to increase it (#### = cap size in KB).");
+            config.getError().println("Error: Your application used more stack memory than the safety cap" + message + ".");
+            config.getError().println("Specify -J-Xss####k to increase it (#### = cap size in KB).");
             
             if (config.getVerbose()) {
-                System.err.println("Exception trace follows:");
+                config.getError().println("Exception trace follows:");
                 soe.printStackTrace();
             } else {
-                System.err.println("Specify -w for full StackOverflowError stack trace");
+                config.getError().println("Specify -w for full StackOverflowError stack trace");
             }
-            return 1;
+            return new Status(1);
         } catch (UnsupportedClassVersionError ucve) {
-            System.err.println("Error: Some library (perhaps JRuby) was built with a later JVM version.");
-            System.err.println("Please use libraries built with the version you intend to use or an earlier one.");
+            config.getError().println("Error: Some library (perhaps JRuby) was built with a later JVM version.");
+            config.getError().println("Please use libraries built with the version you intend to use or an earlier one.");
             
             if (config.getVerbose()) {
-                System.err.println("Exception trace follows:");
+                config.getError().println("Exception trace follows:");
                 ucve.printStackTrace();
             } else {
-                System.err.println("Specify -w for full UnsupportedClassVersionError stack trace");
+                config.getError().println("Specify -w for full UnsupportedClassVersionError stack trace");
             }
-            return 1;
+            return new Status(1);
         } catch (ThreadKill kill) {
-            return 0;
+            return new Status();
         }
     }
 
-    public int run() {
+    public Status run() {
         if (config.isShowVersion()) {
             showVersion();
         }
@@ -186,7 +204,7 @@ public class Main {
             if (config.shouldPrintProperties()) {
                 printProperties();
             }
-            return 0;
+            return new Status();
         }
 
         InputStream in   = config.getScriptSource();
@@ -197,22 +215,61 @@ public class Main {
             config.processArguments(args);
         }
         Ruby runtime     = Ruby.newInstance(config);
-        
+
         // set thread context JRuby classloader here, for the main thread
         try {
             Thread.currentThread().setContextClassLoader(runtime.getJRubyClassLoader());
         } catch (SecurityException se) {
             // can't set TC classloader
             if (runtime.getInstanceConfig().isVerbose()) {
-                System.err.println("WARNING: Security restrictions disallowed setting context classloader for main thread.");
+                config.getError().println("WARNING: Security restrictions disallowed setting context classloader for main thread.");
             }
         }
 
         if (in == null) {
             // no script to run, return success below
+        } else if (config.isxFlag() && !config.hasShebangLine()) {
+            // no shebang was found and x option is set
+            throw new MainExitException(1, "jruby: no Ruby script found in input (LoadError)");
         } else if (config.isShouldCheckSyntax()) {
-            runtime.parseFromMain(in, filename);
-            config.getOutput().println("Syntax OK");
+            int status = 0;
+            try {
+                runtime.parseFromMain(in, filename);
+                config.getOutput().println("Syntax OK for " + filename);
+            } catch (RaiseException re) {
+                status = -1;
+                if (re.getException().getMetaClass().getBaseName().equals("SyntaxError")) {
+                    config.getOutput().println("SyntaxError in " + re.getException().message(runtime.getCurrentContext()));
+                } else {
+                    throw re;
+                }
+            }
+            
+            if (config.getArgv().length > 0) {
+                for (String arg : config.getArgv()) {
+                    File argFile = new File(arg);
+                    if (argFile.exists()) {
+                        try {
+                            runtime.parseFromMain(new FileInputStream(argFile), arg);
+                            config.getOutput().println("Syntax OK for " + arg);
+                        } catch (FileNotFoundException fnfe) {
+                            status = -1;
+                            config.getOutput().println("File not found: " + arg);
+                        } catch (RaiseException re) {
+                            status = -1;
+                            if (re.getException().getMetaClass().getBaseName().equals("SyntaxError")) {
+                                config.getOutput().println("SyntaxError in " + re.getException().message(runtime.getCurrentContext()));
+                            } else {
+                                throw re;
+                            }
+                        }
+                    } else {
+                        status = -1;
+                        config.getOutput().println("File not found: " + arg);
+                    }
+                }
+            }
+            return new Status(status);
         } else {
             long now = -1;
 
@@ -244,23 +301,25 @@ public class Main {
                     IRubyObject status = raisedException.callMethod(runtime.getCurrentContext(), "status");
 
                     if (status != null && !status.isNil()) {
-                        return RubyNumeric.fix2int(status);
+                        return new Status(RubyNumeric.fix2int(status));
+                    } else {
+                        return new Status(0);
                     }
                 } else {
                     runtime.printError(raisedException);
-                    return 1;
+                    return new Status(1);
                 }
             }
         }
-        return 0;
+        return new Status();
     }
 
     private void showVersion() {
-        config.getOutput().print(config.getVersionString());
+        config.getOutput().println(config.getVersionString());
     }
 
     private void showCopyright() {
-        config.getOutput().print(config.getCopyrightString());
+        config.getOutput().println(config.getCopyrightString());
     }
 
     public void printUsage() {
@@ -273,7 +332,19 @@ public class Main {
     public void printProperties() {
         config.getOutput().print(config.getPropertyHelp());
     }
-    
+    /**
+     * The intent here is to gather up any options that might have
+     * been specified in the shebang line and return them so they can
+     * be merged into the ones specified on the commandline.  This is
+     * kind of a hopeless task because it's impossible to figure out
+     * where the command invocation stops and the parameters start.
+     * We try to work with the common scenarios where /usr/bin/env is
+     * used to invoke the jruby shell script, and skip any parameters
+     * it might have.  Then we look for the interpreter invokation and
+     * assume that the binary will have the word "ruby" in the name.
+     * This is error prone but should cover more cases than the
+     * previous code.
+     */
     private String[] parseShebangOptions(InputStream in) {
         BufferedReader reader = null;
         String[] result = new String[0];
@@ -282,22 +353,44 @@ public class Main {
             in.mark(1024);
             reader = new BufferedReader(new InputStreamReader(in, "iso-8859-1"), 8192);
             String firstLine = reader.readLine();
+
+            // Search for the shebang line in the given stream
+            // if it wasn't found on the first line and the -x option
+            // was specified
+            if (config.isxFlag()) {
+                while (firstLine != null && !isShebangLine(firstLine)) {
+                    firstLine = reader.readLine();
+                }
+            }
+
+            boolean usesEnv = false;
             if (firstLine.length() > 2 && firstLine.charAt(0) == '#' && firstLine.charAt(1) == '!') {
-                int index = firstLine.indexOf("ruby", 2);
-
-                // JRUBY-3456: This was not considering that the executable
-                // name may actually be "jruby.bat" and the arg processing
-                // should happen after it.
-                if (firstLine.indexOf("ruby.bat", 2) == index) {
-                    index += 4;
+                String[] options = firstLine.substring(2).split("\\s+");
+                int i;
+                for (i = 0; i < options.length; i++) {
+                    // Skip /usr/bin/env if it's first
+                    if (i == 0 && options[i].endsWith("/env")) {
+                        usesEnv = true;
+                        continue;
+                    }
+                    // Skip any assignments if /usr/bin/env is in play
+                    if (usesEnv && options[i].indexOf('=') > 0) {
+                        continue;
+                    }
+                    // Skip any commandline args if /usr/bin/env is in play
+                    if (usesEnv && options[i].startsWith("-")) {
+                        continue;
+                    }
+                    String basename = (new File(options[i])).getName();
+                    if (basename.indexOf("ruby") > 0) {
+                        break;
+                    }
                 }
-
-                if (firstLine.length() < index + 5) {
-                    in.reset();
-                    return result;
-                }
-                String option = firstLine.substring(index + 5);
-                result = option.split("\\s");
+                config.setHasShebangLine(true);
+                System.arraycopy(options, i, result, 0, options.length - i);
+            } else {
+                // No shebang line found
+                config.setHasShebangLine(false);
             }
         } catch (Exception ex) {
             // ignore error
@@ -308,4 +401,35 @@ public class Main {
         }
         return result;
     }
+
+    public static class Status {
+        private boolean isExit = false;
+        private int status = 0;
+
+        /**
+         * Creates a status object with the specified value and with explicit
+         * exit flag. An exit flag means that Kernel.exit() has been explicitly
+         * invoked during the run.
+         *
+         * @param status
+         *            The status value.
+         */
+        Status(int status) {
+            this.isExit = true;
+            this.status = status;
+        }
+
+        /**
+         * Creates a status object with 0 value and no explicit exit flag. 
+         */
+        Status() {}
+
+        public boolean isExit() { return isExit; }
+        public int getStatus() { return status; }
+    }
+
+    protected boolean isShebangLine(String line) {
+        return (line.length() > 2 && line.charAt(0) == '#' && line.charAt(1) == '!');
+    }
 }
+
