@@ -29,21 +29,22 @@ package org.jruby;
 
 import java.io.Serializable;
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 import org.jruby.anno.JRubyMethod;
+import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.javasupport.JavaObject;
+import org.jruby.javasupport.JavaUtil;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ObjectAllocator;
+import org.jruby.runtime.ObjectSpace;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
-import org.jruby.runtime.builtin.InstanceVariableTable;
 import org.jruby.runtime.builtin.InstanceVariables;
 import org.jruby.runtime.builtin.InternalVariables;
 import org.jruby.runtime.builtin.Variable;
@@ -57,23 +58,17 @@ import org.jruby.util.TypeConverter;
  * @author enebo
  */
 public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Comparable<IRubyObject>, CoreObjectType, InstanceVariables, InternalVariables {
-
+    private static final boolean DEBUG = false;
+    private static final Object[] NULL_OBJECT_ARRAY = new Object[0];
+    
     // The class of this object
     protected transient RubyClass metaClass;
 
-    protected int flags; // zeroed by jvm
+    // zeroed by jvm
+    protected int flags;
 
-    /**
-     * The variableTable contains variables for an object, defined as:
-     * <ul>
-     * <li> instance variables
-     * <li> class variables (for classes/modules)
-     * <li> internal variables (such as those used when marshaling RubyRange and RubyException)
-     * </ul>
-     *
-     * Constants are stored separately, see {@link RubyModule}.
-     */
-    protected transient volatile InstanceVariableTable variables;
+    // variable table, lazily allocated as needed (if needed)
+    private volatile Object[] varTable = NULL_OBJECT_ARRAY;
 
     /**
      * The error message used when some one tries to modify an
@@ -109,6 +104,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     public static final int USER6_F = (1<<(FL_USHIFT+6));
     public static final int USER7_F = (1<<(FL_USHIFT+7));
 
+    public static final int COMPARE_BY_IDENTITY_F = (1<<(FL_USHIFT+8));
 
     /**
      *  A value that is used as a null sentinel in among other places
@@ -154,7 +150,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     public static RubyClass createBasicObjectClass(Ruby runtime, RubyClass objectClass) {
         objectClass.index = ClassIndex.OBJECT;
 
-        //objectClass.defineAnnotatedMethods(BasicObjectMethods.class);
+        objectClass.defineAnnotatedMethods(BasicObjectMethods.class);
         objectClass.defineAnnotatedMethods(RubyBasicObject.class);
 
         return objectClass;
@@ -188,6 +184,13 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         if (runtime.getSafeLevel() >= 3) taint(runtime);
     }
 
+    /**
+     * Path for objects that don't taint and don't enter objectspace.
+     */
+    public RubyBasicObject(RubyClass metaClass) {
+        this.metaClass = metaClass;
+    }
+
     /**
      * Path for objects who want to decide whether they don't want to be in
      * ObjectSpace even when it is on. (notably used by objects being
@@ -306,6 +309,14 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         return RuntimeHelpers.invokeSuper(context, this, args, block);
     }
 
+    /**
+     * Will invoke a named method with no arguments and no block if that method or a custom
+     * method missing exists. Otherwise returns null. 1.9: rb_check_funcall
+     */
+    public final IRubyObject checkCallMethod(ThreadContext context, String name) {
+        return RuntimeHelpers.invokeChecked(context, this, name);
+    }
+
     /**
      * Will invoke a named method with no arguments and no block.
      */
@@ -329,6 +340,14 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         return RuntimeHelpers.invoke(context, this, name, args);
     }
 
+    public final IRubyObject callMethod(String name, IRubyObject... args) {
+        return RuntimeHelpers.invoke(getRuntime().getCurrentContext(), this, name, args);
+    }
+
+    public final IRubyObject callMethod(String name) {
+        return RuntimeHelpers.invoke(getRuntime().getCurrentContext(), this, name);
+    }
+
     /**
      * Will invoke a named method with the supplied arguments and
      * supplied block with functional invocation.
@@ -393,6 +412,9 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @param taint should this object be tainted or not?
      */
     public void setTaint(boolean taint) {
+        // JRUBY-4113: callers should not call setTaint on immediate objects
+        if (isImmediate()) return;
+        
         if (taint) {
             flags |= TAINTED_F;
         } else {
@@ -518,10 +540,9 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * class.
      */
     public RubyClass makeMetaClass(RubyClass superClass) {
-        MetaClass klass = new MetaClass(getRuntime(), superClass); // rb_class_boot
+        MetaClass klass = new MetaClass(getRuntime(), superClass, this); // rb_class_boot
         setMetaClass(klass);
 
-        klass.setAttached(this);
         klass.setMetaClass(superClass.getRealClass().getMetaClass());
 
         superClass.addSubclass(klass);
@@ -575,8 +596,9 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * translations.
      */
     public Class getJavaClass() {
-        if (dataGetStruct() instanceof JavaObject) {
-            return ((JavaObject)dataGetStruct()).getValue().getClass();
+        Object obj = dataGetStruct();
+        if (obj instanceof JavaObject) {
+            return ((JavaObject)obj).getValue().getClass();
         }
         return getClass();
     }
@@ -591,7 +613,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     public String asJavaString() {
         IRubyObject asString = checkStringType();
         if(!asString.isNil()) return ((RubyString)asString).asJavaString();
-        throw getRuntime().newTypeError(inspect().toString() + " is not a symbol");
+        throw getRuntime().newTypeError(inspect().toString() + " is not a string");
     }
 
     /** rb_obj_as_string
@@ -691,6 +713,21 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         return str;
     }
 
+    /** rb_check_string_type
+     *
+     * Tries to return a coerced string representation of this object,
+     * using "to_str". If that returns something other than a String
+     * or nil, an empty String will be returned.
+     *
+     */
+    public IRubyObject checkStringType19() {
+        IRubyObject str = TypeConverter.convertToTypeWithCheck19(this, getRuntime().getString(), "to_str");
+        if(!str.isNil() && !(str instanceof RubyString)) {
+            str = RubyString.newEmptyString(getRuntime());
+        }
+        return str;
+    }
+
     /** rb_check_array_type
     *
     * Returns the result of trying to convert this object to an Array
@@ -707,6 +744,36 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         return conv instanceof RubyInteger ? conv : obj.getRuntime().getNil();
     }
 
+    /**
+     * @see IRubyObject.toJava
+     */
+    public Object toJava(Class target) {
+        // for callers that unconditionally pass null retval type (JRUBY-4737)
+        if (target == void.class) return null;
+
+        if (dataGetStruct() instanceof JavaObject) {
+            // for interface impls
+
+            JavaObject innerWrapper = (JavaObject)dataGetStruct();
+
+            // ensure the object is associated with the wrapper we found it in,
+            // so that if it comes back we don't re-wrap it
+            if (target.isAssignableFrom(innerWrapper.getValue().getClass())) {
+                getRuntime().getJavaSupport().getObjectProxyCache().put(innerWrapper.getValue(), this);
+
+                return innerWrapper.getValue();
+            }
+        } else if (JavaUtil.isDuckTypeConvertable(getClass(), target)) {
+            if (!respondsTo("java_object")) {
+                return JavaUtil.convertProcToInterface(getRuntime().getCurrentContext(), this, target);
+            }
+        } else if (target.isAssignableFrom(getClass())) {
+            return this;
+        }
+        
+        throw getRuntime().newTypeError("cannot convert instance of " + getClass() + " to " + target);
+    }
+
     public IRubyObject dup() {
         if (isImmediate()) throw getRuntime().newTypeError("can't dup " + getMetaClass().getName());
 
@@ -777,31 +844,31 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @return either a real class, or a clone of the current singleton class
      */
     protected RubyClass getSingletonClassClone() {
-       RubyClass klass = getMetaClass();
+        RubyClass klass = getMetaClass();
 
-       if (!klass.isSingleton()) return klass;
+        if (!klass.isSingleton()) {
+            return klass;
+        }
 
-       MetaClass clone = new MetaClass(getRuntime());
-       clone.flags = flags;
+        MetaClass clone = new MetaClass(getRuntime(), klass.getSuperClass(), ((MetaClass) klass).getAttached());
+        clone.flags = flags;
 
-       if (this instanceof RubyClass) {
-           clone.setMetaClass(clone);
-       } else {
-           clone.setMetaClass(klass.getSingletonClassClone());
-       }
+        if (this instanceof RubyClass) {
+            clone.setMetaClass(clone);
+        } else {
+            clone.setMetaClass(klass.getSingletonClassClone());
+        }
 
-       clone.setSuperClass(klass.getSuperClass());
+        if (klass.hasVariables()) {
+            clone.syncVariables(klass.getVariableList());
+        }
+        clone.syncConstants(klass);
 
-       if (klass.hasVariables()) clone.syncVariables(klass.getVariableList());
-       clone.syncConstants(klass);
+        klass.cloneMethods(clone);
 
-       klass.cloneMethods(clone);
+        ((MetaClass) clone.getMetaClass()).setAttached(clone);
 
-       ((MetaClass)clone.getMetaClass()).setAttached(clone);
-
-       ((MetaClass)clone).setAttached(((MetaClass)klass).getAttached());
-
-       return clone;
+        return clone;
     }
 
     /**
@@ -846,14 +913,67 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         return fastGetInternalVariable("__wrap_struct__");
     }
 
+    // Equivalent of Data_Get_Struct
+    // This will first check that the object in question is actually a T_DATA equivalent.
+    public synchronized Object dataGetStructChecked() {
+        TypeConverter.checkData(this);
+        return this.fastGetInternalVariable("__wrap_struct__");
+    }
+
     /** rb_obj_id
      *
      * Return the internal id of an object.
-     *
-     * FIXME: Should this be renamed to match its ruby name?
      */
-    public synchronized IRubyObject id() {
-        return getRuntime().newFixnum(getRuntime().getObjectSpace().idOf(this));
+    public IRubyObject id() {
+        return getRuntime().newFixnum(getObjectId());
+    }
+
+    /**
+     * The logic here is to use the special objectId accessor slot from the
+     * parent as a lazy store for an object ID. IDs are generated atomically,
+     * in serial, and guaranteed unique for up to 2^63 objects. The special
+     * objectId slot is managed separately from the "normal" vars so it
+     * does not marshal, clone/dup, or refuse to be initially set when the
+     */
+    protected synchronized long getObjectId() {
+        // object is frozen.
+        synchronized (this) {
+            RubyClass.VariableAccessor objectIdAccessor = getMetaClass().getRealClass().getObjectIdAccessorForWrite();
+            Long id = (Long)objectIdAccessor.get(this);
+            if (id == null) {
+                return initObjectId(objectIdAccessor);
+            }
+            return id;
+        }
+    }
+
+    /**
+     * We lazily stand up the object ID since it forces us to stand up
+     * per-object state for a given object. We also check for ObjectSpace here,
+     * and normally we do not register a given object ID into ObjectSpace due
+     * to the high cost associated with constructing the related weakref. Most
+     * uses of id/object_id will only ever need it to be a unique identifier,
+     * and the id2ref behavior provided by ObjectSpace is considered internal
+     * and not generally supported.
+     * 
+     * @param objectIdAccessor The variable accessor to use for storing the
+     * generated object ID
+     * @return The generated object ID
+     */
+    protected synchronized long initObjectId(RubyClass.VariableAccessor objectIdAccessor) {
+        Ruby runtime = getRuntime();
+        long id;
+        
+        if (runtime.isObjectSpaceEnabled()) {
+            id = runtime.getObjectSpace().createAndRegisterObjectId(this);
+        } else {
+            id = ObjectSpace.calculateObjectId(this);
+        }
+        
+        // we use a direct path here to avoid frozen checks
+        setObjectId(objectIdAccessor.getIndex(), id);
+
+        return id;
     }
 
     /** rb_obj_inspect
@@ -871,28 +991,44 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     public IRubyObject inspect() {
         Ruby runtime = getRuntime();
         if ((!isImmediate()) && !(this instanceof RubyModule) && hasVariables()) {
-            StringBuilder part = new StringBuilder();
-            String cname = getMetaClass().getRealClass().getName();
-            part.append("#<").append(cname).append(":0x");
-            part.append(Integer.toHexString(System.identityHashCode(this)));
-
-            if (runtime.isInspecting(this)) {
-                /* 6:tags 16:addr 1:eos */
-                part.append(" ...>");
-                return runtime.newString(part.toString());
-            }
-            try {
-                runtime.registerInspecting(this);
-                return runtime.newString(inspectObj(part).toString());
-            } finally {
-                runtime.unregisterInspecting(this);
-            }
+            return hashyInspect();
         }
 
         if (isNil()) return RubyNil.inspect(this);
         return RuntimeHelpers.invoke(runtime.getCurrentContext(), this, "to_s");
     }
 
+    public IRubyObject hashyInspect() {
+        Ruby runtime = getRuntime();
+        StringBuilder part = new StringBuilder();
+        String cname = getMetaClass().getRealClass().getName();
+        part.append("#<").append(cname).append(":0x");
+        part.append(Integer.toHexString(inspectHashCode()));
+
+        if (runtime.isInspecting(this)) {
+            /* 6:tags 16:addr 1:eos */
+            part.append(" ...>");
+            return runtime.newString(part.toString());
+        }
+        try {
+            runtime.registerInspecting(this);
+            return runtime.newString(inspectObj(part).toString());
+        } finally {
+            runtime.unregisterInspecting(this);
+        }
+    }
+
+    /**
+     * For most objects, the hash used in the default #inspect is just the
+     * identity hashcode of the actual object.
+     *
+     * See org.jruby.java.proxies.JavaProxy for a divergent case.
+     *
+     * @return The identity hashcode of this object
+     */
+    protected int inspectHashCode() {
+        return System.identityHashCode(this);
+    }
 
     /** inspect_obj
      *
@@ -922,32 +1058,38 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
 
     @JRubyMethod(name = "!=", required = 1, compat = CompatVersion.RUBY1_9)
     public IRubyObject op_not_equal(ThreadContext context, IRubyObject other) {
-        return context.getRuntime().newBoolean(!equalInternal(context, this, other));
+        return context.getRuntime().newBoolean(!op_equal(context, other).isTrue());
     }
 
     public int compareTo(IRubyObject other) {
         return (int)callMethod(getRuntime().getCurrentContext(), "<=>", other).convertToInteger().getLongValue();
     }
 
+    public IRubyObject op_equal(ThreadContext context, IRubyObject obj) {
+        // Remain unimplemented due to problems with the double java hierarchy
+        return context.getRuntime().getNil();
+    }
+
     /** rb_obj_equal
      *
      * Will by default use identity equality to compare objects. This
      * follows the Ruby semantics.
+     *
+     * The name of this method doesn't follow the convention because hierarchy problems
      */
     @JRubyMethod(name = "==", required = 1, compat = CompatVersion.RUBY1_9)
-    public IRubyObject op_equal(ThreadContext context, IRubyObject obj) {
+    public IRubyObject op_equal_19(ThreadContext context, IRubyObject obj) {
         return this == obj ? context.getRuntime().getTrue() : context.getRuntime().getFalse();
     }
 
-    /** rb_equal
-     *
-     * The Ruby "===" method is used by default in case/when
-     * statements. The Object implementation first checks Java identity
-     * equality and then calls the "==" method too.
-     */
-    @JRubyMethod(name = "equal?", required = 1, compat = CompatVersion.RUBY1_9)
     public IRubyObject op_eqq(ThreadContext context, IRubyObject other) {
-        return context.getRuntime().newBoolean(equalInternal(context, this, other));
+        // Remain unimplemented due to problems with the double java hierarchy
+        return context.getRuntime().getNil();
+    }
+
+    @JRubyMethod(name = "equal?", required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject equal_p19(ThreadContext context, IRubyObject other) {
+        return op_equal_19(context, other);
     }
 
     /**
@@ -972,7 +1114,16 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     public void addFinalizer(IRubyObject f) {
         Finalizer finalizer = (Finalizer)fastGetInternalVariable("__finalizer__");
         if (finalizer == null) {
-            finalizer = new Finalizer(getRuntime().getObjectSpace().idOf(this));
+            // since this is the first time we're registering a finalizer, we
+            // must also register this object in ObjectSpace, so that future
+            // calls to undefine_finalizer, which takes an object ID, can
+            // locate the object properly. See JRUBY-4839.
+            long id = getObjectId();
+            RubyFixnum fixnumId = (RubyFixnum)id();
+
+            getRuntime().getObjectSpace().registerObjectId(id, this);
+
+            finalizer = new Finalizer(fixnumId);
             fastSetInternalVariable("__finalizer__", finalizer);
             getRuntime().addFinalizer(finalizer);
         }
@@ -991,14 +1142,47 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         }
     }
 
+    private Object[] getVariableTableForRead() {
+        return varTable;
+    }
+
+    private synchronized Object[] getVariableTableForWrite(int index) {
+        if (varTable == NULL_OBJECT_ARRAY) {
+            if (DEBUG) System.out.println("resizing from " + varTable.length + " to " + getMetaClass().getRealClass().getVariableTableSizeWithObjectId());
+            varTable = new Object[getMetaClass().getRealClass().getVariableTableSizeWithObjectId()];
+        } else if (varTable.length <= index) {
+            if (DEBUG) System.out.println("resizing from " + varTable.length + " to " + getMetaClass().getRealClass().getVariableTableSizeWithObjectId());
+            Object[] newTable = new Object[getMetaClass().getRealClass().getVariableTableSizeWithObjectId()];
+            System.arraycopy(varTable, 0, newTable, 0, varTable.length);
+            varTable = newTable;
+        }
+        return varTable;
+    }
+
+    public Object getVariable(int index) {
+        if (index < 0) return null;
+        Object[] ivarTable = getVariableTableForRead();
+        if (ivarTable.length > index) return ivarTable[index];
+        return null;
+    }
+
+    public synchronized void setVariable(int index, Object value) {
+        ensureInstanceVariablesSettable();
+        if (index < 0) return;
+        Object[] ivarTable = getVariableTableForWrite(index);
+        ivarTable[index] = value;
+    }
+
+    private synchronized void setObjectId(int index, long value) {
+        if (index < 0) return;
+        Object[] ivarTable = getVariableTableForWrite(index);
+        ivarTable[index] = value;
+    }
+
     //
     // COMMON VARIABLE METHODS
     //
 
-    public InstanceVariableTable getVariables() {
-        return variables;
-    }
-
     /**
      * Returns true if object has any variables, defined as:
      * <ul>
@@ -1010,30 +1194,31 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @return true if object has any variables, else false
      */
     public boolean hasVariables() {
-        return variableTableGetSize() > 0;
+        // we check both to exclude object_id
+        return getMetaClass().getRealClass().getVariableTableSize() > 0 && varTable.length > 0;
     }
 
     /**
      * Returns the amount of instance variables, class variables,
      * constants and internal variables this object has.
      */
+    @Deprecated
     public int getVariableCount() {
-        return variableTableGetSize();
+        // we use min to exclude object_id
+        return Math.min(varTable.length, getMetaClass().getRealClass().getVariableTableSize());
     }
 
     /**
      * Gets a list of all variables in this object.
      */
     // TODO: must override in RubyModule to pick up constants
-    public List<Variable<IRubyObject>> getVariableList() {
-        final ArrayList<Variable<IRubyObject>> list = new ArrayList<Variable<IRubyObject>>();
-        InstanceVariableTable variables = getVariables();
-        if (variables != null) {
-            variables.visit(new InstanceVariableTable.TryLockVisitor(this) {
-                public void visit(String name, Object value) {
-                    list.add(new VariableEntry<IRubyObject>(name, (IRubyObject)value));
-                }
-            });
+    public List<Variable<Object>> getVariableList() {
+        Map<String, RubyClass.VariableAccessor> ivarAccessors = getMetaClass().getRealClass().getVariableAccessorsForRead();
+        ArrayList<Variable<Object>> list = new ArrayList<Variable<Object>>();
+        for (Map.Entry<String, RubyClass.VariableAccessor> entry : ivarAccessors.entrySet()) {
+            Object value = entry.getValue().get(this);
+            if (value == null) continue;
+            list.add(new VariableEntry<Object>(entry.getKey(), value));
         }
         return list;
     }
@@ -1043,36 +1228,14 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
    // TODO: must override in RubyModule to pick up constants
    public List<String> getVariableNameList() {
-       final ArrayList<String> list = new ArrayList<String>();
-       InstanceVariableTable variables = getVariables();
-       if (variables != null) {
-           variables.visit(new InstanceVariableTable.Visitor() {
-               public void visit(String name, Object value) {
-                   list.add(name);
-               }
-           });
-       }
-       return list;
-    }
-
-    /**
-     * Gets internal access to the getmap for variables.
-     */
-    @SuppressWarnings("unchecked")
-    @Deprecated // born deprecated
-    public Map getVariableMap() {
-        return variableTableGetMap();
-    }
-
-    /**
-     * Check the syntax of a Ruby variable, including that it's longer
-     * than zero characters, and starts with either an @ or a capital
-     * letter.
-     */
-    // FIXME: this should go somewhere more generic -- maybe IdUtil
-    protected static final boolean isRubyVariable(String name) {
-        char c;
-        return name.length() > 0 && ((c = name.charAt(0)) == '@' || (c <= 'Z' && c >= 'A'));
+        Map<String, RubyClass.VariableAccessor> ivarAccessors = getMetaClass().getRealClass().getVariableAccessorsForRead();
+        ArrayList<String> list = new ArrayList<String>();
+        for (Map.Entry<String, RubyClass.VariableAccessor> entry : ivarAccessors.entrySet()) {
+            Object value = entry.getValue().get(this);
+            if (value == null) continue;
+            list.add(entry.getKey());
+        }
+        return list;
     }
 
     /**
@@ -1080,7 +1243,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * specified name.
      */
     protected boolean variableTableContains(String name) {
-        return variables == null ? false : variables.contains(name);
+        return getMetaClass().getRealClass().getVariableAccessorForRead(name).get(this) != null;
     }
 
     /**
@@ -1089,7 +1252,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * an interned Java String.
      */
     protected boolean variableTableFastContains(String internedName) {
-        return variables == null ? false : variables.fastContains(internedName);
+        return variableTableContains(internedName);
     }
 
     /**
@@ -1097,8 +1260,8 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      *
      * @return the object or null if not found
      */
-    protected IRubyObject variableTableFetch(String name) {
-        return variables == null ? null : (IRubyObject)variables.fetch(name);
+    protected Object variableTableFetch(String name) {
+        return getMetaClass().getRealClass().getVariableAccessorForRead(name).get(this);
     }
 
     /**
@@ -1108,20 +1271,14 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @return the object or null if not found
      */
     protected Object variableTableFastFetch(String internedName) {
-        return variables == null ? null : variables.fastFetch(internedName);
+        return variableTableFetch(internedName);
     }
 
     /**
      * Store a value in the variable store under the specific name.
      */
     protected Object variableTableStore(String name, Object value) {
-        synchronized(this) {
-            if (variables == null) {
-                variables = new InstanceVariableTable(name.intern(), value);
-            } else {
-                variables.store(name, value);
-            }
-        }
+        getMetaClass().getRealClass().getVariableAccessorForWrite(name).set(this, value);
         return value;
     }
 
@@ -1130,15 +1287,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * needs to be an interned Java String.
      */
     protected Object variableTableFastStore(String internedName, Object value) {
-        assert internedName == internedName.intern() : internedName + " not interned";
-        synchronized(this) {
-            if (variables == null) {
-                variables = new InstanceVariableTable(internedName, value);
-            } else {
-                variables.fastStore(internedName, value);
-            }
-        }
-        return value;
+        return variableTableStore(internedName, value);
     }
 
     /**
@@ -1147,46 +1296,24 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
     protected Object variableTableRemove(String name) {
         synchronized(this) {
-            return variables == null ? null : variables.remove(name);
+            Object value = getMetaClass().getRealClass().getVariableAccessorForRead(name).get(this);
+            getMetaClass().getRealClass().getVariableAccessorForWrite(name).set(this, null);
+            return value;
         }
     }
 
-    /**
-     * Get the size of the variable table.
-     */
-    protected int variableTableGetSize() {
-        return variables == null ? 0 : variables.getSize(); 
-    }
-
     /**
      * Synchronize the variable table with the argument. In real terms
      * this means copy all entries into a newly allocated table.
      */
-    protected void variableTableSync(List<Variable<IRubyObject>> vars) {
+    protected void variableTableSync(List<Variable<Object>> vars) {
         synchronized(this) {
-            variables = new InstanceVariableTable(vars);
+            for (Variable<Object> var : vars) {
+                variableTableStore(var.getName(), var.getValue());
+            }
         }
     }
 
-    /**
-     * Method to help ease transition to new variables implementation.
-     * Will likely be deprecated in the near future.
-     */
-    protected Map variableTableGetMap() {
-        HashMap map = new HashMap();
-        if (variables != null) variables.getMap(this, map);
-        return map;
-    }
-
-    /**
-     * Method to help ease transition to new variables implementation.
-     * Will likely be deprecated in the near future.
-     */
-    protected Map variableTableGetMap(Map map) {
-        if (variables != null) variables.getMap(this, map);
-        return map;
-    }
-
     //
     // INTERNAL VARIABLE METHODS
     //
@@ -1204,7 +1331,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @see org.jruby.runtime.builtin.InternalVariables#hasInternalVariable
      */
     public boolean hasInternalVariable(String name) {
-        assert !isRubyVariable(name);
+        assert !IdUtil.isRubyVariable(name);
         return variableTableContains(name);
     }
 
@@ -1212,15 +1339,15 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @see org.jruby.runtime.builtin.InternalVariables#fastHasInternalVariable
      */
     public boolean fastHasInternalVariable(String internedName) {
-        assert !isRubyVariable(internedName);
+        assert !IdUtil.isRubyVariable(internedName);
         return variableTableFastContains(internedName);
     }
 
     /**
      * @see org.jruby.runtime.builtin.InternalVariables#getInternalVariable
      */
-    public IRubyObject getInternalVariable(String name) {
-        assert !isRubyVariable(name);
+    public Object getInternalVariable(String name) {
+        assert !IdUtil.isRubyVariable(name);
         return variableTableFetch(name);
     }
 
@@ -1228,7 +1355,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @see org.jruby.runtime.builtin.InternalVariables#fastGetInternalVariable
      */
     public Object fastGetInternalVariable(String internedName) {
-        assert !isRubyVariable(internedName);
+        assert !IdUtil.isRubyVariable(internedName);
         return variableTableFastFetch(internedName);
     }
 
@@ -1236,7 +1363,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @see org.jruby.runtime.builtin.InternalVariables#setInternalVariable
      */
     public void setInternalVariable(String name, Object value) {
-        assert !isRubyVariable(name);
+        assert !IdUtil.isRubyVariable(name);
         variableTableStore(name, value);
     }
 
@@ -1244,7 +1371,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @see org.jruby.runtime.builtin.InternalVariables#fastSetInternalVariable
      */
     public void fastSetInternalVariable(String internedName, Object value) {
-        assert !isRubyVariable(internedName);
+        assert !IdUtil.isRubyVariable(internedName);
         variableTableFastStore(internedName, value);
     }
 
@@ -1252,7 +1379,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @see org.jruby.runtime.builtin.InternalVariables#removeInternalVariable
      */
     public Object removeInternalVariable(String name) {
-        assert !isRubyVariable(name);
+        assert !IdUtil.isRubyVariable(name);
         return variableTableRemove(name);
     }
 
@@ -1260,28 +1387,10 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * Sync one variable table with another - this is used to make
      * rbClone work correctly.
      */
-    public void syncVariables(List<Variable<IRubyObject>> variables) {
+    public void syncVariables(List<Variable<Object>> variables) {
         variableTableSync(variables);
     }
 
-    /**
-     * @see org.jruby.runtime.builtin.InternalVariables#getInternalVariableList
-     */
-    public List<Variable<Object>> getInternalVariableList() {
-        final ArrayList<Variable<Object>> list = new ArrayList<Variable<Object>>();
-        InstanceVariableTable variables = getVariables();
-        if (variables != null) {
-            variables.visit(new InstanceVariableTable.TryLockVisitor(this) {
-                public void visit(String name, Object value) {
-                    if (!isRubyVariable(name)) {
-                        list.add(new VariableEntry<Object>(name, value));
-                    }
-                }
-            });
-        }
-        return list;
-    }
-
     //
     // INSTANCE VARIABLE API METHODS
     //
@@ -1316,7 +1425,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
     public IRubyObject getInstanceVariable(String name) {
         assert IdUtil.isInstanceVariable(name);
-        return variableTableFetch(name);
+        return (IRubyObject)variableTableFetch(name);
     }
 
     /**
@@ -1356,37 +1465,31 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     }
 
     /**
-     * @see org.jruby.runtime.builtin.InstanceVariables#getInstanceVariableList
+     * Gets a list of all variables in this object.
      */
+    // TODO: must override in RubyModule to pick up constants
     public List<Variable<IRubyObject>> getInstanceVariableList() {
-        final ArrayList<Variable<IRubyObject>> list = new ArrayList<Variable<IRubyObject>>();
-        InstanceVariableTable variables = getVariables();
-        if (variables != null) {
-            variables.visit(new InstanceVariableTable.TryLockVisitor(this) {
-                public void visit(String name, Object value) {
-                    if (IdUtil.isInstanceVariable(name)) {
-                        list.add(new VariableEntry<IRubyObject>(name, (IRubyObject)value));
-                    }                    
-                }
-            });
+        Map<String, RubyClass.VariableAccessor> ivarAccessors = getMetaClass().getVariableAccessorsForRead();
+        ArrayList<Variable<IRubyObject>> list = new ArrayList<Variable<IRubyObject>>();
+        for (Map.Entry<String, RubyClass.VariableAccessor> entry : ivarAccessors.entrySet()) {
+            Object value = entry.getValue().get(this);
+            if (value == null || !(value instanceof IRubyObject) || !IdUtil.isInstanceVariable(entry.getKey())) continue;
+            list.add(new VariableEntry<IRubyObject>(entry.getKey(), (IRubyObject)value));
         }
         return list;
     }
 
     /**
-     * @see org.jruby.runtime.builtin.InstanceVariables#getInstanceVariableNameList
+     * Gets a name list of all variables in this object.
      */
-    public List<String> getInstanceVariableNameList() {
-        final ArrayList<String> list = new ArrayList<String>();
-        InstanceVariableTable variables = getVariables();
-        if (variables != null) {
-            variables.visit(new InstanceVariableTable.Visitor() {
-                public void visit(String name, Object value) {
-                    if (IdUtil.isInstanceVariable(name)) {
-                        list.add(name);
-                    }
-                }
-            });
+   // TODO: must override in RubyModule to pick up constants
+   public List<String> getInstanceVariableNameList() {
+        Map<String, RubyClass.VariableAccessor> ivarAccessors = getMetaClass().getRealClass().getVariableAccessorsForRead();
+        ArrayList<String> list = new ArrayList<String>();
+        for (Map.Entry<String, RubyClass.VariableAccessor> entry : ivarAccessors.entrySet()) {
+            Object value = entry.getValue().get(this);
+            if (value == null || !(value instanceof IRubyObject) || !IdUtil.isInstanceVariable(entry.getKey())) continue;
+            list.add(entry.getKey());
         }
         return list;
     }
@@ -1395,15 +1498,10 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @see org.jruby.runtime.builtin.InstanceVariables#getInstanceVariableNameList
      */
     public void copyInstanceVariablesInto(final InstanceVariables other) {
-        InstanceVariableTable variables = getVariables();
-        if (variables != null) {
-            variables.visit(new InstanceVariableTable.Visitor() {
-                public void visit(String name, Object value) {
-                    if (IdUtil.isInstanceVariable(name)) { 
-                        other.setInstanceVariable(name, (IRubyObject)value);
-                    }
-                }
-            });
+        for (Variable<IRubyObject> var : getInstanceVariableList()) {
+            synchronized (this) {
+                other.setInstanceVariable(var.getName(), var.getValue());
+            }
         }
     }
 
@@ -1433,17 +1531,31 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         throw new UnsupportedOperationException("Not supported yet.");
     }
 
+    /**
+     * A method to determine whether the method named by methodName is a builtin
+     * method.  This means a method with a JRubyMethod annotation written in
+     * Java.
+     *
+     * @param methodName to look for.
+     * @return true if so
+     */
+    public boolean isBuiltin(String methodName) {
+        DynamicMethod method = getMetaClass().searchMethodInner(methodName);
+
+        return method != null && method.isBuiltin();
+    }
+
     /**
      * Class that keeps track of the finalizers for the object under
      * operation.
      */
-    public class Finalizer implements Finalizable {
-        private long id;
+    public static class Finalizer implements Finalizable {
+        private RubyFixnum id;
         private IRubyObject firstFinalizer;
         private List<IRubyObject> finalizers;
         private AtomicBoolean finalized;
 
-        public Finalizer(long id) {
+        public Finalizer(RubyFixnum id) {
             this.id = id;
             this.finalized = new AtomicBoolean(false);
         }
@@ -1477,7 +1589,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         private void callFinalizer(IRubyObject finalizer) {
             RuntimeHelpers.invoke(
                     finalizer.getRuntime().getCurrentContext(),
-                    finalizer, "call", RubyBasicObject.this.id());
+                    finalizer, "call", id);
         }
     }
 }
