@@ -42,7 +42,6 @@ import org.jruby.anno.JRubyConstant;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
-import org.jruby.runtime.CallbackFactory;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
@@ -77,7 +76,7 @@ public class RubyBigDecimal extends RubyNumeric {
     @JRubyConstant
     public final static int SIGN_POSITIVE_INFINITE=3;
     @JRubyConstant
-    public final static int EXCEPTION_OVERFLOW=1;
+    public final static int EXCEPTION_OVERFLOW=8;
     @JRubyConstant
     public final static int SIGN_POSITIVE_ZERO=1;
     @JRubyConstant
@@ -112,8 +111,6 @@ public class RubyBigDecimal extends RubyNumeric {
     public static RubyClass createBigDecimal(Ruby runtime) {
         RubyClass result = runtime.defineClass("BigDecimal",runtime.getNumeric(), BIGDECIMAL_ALLOCATOR);
 
-        CallbackFactory callbackFactory = runtime.callbackFactory(RubyBigDecimal.class);
-
         runtime.getKernel().defineAnnotatedMethods(BigDecimalKernelMethods.class);
 
         result.setInternalModuleVariable("vpPrecLimit", RubyFixnum.zero(runtime));
@@ -162,9 +159,9 @@ public class RubyBigDecimal extends RubyNumeric {
 
     @JRubyMethod(name = "_dump", optional = 1, frame = true)
     public IRubyObject dump(IRubyObject[] args, Block unusedBlock) {
-        RubyString precision = RubyString.newUnicodeString(args[0].getRuntime(), "0:");
-        RubyString str = this.asString();
-        return precision.append(str);
+        RubyString precision = RubyString.newUnicodeString(getRuntime(), "0:");
+
+        return precision.append(asString());
     }
         
     @JRubyMethod(name = "_load", required = 1, frame = true, meta = true)
@@ -245,6 +242,17 @@ public class RubyBigDecimal extends RubyNumeric {
                 newExceptionMode = (value.isTrue()) ? (RubyFixnum)currentExceptionMode.callCoerced(context, "|", EXCEPTION_NaN)
                         : (RubyFixnum)currentExceptionMode.callCoerced(context, "&", new RubyFixnum(runtime, ~(EXCEPTION_NaN).getLongValue()));
             }
+            
+            RubyFixnum EXCEPTION_UNDERFLOW = (RubyFixnum)clazz.fastGetConstant("EXCEPTION_UNDERFLOW");
+            if ((longMode & EXCEPTION_UNDERFLOW.getLongValue()) != 0) {
+                newExceptionMode = (value.isTrue()) ? (RubyFixnum)currentExceptionMode.callCoerced(context, "|", EXCEPTION_UNDERFLOW)
+                        : (RubyFixnum)currentExceptionMode.callCoerced(context, "&", new RubyFixnum(runtime, ~(EXCEPTION_UNDERFLOW).getLongValue()));
+            }
+            RubyFixnum EXCEPTION_OVERFLOW = (RubyFixnum)clazz.fastGetConstant("EXCEPTION_OVERFLOW");
+            if ((longMode & EXCEPTION_OVERFLOW.getLongValue()) != 0) {
+                newExceptionMode = (value.isTrue()) ? (RubyFixnum)currentExceptionMode.callCoerced(context, "|", EXCEPTION_OVERFLOW)
+                        : (RubyFixnum)currentExceptionMode.callCoerced(context, "&", new RubyFixnum(runtime, ~(EXCEPTION_OVERFLOW).getLongValue()));
+            }
             c.setInternalModuleVariable("vpExceptionMode", newExceptionMode);
             return newExceptionMode;
         }
@@ -281,6 +289,30 @@ public class RubyBigDecimal extends RubyNumeric {
         return RoundingMode.valueOf((int)roundingMode.getLongValue());
     }
 
+    private static boolean isNaNExceptionMode(Ruby runtime) {
+        RubyFixnum currentExceptionMode = (RubyFixnum)runtime.fastGetClass("BigDecimal")
+                .searchInternalModuleVariable("vpExceptionMode");
+        RubyFixnum EXCEPTION_NaN = (RubyFixnum)runtime.fastGetClass("BigDecimal")
+                .fastGetConstant("EXCEPTION_NaN");
+        return (currentExceptionMode.getLongValue() & EXCEPTION_NaN.getLongValue()) != 0;
+    }
+
+    private static boolean isInfinityExceptionMode(Ruby runtime) {
+        RubyFixnum currentExceptionMode = (RubyFixnum)runtime.fastGetClass("BigDecimal")
+                .searchInternalModuleVariable("vpExceptionMode");
+        RubyFixnum EXCEPTION_INFINITY = (RubyFixnum)runtime.fastGetClass("BigDecimal")
+                .fastGetConstant("EXCEPTION_INFINITY");
+        return (currentExceptionMode.getLongValue() & EXCEPTION_INFINITY.getLongValue()) != 0;
+    }
+    
+    private static boolean isOverflowExceptionMode(Ruby runtime) {
+        RubyFixnum currentExceptionMode = (RubyFixnum)runtime.fastGetClass("BigDecimal")
+                .searchInternalModuleVariable("vpExceptionMode");
+        RubyFixnum EXCEPTION_OVERFLOW = (RubyFixnum)runtime.fastGetClass("BigDecimal")
+                .fastGetConstant("EXCEPTION_OVERFLOW");
+        return (currentExceptionMode.getLongValue() & EXCEPTION_OVERFLOW.getLongValue()) != 0;
+    }
+
     private static RubyBigDecimal getVpValue(IRubyObject v, boolean must) {
         if(v instanceof RubyBigDecimal) {
             return (RubyBigDecimal)v;
@@ -313,13 +345,14 @@ public class RubyBigDecimal extends RubyNumeric {
     @JRubyMethod(name = "new", required = 1, optional = 1, meta = true)
     public static RubyBigDecimal newInstance(IRubyObject recv, IRubyObject[] args) {
         BigDecimal decimal;
+        Ruby runtime = recv.getRuntime();
         if (args.length == 0) { 
             decimal = new BigDecimal(0);
         } else {
             String strValue = args[0].convertToString().toString();
             strValue = strValue.trim();
             if ("NaN".equals(strValue)) {
-                return newNaN(recv.getRuntime());
+                return newNaN(runtime);
             }
 
             Matcher m = INFINITY_PATTERN.matcher(strValue);
@@ -329,7 +362,7 @@ public class RubyBigDecimal extends RubyNumeric {
                 if ("-".equals(signGroup)) {
                     sign = -1;
                 }
-                return newInfinity(recv.getRuntime(), sign);
+                return newInfinity(runtime, sign);
             }
 
             // Clean-up string representation so that it could be understood
@@ -344,18 +377,21 @@ public class RubyBigDecimal extends RubyNumeric {
             try {
                 decimal = new BigDecimal(strValue);
             } catch(NumberFormatException e) {
+                if (isOverflowExceptionMode(runtime)) {
+                    throw runtime.newFloatDomainError("exponent overflow");
+                }
                 decimal = new BigDecimal(0);
             }
             if (decimal.signum() == 0) {
                 // MRI behavior: -0 and +0 are two different things
                 if (strValue.matches("^\\s*-.*")) {
-                    return newZero(recv.getRuntime(), -1);
+                    return newZero(runtime, -1);
                 } else {
-                    return newZero(recv.getRuntime(), 1);
+                    return newZero(runtime, 1);
                 }
             }
         }
-        return new RubyBigDecimal(recv.getRuntime(), decimal);
+        return new RubyBigDecimal(runtime, decimal);
     }
 
     private static RubyBigDecimal newZero(Ruby runtime, int sign) {
@@ -369,6 +405,9 @@ public class RubyBigDecimal extends RubyNumeric {
     }
 
     private static RubyBigDecimal newNaN(Ruby runtime) {
+        if (isNaNExceptionMode(runtime)) {
+            throw runtime.newFloatDomainError("Computation results to 'NaN'(Not a Number)");
+        }
         RubyBigDecimal rbd =  new RubyBigDecimal(runtime, BigDecimal.ZERO);
         rbd.isNaN = true;
         return rbd;
@@ -376,6 +415,9 @@ public class RubyBigDecimal extends RubyNumeric {
     
     private static RubyBigDecimal newInfinity(Ruby runtime, int sign) {
         RubyBigDecimal rbd =  new RubyBigDecimal(runtime, BigDecimal.ZERO);
+        if (isInfinityExceptionMode(runtime)) {
+            throw runtime.newFloatDomainError("Computation results to 'Infinity'");
+        }
         if (sign < 0) {
             rbd.infinitySign = -1;
         } else {
@@ -738,12 +780,18 @@ public class RubyBigDecimal extends RubyNumeric {
         if(rb == null) {
             IRubyObject ee = callCoerced(context, "<=>",r);
             if(ee.isNil()) {
-                return getRuntime().getNil();
+                if (op == '*') {
+                    return getRuntime().getNil();
+                } else if (op == '=' || isNaN()){
+                    return getRuntime().getFalse();
+                } else {
+                    throw getRuntime().newArgumentError("nil could not be coerced into a BigDecmil");
+                }
             }
             e = RubyNumeric.fix2int(ee);
         } else {
             if (isNaN() | rb.isNaN()) {
-                return getRuntime().getNil();
+                return (op == '*') ? getRuntime().getNil() : getRuntime().getFalse();
             }
             if (infinitySign != 0 || rb.infinitySign != 0) {
                 e = infinitySign - rb.infinitySign;
@@ -828,6 +876,7 @@ public class RubyBigDecimal extends RubyNumeric {
     }
 
     @JRubyMethod(name = "coerce", required = 1)
+    @Override
     public IRubyObject coerce(IRubyObject other) {
         IRubyObject obj;
         if(other instanceof RubyFloat) {
@@ -838,9 +887,17 @@ public class RubyBigDecimal extends RubyNumeric {
         return obj;
     }
 
+    @Override
     public double getDoubleValue() { return value.doubleValue(); }
+    
+    @Override
     public long getLongValue() { return value.longValue(); }
 
+    @Override
+    public BigInteger getBigIntegerValue() {
+        return value.toBigInteger();
+    }
+
     public RubyNumeric multiplyWith(ThreadContext context, RubyInteger value) { 
         return (RubyNumeric)op_mul(context, value);
     }
@@ -1139,6 +1196,21 @@ public class RubyBigDecimal extends RubyNumeric {
             return RubyFloat.newFloat(getRuntime(),
                     zeroSign < 0 ? -0.0 : 0.0);
         }
+        if (-value.scale() > RubyFloat.MAX_10_EXP) {
+            switch (value.signum()) {
+            case -1:
+                return RubyFloat.newFloat(getRuntime(), Double.NEGATIVE_INFINITY);
+            case 0:
+                return RubyFloat.newFloat(getRuntime(), 0);
+            case 1:
+                return RubyFloat.newFloat(getRuntime(), Double.POSITIVE_INFINITY);
+            default:
+                // eh?!
+            }
+        }
+        if (-value.scale() < RubyFloat.MIN_10_EXP) {
+            return RubyFloat.newFloat(getRuntime(), 0);
+        }
         return RubyFloat.newFloat(getRuntime(), value.doubleValue());
     }
 
