@@ -56,7 +56,9 @@ public class RubyBignum extends RubyInteger {
         RubyClass bignum = runtime.defineClass("Bignum", runtime.getInteger(),
                 ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
         runtime.setBignum(bignum);
+
         bignum.index = ClassIndex.BIGNUM;
+        bignum.setReifiedClass(RubyBignum.class);
         
         bignum.defineAnnotatedMethods(RubyBignum.class);
 
@@ -75,10 +77,12 @@ public class RubyBignum extends RubyInteger {
         this.value = value;
     }
     
+    @Override
     public int getNativeTypeIndex() {
         return ClassIndex.BIGNUM;
     }
     
+    @Override
     public Class<?> getJavaClass() {
         return BigInteger.class;
     }
@@ -99,14 +103,21 @@ public class RubyBignum extends RubyInteger {
         return new RubyBignum(runtime, new BigInteger(value));
     }
 
+    @Override
     public double getDoubleValue() {
         return big2dbl(this);
     }
 
+    @Override
     public long getLongValue() {
         return big2long(this);
     }
 
+    @Override
+    public BigInteger getBigIntegerValue() {
+        return value;
+    }
+
     /** Getter for property value.
      * @return Value of property value.
      */
@@ -187,7 +198,11 @@ public class RubyBignum extends RubyInteger {
      * 
      */
     public static BigInteger fix2big(RubyFixnum arg) {
-        return BigInteger.valueOf(arg.getLongValue());
+        return long2big(arg.getLongValue());
+    }
+
+    public static BigInteger long2big(long arg) {
+        return BigInteger.valueOf(arg);
     }
 
     /*  ================
@@ -198,12 +213,30 @@ public class RubyBignum extends RubyInteger {
     /** rb_big_to_s
      * 
      */
-    @JRubyMethod(name = "to_s", optional = 1)
     public IRubyObject to_s(IRubyObject[] args) {
-        int base = args.length == 0 ? 10 : num2int(args[0]);
+        switch (args.length) {
+        case 0:
+            return to_s();
+        case 1:
+            return to_s(args[0]);
+        default:
+            throw getRuntime().newArgumentError(args.length, 1);
+        }
+    }
+    
+    @JRubyMethod(name = "to_s")
+    @Override
+    public IRubyObject to_s() {
+        int base = 10;
+        return getRuntime().newString(getValue().toString(base));
+    }
+
+    @JRubyMethod(name = "to_s")
+    public IRubyObject to_s(IRubyObject arg0) {
+        int base = num2int(arg0);
         if (base < 2 || base > 36) {
             throw getRuntime().newArgumentError("illegal radix " + base);
-    }
+        }
         return getRuntime().newString(getValue().toString(base));
     }
 
@@ -211,6 +244,7 @@ public class RubyBignum extends RubyInteger {
      * 
      */
     @JRubyMethod(name = "coerce", required = 1)
+    @Override
     public IRubyObject coerce(IRubyObject other) {
         if (other instanceof RubyFixnum) {
             return getRuntime().newArray(newBignum(getRuntime(), ((RubyFixnum) other).getLongValue()), this);
@@ -243,6 +277,10 @@ public class RubyBignum extends RubyInteger {
         }
         return addOther(context, other);
     }
+    
+    public IRubyObject op_plus(ThreadContext context, long other) {
+        return addFixnum(other);
+    }
 
     private IRubyObject addFixnum(long other) {
         BigInteger result = value.add(BigInteger.valueOf(other));
@@ -279,6 +317,10 @@ public class RubyBignum extends RubyInteger {
         return subtractOther(context, other);
     }
 
+    public IRubyObject op_minus(ThreadContext context, long other) {
+        return subtractFixnum(other);
+    }
+
     private IRubyObject subtractFixnum(long other) {
         BigInteger result = value.subtract(BigInteger.valueOf(other));
         if (value.signum() < 0 && other > 0) return new RubyBignum(getRuntime(), result);
@@ -318,6 +360,12 @@ public class RubyBignum extends RubyInteger {
         return coerceBin(context, "*", other);
     }
 
+    public IRubyObject op_mul(ThreadContext context, long other) {
+        Ruby runtime = context.getRuntime();
+        BigInteger result = value.multiply(long2big(other));
+        return result.signum() == 0 ? RubyFixnum.zero(runtime) : new RubyBignum(runtime, result);
+    }
+
     /**
      * rb_big_divide. Shared part for both "/" and "div" operations.
      */
@@ -370,7 +418,8 @@ public class RubyBignum extends RubyInteger {
     /** rb_big_divmod
      * 
      */
-    @JRubyMethod(name = "divmod", required = 1)
+    @JRubyMethod(name = "divmod", required = 1, compat = CompatVersion.RUBY1_8)
+    @Override
     public IRubyObject divmod(ThreadContext context, IRubyObject other) {
         Ruby runtime = context.getRuntime();
         final BigInteger otherValue;
@@ -391,11 +440,20 @@ public class RubyBignum extends RubyInteger {
     	}
         return RubyArray.newArray(runtime, bignorm(runtime, results[0]), bignorm(runtime, results[1]));
     }
+    
+    @JRubyMethod(name = "divmod", required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject divmod19(ThreadContext context, IRubyObject other) {
+        if (!other.isNil() && other instanceof RubyFloat
+                && ((RubyFloat)other).getDoubleValue() == 0) {
+            throw context.getRuntime().newZeroDivisionError(); 
+        }
+        return divmod(context, other);
+    }
 
     /** rb_big_modulo
      * 
      */
-    @JRubyMethod(name = {"%", "modulo"}, required = 1)
+    @JRubyMethod(name = {"%", "modulo"}, required = 1, compat = CompatVersion.RUBY1_8)
     public IRubyObject op_mod(ThreadContext context, IRubyObject other) {
         final BigInteger otherValue;
         if (other instanceof RubyFixnum) {
@@ -411,13 +469,25 @@ public class RubyBignum extends RubyInteger {
         BigInteger result = value.mod(otherValue.abs());
         if (otherValue.signum() == -1 && result.signum() != 0) result = otherValue.add(result);
         return bignorm(runtime, result);
+    }
 
+    /** rb_big_modulo
+     * 
+     */
+    @JRubyMethod(name = {"%", "modulo"}, required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_mod19(ThreadContext context, IRubyObject other) {
+        if (!other.isNil() && other instanceof RubyFloat
+                && ((RubyFloat)other).getDoubleValue() == 0) {
+            throw context.getRuntime().newZeroDivisionError(); 
+        }
+        return op_mod(context, other);
     }
 
     /** rb_big_remainder
      * 
      */
-    @JRubyMethod(name = "remainder", required = 1)
+    @JRubyMethod(name = "remainder", required = 1, compat = CompatVersion.RUBY1_8)
+    @Override
     public IRubyObject remainder(ThreadContext context, IRubyObject other) {
         final BigInteger otherValue;
         if (other instanceof RubyFixnum) {
@@ -431,12 +501,21 @@ public class RubyBignum extends RubyInteger {
         if (otherValue.signum() == 0) throw runtime.newZeroDivisionError();
         return bignorm(runtime, value.remainder(otherValue));
     }
+    
+    @JRubyMethod(name = "remainder", required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject remainder19(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyFloat && ((RubyFloat) other).getDoubleValue() == 0) {
+            throw context.getRuntime().newZeroDivisionError();
+        }
+        return remainder(context, other);
+    }
 
     /** rb_big_quo
 
      * 
      */
-    @JRubyMethod(name = "quo", required = 1)
+    @JRubyMethod(name = "quo", required = 1, compat = CompatVersion.RUBY1_8)
+    @Override
     public IRubyObject quo(ThreadContext context, IRubyObject other) {
     	if (other instanceof RubyNumeric) {
             return RubyFloat.newFloat(getRuntime(), big2dbl(this) / ((RubyNumeric) other).getDoubleValue());
@@ -444,6 +523,14 @@ public class RubyBignum extends RubyInteger {
             return coerceBin(context, "quo", other);
     	}
     }
+    
+    @JRubyMethod(name = "quo", required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject quo19(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyInteger && ((RubyInteger) other).getDoubleValue() == 0) {
+            throw context.getRuntime().newZeroDivisionError();
+        }
+        return quo(context, other);
+    }
 
     /** rb_big_pow
      * 
@@ -479,7 +566,7 @@ public class RubyBignum extends RubyInteger {
      * 
      */
     @JRubyMethod(name = {"**", "power"}, required = 1, compat = CompatVersion.RUBY1_9)
-    public IRubyObject op_pow_19(ThreadContext context, IRubyObject other) {
+    public IRubyObject op_pow19(ThreadContext context, IRubyObject other) {
         Ruby runtime = context.getRuntime();
         if (other == RubyFixnum.zero(runtime)) return RubyFixnum.one(runtime);
         double d;
@@ -504,6 +591,10 @@ public class RubyBignum extends RubyInteger {
             getRuntime().getWarnings().warn(ID.MAY_BE_TOO_BIG, "in a**b, b may be too big", d);
         } else if (other instanceof RubyFloat) {
             d = ((RubyFloat) other).getDoubleValue();
+            if (this.compareTo(RubyFixnum.zero(runtime)) == -1
+                    &&  d != Math.round(d)) {
+                return RubyComplex.newComplexRaw(getRuntime(), this).callMethod(context, "**", other);
+            }
         } else {
             return coerceBin(context, "**", other);
 
@@ -514,21 +605,26 @@ public class RubyBignum extends RubyInteger {
     /** rb_big_and
      * 
      */
-    @JRubyMethod(name = "&", required = 1)
+    @JRubyMethod(name = "&", required = 1, compat = CompatVersion.RUBY1_8)
     public IRubyObject op_and(ThreadContext context, IRubyObject other) {
         other = other.convertToInteger();
         if (other instanceof RubyBignum) {
             return bignorm(getRuntime(), value.and(((RubyBignum) other).value));
-        } else if(other instanceof RubyFixnum) {
+        } else if (other instanceof RubyFixnum) {
             return bignorm(getRuntime(), value.and(fix2big((RubyFixnum)other)));
         }
         return coerceBin(context, "&", other);
     }
+    
+    @JRubyMethod(name = "&", required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_and19(ThreadContext context, IRubyObject other) {
+        return op_and(context, convertToInteger(context, other));
+    }
 
     /** rb_big_or
      * 
      */
-    @JRubyMethod(name = "|", required = 1)
+    @JRubyMethod(name = "|", required = 1, compat = CompatVersion.RUBY1_8)
     public IRubyObject op_or(ThreadContext context, IRubyObject other) {
         other = other.convertToInteger();
         if (other instanceof RubyBignum) {
@@ -539,21 +635,38 @@ public class RubyBignum extends RubyInteger {
         }
         return coerceBin(context, "|", other);
     }
+    
+    @JRubyMethod(name = "|", required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_or19(ThreadContext context, IRubyObject other) {
+        return op_or(context, convertToInteger(context, other));
+    }
 
     /** rb_big_xor
      * 
      */
-    @JRubyMethod(name = "^", required = 1)
+    @JRubyMethod(name = "^", required = 1, compat = CompatVersion.RUBY1_8)
     public IRubyObject op_xor(ThreadContext context, IRubyObject other) {
         other = other.convertToInteger();
         if (other instanceof RubyBignum) {
             return bignorm(getRuntime(), value.xor(((RubyBignum) other).value));
-		}
+        }
         if (other instanceof RubyFixnum) {
             return bignorm(getRuntime(), value.xor(BigInteger.valueOf(((RubyFixnum) other).getLongValue())));
         }
         return coerceBin(context, "^", other);
     }
+    
+    @JRubyMethod(name = "^", required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_xor19(ThreadContext context, IRubyObject other) {
+        return op_xor(context, convertToInteger(context, other));
+    }
+    
+    private IRubyObject convertToInteger(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyFloat) {
+            throw context.getRuntime().newTypeError("can't convert Float into Integer");
+        }
+        return other.convertToInteger();
+    }
 
     /** rb_big_neg     
      * 
@@ -665,12 +778,17 @@ public class RubyBignum extends RubyInteger {
         } else if (other instanceof RubyBignum) {
             otherValue = ((RubyBignum) other).value;
         } else if (other instanceof RubyFloat) {
-            return dbl_cmp(getRuntime(), big2dbl(this), ((RubyFloat) other).getDoubleValue());
+            RubyFloat flt = (RubyFloat)other;
+            if (flt.infinite_p().isTrue()) {
+                if (flt.getDoubleValue() > 0.0) return RubyFixnum.minus_one(getRuntime());
+                return RubyFixnum.one(getRuntime());
+            }
+            return dbl_cmp(getRuntime(), big2dbl(this), flt.getDoubleValue());
         } else {
             return coerceCmp(context, "<=>", other);
         }
 
-        // wow, the only time we can use the java protocol ;)        
+        // wow, the only time we can use the java protocol ;)
         return RubyFixnum.newFixnum(getRuntime(), value.compareTo(otherValue));
     }
 
@@ -700,6 +818,7 @@ public class RubyBignum extends RubyInteger {
      * 
      */
     @JRubyMethod(name = {"eql?", "==="}, required = 1)
+    @Override
     public IRubyObject eql_p(IRubyObject other) {
         if (other instanceof RubyBignum) {
             return value.compareTo(((RubyBignum)other).value) == 0 ? getRuntime().getTrue() : getRuntime().getFalse();
@@ -711,6 +830,7 @@ public class RubyBignum extends RubyInteger {
      * 
      */
     @JRubyMethod(name = "hash")
+    @Override
     public RubyFixnum hash() {
         return getRuntime().newFixnum(value.hashCode());
         }
