@@ -29,7 +29,9 @@ package org.jruby;
 
 import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyClass;
+import org.jruby.exceptions.Unrescuable;
 import org.jruby.runtime.Block;
+import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
@@ -40,8 +42,13 @@ import org.jruby.runtime.builtin.IRubyObject;
  */
 @JRubyClass(name="Continuation")
 public class RubyContinuation extends RubyObject {
-    public static class Continuation extends Error {
-        public IRubyObject[] args;
+    public static class Continuation extends Error implements Unrescuable {
+        public Continuation() {tag = null;}
+        public Continuation(String tag) {
+            this.tag = tag.intern();
+        }
+        public IRubyObject[] args = IRubyObject.NULL_ARRAY;
+        public final String tag;
         
         @Override
         public synchronized Throwable fillInStackTrace() {
@@ -54,6 +61,10 @@ public class RubyContinuation extends RubyObject {
     
     public static void createContinuation(Ruby runtime) {
         RubyClass cContinuation = runtime.defineClass("Continuation",runtime.getObject(),runtime.getObject().getAllocator());
+
+        cContinuation.index = ClassIndex.CONTINUATION;
+        cContinuation.setReifiedClass(RubyContinuation.class);
+        
         cContinuation.defineAnnotatedMethods(RubyContinuation.class);
         cContinuation.getSingletonClass().undefineMethod("new");
         
@@ -65,6 +76,21 @@ public class RubyContinuation extends RubyObject {
         this.continuation = new Continuation();
     }
 
+    /**
+     * A RubyContinuation used for catch/throw, which have a tag associated
+     *
+     * @param runtime Current JRuby runtime
+     * @param tag The tag to use
+     */
+    public RubyContinuation(Ruby runtime, String tag) {
+        super(runtime, runtime.getContinuation());
+        this.continuation = new Continuation(tag);
+    }
+
+    public Continuation getContinuation() {
+        return continuation;
+    }
+
     @JRubyMethod(name = {"call", "[]"}, rest = true, frame = true)
     public IRubyObject call(ThreadContext context, IRubyObject[] args) {
         if (disabled) {
