@@ -337,20 +337,39 @@ public class RubyEnumerable {
         return result;
     }
 
-    @JRubyMethod(name = {"to_a", "entries"})
-    public static IRubyObject to_a19(ThreadContext context, IRubyObject self) {
+    @JRubyMethod(name = {"to_a", "entries"}, compat = CompatVersion.RUBY1_8)
+    public static IRubyObject to_a(ThreadContext context, IRubyObject self) {
         Ruby runtime = context.getRuntime();
         RubyArray result = runtime.newArray();
         callEach(runtime, context, self, new AppendBlockCallback(runtime, result));
         return result;
     }
 
-    @JRubyMethod(name = {"to_a", "entries"}, rest = true)
+    @JRubyMethod(name = {"to_a", "entries"}, rest = true, compat = CompatVersion.RUBY1_8)
+    public static IRubyObject to_a(ThreadContext context, IRubyObject self, IRubyObject[] args) {
+        Ruby runtime = context.getRuntime();
+        RubyArray result = runtime.newArray();
+        RuntimeHelpers.invoke(context, self, "each", args, CallBlock.newCallClosure(self, runtime.getEnumerable(), 
+                Arity.OPTIONAL, new AppendBlockCallback(runtime, result), context));
+        return result;
+    }
+
+    @JRubyMethod(name = {"to_a", "entries"}, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject to_a19(ThreadContext context, IRubyObject self) {
+        Ruby runtime = context.getRuntime();
+        RubyArray result = runtime.newArray();
+        callEach(runtime, context, self, new AppendBlockCallback(runtime, result));
+        result.infectBy(self);
+        return result;
+    }
+
+    @JRubyMethod(name = {"to_a", "entries"}, rest = true, compat = CompatVersion.RUBY1_9)
     public static IRubyObject to_a19(ThreadContext context, IRubyObject self, IRubyObject[] args) {
         Ruby runtime = context.getRuntime();
         RubyArray result = runtime.newArray();
         RuntimeHelpers.invoke(context, self, "each", args, CallBlock.newCallClosure(self, runtime.getEnumerable(), 
                 Arity.OPTIONAL, new AppendBlockCallback(runtime, result), context));
+        result.infectBy(self);
         return result;
     }
 
@@ -365,7 +384,7 @@ public class RubyEnumerable {
         return result;
     }
 
-    public static IRubyObject sort_by(ThreadContext context, IRubyObject self, final Block block) {
+    public static IRubyObject sort_byCommon(ThreadContext context, IRubyObject self, final Block block) {
         final Ruby runtime = context.getRuntime();
         final ThreadContext localContext = context; // MUST NOT be used across threads
 
@@ -427,8 +446,8 @@ public class RubyEnumerable {
     }
 
     @JRubyMethod(name = "sort_by", frame = true)
-    public static IRubyObject sort_by19(ThreadContext context, IRubyObject self, final Block block) {
-        return block.isGiven() ? sort_by(context, self, block) : enumeratorize(context.getRuntime(), self, "sort_by");
+    public static IRubyObject sort_by(ThreadContext context, IRubyObject self, final Block block) {
+        return block.isGiven() ? sort_byCommon(context, self, block) : enumeratorize(context.getRuntime(), self, "sort_by");
     }
 
     @JRubyMethod(name = "grep", required = 1, frame = true)
@@ -468,11 +487,11 @@ public class RubyEnumerable {
         return result;
     }
 
-    public static IRubyObject detect(ThreadContext context, IRubyObject self, final Block block) {
-        return detect(context, self, null, block);
+    public static IRubyObject detectCommon(ThreadContext context, IRubyObject self, final Block block) {
+        return detectCommon(context, self, null, block);
     }
 
-    public static IRubyObject detect(ThreadContext context, IRubyObject self, IRubyObject ifnone, final Block block) {
+    public static IRubyObject detectCommon(ThreadContext context, IRubyObject self, IRubyObject ifnone, final Block block) {
         final Ruby runtime = context.getRuntime();
         final IRubyObject result[] = new IRubyObject[] { null };
         final ThreadContext localContext = context;
@@ -497,31 +516,65 @@ public class RubyEnumerable {
     }
 
     @JRubyMethod(name = "detect", frame = true)
-    public static IRubyObject detect19(ThreadContext context, IRubyObject self, final Block block) {
-        return block.isGiven() ? detect(context, self, block) : enumeratorize(context.getRuntime(), self, "detect");
+    public static IRubyObject detect(ThreadContext context, IRubyObject self, final Block block) {
+        boolean blockGiven = block.isGiven();
+
+        if (self instanceof RubyArray && blockGiven) return ((RubyArray) self).find(context, null, block);
+        
+        return block.isGiven() ? detectCommon(context, self, block) : enumeratorize(context.getRuntime(), self, "detect");
     }
 
     @JRubyMethod(name = "detect", frame = true)
-    public static IRubyObject detect19(ThreadContext context, IRubyObject self, IRubyObject ifnone, final Block block) {
-        return block.isGiven() ? detect(context, self, ifnone, block) : enumeratorize(context.getRuntime(), self, "detect", ifnone);
+    public static IRubyObject detect(ThreadContext context, IRubyObject self, IRubyObject ifnone, final Block block) {
+        boolean blockGiven = block.isGiven();
+
+        if (self instanceof RubyArray && blockGiven) return ((RubyArray) self).find(context, null, block);
+
+        return block.isGiven() ? detectCommon(context, self, ifnone, block) : enumeratorize(context.getRuntime(), self, "detect", ifnone);
+    }
+
+    // FIXME: Custom Array enumeratorize should be made for all of these methods which skip Array without a supplied block.
+    @JRubyMethod(name = "find", frame = true)
+    public static IRubyObject find(ThreadContext context, IRubyObject self, final Block block) {
+        boolean blockGiven = block.isGiven();
+
+        if (self instanceof RubyArray && blockGiven) return ((RubyArray) self).find(context, null, block);
+
+        return blockGiven ? detectCommon(context, self, block) : enumeratorize(context.getRuntime(), self, "find");
     }
 
     @JRubyMethod(name = "find", frame = true)
-    public static IRubyObject find19(ThreadContext context, IRubyObject self, final Block block) {
-        return block.isGiven() ? detect(context, self, block) : enumeratorize(context.getRuntime(), self, "find");
+    public static IRubyObject find(ThreadContext context, IRubyObject self, IRubyObject ifnone, final Block block) {
+        boolean blockGiven = block.isGiven();
+
+        if (self instanceof RubyArray && blockGiven) return ((RubyArray) self).find(context, ifnone, block);
+
+        return blockGiven ? detectCommon(context, self, ifnone, block) :
+            enumeratorize(context.getRuntime(), self, "find", ifnone);
     }
 
-    @JRubyMethod(name = "find", frame = true)
-    public static IRubyObject find19(ThreadContext context, IRubyObject self, IRubyObject ifnone, final Block block) {
-        return block.isGiven() ? detect(context, self, ifnone, block) : enumeratorize(context.getRuntime(), self, "find", ifnone);
-    }
-    
     @JRubyMethod(name = "find_index", frame = true)
     public static IRubyObject find_index(ThreadContext context, IRubyObject self, final Block block) {
+        boolean blockGiven = block.isGiven();
+
+        if (self instanceof RubyArray && blockGiven) return ((RubyArray) self).find_index(context, block);
+        
+        return blockGiven ? find_indexCommon(context, self, block) :
+            enumeratorize(context.getRuntime(), self, "find_index");
+    }
+
+    @JRubyMethod(name = "find_index", frame = true)
+    public static IRubyObject find_index(ThreadContext context, IRubyObject self, final IRubyObject cond, final Block block) {
         final Ruby runtime = context.getRuntime();
 
-        if (!block.isGiven()) return enumeratorize(runtime, self, "find_index");
+        if (block.isGiven()) runtime.getWarnings().warn(ID.BLOCK_UNUSED , "given block not used");
+        if (self instanceof RubyArray) return ((RubyArray) self).find_index(context, cond);
 
+        return find_indexCommon(context, self, cond);
+    }
+
+    public static IRubyObject find_indexCommon(ThreadContext context, IRubyObject self, final Block block) {
+        final Ruby runtime = context.getRuntime();
         final long result[] = new long[] {0};
 
         try {
@@ -536,15 +589,12 @@ public class RubyEnumerable {
         } catch (JumpException.SpecialJump sj) {
             return RubyFixnum.newFixnum(runtime, result[0]);
         }
+        
         return runtime.getNil();
     }
 
-    @JRubyMethod(name = "find_index", frame = true)
-    public static IRubyObject find_index(ThreadContext context, IRubyObject self, final IRubyObject cond, final Block block) {
+    public static IRubyObject find_indexCommon(ThreadContext context, IRubyObject self, final IRubyObject cond) {
         final Ruby runtime = context.getRuntime();
-
-        if (block.isGiven()) runtime.getWarnings().warn(ID.BLOCK_UNUSED , "given block not used");
-
         final long result[] = new long[] {0};
 
         try {
@@ -559,10 +609,11 @@ public class RubyEnumerable {
         } catch (JumpException.SpecialJump sj) {
             return RubyFixnum.newFixnum(runtime, result[0]);
         }
+
         return runtime.getNil();
     }
 
-    public static IRubyObject select(ThreadContext context, IRubyObject self, final Block block) {
+    public static IRubyObject selectCommon(ThreadContext context, IRubyObject self, final Block block) {
         final Ruby runtime = context.getRuntime();
         final RubyArray result = runtime.newArray();
 
@@ -582,16 +633,16 @@ public class RubyEnumerable {
     }
 
     @JRubyMethod(name = "select", frame = true)
-    public static IRubyObject select19(ThreadContext context, IRubyObject self, final Block block) {
-        return block.isGiven() ? select(context, self, block) : enumeratorize(context.getRuntime(), self, "select");
+    public static IRubyObject select(ThreadContext context, IRubyObject self, final Block block) {
+        return block.isGiven() ? selectCommon(context, self, block) : enumeratorize(context.getRuntime(), self, "select");
     }
 
     @JRubyMethod(name = "find_all", frame = true)
-    public static IRubyObject find_all19(ThreadContext context, IRubyObject self, final Block block) {
-        return block.isGiven() ? select(context, self, block) : enumeratorize(context.getRuntime(), self, "find_all");
+    public static IRubyObject find_all(ThreadContext context, IRubyObject self, final Block block) {
+        return block.isGiven() ? selectCommon(context, self, block) : enumeratorize(context.getRuntime(), self, "find_all");
     }
 
-    public static IRubyObject reject(ThreadContext context, IRubyObject self, final Block block) {
+    public static IRubyObject rejectCommon(ThreadContext context, IRubyObject self, final Block block) {
         final Ruby runtime = context.getRuntime();
         final RubyArray result = runtime.newArray();
 
@@ -611,8 +662,8 @@ public class RubyEnumerable {
     }
 
     @JRubyMethod(name = "reject", frame = true)
-    public static IRubyObject reject19(ThreadContext context, IRubyObject self, final Block block) {
-        return block.isGiven() ? reject(context, self, block) : enumeratorize(context.getRuntime(), self, "reject");
+    public static IRubyObject reject(ThreadContext context, IRubyObject self, final Block block) {
+        return block.isGiven() ? rejectCommon(context, self, block) : enumeratorize(context.getRuntime(), self, "reject");
     }
 
     @JRubyMethod(name = {"collect", "map"}, frame = true, compat = CompatVersion.RUBY1_8)
@@ -674,7 +725,43 @@ public class RubyEnumerable {
         return result;
     }
 
-    public static IRubyObject inject(ThreadContext context, IRubyObject self, IRubyObject init, final Block block) {
+    @JRubyMethod(name = {"flat_map"}, frame = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject flat_map19(ThreadContext context, IRubyObject self, final Block block) {
+        return flatMapCommon19(context, self, block, "flat_map");
+    }
+
+    @JRubyMethod(name = {"collect_concat"}, frame = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject collect_concat19(ThreadContext context, IRubyObject self, final Block block) {
+        return flatMapCommon19(context, self, block, "collect_concat");
+    }
+
+    private static IRubyObject flatMapCommon19(ThreadContext context, IRubyObject self, final Block block, String methodName) {
+        final Ruby runtime = context.getRuntime();
+        if(block.isGiven()) {
+            final RubyArray ary = runtime.newArray();
+
+            callEach(runtime, context, self, new BlockCallback() {
+                public IRubyObject call(ThreadContext ctx, IRubyObject[] largs, Block blk) {
+                    IRubyObject larg = checkArgs(runtime, largs);
+                    IRubyObject i = block.yield(ctx, larg);
+                    IRubyObject tmp = i.checkArrayType();
+                    synchronized(ary) {
+                        if(tmp.isNil()) {
+                            ary.append(i);
+                        } else {
+                            ary.concat(tmp);
+                        }
+                    }
+                    return runtime.getNil();
+                }
+            });
+            return ary;
+        } else {
+            return enumeratorize(runtime, self, methodName);
+        }
+    }
+
+    public static IRubyObject injectCommon(ThreadContext context, IRubyObject self, IRubyObject init, final Block block) {
         final Ruby runtime = context.getRuntime();
         final IRubyObject result[] = new IRubyObject[] { init };
         final ThreadContext localContext = context;
@@ -684,7 +771,7 @@ public class RubyEnumerable {
                 IRubyObject larg = checkArgs(runtime, largs);
                 checkContext(localContext, ctx, "inject");
                 result[0] = result[0] == null ? 
-                        larg : block.yield(ctx, runtime.newArray(result[0], larg), null, null, true);
+                        larg : block.yieldArray(ctx, runtime.newArray(result[0], larg), null, null);
 
                 return runtime.getNil();
             }
@@ -694,17 +781,17 @@ public class RubyEnumerable {
     }
 
     @JRubyMethod(name = {"inject", "reduce"}, frame = true)
-    public static IRubyObject inject19(ThreadContext context, IRubyObject self, final Block block) {
-        return inject(context, self, null, block);
+    public static IRubyObject inject(ThreadContext context, IRubyObject self, final Block block) {
+        return injectCommon(context, self, null, block);
     }
     
     @JRubyMethod(name = {"inject", "reduce"}, frame = true)
-    public static IRubyObject inject19(ThreadContext context, IRubyObject self, IRubyObject arg, final Block block) {
-        return block.isGiven() ? inject(context, self, arg, block) : inject19(context, self, null, arg, block);
+    public static IRubyObject inject(ThreadContext context, IRubyObject self, IRubyObject arg, final Block block) {
+        return block.isGiven() ? injectCommon(context, self, arg, block) : inject(context, self, null, arg, block);
     }
 
     @JRubyMethod(name = {"inject", "reduce"}, frame = true)
-    public static IRubyObject inject19(ThreadContext context, IRubyObject self, IRubyObject init, IRubyObject method, final Block block) {
+    public static IRubyObject inject(ThreadContext context, IRubyObject self, IRubyObject init, IRubyObject method, final Block block) {
         final Ruby runtime = context.getRuntime();
 
         if (block.isGiven()) runtime.getWarnings().warn(ID.BLOCK_UNUSED , "given block not used");
@@ -722,7 +809,7 @@ public class RubyEnumerable {
         return result[0] == null ? runtime.getNil() : result[0];
     }
 
-    public static IRubyObject partition(ThreadContext context, IRubyObject self, final Block block) {
+    public static IRubyObject partitionCommon(ThreadContext context, IRubyObject self, final Block block) {
         final Ruby runtime = context.getRuntime();
         final RubyArray arr_true = runtime.newArray();
         final RubyArray arr_false = runtime.newArray();
@@ -748,8 +835,8 @@ public class RubyEnumerable {
     }
 
     @JRubyMethod(name = "partition", frame = true)
-    public static IRubyObject partition19(ThreadContext context, IRubyObject self, final Block block) {
-        return block.isGiven() ? partition(context, self, block) : enumeratorize(context.getRuntime(), self, "partition");
+    public static IRubyObject partition(ThreadContext context, IRubyObject self, final Block block) {
+        return block.isGiven() ? partitionCommon(context, self, block) : enumeratorize(context.getRuntime(), self, "partition");
     }
 
     private static class EachWithIndex implements BlockCallback {
@@ -763,35 +850,54 @@ public class RubyEnumerable {
         }
 
         public IRubyObject call(ThreadContext context, IRubyObject[] iargs, Block block) {
-            this.block.call(context, new IRubyObject[] { runtime.newArray(checkArgs(runtime, iargs), runtime.newFixnum(index++)) });
+            switch (iargs.length) {
+            case 0:
+                // FIXME: Does this ever happen?
+            case 1:
+                this.block.call(context, checkArgs(runtime, iargs), runtime.newFixnum(index++));
+                break;
+            case 2:
+                this.block.call(context, runtime.newArrayNoCopy(iargs), runtime.newFixnum(index++));
+                break;
+            }
             return runtime.getNil();            
         }
     }
 
-    public static IRubyObject each_with_index(ThreadContext context, IRubyObject self, Block block) {
+    public static IRubyObject each_with_indexCommon(ThreadContext context, IRubyObject self, Block block) {
         callEach(context.getRuntime(), context, self, new EachWithIndex(context, block));
         return self;
     }
 
-    @JRubyMethod(name = "each_with_index", frame = true)
-    public static IRubyObject each_with_index19(ThreadContext context, IRubyObject self, Block block) {
-        return block.isGiven() ? each_with_index(context, self, block) : enumeratorize(context.getRuntime(), self, "each_with_index");
+    public static IRubyObject each_with_indexCommon19(ThreadContext context, IRubyObject self, Block block, IRubyObject[] args) {
+        callEach(context.getRuntime(), context, self, args, new EachWithIndex(context, block));
+        return self;
+    }
+
+    @JRubyMethod(name = "each_with_index", frame = true, compat = CompatVersion.RUBY1_8)
+    public static IRubyObject each_with_index(ThreadContext context, IRubyObject self, Block block) {
+        return block.isGiven() ? each_with_indexCommon(context, self, block) : enumeratorize(context.getRuntime(), self, "each_with_index");
+    }
+
+    @JRubyMethod(name = "each_with_index", frame = true, rest = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject each_with_index19(ThreadContext context, IRubyObject self, IRubyObject[] args, Block block) {
+        return block.isGiven() ? each_with_indexCommon19(context, self, block, args) : enumeratorize(context.getRuntime(), self, "each_with_index", args);
     }
 
     @JRubyMethod(name = "enum_with_index", frame = true)
-    public static IRubyObject enum_with_index19(ThreadContext context, IRubyObject self, Block block) {
-        return block.isGiven() ? each_with_index(context, self, block) : enumeratorize(context.getRuntime(), self, "enum_with_index");
+    public static IRubyObject enum_with_index(ThreadContext context, IRubyObject self, Block block) {
+        return block.isGiven() ? each_with_indexCommon(context, self, block) : enumeratorize(context.getRuntime(), self, "enum_with_index");
     }
 
     @JRubyMethod(name = "reverse_each", frame = true)
-    public static IRubyObject reverse_each19(ThreadContext context, IRubyObject self, Block block) {
-        return block.isGiven() ? reverse_eachInternal(context, self, to_a19(context, self), block) :
+    public static IRubyObject reverse_each(ThreadContext context, IRubyObject self, Block block) {
+        return block.isGiven() ? reverse_eachInternal(context, self, to_a(context, self), block) :
             enumeratorize(context.getRuntime(), self, "reverse_each");
     }
 
     @JRubyMethod(name = "reverse_each", frame = true, rest = true)
-    public static IRubyObject reverse_each19(ThreadContext context, IRubyObject self, IRubyObject[] args, Block block) {
-        return block.isGiven() ? reverse_eachInternal(context, self, to_a19(context, self, args), block) :
+    public static IRubyObject reverse_each(ThreadContext context, IRubyObject self, IRubyObject[] args, Block block) {
+        return block.isGiven() ? reverse_eachInternal(context, self, to_a(context, self, args), block) :
             enumeratorize(context.getRuntime(), self, "reverse_each", args);
     }
 
@@ -834,8 +940,8 @@ public class RubyEnumerable {
                 public IRubyObject call(ThreadContext ctx, IRubyObject[] largs, Block blk) {
                     IRubyObject larg = checkArgs(runtime, largs);
                     checkContext(localContext, ctx, "max{}");
-                    if (result[0] == null || RubyComparable.cmpint(ctx, block.yield(ctx, 
-                            runtime.newArray(larg, result[0]), null, null, true), larg, result[0]) > 0) {
+                    if (result[0] == null || RubyComparable.cmpint(ctx, block.yieldArray(ctx,
+                            runtime.newArray(larg, result[0]), null, null), larg, result[0]) > 0) {
                         result[0] = larg;
                     }
                     return runtime.getNil();
@@ -1110,9 +1216,15 @@ public class RubyEnumerable {
         }
         return result[0] ? runtime.getTrue() : runtime.getFalse();
     }
-    
+
     @JRubyMethod(name = "all?", frame = true)
     public static IRubyObject all_p(ThreadContext context, IRubyObject self, final Block block) {
+        if (self instanceof RubyArray) return ((RubyArray) self).all_p(context, block);
+
+        return all_pCommon(context, self, block);
+    }
+
+    public static IRubyObject all_pCommon(ThreadContext context, IRubyObject self, final Block block) {
         final Ruby runtime = context.getRuntime();
         final ThreadContext localContext = context;
 
@@ -1146,9 +1258,15 @@ public class RubyEnumerable {
 
         return runtime.getTrue();
     }
-
+    
     @JRubyMethod(name = "any?", frame = true)
     public static IRubyObject any_p(ThreadContext context, IRubyObject self, final Block block) {
+        if (self instanceof RubyArray) return ((RubyArray) self).any_p(context, block);
+        
+        return any_pCommon(context, self, block);
+    }
+
+    public static IRubyObject any_pCommon(ThreadContext context, IRubyObject self, final Block block) {
         final Ruby runtime = context.getRuntime();
         final ThreadContext localContext = context;
 
@@ -1193,15 +1311,21 @@ public class RubyEnumerable {
         return zipCommon(context, self, args, block, "to_ary");
     }
 
-    private static IRubyObject zipCommon(ThreadContext context, IRubyObject self,
-            final IRubyObject[] args, final Block block, String methodConverter) {
-        final Ruby runtime = context.getRuntime();
+    public static IRubyObject[] zipCommonConvert(Ruby runtime, IRubyObject[] args, String methodConverter) {
+        RubyClass array = runtime.getArray();
 
         for (int i = 0; i < args.length; i++) {
-            args[i] = TypeConverter.convertToType(args[i], runtime.getArray(), methodConverter);
+            args[i] = TypeConverter.convertToType(args[i], array, methodConverter);
         }
 
-        final int aLen = args.length + 1;
+        return args;
+    }
+
+    public static IRubyObject zipCommon(ThreadContext context, IRubyObject self,
+            IRubyObject[] aArgs, final Block block, String methodConverter) {
+        final Ruby runtime = context.getRuntime();
+        final int aLen = aArgs.length + 1;
+        final IRubyObject[] args = zipCommonConvert(runtime, aArgs, methodConverter);
 
         if (block.isGiven()) {
             callEach(runtime, context, self, new BlockCallback() {
@@ -1247,7 +1371,7 @@ public class RubyEnumerable {
     public static IRubyObject group_by(ThreadContext context, IRubyObject self, final Block block) {
         final Ruby runtime = context.getRuntime();
         
-        if (!block.isGiven()) return RubyEnumerator.enumeratorize(runtime, self, "group_by");
+        if (!block.isGiven()) return enumeratorize(runtime, self, "group_by");
         
         final RubyHash result = new RubyHash(runtime);
 
@@ -1271,6 +1395,112 @@ public class RubyEnumerable {
         return result;
     }
 
+    @JRubyMethod(name = "chunk", required = 0, optional = 1, frame = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject chunk(ThreadContext context, IRubyObject self, final IRubyObject[] args, final Block block) {
+        IRubyObject initialState = context.getRuntime().getNil();
+        if(!block.isGiven()) {
+            throw context.getRuntime().newArgumentError("no block given");
+        }
+
+        if(args.length > 0) {
+            initialState = args[0];
+        }
+
+        IRubyObject enumerator = context.getRuntime().getEnumerator().allocate();
+        enumerator.getInternalVariables().setInternalVariable("chunk_enumerable", self);
+        enumerator.getInternalVariables().setInternalVariable("chunk_categorize", RubyProc.newProc(context.getRuntime(), block, block.type));
+        enumerator.getInternalVariables().setInternalVariable("chunk_initial_state", initialState);
+
+        RuntimeHelpers.invoke(context, enumerator, "initialize", 
+                              CallBlock.newCallClosure(self, context.getRuntime().getEnumerable(), Arity.noArguments(), 
+                                                       new ChunkedBlockCallback(context.getRuntime(), enumerator), context));
+        return enumerator;
+    }
+
+    static class ChunkArg {
+        public IRubyObject categorize;
+        public IRubyObject state;
+        public IRubyObject prev_value;
+        public IRubyObject prev_elts;
+        public IRubyObject yielder;
+    }
+
+    // chunk_i
+    public static final class ChunkedBlockCallback implements BlockCallback {
+        private final Ruby runtime;
+        private final IRubyObject enumerator;
+
+        public ChunkedBlockCallback(Ruby runtime, IRubyObject enumerator) {
+            this.runtime = runtime;
+            this.enumerator = enumerator;
+        }
+
+        public IRubyObject call(ThreadContext context, IRubyObject[] largs, Block blk) {
+            IRubyObject args = checkArgs(runtime, largs);
+            final ChunkArg arg = new ChunkArg();
+            IRubyObject enumerable = (IRubyObject)enumerator.getInternalVariables().getInternalVariable("chunk_enumerable");
+            arg.categorize = (IRubyObject)enumerator.getInternalVariables().getInternalVariable("chunk_categorize");
+            arg.state = (IRubyObject)enumerator.getInternalVariables().getInternalVariable("chunk_initial_state");
+            arg.prev_value = runtime.getNil();
+            arg.prev_elts = runtime.getNil();
+            arg.yielder = args;
+
+            if(!arg.state.isNil()) {
+                arg.state = arg.state.dup();
+            }
+
+            final IRubyObject alone = runtime.newSymbol("_alone");
+            final IRubyObject separator = runtime.newSymbol("_separator");
+
+            callEach(runtime, context, enumerable, new BlockCallback() {
+                    public IRubyObject call(ThreadContext ctx, IRubyObject[] largs, Block blk) {
+                        IRubyObject i = checkArgs(runtime, largs);
+                        IRubyObject v;
+                        if(arg.state.isNil()) {
+                            v = arg.categorize.callMethod(ctx, "call", i);
+                        } else {
+                            v = arg.categorize.callMethod(ctx, "call", new IRubyObject[]{i, arg.state});  
+                        }
+                        
+                        if(v == alone) {
+                            if(!arg.prev_value.isNil()) {
+                                arg.yielder.callMethod(ctx, "<<", runtime.newArray(arg.prev_value, arg.prev_elts));
+                                arg.prev_value = arg.prev_elts = runtime.getNil();
+                            }
+                            arg.yielder.callMethod(ctx, "<<", runtime.newArray(v, runtime.newArray(i)));
+                        } else if(v.isNil() || v == separator) {
+                            if(!arg.prev_value.isNil()) {
+                                arg.yielder.callMethod(ctx, "<<", runtime.newArray(arg.prev_value, arg.prev_elts));
+                                arg.prev_value = arg.prev_elts = runtime.getNil();
+                            }
+                        } else if((v instanceof RubySymbol) && v.toString().charAt(0) == '_') {
+                            throw runtime.newRuntimeError("symbol begins with an underscore is reserved");
+                        } else {
+                            if(arg.prev_value.isNil()) {
+                                arg.prev_value = v;
+                                arg.prev_elts = runtime.newArray(i);
+                            } else {
+                                if(arg.prev_value.equals(v)) {
+                                    ((RubyArray)arg.prev_elts).append(i);
+                                } else {
+                                    arg.yielder.callMethod(ctx, "<<", runtime.newArray(arg.prev_value, arg.prev_elts));
+                                    arg.prev_value = v;
+                                    arg.prev_elts = runtime.newArray(i);
+                                }
+                            }
+                        }
+                        return runtime.getNil();
+                    }
+                });
+
+            if(!arg.prev_elts.isNil()) {
+                arg.yielder.callMethod(context, "<<", runtime.newArray(arg.prev_value, arg.prev_elts));
+            }
+
+            return runtime.getNil();
+        }
+    }
+
     public static final class AppendBlockCallback implements BlockCallback {
         private Ruby runtime;
         private RubyArray result;
@@ -1285,4 +1515,14 @@ public class RubyEnumerable {
             return runtime.getNil();
         }
     }
+
+    @JRubyMethod(name = "join", compat = CompatVersion.RUBY1_9)
+    public static IRubyObject join(ThreadContext context, IRubyObject self) {
+        return join(context, self, context.getRuntime().getGlobalVariables().get("$,"));
+    }
+
+    @JRubyMethod(name = "join", compat = CompatVersion.RUBY1_9)
+    public static IRubyObject join(ThreadContext context, IRubyObject self, final IRubyObject sep) {
+        return ((RubyArray)to_a19(context, self)).join19(context, sep);
+    }
 }
