@@ -70,7 +70,7 @@ public class RubyEnumerator extends RubyObject {
         runtime.setEnumerator(enmr);
 
         if (runtime.is1_9()) {
-            runtime.getLoadService().lockAndRequire("generator");
+            runtime.getLoadService().lockAndRequire("generator_internal");
             RubyYielder.createYielderClass(runtime);
         }
     }
@@ -104,21 +104,58 @@ public class RubyEnumerator extends RubyObject {
         return new RubyEnumerator(runtime, object, runtime.fastNewSymbol(method), args); // TODO: make sure it's really safe to not to copy it
     }
 
-    @JRubyMethod(name = "initialize", visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "initialize", visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_8)
     public IRubyObject initialize(ThreadContext context) {
         throw context.getRuntime().newArgumentError(0, 1);
     }
 
-    @JRubyMethod(name = "initialize", visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "initialize", frame = true, visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_9)
+    public IRubyObject initialize19(ThreadContext context, Block block) {
+        if(!block.isGiven()) {
+            throw context.getRuntime().newArgumentError(0, 1);
+        }
+
+        IRubyObject obj = context.getRuntime().getClass("Generator").callMethod(context, "new", new IRubyObject[0], block);
+        return initialize19(context, obj, block);
+    }
+
+    @JRubyMethod(name = "initialize", visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_8)
     public IRubyObject initialize(ThreadContext context, IRubyObject object) {
         return initialize(object, context.getRuntime().fastNewSymbol("each"), NULL_ARRAY);
     }
 
-    @JRubyMethod(name = "initialize", visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "initialize", frame = true, visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_9)
+    public IRubyObject initialize19(ThreadContext context, IRubyObject object, Block block) {
+        return initialize(object, context.getRuntime().fastNewSymbol("each"), NULL_ARRAY);
+    }
+
+    @JRubyMethod(name = "initialize", visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_8)
     public IRubyObject initialize(ThreadContext context, IRubyObject object, IRubyObject method) {
         return initialize(object, method, NULL_ARRAY);
     }
 
+    @JRubyMethod(name = "initialize", frame = true, visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_9)
+    public IRubyObject initialize19(ThreadContext context, IRubyObject object, IRubyObject method, Block block) {
+        return initialize(object, method, NULL_ARRAY);
+    }
+
+    @JRubyMethod(name = "initialize", visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_8)
+    public IRubyObject initialize(ThreadContext context, IRubyObject object, IRubyObject method, IRubyObject methodArg) {
+        return initialize(object, method, new IRubyObject[] { methodArg });
+    }
+
+    @JRubyMethod(name = "initialize", frame = true, visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_9)
+    public IRubyObject initialize19(ThreadContext context, IRubyObject object, IRubyObject method, IRubyObject methodArg, Block block) {
+        return initialize(object, method, new IRubyObject[] { methodArg });
+    }
+
+    @JRubyMethod(name = "initialize", required = 1, rest = true, visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_8)
+    public IRubyObject initialize(ThreadContext context, IRubyObject[] args) {
+        IRubyObject[] methArgs = new IRubyObject[args.length - 2];
+        System.arraycopy(args, 2, methArgs, 0, methArgs.length);
+        return initialize(args[0], args[1], methArgs);
+    }
+
     private IRubyObject initialize(IRubyObject object, IRubyObject method, IRubyObject[] methodArgs) {
         this.object = object;
         this.method = method.asJavaString();
@@ -129,18 +166,6 @@ public class RubyEnumerator extends RubyObject {
         return this;
     }
 
-    @JRubyMethod(name = "initialize", visibility = Visibility.PRIVATE)
-    public IRubyObject initialize(ThreadContext context, IRubyObject object, IRubyObject method, IRubyObject methodArg) {
-        return initialize(object, method, new IRubyObject[] { methodArg });
-    }
-
-    @JRubyMethod(name = "initialize", required = 1, rest = true, visibility = Visibility.PRIVATE)
-    public IRubyObject initialize(ThreadContext context, IRubyObject[] args) {
-        IRubyObject[] methArgs = new IRubyObject[args.length - 2];
-        System.arraycopy(args, 2, methArgs, 0, methArgs.length);
-        return initialize(args[0], args[1], methArgs);
-    }
-
     /**
      * Send current block and supplied args to method on target. According to MRI
      * Block may not be given and "each" should just ignore it and call on through to
@@ -327,9 +352,10 @@ public class RubyEnumerator extends RubyObject {
         private final Block block;
         private final Ruby runtime;
 
-        public EachWithIndex(ThreadContext ctx, Block block) {
+        public EachWithIndex(ThreadContext ctx, Block block, int index) {
             this.block = block;
             this.runtime = ctx.getRuntime();
+            this.index = index;
         }
 
         public IRubyObject call(ThreadContext context, IRubyObject[] iargs, Block block) {
@@ -338,9 +364,13 @@ public class RubyEnumerator extends RubyObject {
     }
 
     private static IRubyObject with_index_common(ThreadContext context, IRubyObject self, 
-            final Block block, final String rubyMethodName) {
+            final Block block, final String rubyMethodName, IRubyObject arg) {
         final Ruby runtime = context.getRuntime();
-        if (!block.isGiven()) return enumeratorize(runtime, self , rubyMethodName);
+        int index = arg.isNil() ? 0 : RubyNumeric.num2int(arg);
+        if (!block.isGiven()) {
+            return arg.isNil() ? enumeratorize(runtime, self , rubyMethodName) :
+                enumeratorize(runtime, self , rubyMethodName, runtime.newFixnum(index));
+        }
         
         IRubyObject[] args = new IRubyObject[0];
 
@@ -349,28 +379,38 @@ public class RubyEnumerator extends RubyObject {
             args = e.methodArgs;
         }
 
-        return RubyEnumerable.callEach(runtime, context, self, args, new EachWithIndex(context, block));
+        return RubyEnumerable.callEach(runtime, context, self, args, new EachWithIndex(context, block, index));
     }
 
     @JRubyMethod(name = "each_with_index", frame = true)
     public static IRubyObject each_with_index(ThreadContext context, IRubyObject self, final Block block) {
-        return with_index_common(context, self, block, "each_with_index");
+        return with_index_common(context, self, block, "each_with_index", context.getRuntime().getNil());
     }
 
-    @JRubyMethod(name = "with_index", frame = true)
+    @JRubyMethod(name = "with_index", frame = true, compat = CompatVersion.RUBY1_8)
     public static IRubyObject with_index(ThreadContext context, IRubyObject self, final Block block) {
-        return with_index_common(context, self, block, "with_index");
+        return with_index_common(context, self, block, "with_index", context.getRuntime().getNil());
+    }
+
+    @JRubyMethod(name = "with_index", frame = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject with_index19(ThreadContext context, IRubyObject self, final Block block) {
+        return with_index_common(context, self, block, "with_index", context.getRuntime().getNil());
+    }
+
+    @JRubyMethod(name = "with_index", frame = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject with_index19(ThreadContext context, IRubyObject self, IRubyObject arg, final Block block) {
+        return with_index_common(context, self, block, "with_index", arg);
     }
 
     @JRubyMethod(name = "next", frame = true)
     public static IRubyObject next(ThreadContext context, IRubyObject self) {
-        context.getRuntime().getLoadService().lockAndRequire("generator");
+        context.getRuntime().getLoadService().lockAndRequire("generator_internal");
         return self.callMethod(context, "next");
     }
 
     @JRubyMethod(name = "rewind", frame = true)
     public static IRubyObject rewind(ThreadContext context, IRubyObject self) {
-        context.getRuntime().getLoadService().lockAndRequire("generator");
+        context.getRuntime().getLoadService().lockAndRequire("generator_internal");
         return self.callMethod(context, "rewind");
     }
 }
