@@ -11,7 +11,7 @@
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
- * Copyright (C) 2004-2005 Charles O Nutter <headius@headius.com>
+ * Copyright (C) 2004-2005, 2009 Charles O Nutter <headius@headius.com>
  * Copyright (C) 2004 Thomas E Enebo <enebo@acm.org>
  * Copyright (C) 2004 Jan Arne Petersen <jpetersen@uni-bonn.de>
  * Copyright (C) 2004 Stefan Matthias Aust <sma@3plus4.de>
@@ -30,6 +30,11 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
+import java.io.File;
+import java.io.IOException;
+import java.io.InputStream;
+import java.util.jar.JarFile;
+import java.util.zip.ZipEntry;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyModule;
 import org.jruby.exceptions.RaiseException;
@@ -37,14 +42,17 @@ import org.jruby.exceptions.RaiseException;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.JRubyFile;
 
-@JRubyModule(name="FileTest")
+import static org.jruby.RubyFile.get_path;
+
+@JRubyModule(name = "FileTest")
 public class RubyFileTest {
+
     public static RubyModule createFileTestModule(Ruby runtime) {
         RubyModule fileTestModule = runtime.defineModule("FileTest");
         runtime.setFileTest(fileTestModule);
-        
+
         fileTestModule.defineAnnotatedMethods(RubyFileTest.class);
-        
+
         return fileTestModule;
     }
 
@@ -52,92 +60,97 @@ public class RubyFileTest {
     public static IRubyObject blockdev_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isBlockDev());
     }
-    
+
     @JRubyMethod(name = "chardev?", required = 1, module = true)
     public static IRubyObject chardev_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isCharDev());
     }
 
     @JRubyMethod(name = "directory?", required = 1, module = true)
     public static IRubyObject directory_p(IRubyObject recv, IRubyObject filename) {
-        Ruby runtime = recv.getRuntime();
+        return directory_p(recv.getRuntime(), filename);
+    }
+
+    public static IRubyObject directory_p(Ruby runtime, IRubyObject filename) {
+        ZipEntry entry = file_in_archive(filename);
+        if (entry != null) {
+            return entry.isDirectory() ? runtime.getTrue() : runtime.getFalse();
+        }
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isDirectory());
     }
-    
+
     @JRubyMethod(name = "executable?", required = 1, module = true)
     public static IRubyObject executable_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isExecutable());
     }
-    
+
     @JRubyMethod(name = "executable_real?", required = 1, module = true)
     public static IRubyObject executable_real_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isExecutableReal());
     }
-    
+
     @JRubyMethod(name = {"exist?", "exists?"}, required = 1, module = true)
     public static IRubyObject exist_p(IRubyObject recv, IRubyObject filename) {
+        Ruby runtime = recv.getRuntime();
+        if (existsOnClasspath(filename)) {
+            return runtime.getTrue();
+        }
+
         if (Ruby.isSecurityRestricted()) {
-            return recv.getRuntime().getFalse();
+            return runtime.getFalse();
         }
 
-        if(filename.convertToString().toString().startsWith("file:")) {
-            String file = filename.convertToString().toString().substring(5);
-            int bang = file.indexOf('!');
-            if (bang == -1 || bang == file.length() - 1) {
-                return recv.getRuntime().getFalse();
-            }
-            String jar = file.substring(0, bang);
-            String after = file.substring(bang + 2);
-            try {
-                java.util.jar.JarFile jf = new java.util.jar.JarFile(jar);
-                if(jf.getJarEntry(after) != null) {
-                    return recv.getRuntime().getTrue();
-                } else {
-                    return recv.getRuntime().getFalse();
-                }
-            } catch(Exception e) {
-                return recv.getRuntime().getFalse();
-            }
+
+        if (file_in_archive(filename) != null) {
+            return runtime.getTrue();
         }
 
-        return recv.getRuntime().newBoolean(file(filename).exists());
+        return runtime.newBoolean(file(filename).exists());
     }
 
     @JRubyMethod(name = "file?", required = 1, module = true)
     public static RubyBoolean file_p(IRubyObject recv, IRubyObject filename) {
+        Ruby runtime = recv.getRuntime();
+        ZipEntry entry = file_in_archive(filename);
+        if (entry != null) {
+            return entry.isDirectory() ?
+                recv.getRuntime().getFalse() :
+                recv.getRuntime().getTrue();
+        }
+
         JRubyFile file = file(filename);
 
-        return filename.getRuntime().newBoolean(file.exists() && file.isFile());
+        return runtime.newBoolean(file.exists() && file.isFile());
     }
 
     @JRubyMethod(name = "grpowned?", required = 1, module = true)
     public static IRubyObject grpowned_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isGroupOwned());
     }
-    
+
     @JRubyMethod(name = "identical?", required = 2, module = true)
     public static IRubyObject identical_p(IRubyObject recv, IRubyObject filename1, IRubyObject filename2) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file1 = file(filename1);
         JRubyFile file2 = file(filename2);
-        
+
         return runtime.newBoolean(file1.exists() && file2.exists() &&
                 runtime.getPosix().stat(file1.getAbsolutePath()).isIdentical(runtime.getPosix().stat(file2.getAbsolutePath())));
     }
@@ -146,91 +159,118 @@ public class RubyFileTest {
     public static IRubyObject owned_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isOwned());
     }
-    
+
     @JRubyMethod(name = "pipe?", required = 1, module = true)
     public static IRubyObject pipe_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isNamedPipe());
     }
 
     // We use file test since it is faster than a stat; also euid == uid in Java always
     @JRubyMethod(name = {"readable?", "readable_real?"}, required = 1, module = true)
     public static IRubyObject readable_p(IRubyObject recv, IRubyObject filename) {
+        Ruby runtime = recv.getRuntime();
+        ZipEntry entry = file_in_archive(filename);
+        if (entry != null) {
+            return entry.isDirectory() ?
+                recv.getRuntime().getFalse() :
+                recv.getRuntime().getTrue();
+        }
+
         JRubyFile file = file(filename);
 
-        return recv.getRuntime().newBoolean(file.exists() && file.canRead());
+        return runtime.newBoolean(file.exists() && file.canRead());
     }
 
     // Not exposed by filetest, but so similiar in nature that it is stored here
     public static IRubyObject rowned_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isROwned());
     }
-    
+
     @JRubyMethod(name = "setgid?", required = 1, module = true)
     public static IRubyObject setgid_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isSetgid());
     }
-    
+
     @JRubyMethod(name = "setuid?", required = 1, module = true)
     public static IRubyObject setuid_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isSetuid());
     }
-    
+
     @JRubyMethod(name = "size", required = 1, module = true)
     public static IRubyObject size(IRubyObject recv, IRubyObject filename) {
-        JRubyFile file = file(filename);
+        Ruby runtime = recv.getRuntime();
+        ZipEntry entry = file_in_archive(filename);
+        if (entry != null) {
+            return runtime.newFixnum(entry.getSize());
+        }
 
-        if (!file.exists()) noFileError(filename);
-        
-        return recv.getRuntime().newFixnum(file.length());
-    }
-    
-    @JRubyMethod(name = "size?", required = 1, module = true)
-    public static IRubyObject size_p(IRubyObject recv, IRubyObject filename) {
         JRubyFile file = file(filename);
 
         if (!file.exists()) {
-            return recv.getRuntime().getNil();
+            noFileError(filename);
         }
 
-        long length =  file.length();
+        return runtime.newFixnum(file.length());
+    }
+
+    @JRubyMethod(name = "size?", required = 1, module = true)
+    public static IRubyObject size_p(IRubyObject recv, IRubyObject filename) {
+        Ruby runtime = recv.getRuntime();
+        ZipEntry entry = file_in_archive(filename);
+        if (entry != null) {
+            long size = entry.getSize();
+            if (size > 0) {
+                return runtime.newFixnum(size);
+            } else {
+                return runtime.getNil();
+            }
+        }
+
+        JRubyFile file = file(filename);
+
+        if (!file.exists()) {
+            return runtime.getNil();
+        }
+
+        long length = file.length();
         if (length > 0) {
-            return recv.getRuntime().newFixnum(length);
+            return runtime.newFixnum(length);
         } else {
-            return recv.getRuntime().getNil();
+            return runtime.getNil();
         }
     }
-    
+
     @JRubyMethod(name = "socket?", required = 1, module = true)
     public static IRubyObject socket_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isSocket());
     }
-    
+
     @JRubyMethod(name = "sticky?", required = 1, module = true)
     public static IRubyObject sticky_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
         JRubyFile file = file(filename);
-        
+
         return runtime.newBoolean(file.exists() && runtime.getPosix().stat(file.getAbsolutePath()).isSticky());
     }
-        
+
     @JRubyMethod(name = "symlink?", required = 1, module = true)
     public static RubyBoolean symlink_p(IRubyObject recv, IRubyObject filename) {
         Ruby runtime = recv.getRuntime();
@@ -243,6 +283,8 @@ public class RubyFileTest {
             // but in the try-catch block.
             // MRI behavior: symlink? on broken symlink should return true.
             return runtime.newBoolean(runtime.getPosix().lstat(file.getAbsolutePath()).isSymlink());
+        } catch (SecurityException re) {
+            return runtime.getFalse();
         } catch (RaiseException re) {
             return runtime.getFalse();
         }
@@ -254,26 +296,92 @@ public class RubyFileTest {
     public static RubyBoolean writable_p(IRubyObject recv, IRubyObject filename) {
         return filename.getRuntime().newBoolean(file(filename).canWrite());
     }
-    
+
     @JRubyMethod(name = "zero?", required = 1, module = true)
     public static RubyBoolean zero_p(IRubyObject recv, IRubyObject filename) {
+        Ruby runtime = recv.getRuntime();
+        ZipEntry entry = file_in_archive(filename);
+        if (entry != null) {
+            return runtime.newBoolean(entry.getSize() == 0L);
+        }
+
         JRubyFile file = file(filename);
-        
-        return filename.getRuntime().newBoolean(file.exists() && file.length() == 0L);
+
+        return runtime.newBoolean(file.exists() && file.length() == 0L);
     }
 
     private static JRubyFile file(IRubyObject pathOrFile) {
         Ruby runtime = pathOrFile.getRuntime();
 
         if (pathOrFile instanceof RubyFile) {
-            return JRubyFile.create(runtime.getCurrentDirectory(), ((RubyFile)pathOrFile).getPath());
+            return JRubyFile.create(runtime.getCurrentDirectory(), ((RubyFile) pathOrFile).getPath());
         } else {
-            return JRubyFile.create(runtime.getCurrentDirectory(), pathOrFile.convertToString().getUnicodeValue());
+            RubyString path = get_path(runtime.getCurrentContext(), pathOrFile);
+            return JRubyFile.create(runtime.getCurrentDirectory(), path.getUnicodeValue());
         }
     }
-    
+
+    private static ZipEntry file_in_archive(IRubyObject path) {
+        Ruby runtime = path.getRuntime();
+
+        if (path instanceof RubyFile) {
+            return null;
+        }
+
+        RubyString pathStr = get_path(runtime.getCurrentContext(), path);
+        String pathJStr = pathStr.getUnicodeValue();
+        if (pathJStr.startsWith("file:")) {
+            String file = pathJStr.substring(5);
+            int bang = file.indexOf('!');
+            if (bang == -1 || bang == file.length() - 1) {
+                return null;
+            }
+            String jar = file.substring(0, bang);
+            String after = file.substring(bang + 2);
+            try {
+                JarFile jf = new JarFile(jar);
+                ZipEntry entry = RubyFile.getDirOrFileEntry(jf, after);
+                return entry;
+            } catch (Exception e) {
+            }
+        }
+
+        return null;
+    }
+
+    private static boolean existsOnClasspath(IRubyObject path) {
+        if (path instanceof RubyFile) {
+            return false;
+        }
+
+        Ruby runtime = path.getRuntime();
+        RubyString pathStr = get_path(runtime.getCurrentContext(), path);
+        String pathJStr = pathStr.getUnicodeValue();
+        if (pathJStr.startsWith("classpath:/")) {
+            pathJStr = pathJStr.substring("classpath:/".length());
+
+            ClassLoader classLoader = runtime.getJRubyClassLoader();
+            // handle security-sensitive case
+            if (Ruby.isSecurityRestricted() && classLoader == null) {
+                classLoader = runtime.getInstanceConfig().getLoader();
+            }
+
+            InputStream is = classLoader.getResourceAsStream(pathJStr);
+            if (is != null) {
+                try {
+                    is.close();
+                } catch (IOException ignore) {
+                } catch (NullPointerException wtf) {
+                    // that's what sometimes happens, weird
+                }
+                return true;
+            }
+        }
+        return false;
+    }
+
     private static void noFileError(IRubyObject filename) {
-        throw filename.getRuntime().newErrnoENOENTError("No such file or directory - " + 
+        throw filename.getRuntime().newErrnoENOENTError("No such file or directory - " +
                 filename.convertToString());
     }
 }
