@@ -38,10 +38,19 @@
 package org.jruby;
 
 import java.math.BigInteger;
+import static org.jruby.util.Numeric.f_abs;
+import static org.jruby.util.Numeric.f_add;
 import static org.jruby.util.Numeric.f_expt;
+import static org.jruby.util.Numeric.f_lshift;
 import static org.jruby.util.Numeric.f_mul;
+import static org.jruby.util.Numeric.f_negate;
+import static org.jruby.util.Numeric.f_negative_p;
+import static org.jruby.util.Numeric.f_sub;
+import static org.jruby.util.Numeric.f_to_r;
+import static org.jruby.util.Numeric.f_zero_p;
 import static org.jruby.util.Numeric.frexp;
 import static org.jruby.util.Numeric.ldexp;
+import static org.jruby.util.Numeric.nurat_rationalize_internal;
 
 import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
@@ -58,6 +67,7 @@ import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.runtime.marshal.MarshalStream;
 import org.jruby.runtime.marshal.UnmarshalStream;
 import org.jruby.util.ByteList;
+import org.jruby.util.ConvertDouble;
 import org.jruby.util.Sprintf;
 
 /**
@@ -74,11 +84,16 @@ public class RubyFloat extends RubyNumeric {
     public static final int MAX_10_EXP = 308;
     public static final int MIN_10_EXP = -307;
     public static final double EPSILON = 2.2204460492503131e-16;
+    public static final double INFINITY = Double.POSITIVE_INFINITY;
+    public static final double NAN = Double.NaN;
 
     public static RubyClass createFloatClass(Ruby runtime) {
         RubyClass floatc = runtime.defineClass("Float", runtime.getNumeric(), ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
         runtime.setFloat(floatc);
+
         floatc.index = ClassIndex.FLOAT;
+        floatc.setReifiedClass(RubyFloat.class);
+        
         floatc.kindOf = new RubyModule.KindOf() {
             @Override
             public boolean isKindOf(IRubyObject obj, RubyModule type) {
@@ -106,7 +121,12 @@ public class RubyFloat extends RubyNumeric {
         floatc.defineConstant("MIN", RubyFloat.newFloat(runtime, Double.MIN_VALUE));
         floatc.defineConstant("MAX", RubyFloat.newFloat(runtime, Double.MAX_VALUE));
         floatc.defineConstant("EPSILON", RubyFloat.newFloat(runtime, EPSILON));
-        
+
+        if (runtime.is1_9()) {
+            floatc.defineConstant("INFINITY", RubyFloat.newFloat(runtime, INFINITY));
+            floatc.defineConstant("NAN", RubyFloat.newFloat(runtime, NAN));
+        }
+
         floatc.defineAnnotatedMethods(RubyFloat.class);
 
         return floatc;
@@ -210,20 +230,20 @@ public class RubyFloat extends RubyNumeric {
         ByteList buf = new ByteList();
         Sprintf.sprintf(buf, Locale.US, "%#.15g", this);
         int e = buf.indexOf('e');
-        if (e == -1) e = buf.realSize;
+        if (e == -1) e = buf.getRealSize();
         ASCIIEncoding ascii = ASCIIEncoding.INSTANCE; 
 
         if (!ascii.isDigit(buf.get(e - 1))) {
-            buf.realSize = 0;
+            buf.setRealSize(0);
             Sprintf.sprintf(buf, Locale.US, "%#.14e", this);
             e = buf.indexOf('e');
-            if (e == -1) e = buf.realSize;
+            if (e == -1) e = buf.getRealSize();
         }
 
         int p = e;
         while (buf.get(p - 1) == '0' && ascii.isDigit(buf.get(p - 2))) p--;
-        System.arraycopy(buf.bytes, e, buf.bytes, p, buf.realSize - e);
-        buf.realSize = p + buf.realSize - e;
+        System.arraycopy(buf.getUnsafeBytes(), e, buf.getUnsafeBytes(), p, buf.getRealSize() - e);
+        buf.setRealSize(p + buf.getRealSize() - e);
         return runtime.newString(buf);
     }
 
@@ -316,7 +336,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_mod
      * 
      */
-    @JRubyMethod(name = {"%", "modulo"}, required = 1)
+    @JRubyMethod(name = {"%", "modulo"}, required = 1, compat = CompatVersion.RUBY1_8)
     public IRubyObject op_mod(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().index) {
         case ClassIndex.FIXNUM:
@@ -337,10 +357,22 @@ public class RubyFloat extends RubyNumeric {
         }
     }
 
+    /** flo_mod
+     * 
+     */
+    @JRubyMethod(name = {"%", "modulo"}, required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_mod19(ThreadContext context, IRubyObject other) {
+        if (!other.isNil() && other instanceof RubyNumeric
+            && ((RubyNumeric)other).getDoubleValue() == 0) {
+            throw context.getRuntime().newZeroDivisionError(); 
+        }
+        return op_mod(context, other);
+    }
+
     /** flo_divmod
      * 
      */
-    @JRubyMethod(name = "divmod", required = 1)
+    @JRubyMethod(name = "divmod", required = 1, compat = CompatVersion.RUBY1_8)
     @Override
     public IRubyObject divmod(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().index) {
@@ -369,6 +401,18 @@ public class RubyFloat extends RubyNumeric {
         }
     }
     	
+    /** flo_divmod
+     * 
+     */
+    @JRubyMethod(name = "divmod", required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject divmod19(ThreadContext context, IRubyObject other) {
+        if (!other.isNil() && other instanceof RubyNumeric
+            && ((RubyNumeric)other).getDoubleValue() == 0) {
+            throw context.getRuntime().newZeroDivisionError(); 
+        }
+        return divmod(context, other);
+    }
+    	
     /** flo_pow
      * 
      */
@@ -384,6 +428,16 @@ public class RubyFloat extends RubyNumeric {
             return coerceBin(context, "**", other);
         }
     }
+    
+    @JRubyMethod(name = "**", required = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_pow19(ThreadContext context, IRubyObject other) {
+        double d_other = ((RubyNumeric) other).getDoubleValue();
+        if (value < 0 && (d_other != Math.round(d_other))) {
+            return RubyComplex.newComplexRaw(getRuntime(), this).callMethod(context, "**", other);
+        } else {
+            return op_pow(context, other);
+        }
+    }
 
     /** flo_eq
      * 
@@ -623,6 +677,55 @@ public class RubyFloat extends RubyNumeric {
         return f_mul(context, rf, f_expt(context, RubyFixnum.newFixnum(runtime, FLT_RADIX), rn));
     }
 
+    /** float_rationalize
+     *
+     */
+    @JRubyMethod(name = "rationalize", optional = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject rationalize(ThreadContext context, IRubyObject[] args) {
+        if (f_negative_p(context, this))
+            return f_negate(context, ((RubyFloat) f_abs(context, this)).rationalize(context, args));
+
+        Ruby runtime = context.getRuntime();
+        RubyFixnum one = RubyFixnum.one(runtime);
+        RubyFixnum two = RubyFixnum.two(runtime);
+
+        IRubyObject eps, a, b;
+        if (args.length != 0) {
+            eps = f_abs(context, args[0]);
+            a = f_sub(context, this, eps);
+            b = f_add(context, this, eps);
+        } else {
+            long[] exp = new long[1];
+            double f = frexp(value, exp);
+            f = ldexp(f, DBL_MANT_DIG);
+            long n = exp[0] - DBL_MANT_DIG;
+
+
+            IRubyObject rf = RubyNumeric.dbl2num(runtime, f);
+            IRubyObject rn = RubyFixnum.newFixnum(runtime, n);
+
+            if (f_zero_p(context, rf) || !(f_negative_p(context, rn) || f_zero_p(context, rn)))
+                return RubyRational.newRationalRaw(runtime, f_lshift(context,rf,rn));
+
+            a = RubyRational.newRationalRaw(runtime,
+                    f_sub(context,f_mul(context, two, rf),one),
+                    f_lshift(context, one, f_sub(context,one,rn)));
+            b = RubyRational.newRationalRaw(runtime,
+                    f_add(context,f_mul(context, two, rf),one),
+                    f_lshift(context, one, f_sub(context,one,rn)));
+        }
+
+        if (a.callMethod(context, "==", b).isTrue()) return f_to_r(context, this);
+
+        IRubyObject[] ary = new IRubyObject[2];
+        ary[0] = a;
+        ary[1] = b;
+        IRubyObject[] ans = nurat_rationalize_internal(context, ary);
+
+        return RubyRational.newRationalRaw(runtime, ans[0], ans[1]);
+
+    }
+
     /** floor
      * 
      */
@@ -709,8 +812,23 @@ public class RubyFloat extends RubyNumeric {
     }
         
     public static RubyFloat unmarshalFrom(UnmarshalStream input) throws java.io.IOException {
-        RubyFloat result = RubyFloat.newFloat(input.getRuntime(), org.jruby.util.Convert.byteListToDouble(input.unmarshalString(),false));
+        ByteList value = input.unmarshalString();
+        RubyFloat result;
+        if (value.equals(NAN_BYTELIST)) {
+            result = RubyFloat.newFloat(input.getRuntime(), RubyFloat.NAN);
+        } else if (value.equals(NEGATIVE_INFINITY_BYTELIST)) {
+            result = RubyFloat.newFloat(input.getRuntime(), Double.NEGATIVE_INFINITY);
+        } else if (value.equals(INFINITY_BYTELIST)) {
+            result = RubyFloat.newFloat(input.getRuntime(), Double.POSITIVE_INFINITY);
+        } else {
+            result = RubyFloat.newFloat(input.getRuntime(),
+                    ConvertDouble.byteListToDouble(value, false));
+        }
         input.registerLinkTarget(result);
         return result;
     }
+
+    private static final ByteList NAN_BYTELIST = new ByteList(new String("nan").getBytes());
+    private static final ByteList NEGATIVE_INFINITY_BYTELIST = new ByteList(new String("-inf").getBytes());
+    private static final ByteList INFINITY_BYTELIST = new ByteList(new String("inf").getBytes());
 }
