@@ -62,15 +62,49 @@ import org.jruby.util.KCode;
 public class RubyGlobal {
     
     /**
-     * Obligate string-keyed and string-valued hash, used for ENV and ENV_JAVA
+     * Obligate string-keyed and string-valued hash, used for ENV.
+     * On Windows, the keys are case-insensitive for ENV
      * 
      */
-    public static class StringOnlyRubyHash extends RubyHash {
+    public static class CaseInsensitiveStringOnlyRubyHash extends StringOnlyRubyHash {
         
+        public CaseInsensitiveStringOnlyRubyHash(Ruby runtime, Map valueMap, IRubyObject defaultValue) {
+            super(runtime, valueMap, defaultValue);
+        }
+
+        @Override
+        public IRubyObject op_aref(ThreadContext context, IRubyObject key) {
+            return case_aware_op_aref(context, key, false);
+        }
+
+        @Override
+        public IRubyObject op_aset(ThreadContext context, IRubyObject key, IRubyObject value) {
+            return case_aware_op_aset(context, key, value, false);
+        }
+
+        @JRubyMethod
+        @Override
+        public IRubyObject to_s(){
+            return getRuntime().newString("ENV");
+        }
+
+    }
+
+    /**
+     * A Pseudo-hash whose keys and values are required to be Strings.
+     * On all platforms, the keys are case-sensitive.
+     * Used for ENV_JAVA.
+     */
+    public static class StringOnlyRubyHash extends RubyHash {
         public StringOnlyRubyHash(Ruby runtime, Map valueMap, IRubyObject defaultValue) {
             super(runtime, valueMap, defaultValue);
         }
 
+        @Override
+        public IRubyObject op_aref(ThreadContext context, IRubyObject key) {
+            return case_aware_op_aref(context, key, true);
+        }
+
         @Override
         public RubyHash to_hash() {
             Ruby runtime = getRuntime();
@@ -80,36 +114,40 @@ public class RubyGlobal {
         }
 
         @Override
-        public IRubyObject op_aref(ThreadContext context, IRubyObject key) {
-            RubyString actualKey = getCorrectKey(key, context);
-            return super.op_aref(context, actualKey);
+        public IRubyObject op_aset(ThreadContext context, IRubyObject key, IRubyObject value) {
+            return case_aware_op_aset(context, key, value, true);
         }
 
         @Override
-        public IRubyObject op_aset(ThreadContext context, IRubyObject key, IRubyObject value) {
+        public IRubyObject op_aset19(ThreadContext context, IRubyObject key, IRubyObject value) {
+            return op_aset(context, key, value);
+        }
+
+        protected IRubyObject case_aware_op_aref(ThreadContext context, IRubyObject key, boolean caseSensitive) {
+            if (! caseSensitive)
+                key = getCorrectKey(key, context);
+            return super.op_aref(context, key);
+        }
+
+        protected IRubyObject case_aware_op_aset(ThreadContext context, IRubyObject key, IRubyObject value, boolean caseSensitive) {
             if (!key.respondsTo("to_str")) {
                 throw getRuntime().newTypeError("can't convert " + key.getMetaClass() + " into String");
             }
             if (!value.respondsTo("to_str") && !value.isNil()) {
                 throw getRuntime().newTypeError("can't convert " + value.getMetaClass() + " into String");
             }
-            
-            RubyString actualKey = getCorrectKey(key, context);
+
+            if (! caseSensitive)
+                key = getCorrectKey(key, context);
 
             if (value.isNil()) {
-                return super.delete(context, actualKey, org.jruby.runtime.Block.NULL_BLOCK);
+                return super.delete(context, key, org.jruby.runtime.Block.NULL_BLOCK);
             }
-            
+
             //return super.aset(getRuntime().newString("sadfasdF"), getRuntime().newString("sadfasdF"));
-            return super.op_aset(context, RuntimeHelpers.invoke(context, actualKey, "to_str"),
+            return super.op_aset(context, RuntimeHelpers.invoke(context, key, "to_str"),
                     value.isNil() ? getRuntime().getNil() : RuntimeHelpers.invoke(context, value, "to_str"));
         }
-        
-        @JRubyMethod
-        @Override
-        public IRubyObject to_s(){
-            return getRuntime().newString("ENV");
-        }
 
         private RubyString getCorrectKey(IRubyObject key, ThreadContext context) {
             RubyString originalKey = key.convertToString();
@@ -297,9 +335,9 @@ public class RubyGlobal {
     		environmentVariableMap = new HashMap();
     	}
 
-        StringOnlyRubyHash h1 = new StringOnlyRubyHash(runtime,
+        CaseInsensitiveStringOnlyRubyHash h1 = new CaseInsensitiveStringOnlyRubyHash(runtime,
                                                        environmentVariableMap, runtime.getNil());
-        h1.getSingletonClass().defineAnnotatedMethods(StringOnlyRubyHash.class);
+        h1.getSingletonClass().defineAnnotatedMethods(CaseInsensitiveStringOnlyRubyHash.class);
         runtime.defineGlobalConstant("ENV", h1);
 
         // Define System.getProperties() in ENV_JAVA
