@@ -14,6 +14,7 @@
  *
  * Copyright (C) 2005 Thomas E Enebo <enebo@acm.org>
  * Copyright (C) 2009 MenTaLguY <mental@rydia.net>
+ * Copyright (C) 2010 Charles Oliver Nutter <headius@headius.com>
  * 
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -34,8 +35,11 @@ import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.lang.management.ManagementFactory;
 import java.lang.management.ThreadMXBean;
+import java.net.URL;
+import java.net.URLDecoder;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.List;
 import org.jruby.anno.JRubyMethod;
@@ -67,8 +71,10 @@ import org.objectweb.asm.ClassReader;
 import org.objectweb.asm.util.TraceClassVisitor;
 
 import java.util.Map;
+import org.jruby.java.proxies.JavaProxy;
 import org.jruby.runtime.ExecutionContext;
 import org.jruby.runtime.ObjectAllocator;
+import org.jruby.runtime.Visibility;
 
 /**
  * Module which defines JRuby-specific methods for use. 
@@ -79,12 +85,15 @@ public class RubyJRuby {
         ThreadContext context = runtime.getCurrentContext();
         runtime.getKernel().callMethod(context, "require", runtime.newString("java"));
         RubyModule jrubyModule = runtime.defineModule("JRuby");
-        
+
         jrubyModule.defineAnnotatedMethods(RubyJRuby.class);
+        jrubyModule.defineAnnotatedMethods(UtilLibrary.class);
 
         RubyClass compiledScriptClass = jrubyModule.defineClassUnder("CompiledScript",runtime.getObject(), runtime.getObject().getAllocator());
 
-        compiledScriptClass.attr_accessor(context, new IRubyObject[]{runtime.newSymbol("name"), runtime.newSymbol("class_name"), runtime.newSymbol("original_script"), runtime.newSymbol("code")});
+        for (String name : new String[] {"name", "class_name", "original_script", "code"}) {
+            compiledScriptClass.addReadWriteAttribute(context, name);
+        }
         compiledScriptClass.defineAnnotatedMethods(JRubyCompiledScript.class);
 
         RubyClass threadLocalClass = jrubyModule.defineClassUnder("ThreadLocal", runtime.getObject(), JRubyThreadLocal.ALLOCATOR);
@@ -113,6 +122,29 @@ public class RubyJRuby {
         runtime.getString().defineAnnotatedMethods(JRubyStringExtensions.class);
     }
 
+    public static class JRubySynchronizedMeta {
+        @JRubyMethod(frame = true, visibility = Visibility.PRIVATE)
+        public static IRubyObject append_features(IRubyObject self, IRubyObject target) {
+            if (target instanceof RubyClass && self instanceof RubyModule) { // should always be true
+                RubyClass targetModule = ((RubyClass)target);
+                targetModule.becomeSynchronized();
+                return ((RubyModule)self).append_features(target);
+            }
+            throw target.getRuntime().newTypeError(self + " can only be included into classes");
+        }
+
+        @JRubyMethod(frame = true, visibility = Visibility.PRIVATE)
+        public static IRubyObject extend_object(IRubyObject self, IRubyObject obj) {
+            if (self instanceof RubyModule) {
+                RubyClass singletonClass = obj.getSingletonClass();
+                singletonClass.becomeSynchronized();
+                return ((RubyModule)self).extend_object(obj);
+            }
+            // should never happen
+            throw self.getRuntime().newTypeError("JRuby::Singleton.extend_object called against " + self);
+        }
+    }
+
     public static class ExtLibrary implements Library {
         public void load(Ruby runtime, boolean wrap) throws IOException {
             RubyJRuby.createJRubyExt(runtime);
@@ -126,6 +158,16 @@ public class RubyJRuby {
             RubyJRuby.createJRubyCoreExt(runtime);
         }
     }
+
+    public static class SynchronizedLibrary implements Library {
+        public void load(Ruby runtime, boolean wrap) throws IOException {
+            RubyModule syncModule = runtime.getOrCreateModule("JRuby").defineModuleUnder("Synchronized");
+            syncModule.getSingletonClass().defineAnnotatedMethods(JRubySynchronizedMeta.class);
+
+            // make Synchronized itself be synchronized, so subclasses created later pick that up
+            syncModule.becomeSynchronized();
+        }
+    }
     
     public static class TypeLibrary implements Library {
         public void load(Ruby runtime, boolean wrap) throws IOException {
@@ -151,11 +193,63 @@ public class RubyJRuby {
         }
     }
 
-    @JRubyMethod(module = true)
-    public static void gc(IRubyObject recv) {
-        System.gc();
+    /**
+     * Utilities library for all those methods that don't need the full 'java' library
+     * to be loaded. This is done mostly for performance reasons. For example, for those
+     * who only need to enable the object space, not loading 'java' might save 200-300ms
+     * of startup time, like in case of jirb.
+     */
+    public static class UtilLibrary implements Library {
+        public void load(Ruby runtime, boolean wrap) throws IOException {
+            RubyModule mJRubyUtil = runtime.getOrCreateModule("JRuby").defineModuleUnder("Util");
+            mJRubyUtil.defineAnnotatedMethods(UtilLibrary.class);
+        }
+        @JRubyMethod(module = true)
+        public static void gc(IRubyObject recv) {
+            System.gc();
+        }
+        @JRubyMethod(name = "objectspace", module = true)
+        public static IRubyObject getObjectSpaceEnabled(IRubyObject recv) {
+            Ruby runtime = recv.getRuntime();
+            return RubyBoolean.newBoolean(
+                    runtime, runtime.isObjectSpaceEnabled());
+        }
+        @JRubyMethod(name = "objectspace=", module = true)
+        public static IRubyObject setObjectSpaceEnabled(
+                IRubyObject recv, IRubyObject arg) {
+            Ruby runtime = recv.getRuntime();
+            runtime.setObjectSpaceEnabled(arg.isTrue());
+            return runtime.getNil();
+        }
+        @SuppressWarnings("deprecation")
+        @JRubyMethod(name = "classloader_resources", module = true)
+        public static IRubyObject getClassLoaderResources(IRubyObject recv, IRubyObject arg) {
+            Ruby runtime = recv.getRuntime();
+            String resource = arg.convertToString().toString();
+            final List<RubyString> urlStrings = new ArrayList<RubyString>();
+
+            try {
+                Enumeration<URL> urls = runtime.getJRubyClassLoader().getResources(resource);
+                while (urls.hasMoreElements()) {
+                    URL url = urls.nextElement();
+
+                    String urlString;
+                    if ("jar".equals(url.getProtocol()) && url.getFile().startsWith("file:/")) {
+                        urlString = URLDecoder.decode(url.getFile());
+                    } else {
+                        urlString = url.getFile();
+                    }
+
+                    urlStrings.add(runtime.newString(urlString));
+                }
+                return RubyArray.newArrayNoCopy(runtime,
+                        urlStrings.toArray(new IRubyObject[urlStrings.size()]));
+            } catch (IOException ignore) {}
+
+            return runtime.newEmptyArray();
+        }
     }
-    
+
     @JRubyMethod(name = "runtime", frame = true, module = true)
     public static IRubyObject runtime(IRubyObject recv, Block unusedBlock) {
         return JavaUtil.convertJavaToUsableRubyObject(recv.getRuntime(), recv.getRuntime());
@@ -178,21 +272,6 @@ public class RubyJRuby {
         return currentRuntime.getNil();
     }
 
-    @JRubyMethod(name = "objectspace", frame = true, module = true)
-    public static IRubyObject getObjectSpaceEnabled(IRubyObject recv, Block b) {
-        Ruby runtime = recv.getRuntime();
-        return RubyBoolean.newBoolean(
-                runtime, runtime.isObjectSpaceEnabled());
-    }
-
-    @JRubyMethod(name = "objectspace=", required = 1, frame = true, module = true)
-    public static IRubyObject setObjectSpaceEnabled(
-            IRubyObject recv, IRubyObject arg, Block b) {
-        Ruby runtime = recv.getRuntime();
-        runtime.setObjectSpaceEnabled(arg.isTrue());
-        return runtime.getNil();
-    }
-
     @JRubyMethod(name = {"parse", "ast_for"}, optional = 3, frame = true, module = true)
     public static IRubyObject parse(IRubyObject recv, IRubyObject[] args, Block block) {
         if(block.isGiven()) {
@@ -274,16 +353,20 @@ public class RubyJRuby {
     public static IRubyObject reference(ThreadContext context, IRubyObject recv, IRubyObject obj) {
         Ruby runtime = context.getRuntime();
 
-        return Java.wrap(runtime, JavaObject.wrap(runtime, obj));
+        return Java.getInstance(runtime, obj);
     }
 
     @JRubyMethod(name = "dereference", required = 1, module = true)
     public static IRubyObject dereference(ThreadContext context, IRubyObject recv, IRubyObject obj) {
-        if (!(obj.dataGetStruct() instanceof JavaObject)) {
+        Object unwrapped;
+
+        if (obj instanceof JavaProxy) {
+            unwrapped = ((JavaProxy)obj).getObject();
+        } else if (obj.dataGetStruct() instanceof JavaObject) {
+            unwrapped = JavaUtil.unwrapJavaObject(obj);
+        } else {
             throw context.getRuntime().newTypeError("got " + obj + ", expected wrapped Java object");
         }
-        
-        Object unwrapped = JavaUtil.unwrapJavaObject(obj);
 
         if (!(unwrapped instanceof IRubyObject)) {
             throw context.getRuntime().newTypeError("got " + obj + ", expected Java-wrapped Ruby object");
@@ -533,8 +616,11 @@ public class RubyJRuby {
                 throw context.getRuntime().newTypeError(maybeClass, context.getRuntime().getClassClass());
             }
 
-            if (clazz.getReifiedClass() == null) {
-                return context.getRuntime().getNil();
+            for (RubyClass current = clazz; current != null; current = current.getSuperClass()) {
+                if (current.getReifiedClass() != null) {
+                    clazz = current;
+                    break;
+                }
             }
 
             return JavaUtil.convertJavaToUsableRubyObject(context.getRuntime(), clazz.getReifiedClass());
@@ -654,31 +740,41 @@ public class RubyJRuby {
             DynamicMethod method = rubyMethod.method;
             
             if (method instanceof MethodArgs) {
+                RubySymbol req = runtime.newSymbol("req");
+                RubySymbol opt = runtime.newSymbol("opt");
+                RubySymbol rest = runtime.newSymbol("rest");
+                RubySymbol block = runtime.newSymbol("block");
                 MethodArgs interpMethod = (MethodArgs)method;
                 ArgsNode args = interpMethod.getArgsNode();
                 RubyArray argsArray = RubyArray.newArray(runtime);
                 
-                RubyArray reqArray = RubyArray.newArray(runtime);
                 ListNode requiredArgs = args.getPre();
                 for (int i = 0; requiredArgs != null && i < requiredArgs.size(); i++) {
-                    reqArray.append(getNameFrom(runtime, (INameNode) requiredArgs.get(i)));
+                    argsArray.append(RubyArray.newArray(runtime, req, getNameFrom(runtime, (INameNode) requiredArgs.get(i))));
                 }
-                argsArray.append(reqArray);
                 
-                RubyArray optArray = RubyArray.newArray(runtime);
                 ListNode optArgs = args.getOptArgs();
                 for (int i = 0; optArgs != null && i < optArgs.size(); i++) {
-                    optArray.append(getNameFrom(runtime, (INameNode) optArgs.get(i)));
+                    argsArray.append(RubyArray.newArray(runtime, opt, getNameFrom(runtime, (INameNode) optArgs.get(i))));
+                }
+                
+                ListNode requiredArgsPost = args.getPost();
+                for (int i = 0; requiredArgsPost != null && i < requiredArgsPost.size(); i++) {
+                    argsArray.append(RubyArray.newArray(runtime, req, getNameFrom(runtime, (INameNode) requiredArgsPost.get(i))));
                 }
-                argsArray.append(optArray);
 
-                argsArray.append(getNameFrom(runtime, args.getRestArgNode()));
-                argsArray.append(getNameFrom(runtime, args.getBlock()));
+                if (args.getRestArg() >= 0) {
+                    argsArray.append(RubyArray.newArray(runtime, rest, getNameFrom(runtime, args.getRestArgNode())));
+                }
+
+                if (args.getBlock() != null) {
+                    argsArray.append(RubyArray.newArray(runtime, block, getNameFrom(runtime, args.getBlock())));
+                }
                 
                 return argsArray;
             }
             
-            throw runtime.newTypeError("Method args are only available for standard interpreted or jitted methods");
+            return runtime.getNil();
         }
     }
 
