@@ -58,10 +58,10 @@ import org.jruby.evaluator.ASTInterpreter;
 import org.jruby.exceptions.JumpException;
 import org.jruby.exceptions.MainExitException;
 import org.jruby.exceptions.RaiseException;
-import org.jruby.internal.runtime.JumpTarget;
 import org.jruby.internal.runtime.methods.CallConfiguration;
 import org.jruby.internal.runtime.methods.JavaMethod.JavaMethodNBlock;
 import org.jruby.javasupport.util.RuntimeHelpers;
+import org.jruby.platform.Platform;
 import org.jruby.runtime.Binding;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallType;
@@ -179,12 +179,11 @@ public class RubyKernel {
             throw runtime.newNameError("autoload must be constant name", nonInternedName);
         }
 
-        if (!(file instanceof RubyString)) throw runtime.newTypeError(file, runtime.getString());
-        RubyString fileString = file.convertToString();
+        if (!runtime.is1_9() && !(file instanceof RubyString)) throw runtime.newTypeError(file, runtime.getString());
+
+        RubyString fileString = RubyFile.get_path(runtime.getCurrentContext(), file);
         
-        if (fileString.isEmpty()) {
-            throw runtime.newArgumentError("empty file name");
-        }
+        if (fileString.isEmpty()) throw runtime.newArgumentError("empty file name");
         
         final String baseName = symbol.asJavaString().intern(); // interned, OK for "fast" methods
         final RubyModule module = recv instanceof RubyModule ? (RubyModule) recv : runtime.getObject();
@@ -293,7 +292,7 @@ public class RubyKernel {
 
     @JRubyMethod(name = "gets", optional = 1, module = true, visibility = PRIVATE)
     public static IRubyObject gets(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
-        return RubyArgsFile.gets(context, context.getRuntime().getGlobalVariables().get("$<"), args);
+        return RubyArgsFile.gets(context, context.getRuntime().getArgsFile(), args);
     }
 
     @JRubyMethod(name = "abort", optional = 1, module = true, visibility = PRIVATE)
@@ -306,18 +305,7 @@ public class RubyKernel {
 
     @JRubyMethod(name = "Array", required = 1, module = true, visibility = PRIVATE)
     public static IRubyObject new_array(ThreadContext context, IRubyObject recv, IRubyObject object) {
-        IRubyObject value = object.checkArrayType();
-
-        if (value.isNil()) {
-            if (object.getMetaClass().searchMethod("to_a").getImplementationClass() != context.getRuntime().getKernel()) {
-                value = object.callMethod(context, "to_a");
-                if (!(value instanceof RubyArray)) throw context.getRuntime().newTypeError("`to_a' did not return Array");
-                return value;
-            } else {
-                return context.getRuntime().newArray(object);
-            }
-        }
-        return value;
+        return RuntimeHelpers.arrayValue(context, context.getRuntime(), object);
     }
 
     @JRubyMethod(name = "Complex", module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
@@ -346,7 +334,7 @@ public class RubyKernel {
         return RuntimeHelpers.invoke(context, context.getRuntime().getRational(), "convert", arg0, arg1);
     }
 
-    @JRubyMethod(name = "Float", module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "Float", module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static RubyFloat new_float(IRubyObject recv, IRubyObject object) {
         if(object instanceof RubyFixnum){
             return RubyFloat.newFloat(object.getRuntime(), ((RubyFixnum)object).getDoubleValue());
@@ -355,7 +343,7 @@ public class RubyKernel {
         }else if(object instanceof RubyBignum){
             return RubyFloat.newFloat(object.getRuntime(), RubyBignum.big2dbl((RubyBignum)object));
         }else if(object instanceof RubyString){
-            if(((RubyString)object).getByteList().realSize == 0){ // rb_cstr_to_dbl case
+            if(((RubyString) object).getByteList().getRealSize() == 0){ // rb_cstr_to_dbl case
                 throw recv.getRuntime().newArgumentError("invalid value for Float(): " + object.inspect());
             }
             return RubyNumeric.str2fnum(recv.getRuntime(),(RubyString)object,true);
@@ -368,7 +356,28 @@ public class RubyKernel {
         }
     }
 
-    @JRubyMethod(name = "Integer", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "Float", module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static RubyFloat new_float19(IRubyObject recv, IRubyObject object) {
+        if(object instanceof RubyFixnum){
+            return RubyFloat.newFloat(object.getRuntime(), ((RubyFixnum)object).getDoubleValue());
+        }else if(object instanceof RubyFloat){
+            return (RubyFloat)object;
+        }else if(object instanceof RubyBignum){
+            return RubyFloat.newFloat(object.getRuntime(), RubyBignum.big2dbl((RubyBignum)object));
+        }else if(object instanceof RubyString){
+            if(((RubyString) object).getByteList().getRealSize() == 0){ // rb_cstr_to_dbl case
+                throw recv.getRuntime().newArgumentError("invalid value for Float(): " + object.inspect());
+            }
+            return RubyNumeric.str2fnum(recv.getRuntime(),(RubyString)object,true);
+        }else if(object.isNil()){
+            throw recv.getRuntime().newTypeError("can't convert nil into Float");
+        } else {
+            RubyFloat rFloat = (RubyFloat)TypeConverter.convertToType19(object, recv.getRuntime().getFloat(), "to_f");
+            return rFloat;
+        }
+    }
+
+    @JRubyMethod(name = "Integer", required = 1, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject new_integer(ThreadContext context, IRubyObject recv, IRubyObject object) {
         if (object instanceof RubyFloat) {
             double val = ((RubyFloat)object).getDoubleValue(); 
@@ -386,11 +395,50 @@ public class RubyKernel {
         return tmp;
     }
 
-    @JRubyMethod(name = "String", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "Integer", module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject new_integer19(ThreadContext context, IRubyObject recv, IRubyObject object) {
+        if (object instanceof RubyFloat) {
+            double val = ((RubyFloat)object).getDoubleValue(); 
+            if (val > (double) RubyFixnum.MAX && val < (double) RubyFixnum.MIN) {
+                return RubyNumeric.dbl2num(context.getRuntime(),((RubyFloat)object).getDoubleValue());
+            }
+        } else if (object instanceof RubyFixnum || object instanceof RubyBignum) {
+            return object;
+        } else if (object instanceof RubyString) {
+            return RubyNumeric.str2inum(context.getRuntime(),(RubyString)object,0,true);
+        } else if(object instanceof RubyNil) {
+            throw context.getRuntime().newTypeError("can't convert nil into Integer");
+        }
+        
+        IRubyObject tmp = TypeConverter.convertToType(object, context.getRuntime().getInteger(), "to_int", false);
+        if (tmp.isNil()) return object.convertToInteger("to_i");
+        return tmp;
+    }
+
+    @JRubyMethod(name = "Integer", module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject new_integer19(ThreadContext context, IRubyObject recv, IRubyObject object, IRubyObject base) {
+        int bs = RubyNumeric.num2int(base);
+        if(object instanceof RubyString) {
+            return RubyNumeric.str2inum(context.getRuntime(),(RubyString)object,bs,true);
+        } else {
+            IRubyObject tmp = object.checkStringType();
+            if(!tmp.isNil()) {
+                return RubyNumeric.str2inum(context.getRuntime(),(RubyString)tmp,bs,true);
+            }
+        }
+        throw context.getRuntime().newArgumentError("base specified for non string value");
+    }
+
+    @JRubyMethod(name = "String", required = 1, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject new_string(ThreadContext context, IRubyObject recv, IRubyObject object) {
         return TypeConverter.convertToType(object, context.getRuntime().getString(), "to_s");
     }
 
+    @JRubyMethod(name = "String", required = 1, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject new_string19(ThreadContext context, IRubyObject recv, IRubyObject object) {
+        return TypeConverter.convertToType19(object, context.getRuntime().getString(), "to_s");
+    }
+
     @JRubyMethod(name = "p", rest = true, module = true, visibility = PRIVATE)
     public static IRubyObject p(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = context.getRuntime();
@@ -469,7 +517,7 @@ public class RubyKernel {
 
     @JRubyMethod(name = "readlines", optional = 1, module = true, visibility = PRIVATE)
     public static IRubyObject readlines(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
-        return RubyArgsFile.readlines(context, context.getRuntime().getGlobalVariables().get("$<"), args);
+        return RubyArgsFile.readlines(context, context.getRuntime().getArgsFile(), args);
     }
 
     /** Returns value of $_.
@@ -606,7 +654,7 @@ public class RubyKernel {
     public static IRubyObject chop(ThreadContext context, IRubyObject recv, Block block) {
         RubyString str = getLastlineString(context, context.getRuntime());
 
-        if (str.getByteList().realSize > 0) {
+        if (str.getByteList().getRealSize() > 0) {
             str = (RubyString) str.dup();
             str.chop_bang(context);
             context.getCurrentScope().setLastLine(str);
@@ -761,17 +809,18 @@ public class RubyKernel {
         exit(recv.getRuntime(), args, true);
         return recv.getRuntime().getNil(); // not reached
     }
-    
+
     private static void exit(Ruby runtime, IRubyObject[] args, boolean hard) {
         runtime.secure(4);
 
-        int status = 1;
+        int status = 0;
+
         if (args.length > 0) {
-            RubyObject argument = (RubyObject)args[0];
-            if (argument instanceof RubyFixnum) {
-                status = RubyNumeric.fix2int(argument);
-            } else {
+            RubyObject argument = (RubyObject) args[0];
+            if (argument instanceof RubyBoolean) {
                 status = argument.isFalse() ? 1 : 0;
+            } else {
+                status = RubyNumeric.fix2int(argument);
             }
         }
 
@@ -898,12 +947,12 @@ public class RubyKernel {
     private static void printExceptionSummary(ThreadContext context, Ruby runtime, RubyException rEx) {
         Frame currentFrame = context.getCurrentFrame();
 
-        String msg = String.format("Exception `%s':%s at %s:%s - %s\n",
-                rEx.getMetaClass(), rEx.convertToString().toString(),
+        String msg = String.format("Exception `%s' at %s:%s - %s\n",
+                rEx.getMetaClass(),
                 currentFrame.getFile(), currentFrame.getLine() + 1,
-                rEx.to_s());
+                rEx.convertToString().toString());
 
-        runtime.getErr().print(msg);
+        runtime.getErrorStream().print(msg);
     }
 
     /**
@@ -913,24 +962,53 @@ public class RubyKernel {
      * @param recv ruby object used to call require (any object will do and it won't be used anyway).
      * @param name the name of the file to require
      **/
-    @JRubyMethod(name = "require", required = 1, frame = true, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "require", required = 1, frame = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject require(IRubyObject recv, IRubyObject name, Block block) {
-        Ruby runtime = recv.getRuntime();
-        
+        return requireCommon(recv.getRuntime(), recv, name, block);
+    }
+
+    @JRubyMethod(name = "require", required = 1, frame = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject require19(ThreadContext context, IRubyObject recv, IRubyObject name, Block block) {
+        Ruby runtime = context.getRuntime();
+
+        IRubyObject tmp = name.checkStringType();
+        if (!tmp.isNil()) {
+            return requireCommon(runtime, recv, tmp, block);
+        }
+
+        return requireCommon(runtime, recv,
+                name.respondsTo("to_path") ? name.callMethod(context, "to_path") : name, block);
+    }
+
+    private static IRubyObject requireCommon(Ruby runtime, IRubyObject recv, IRubyObject name, Block block) {
         if (runtime.getLoadService().lockAndRequire(name.convertToString().toString())) {
             return runtime.getTrue();
         }
         return runtime.getFalse();
     }
 
-    @JRubyMethod(name = "load", required = 1, optional = 1, frame = true, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "load", required = 1, optional = 1, frame = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject load(IRubyObject recv, IRubyObject[] args, Block block) {
-        Ruby runtime = recv.getRuntime();
-        RubyString file = args[0].convertToString();
+        return loadCommon(args[0], recv.getRuntime(), args, block);
+    }
+
+    @JRubyMethod(name = "load", required = 1, optional = 1, frame = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject load19(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
+        IRubyObject file = args[0];
+        if (!(file instanceof RubyString) && file.respondsTo("to_path")) {
+            file = file.callMethod(context, "to_path");
+        }
+
+        return loadCommon(file, context.getRuntime(), args, block);
+    }
+
+    private static IRubyObject loadCommon(IRubyObject fileName, Ruby runtime, IRubyObject[] args, Block block) {
+        RubyString file = fileName.convertToString();
+
         boolean wrap = args.length == 2 ? args[1].isTrue() : false;
 
         runtime.getLoadService().load(file.getByteList().toString(), wrap);
-        
+
         return runtime.getTrue();
     }
 
@@ -957,7 +1035,8 @@ public class RubyKernel {
         }
         if (args.length > 3) {
             // file given, use it and force it into binding
-            binding.setLine((int) args[3].convertToInteger().getLongValue());
+            // -1 because parser uses zero offsets and other code compensates
+            binding.setLine(((int) args[3].convertToInteger().getLongValue()) - 1);
         } else {
             // no binding given, use 0 for both
             binding.setLine(0);
@@ -989,7 +1068,7 @@ public class RubyKernel {
         int level = args.length > 0 ? RubyNumeric.fix2int(args[0]) : 1;
 
         if (level < 0) {
-            throw context.getRuntime().newArgumentError("negative level(" + level + ')');
+            throw context.getRuntime().newArgumentError("negative level (" + level + ')');
         }
 
         return context.createCallerBacktrace(context.getRuntime(), level);
@@ -1000,7 +1079,7 @@ public class RubyKernel {
         int level = args.length > 0 ? RubyNumeric.fix2int(args[0]) : 1;
 
         if (level < 0) {
-            throw context.getRuntime().newArgumentError("negative level(" + level + ')');
+            throw context.getRuntime().newArgumentError("negative level (" + level + ')');
         }
 
         return context.createCallerBacktrace(context.getRuntime(), level);
@@ -1008,44 +1087,39 @@ public class RubyKernel {
 
     @JRubyMethod(name = "catch", required = 1, frame = true, module = true, visibility = PRIVATE)
     public static IRubyObject rbCatch(ThreadContext context, IRubyObject recv, IRubyObject tag, Block block) {
-        CatchTarget target = new CatchTarget(tag.asJavaString());
+        RubyContinuation rbContinuation = new RubyContinuation(context.getRuntime(), tag.asJavaString());
         try {
-            context.pushCatch(target);
-            return block.yield(context, tag);
-        } catch (JumpException.ThrowJump tj) {
-            if (tj.getTarget() == target) return (IRubyObject) tj.getValue();
-            
-            throw tj;
+            context.pushCatch(rbContinuation.getContinuation());
+            return rbContinuation.enter(context, block);
         } finally {
             context.popCatch();
         }
     }
-    
-    public static class CatchTarget implements JumpTarget {
-        private final String tag;
-        public CatchTarget(String tag) { this.tag = tag; }
-        public String getTag() { return tag; }
+
+    @JRubyMethod(name = "throw", frame = true, module = true, visibility = PRIVATE)
+    public static IRubyObject rbThrow(ThreadContext context, IRubyObject recv, IRubyObject tag, Block block) {
+        return rbThrowInternal(context, tag.asJavaString(), IRubyObject.NULL_ARRAY, block);
     }
 
-    @JRubyMethod(name = "throw", required = 1, frame = true, optional = 1, module = true, visibility = PRIVATE)
-    public static IRubyObject rbThrow(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
+    @JRubyMethod(name = "throw", frame = true, module = true, visibility = PRIVATE)
+    public static IRubyObject rbThrow(ThreadContext context, IRubyObject recv, IRubyObject tag, IRubyObject arg, Block block) {
+        return rbThrowInternal(context, tag.asJavaString(), new IRubyObject[] {arg}, block);
+    }
+
+    public static IRubyObject rbThrowInternal(ThreadContext context, String tag, IRubyObject[] args, Block block) {
         Ruby runtime = context.getRuntime();
 
-        String tag = args[0].asJavaString();
-        CatchTarget[] catches = context.getActiveCatches();
+        RubyContinuation.Continuation continuation = context.getActiveCatch(tag.intern());
 
-        String message = "uncaught throw `" + tag + "'";
-
-        // Ordering of array traversal not important, just intuitive
-        for (int i = catches.length - 1 ; i >= 0 ; i--) {
-            if (tag.equals(catches[i].getTag())) {
-                //Catch active, throw for catch to handle
-                throw new JumpException.ThrowJump(catches[i], args.length > 1 ? args[1] : runtime.getNil());
-            }
+        if (continuation != null) {
+            continuation.args = args;
+            throw continuation;
         }
 
         // No catch active for this throw
+        String message = "uncaught throw `" + tag + "'";
         RubyThread currentThread = context.getThread();
+
         if (currentThread == runtime.getThreadService().getMainThread()) {
             throw runtime.newNameError(message, tag);
         } else {
@@ -1055,7 +1129,7 @@ public class RubyKernel {
 
     @JRubyMethod(name = "trap", required = 1, frame = true, optional = 1, module = true, visibility = PRIVATE)
     public static IRubyObject trap(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
-        context.getRuntime().getLoadService().require("jsignal");
+        context.getRuntime().getLoadService().require("jsignal_internal");
         return RuntimeHelpers.invoke(context, recv, "__jtrap", args, block);
     }
     
@@ -1072,11 +1146,6 @@ public class RubyKernel {
 
     @JRubyMethod(name = "set_trace_func", required = 1, frame = true, module = true, visibility = PRIVATE)
     public static IRubyObject set_trace_func(ThreadContext context, IRubyObject recv, IRubyObject trace_func, Block block) {
-        if (!RubyInstanceConfig.FULL_TRACE_ENABLED) {
-            // without full tracing, many events will not fire
-            context.getRuntime().getWarnings().warn("set_trace_func will not capture all events without --debug flag");
-        }
-        
         if (trace_func.isNil()) {
             context.getRuntime().setTraceFunction(null);
         } else if (!(trace_func instanceof RubyProc)) {
@@ -1298,16 +1367,46 @@ public class RubyKernel {
     @JRubyMethod(name = "`", required = 1, module = true, visibility = PRIVATE)
     public static IRubyObject backquote(ThreadContext context, IRubyObject recv, IRubyObject aString) {
         Ruby runtime = context.getRuntime();
-        ByteArrayOutputStream output = new ByteArrayOutputStream();
-        
         RubyString string = aString.convertToString();
-        int resultCode = ShellLauncher.runAndWait(runtime, new IRubyObject[] {string}, output);
-        
+        IRubyObject[] args = new IRubyObject[] {string};
+        ByteArrayOutputStream output = new ByteArrayOutputStream();
+        int resultCode;
+
+        try {
+            // NOTE: not searching executable path before invoking args
+            resultCode = ShellLauncher.runAndWait(runtime, args, output, false);
+        } catch (Exception e) {
+            resultCode = 127;
+        }
+
         runtime.getGlobalVariables().set("$?", RubyProcess.RubyStatus.newProcessStatus(runtime, resultCode));
-        
-        return RubyString.newStringNoCopy(runtime, output.toByteArray());
+
+        byte[] out = output.toByteArray();
+        int length = out.length;
+
+        if (Platform.IS_WINDOWS) {
+            // MRI behavior, replace '\r\n' by '\n'
+            int newPos = 0;
+            byte curr, next;
+            for (int pos = 0; pos < length; pos++) {
+                curr = out[pos];
+                if (pos == length - 1) {
+                    out[newPos++] = curr;
+                    break;
+                }
+                next = out[pos + 1];
+                if (curr != '\r' || next != '\n') {
+                    out[newPos++] = curr;
+                }
+            }
+
+            // trim the length
+            length = newPos;
+        }
+
+        return RubyString.newStringNoCopy(runtime, out, 0, length);
     }
-    
+
     @JRubyMethod(name = "srand", module = true, visibility = PRIVATE)
     public static RubyInteger srand(ThreadContext context, IRubyObject recv) {
         Ruby runtime = context.getRuntime();
@@ -1379,11 +1478,13 @@ public class RubyKernel {
     public static RubyBoolean system(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = context.getRuntime();
         int resultCode;
+
         try {
             resultCode = ShellLauncher.runAndWait(runtime, args);
         } catch (Exception e) {
             resultCode = 127;
         }
+
         runtime.getGlobalVariables().set("$?", RubyProcess.RubyStatus.newProcessStatus(runtime, resultCode));
         return runtime.newBoolean(resultCode == 0);
     }
@@ -1402,6 +1503,8 @@ public class RubyKernel {
             // TODO: exec should replace the current process.
             // This could be possible with JNA. 
             resultCode = ShellLauncher.execAndWait(runtime, args);
+        } catch (RaiseException e) {
+            throw e; // no need to wrap this exception
         } catch (Exception e) {
             throw runtime.newErrnoENOENTError("cannot execute");
         }
@@ -1475,4 +1578,9 @@ public class RubyKernel {
         String name = f != null ? f.getName() : null;
         return name != null ? context.getRuntime().newSymbol(name) : context.getRuntime().getNil();
     }
+
+    @JRubyMethod(module = true, frame = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject singleton_class(IRubyObject recv) {
+        return recv.getSingletonClass();
+    }
 }
