@@ -68,6 +68,10 @@ public class RubyMatchData extends RubyObject {
     public static RubyClass createMatchDataClass(Ruby runtime) {
         RubyClass matchDataClass = runtime.defineClass("MatchData", runtime.getObject(), MATCH_DATA_ALLOCATOR);
         runtime.setMatchData(matchDataClass);
+
+        matchDataClass.index = ClassIndex.MATCHDATA;
+        matchDataClass.setReifiedClass(RubyMatchData.class);
+        
         runtime.defineGlobalConstant("MatchingData", matchDataClass);
         matchDataClass.kindOf = new RubyModule.KindOf() {
             @Override
@@ -97,7 +101,7 @@ public class RubyMatchData extends RubyObject {
 
     @Override
     public int getNativeTypeIndex() {
-        return ClassIndex.MATCH;
+        return ClassIndex.MATCHDATA;
     }
 
     private static final class Pair implements Comparable {
@@ -131,9 +135,9 @@ public class RubyMatchData extends RubyObject {
         Arrays.sort(pairs);
 
         ByteList value = str.getByteList();
-        Encoding enc = value.encoding;
-        byte[]bytes = value.bytes;
-        int p = value.begin;
+        Encoding enc = value.getEncoding();
+        byte[]bytes = value.getUnsafeBytes();
+        int p = value.getBegin();
         int s = p;
 
         int c = 0;
@@ -254,7 +258,7 @@ public class RubyMatchData extends RubyObject {
             if (v.isNil()) {
                 result.cat("nil".getBytes());
             } else {
-                result.append(((RubyString) str).inspectCommon(runtime.is1_9()));
+                result.append(((RubyString) v).inspectCommon(runtime.is1_9()));
             }
         }
 
@@ -321,7 +325,7 @@ public class RubyMatchData extends RubyObject {
     private int nameToBackrefNumber(RubyString str) {
         ByteList value = str.getByteList();
         try {
-            return pattern.nameToBackrefNumber(value.bytes, value.begin, value.begin + value.realSize, regs);
+            return pattern.nameToBackrefNumber(value.getUnsafeBytes(), value.getBegin(), value.getBegin() + value.getRealSize(), regs);
         } catch (JOniException je) {
             throw getRuntime().newIndexError(je.getMessage());
         }
