@@ -11,7 +11,7 @@
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
- * Copyright (C) 2007-2009 Koichiro Ohba <koichiro@meadowy.org>
+ * Copyright (C) 2007-2010 Koichiro Ohba <koichiro@meadowy.org>
  * 
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -119,13 +119,19 @@ public class RubyNKF {
 
     @JRubyMethod(name = "guess", required = 1, module = true)
     public static IRubyObject guess(ThreadContext context, IRubyObject recv, IRubyObject s) {
+        // TODO: Fix charset usage for JRUBY-4553
         Ruby runtime = context.getRuntime();
         if (!s.respondsTo("to_str")) {
             throw runtime.newTypeError("can't convert " + s.getMetaClass() + " into String");
         }
         ByteList bytes = s.convertToString().getByteList();
-        ByteBuffer buf = ByteBuffer.wrap(bytes.unsafeBytes(), bytes.begin(), bytes.length());
-        CharsetDecoder decoder = Charset.forName("x-JISAutoDetect").newDecoder();
+        ByteBuffer buf = ByteBuffer.wrap(bytes.getUnsafeBytes(), bytes.begin(), bytes.length());
+        CharsetDecoder decoder;
+        try {
+            decoder = Charset.forName("x-JISAutoDetect").newDecoder();
+        } catch (UnsupportedCharsetException e) {
+            throw runtime.newStandardError("charsets.jar is required to use NKF#guess. Please install JRE which supports m17n.");
+        }
         try {
             decoder.decode(buf);
         } catch (CharacterCodingException e) {
@@ -224,12 +230,13 @@ public class RubyNKF {
             decoder = Charset.forName(decodeCharset).newDecoder();
             encoder = Charset.forName(encodeCharset).newEncoder();
         } catch (UnsupportedCharsetException e) {
-            throw runtime.newArgumentError("invalid encoding");
+            throw runtime.newArgumentError("invalid charset");
         }
         
-        ByteBuffer buf = ByteBuffer.wrap(str.unsafeBytes(), str.begin(), str.length());
+        ByteBuffer buf = ByteBuffer.wrap(str.getUnsafeBytes(), str.begin(), str.length());
         try {
             CharBuffer cbuf = decoder.decode(buf);
+            encoder.onUnmappableCharacter(java.nio.charset.CodingErrorAction.IGNORE);
             buf = encoder.encode(cbuf);
         } catch (CharacterCodingException e) {
             throw runtime.newArgumentError("invalid encoding");
@@ -241,8 +248,7 @@ public class RubyNKF {
     }
 
     private static RubyString encodeMimeString(Ruby runtime, RubyString str, String format) {
-        RubyArray array = RubyArray.newArray(runtime, 1);
-        array.append(str);
+        RubyArray array = RubyArray.newArray(runtime, str);
         return Pack.pack(runtime, array, new ByteList(ByteList.plain(format))).chomp(runtime.getCurrentContext());
     }
 
