@@ -34,7 +34,6 @@ package org.jruby;
 
 import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyClass;
-import org.jruby.javasupport.JavaUtil;
 import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
@@ -61,6 +60,7 @@ public class RubyNil extends RubyObject {
         RubyClass nilClass = runtime.defineClass("NilClass", runtime.getObject(), NIL_ALLOCATOR);
         runtime.setNilClass(nilClass);
         nilClass.index = ClassIndex.NIL;
+        nilClass.setReifiedClass(RubyNil.class);
         
         nilClass.defineAnnotatedMethods(RubyNil.class);
         
@@ -198,6 +198,14 @@ public class RubyNil extends RubyObject {
         return RubyRational.newRationalCanonicalize(context, RubyFixnum.zero(context.getRuntime()));
     }
 
+    /** nilclass_rationalize
+     *
+     */
+    @JRubyMethod(name = "rationalize", optional = 1, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject rationalize(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
+        return to_r(context, recv);
+    }
+
     @Override
     public Object toJava(Class target) {
         if (target.isPrimitive()) {
