@@ -80,7 +80,8 @@ public class RubyNoMethodError extends RubyNameError {
         return args;
     }
 
-    protected void copySpecialInstanceVariables(IRubyObject clone) {
+    @Override
+    public void copySpecialInstanceVariables(IRubyObject clone) {
         super.copySpecialInstanceVariables(clone);
         RubyNoMethodError exception = (RubyNoMethodError)clone;
         exception.args = args;
