@@ -48,13 +48,14 @@ import org.jruby.exceptions.RaiseException;
 import org.jruby.javasupport.JavaUtil;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Block;
+import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
-import org.jruby.util.Convert;
-import org.jruby.util.Convert2;
+import org.jruby.util.ConvertDouble;
+import org.jruby.util.ConvertBytes;
 
 /**
  * Base class for all numerical types in ruby.
@@ -69,6 +70,9 @@ public class RubyNumeric extends RubyObject {
         RubyClass numeric = runtime.defineClass("Numeric", runtime.getObject(), NUMERIC_ALLOCATOR);
         runtime.setNumeric(numeric);
 
+        numeric.index = ClassIndex.NUMERIC;
+        numeric.setReifiedClass(RubyNumeric.class);
+
         numeric.kindOf = new RubyModule.KindOf() {
             @Override
             public boolean isKindOf(IRubyObject obj, RubyModule type) {
@@ -88,7 +92,7 @@ public class RubyNumeric extends RubyObject {
         }
     };
 
-    public static double DBL_EPSILON=2.2204460492503131e-16;
+    public static final double DBL_EPSILON=2.2204460492503131e-16;
 
     private static IRubyObject convertToNum(double val, Ruby runtime) {
 
@@ -323,7 +327,7 @@ public class RubyNumeric extends RubyObject {
      */
     public static RubyInteger str2inum(Ruby runtime, RubyString str, int base, boolean strict) {
         ByteList s = str.getByteList();
-        return Convert2.byteListToInum(runtime, s, base, strict);
+        return ConvertBytes.byteListToInum(runtime, s, base, strict);
     }
 
     public static RubyFloat str2fnum(Ruby runtime, RubyString arg) {
@@ -349,10 +353,17 @@ public class RubyNumeric extends RubyObject {
      *          will be 0.0 if the conversion failed.
      */
     public static RubyFloat str2fnum(Ruby runtime, RubyString arg, boolean strict) {
+        return str2fnumCommon(runtime, arg, strict, biteListCaller18);
+    }
+    
+    public static RubyFloat str2fnum19(Ruby runtime, RubyString arg, boolean strict) {
+        return str2fnumCommon(runtime, arg, strict, biteListCaller19);
+    }
+
+    private static RubyFloat str2fnumCommon(Ruby runtime, RubyString arg, boolean strict, ByteListCaller caller) {
         final double ZERO = 0.0;
-        
         try {
-            return new RubyFloat(runtime,Convert.byteListToDouble(arg.getByteList(),strict));
+            return new RubyFloat(runtime, caller.yield(arg, strict));
         } catch (NumberFormatException e) {
             if (strict) {
                 throw runtime.newArgumentError("invalid value for Float(): "
@@ -361,6 +372,25 @@ public class RubyNumeric extends RubyObject {
             return new RubyFloat(runtime,ZERO);
         }
     }
+
+    private static interface ByteListCaller {
+        public double yield(RubyString arg, boolean strict);
+    }
+
+    private static class ByteListCaller18 implements ByteListCaller {
+        public double yield(RubyString arg, boolean strict) {
+            return ConvertDouble.byteListToDouble(arg.getByteList(),strict);
+        }
+    }
+    private static final ByteListCaller18 biteListCaller18 = new ByteListCaller18();
+
+    private static class ByteListCaller19 implements ByteListCaller {
+        public double yield(RubyString arg, boolean strict) {
+            return ConvertDouble.byteListToDouble19(arg.getByteList(),strict);
+        }
+    }
+    private static final ByteListCaller19 biteListCaller19 = new ByteListCaller19();
+
     
     /** Numeric methods. (num_*)
      *
@@ -519,6 +549,14 @@ public class RubyNumeric extends RubyObject {
         return this;
     }
 
+    /** num_imaginary
+     *
+     */
+    @JRubyMethod(name = "i", compat = CompatVersion.RUBY1_9)
+    public IRubyObject num_imaginary(ThreadContext context) {
+        return RubyComplex.newComplexRaw(context.getRuntime(), RubyFixnum.zero(context.getRuntime()), this);
+    }
+
     /** num_uminus
      *
      */
@@ -890,8 +928,15 @@ public class RubyNumeric extends RubyObject {
      */
     @JRubyMethod(name = {"arg", "angle", "phase"}, compat = CompatVersion.RUBY1_9)
     public IRubyObject arg(ThreadContext context) {
-        if (!f_negative_p(context, this)) return RubyFixnum.zero(context.getRuntime());
-        return context.getRuntime().getMath().fastFetchConstant("PI");
+        double value = this.getDoubleValue();
+        if (Double.isNaN(value)) {
+            return RubyFloat.newFloat(context.getRuntime(), Double.NaN);
+        }
+        if (f_negative_p(context, this) || (value == 0.0 && 1/value == Double.NEGATIVE_INFINITY)) {
+            // negative or -0.0
+            return context.getRuntime().getMath().fastGetConstant("PI");
+        }
+        return RubyFixnum.zero(context.getRuntime());
     }    
 
     /** numeric_rect
