@@ -34,25 +34,24 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
-import java.lang.reflect.InvocationHandler;
-import java.lang.reflect.Method;
-import java.lang.reflect.Proxy;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyClass;
 import org.jruby.exceptions.JumpException;
-import org.jruby.internal.runtime.JumpTarget;
-import org.jruby.java.MiniJava;
+import org.jruby.parser.BlockStaticScope;
+import org.jruby.parser.StaticScope;
 import org.jruby.runtime.Block;
+import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.runtime.marshal.DataType;
 
 /**
  * @author  jpetersen
  */
 @JRubyClass(name="Proc")
-public class RubyProc extends RubyObject implements JumpTarget {
+public class RubyProc extends RubyObject implements DataType {
     private Block block = Block.NULL_BLOCK;
     private Block.Type type;
     private String file;
@@ -77,6 +76,9 @@ public class RubyProc extends RubyObject implements JumpTarget {
     public static RubyClass createProcClass(Ruby runtime) {
         RubyClass procClass = runtime.defineClass("Proc", runtime.getObject(), PROC_ALLOCATOR);
         runtime.setProc(procClass);
+
+        procClass.index = ClassIndex.PROC;
+        procClass.setReifiedClass(RubyProc.class);
         
         procClass.defineAnnotatedMethods(RubyProc.class);
         
@@ -132,6 +134,17 @@ public class RubyProc extends RubyObject implements JumpTarget {
         }
         
         block = procBlock.cloneBlock();
+
+        if (type == Block.Type.THREAD) {
+            // modify the block with a new backref/lastline-grabbing scope
+            StaticScope oldScope = block.getBody().getStaticScope();
+            StaticScope newScope = new BlockStaticScope(oldScope.getEnclosingScope(), oldScope.getVariables());
+            newScope.setBackrefLastlineScope(true);
+            newScope.setPreviousCRefScope(oldScope.getPreviousCRefScope());
+            newScope.setModule(oldScope.getModule());
+            block.getBody().setStaticScope(newScope);
+        }
+
         block.type = type;
         block.setProcObject(this);
 
@@ -141,6 +154,7 @@ public class RubyProc extends RubyObject implements JumpTarget {
     }
     
     @JRubyMethod(name = "clone")
+    @Override
     public IRubyObject rbClone() {
     	RubyProc newProc = new RubyProc(getRuntime(), getRuntime().getProc(), type);
     	newProc.block = getBlock();
@@ -151,6 +165,7 @@ public class RubyProc extends RubyObject implements JumpTarget {
     }
 
     @JRubyMethod(name = "dup")
+    @Override
     public IRubyObject dup() {
         RubyProc newProc = new RubyProc(getRuntime(), getRuntime().getProc(), type);
         newProc.block = getBlock();
@@ -163,7 +178,7 @@ public class RubyProc extends RubyObject implements JumpTarget {
     public IRubyObject op_equal(IRubyObject other) {
         if (!(other instanceof RubyProc)) return getRuntime().getFalse();
         
-        if (this == other || this.block == ((RubyProc)other).block) {
+        if (this == other || this.block.equals(((RubyProc)other).block)) {
             return getRuntime().getTrue();
         }
         
@@ -171,6 +186,7 @@ public class RubyProc extends RubyObject implements JumpTarget {
     }
     
     @JRubyMethod(name = "to_s")
+    @Override
     public IRubyObject to_s() {
         return RubyString.newString(getRuntime(), 
                 "#<Proc:0x" + Integer.toString(block.hashCode(), 16) + "@" + 
@@ -183,6 +199,10 @@ public class RubyProc extends RubyObject implements JumpTarget {
     }
 
     @JRubyMethod(name = {"call", "[]"}, rest = true, frame = true, compat = CompatVersion.RUBY1_8)
+    public IRubyObject call(ThreadContext context, IRubyObject[] args, Block block) {
+        return call(context, args, null, block);
+    }
+
     public IRubyObject call(ThreadContext context, IRubyObject[] args) {
         return call(context, args, null, Block.NULL_BLOCK);
     }
@@ -196,7 +216,7 @@ public class RubyProc extends RubyObject implements JumpTarget {
         assert args != null;
         
         Block newBlock = block.cloneBlock();
-        JumpTarget jumpTarget = newBlock.getBinding().getFrame().getJumpTarget();
+        int jumpTarget = newBlock.getBinding().getFrame().getJumpTarget();
         
         try {
             if (self != null) newBlock.getBinding().setSelf(self);
@@ -205,13 +225,13 @@ public class RubyProc extends RubyObject implements JumpTarget {
         } catch (JumpException.BreakJump bj) {
             return handleBreakJump(getRuntime(), newBlock, bj, jumpTarget);
         } catch (JumpException.ReturnJump rj) {
-            return handleReturnJump(getRuntime(), rj, jumpTarget);
+            return handleReturnJump(context, rj, jumpTarget);
         } catch (JumpException.RetryJump rj) {
             return handleRetryJump(getRuntime(), rj);
         }
     }
 
-    private IRubyObject handleBreakJump(Ruby runtime, Block newBlock, JumpException.BreakJump bj, JumpTarget jumpTarget) {
+    private IRubyObject handleBreakJump(Ruby runtime, Block newBlock, JumpException.BreakJump bj, int jumpTarget) {
         switch(newBlock.type) {
         case LAMBDA: if (bj.getTarget() == jumpTarget) {
             return (IRubyObject) bj.getValue();
@@ -228,14 +248,28 @@ public class RubyProc extends RubyObject implements JumpTarget {
         }
     }
 
-    private IRubyObject handleReturnJump(Ruby runtime, JumpException.ReturnJump rj, JumpTarget jumpTarget) {
-        Object target = rj.getTarget();
+    private IRubyObject handleReturnJump(ThreadContext context, JumpException.ReturnJump rj, int jumpTarget) {
+        int target = rj.getTarget();
+        Ruby runtime = context.getRuntime();
 
-        if (target == jumpTarget && block.type == Block.Type.LAMBDA) return (IRubyObject) rj.getValue();
+        // lambda always just returns the value
+        if (target == jumpTarget && block.type == Block.Type.LAMBDA) {
+            return (IRubyObject) rj.getValue();
+        }
 
+        // returns can't propagate out of threads
         if (type == Block.Type.THREAD) {
             throw runtime.newThreadError("return can't jump across threads");
         }
+
+        // If the block-receiving method is not still active and the original
+        // enclosing frame is no longer on the stack, it's a bad return.
+        // FIXME: this is not very efficient for cases where it won't error
+        if (target == jumpTarget && !context.isJumpTargetAlive(target, 1)) {
+            throw runtime.newLocalJumpError(RubyLocalJumpError.Reason.RETURN, (IRubyObject)rj.getValue(), "unexpected return");
+        }
+
+        // otherwise, let it propagate
         throw rj;
     }
 
@@ -252,22 +286,4 @@ public class RubyProc extends RubyObject implements JumpTarget {
     public RubyProc to_proc() {
     	return this;
     }
-    
-    public IRubyObject as(Class asClass) {
-        final Ruby ruby = getRuntime();
-        if (!asClass.isInterface()) {
-            throw ruby.newTypeError(asClass.getCanonicalName() + " is not an interface");
-        }
-
-        return MiniJava.javaToRuby(ruby, Proxy.newProxyInstance(Ruby.getClassLoader(), new Class[] {asClass}, new InvocationHandler() {
-            public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
-                IRubyObject[] rubyArgs = new IRubyObject[args.length + 1];
-                rubyArgs[0] = RubySymbol.newSymbol(ruby, method.getName());
-                for (int i = 1; i < rubyArgs.length; i++) {
-                    rubyArgs[i] = MiniJava.javaToRuby(ruby, args[i - 1]);
-                }
-                return MiniJava.rubyToJava(call(ruby.getCurrentContext(), rubyArgs));
-            }
-        }));
-    }
 }
