@@ -52,6 +52,7 @@ import org.jruby.runtime.Block;
 import org.jruby.runtime.BlockBody;
 import org.jruby.runtime.BlockCallback;
 import org.jruby.runtime.CallBlock;
+import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ObjectMarshal;
 import org.jruby.runtime.ThreadContext;
@@ -77,6 +78,10 @@ public class RubyRange extends RubyObject {
     public static RubyClass createRangeClass(Ruby runtime) {
         RubyClass result = runtime.defineClass("Range", runtime.getObject(), RANGE_ALLOCATOR);
         runtime.setRange(result);
+
+        result.index = ClassIndex.RANGE;
+        result.setReifiedClass(RubyRange.class);
+
         result.kindOf = new RubyModule.KindOf() {
             public boolean isKindOf(IRubyObject obj, RubyModule type) {
                 return obj instanceof RubyRange;
@@ -127,6 +132,19 @@ public class RubyRange extends RubyObject {
         range.isExclusive = isExclusive;
     }
 
+    final boolean checkBegin(long length) {
+        long beg = RubyNumeric.num2long(this.begin);
+        if(beg < 0) {
+            beg += length;
+            if(beg < 0) {
+                return false;
+            }
+        } else if(length < beg) {
+            return false;
+        }
+        return true;
+    }
+
     final long[] begLen(long len, int err){
         long beg = RubyNumeric.num2long(this.begin);
         long end = RubyNumeric.num2long(this.end);
@@ -371,7 +389,7 @@ public class RubyRange extends RubyObject {
             }
             return RubyArray.newArrayNoCopy(runtime, array);
         } else {
-            return RubyEnumerable.to_a19(context, this);
+            return RubyEnumerable.to_a(context, this);
         }
     }
 
@@ -422,6 +440,8 @@ public class RubyRange extends RubyObject {
             fixnumEach(context, runtime, block);
         } else if (begin instanceof RubyString) {
             ((RubyString) begin).uptoCommon19(context, end, isExclusive, block);
+        } else if (begin instanceof RubySymbol) {
+            begin.asString().uptoCommon19(context, end.asString(), isExclusive, block, true);
         } else {
             if (!begin.respondsTo("succ")) throw getRuntime().newTypeError(
                     "can't iterate from " + begin.getMetaClass().getName());
@@ -571,17 +591,17 @@ public class RubyRange extends RubyObject {
             }
             return runtime.getFalse();
         } else if (begin instanceof RubyString && end instanceof RubyString &&
-                ((RubyString)begin).getByteList().realSize == 1 &&
-                ((RubyString)end).getByteList().realSize == 1) {
+                ((RubyString) begin).getByteList().getRealSize() == 1 &&
+                ((RubyString) end).getByteList().getRealSize() == 1) {
             if (obj.isNil()) return runtime.getFalse();
             if (obj instanceof RubyString) {
                 ByteList Vbytes = ((RubyString)obj).getByteList();
-                if (Vbytes.realSize != 1) return runtime.getFalse();
-                int v = Vbytes.bytes[Vbytes.begin] & 0xff;
+                if (Vbytes.getRealSize() != 1) return runtime.getFalse();
+                int v = Vbytes.getUnsafeBytes()[Vbytes.getBegin()] & 0xff;
                 ByteList Bbytes = ((RubyString)begin).getByteList();
-                int b = Bbytes.bytes[Bbytes.begin] & 0xff;
+                int b = Bbytes.getUnsafeBytes()[Bbytes.getBegin()] & 0xff;
                 ByteList Ebytes = ((RubyString)end).getByteList();
-                int e = Ebytes.bytes[Ebytes.begin] & 0xff;
+                int e = Ebytes.getUnsafeBytes()[Ebytes.getBegin()] & 0xff;
                 if (Encoding.isAscii(v) && Encoding.isAscii(b) && Encoding.isAscii(e)) {
                     if ((b <= v && v < e) || (!isExclusive && v == e)) return runtime.getTrue();
                     return runtime.getFalse();
@@ -683,6 +703,7 @@ public class RubyRange extends RubyObject {
             
             unmarshalStream.registerLinkTarget(range);
 
+            // FIXME: Maybe we can just gank these off the line directly?
             unmarshalStream.defaultVariablesUnmarshal(range);
             
             range.begin = (IRubyObject)range.removeInternalVariable("begin");
