@@ -25,10 +25,11 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
+import static org.jruby.util.Numeric.f_abs;
 import static org.jruby.util.Numeric.f_add;
 import static org.jruby.util.Numeric.f_cmp;
 import static org.jruby.util.Numeric.f_div;
-import static org.jruby.util.Numeric.f_equal_p;
+import static org.jruby.util.Numeric.f_equal;
 import static org.jruby.util.Numeric.f_expt;
 import static org.jruby.util.Numeric.f_floor;
 import static org.jruby.util.Numeric.f_gcd;
@@ -40,6 +41,7 @@ import static org.jruby.util.Numeric.f_mul;
 import static org.jruby.util.Numeric.f_negate;
 import static org.jruby.util.Numeric.f_negative_p;
 import static org.jruby.util.Numeric.f_one_p;
+import static org.jruby.util.Numeric.f_quo;
 import static org.jruby.util.Numeric.f_rshift;
 import static org.jruby.util.Numeric.f_sub;
 import static org.jruby.util.Numeric.f_to_f;
@@ -53,6 +55,7 @@ import static org.jruby.util.Numeric.i_gcd;
 import static org.jruby.util.Numeric.i_ilog2;
 import static org.jruby.util.Numeric.k_exact_p;
 import static org.jruby.util.Numeric.ldexp;
+import static org.jruby.util.Numeric.nurat_rationalize_internal;
 
 import org.jcodings.specific.ASCIIEncoding;
 import org.jruby.anno.JRubyClass;
@@ -80,6 +83,8 @@ public class RubyRational extends RubyNumeric {
         runtime.setRational(rationalc);
 
         rationalc.index = ClassIndex.RATIONAL;
+        rationalc.setReifiedClass(RubyRational.class);
+        
         rationalc.kindOf = new RubyModule.KindOf() {
             @Override
             public boolean isKindOf(IRubyObject obj, RubyModule type) {
@@ -114,7 +119,7 @@ public class RubyRational extends RubyNumeric {
     /** rb_rational_raw
      * 
      */
-    static RubyRational newRationalRaw(Ruby runtime, IRubyObject x, RubyObject y) {
+    static RubyRational newRationalRaw(Ruby runtime, IRubyObject x, IRubyObject y) {
         return new RubyRational(runtime, runtime.getRational(), x, y);
     }
 
@@ -223,7 +228,15 @@ public class RubyRational extends RubyNumeric {
 
         return new RubyRational(runtime, recv, num, den);
     }
-    
+
+    /** nurat_canonicalization
+     *
+     */
+    private static boolean canonicalization = false;
+    public static void setCanonicalization(boolean canonical) {
+        canonicalization = canonical;
+    }
+
     /** nurat_int_check
      * 
      */
@@ -246,7 +259,6 @@ public class RubyRational extends RubyNumeric {
     /** nurat_s_canonicalize_internal
      * 
      */
-    private static boolean canonicalization = false;
     private static IRubyObject canonicalizeInternal(ThreadContext context, IRubyObject clazz, IRubyObject num, IRubyObject den) {
         Ruby runtime = context.getRuntime();
         IRubyObject res = f_cmp(context, den, RubyFixnum.zero(runtime));
@@ -557,7 +569,7 @@ public class RubyRational extends RubyNumeric {
     /** nurat_div
      * 
      */
-    @JRubyMethod(name = {"/", "quo"})
+    @JRubyMethod(name = {"/", "quo"}, backtrace = true)
     public IRubyObject op_div(ThreadContext context, IRubyObject other) {
         if (other instanceof RubyFixnum || other instanceof RubyBignum) {
             if (f_zero_p(context, other)) throw context.getRuntime().newZeroDivisionError();
@@ -575,7 +587,7 @@ public class RubyRational extends RubyNumeric {
     /** nurat_fdiv
      * 
      */
-    @JRubyMethod(name = "fdiv")
+    @JRubyMethod(name = "fdiv", backtrace = true)
     public IRubyObject op_fdiv(ThreadContext context, IRubyObject other) {
         return f_div(context, f_to_f(context, this), other);
     }
@@ -646,34 +658,37 @@ public class RubyRational extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "==")
+    @Override
     public IRubyObject op_equal(ThreadContext context, IRubyObject other) {
         Ruby runtime = context.getRuntime();
         if (other instanceof RubyFixnum || other instanceof RubyBignum) {
             if (f_zero_p(context, num) && f_zero_p(context, other)) return runtime.getTrue();
             if (!(den instanceof RubyFixnum) || ((RubyFixnum)den).getLongValue() != 1) return runtime.getFalse();
-            if (f_equal_p(context, num, other)) return runtime.getTrue();
-            return runtime.getFalse();
+            return f_equal(context, num, other);
         } else if (other instanceof RubyFloat) {
-            return f_equal_p(context, f_to_f(context, this), other) ? runtime.getTrue() : runtime.getFalse();
+            return f_equal(context, f_to_f(context, this), other);
         } else if (other instanceof RubyRational) {
             RubyRational otherRational = (RubyRational)other;
             if (f_zero_p(context, num) && f_zero_p(context, otherRational.num)) return runtime.getTrue();
-            if (f_equal_p(context, num, otherRational.num) && f_equal_p(context, den, otherRational.den)) return runtime.getTrue();
-            return runtime.getFalse();
+            return runtime.newBoolean(f_equal(context, num, otherRational.num).isTrue() &&
+                    f_equal(context, den, otherRational.den).isTrue());
+
         }
-        return f_equal_p(context, other, this) ? runtime.getTrue() : runtime.getFalse();
+        return f_equal(context, other, this);
     }
 
     /** nurat_coerce
      * 
      */
-    @JRubyMethod(name = "coerce")
+    @JRubyMethod(name = "coerce", backtrace = true)
     public IRubyObject op_coerce(ThreadContext context, IRubyObject other) {
         Ruby runtime = context.getRuntime();
         if (other instanceof RubyFixnum || other instanceof RubyBignum) {
             return runtime.newArray(RubyRational.newRationalBang(context, getMetaClass(), other), this);
         } else if (other instanceof RubyFloat) {
             return runtime.newArray(other, f_to_f(context, this));
+        } else if (other instanceof RubyRational) {
+            return runtime.newArray(other, this);
         }
         throw runtime.newTypeError(other.getMetaClass() + " can't be coerced into " + getMetaClass());
     }
@@ -681,29 +696,53 @@ public class RubyRational extends RubyNumeric {
     /** nurat_idiv
      * 
      */
-    @JRubyMethod(name = "div")
+    @JRubyMethod(name = "div", backtrace = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject op_idiv(ThreadContext context, IRubyObject other) {
         return f_floor(context, f_div(context, this, other));
     }
 
+    @JRubyMethod(name = "div", backtrace = true, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_idiv19(ThreadContext context, IRubyObject other) {
+        if (num2dbl(other) == 0.0) {
+            throw context.getRuntime().newZeroDivisionError();
+        }
+        return op_idiv(context, other);
+    }
+
     /** nurat_mod
      * 
      */
-    @JRubyMethod(name = {"modulo", "%"})
+    @JRubyMethod(name = {"modulo", "%"}, backtrace = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject op_mod(ThreadContext context, IRubyObject other) {
         IRubyObject val = f_floor(context, f_div(context, this, other));
         return f_sub(context, this, f_mul(context, other, val));
     }
 
+    @JRubyMethod(name = {"modulo", "%"}, backtrace = true, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_mod19(ThreadContext context, IRubyObject other) {
+        if (num2dbl(other) == 0.0) {
+            throw context.getRuntime().newZeroDivisionError();
+        }
+        return op_mod(context, other);
+    }
+
     /** nurat_divmod
      * 
      */
-    @JRubyMethod(name = "divmod")
+    @JRubyMethod(name = "divmod", backtrace = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject op_divmod(ThreadContext context, IRubyObject other) {
         IRubyObject val = f_floor(context, f_div(context, this, other)); 
         return context.getRuntime().newArray(val, f_sub(context, this, f_mul(context, other, val)));
     }
 
+    @JRubyMethod(name = "divmod", backtrace = true, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_divmod19(ThreadContext context, IRubyObject other) {
+        if (num2dbl(other) == 0.0) {
+            throw context.getRuntime().newZeroDivisionError();
+        }
+        return op_divmod(context, other);
+    }
+
     /** nurat_rem
      * 
      */
@@ -864,8 +903,35 @@ public class RubyRational extends RubyNumeric {
     public IRubyObject to_r(ThreadContext context) {
         return this;
     }
-    
-    /** nurat_to_r
+
+    /** nurat_rationalize
+     *
+     */
+    @JRubyMethod(name = "rationalize", optional = 1, compat = CompatVersion.RUBY1_9)
+    public IRubyObject rationalize(ThreadContext context, IRubyObject[] args) {
+
+        IRubyObject a, b;
+
+        if (args.length == 0) return to_r(context);
+
+        if (f_negative_p(context, this))
+            return f_negate(context,
+                    ((RubyRational) f_abs(context, this)).rationalize(context, args));
+
+        IRubyObject eps = f_abs(context, args[0]);
+        a = f_sub(context, this, eps);
+        b = f_add(context, this, eps);
+
+        if (f_equal(context, a, b).isTrue()) return this;
+        IRubyObject[] ary = new IRubyObject[2];
+        ary[0] = a;
+        ary[1] = b;
+        IRubyObject[] ans = nurat_rationalize_internal(context, ary);
+
+        return newRational(context, this.metaClass, ans[0], ans[1]);
+    }
+
+    /** nurat_hash
      * 
      */
     @JRubyMethod(name = "hash")
@@ -912,7 +978,7 @@ public class RubyRational extends RubyNumeric {
     /** nurat_marshal_load
      * 
      */
-    @JRubyMethod(name = "marshal_load")
+    @JRubyMethod(name = "marshal_load", backtrace = true)
     public IRubyObject marshal_load(ThreadContext context, IRubyObject arg) {
         RubyArray load = arg.convertToArray();
         num = load.size() > 0 ? load.eltInternal(0) : context.getRuntime().getNil();
@@ -928,7 +994,7 @@ public class RubyRational extends RubyNumeric {
         ByteList bytes = s.getByteList();
 
         Ruby runtime = context.getRuntime();
-        if (bytes.realSize == 0) return runtime.newArray(runtime.getNil(), recv);
+        if (bytes.getRealSize() == 0) return runtime.newArray(runtime.getNil(), recv);
 
         IRubyObject m = RubyRegexp.newDummyRegexp(runtime, Numeric.RationalPatterns.rat_pat).callMethod(context, "match", s);
         
@@ -951,9 +1017,9 @@ public class RubyRational extends RubyNumeric {
             if (!fp.isNil()) {
                 bytes = fp.convertToString().getByteList();
                 int count = 0;
-                byte[]buf = bytes.bytes;
-                int i = bytes.begin;
-                int end = i + bytes.realSize;
+                byte[]buf = bytes.getUnsafeBytes();
+                int i = bytes.getBegin();
+                int end = i + bytes.getRealSize();
 
                 while (i < end) {
                     if (ASCIIEncoding.INSTANCE.isDigit(buf[i])) count++;
