@@ -15,6 +15,7 @@ import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
+import org.jruby.util.StringSupport;
 
 /**
  * @author kscott
@@ -120,15 +121,18 @@ public class RubyStringScanner extends RubyObject {
     @JRubyMethod(name = "terminate")
     public IRubyObject terminate() {
         check();
-        pos = str.getByteList().realSize;
+        pos = str.getByteList().getRealSize();
         clearMatched();
         return this;
     }
     
     @JRubyMethod(name = "clear")
-    public IRubyObject clear() {
+    public IRubyObject clear(ThreadContext context) {
         check();
-        getRuntime().getWarnings().warning(ID.DEPRECATED_METHOD, "StringScanner#clear is obsolete; use #terminate instead", "StringScanner#clear", "#terminate");
+        Ruby runtime = context.getRuntime();
+        if (runtime.isVerbose()) {
+            runtime.getWarnings().warning(ID.DEPRECATED_METHOD, "StringScanner#clear is obsolete; use #terminate instead", "StringScanner#clear", "#terminate");
+        }
         return terminate();
     }
     
@@ -162,7 +166,7 @@ public class RubyStringScanner extends RubyObject {
     public IRubyObject set_pos(IRubyObject pos) {
         check();
         int i = RubyNumeric.num2int(pos);
-        int size = str.getByteList().realSize;
+        int size = str.getByteList().getRealSize();
         if (i < 0) i += size;
         if (i < 0 || i > size) throw getRuntime().newRangeError("index out of range.");
         this.pos = i;
@@ -170,18 +174,18 @@ public class RubyStringScanner extends RubyObject {
     }
 
     private IRubyObject extractRange(Ruby runtime, int beg, int end) {
-        int size = str.getByteList().realSize;
+        int size = str.getByteList().getRealSize();
         if (beg > size) return getRuntime().getNil();
         if (end > size) end = size;
-        return str.makeShared(runtime, beg, end - beg);
+        return str.makeSharedString(runtime, beg, end - beg);
     }
     
     private IRubyObject extractBegLen(Ruby runtime, int beg, int len) {
         assert len >= 0;
-        int size = str.getByteList().realSize;
+        int size = str.getByteList().getRealSize();
         if (beg > size) return getRuntime().getNil();
         if (beg + len > size) len = size - beg;
-        return str.makeShared(runtime, beg, len);
+        return str.makeSharedString(runtime, beg, len);
     }
     
     private IRubyObject scan(IRubyObject regex, boolean succptr, boolean getstr, boolean headonly) {
@@ -191,17 +195,17 @@ public class RubyStringScanner extends RubyObject {
         Regex pattern = ((RubyRegexp)regex).getPattern();
 
         clearMatched();
-        int rest = str.getByteList().realSize - pos;
+        int rest = str.getByteList().getRealSize() - pos;
         if (rest < 0) return getRuntime().getNil();
 
         ByteList value = str.getByteList();
-        Matcher matcher = pattern.matcher(value.bytes, value.begin + pos, value.begin + value.realSize);
+        Matcher matcher = pattern.matcher(value.getUnsafeBytes(), value.getBegin() + pos, value.getBegin() + value.getRealSize());
 
         final int ret;
         if (headonly) {
-            ret = matcher.match(value.begin + pos, value.begin + value.realSize, Option.NONE);            
+            ret = matcher.match(value.getBegin() + pos, value.getBegin() + value.getRealSize(), Option.NONE);
         } else {
-            ret = matcher.search(value.begin + pos, value.begin + value.realSize, Option.NONE);
+            ret = matcher.search(value.getBegin() + pos, value.getBegin() + value.getRealSize(), Option.NONE);
         }
 
         regs = matcher.getRegion(); 
@@ -276,33 +280,41 @@ public class RubyStringScanner extends RubyObject {
         end = pos - lastPos;
         regs = null;
     }
-    
-    @JRubyMethod(name = "getch")
+
+    @JRubyMethod(name = "getch", compat = CompatVersion.RUBY1_8)
     public IRubyObject getch(ThreadContext context) {
+        return getchCommon(context, false);
+    }
+
+    @JRubyMethod(name = "getch", compat = CompatVersion.RUBY1_9)
+    public IRubyObject getch19(ThreadContext context) {
+        return getchCommon(context, true);
+    }
+
+    public IRubyObject getchCommon(ThreadContext context, boolean is1_9) {
         check();
         clearMatched();
-        
+
         Ruby runtime = context.getRuntime();
         ByteList value = str.getByteList();
-        
-        if (pos >= value.realSize) return runtime.getNil();
 
-        Encoding enc = runtime.getKCode().getEncoding();
-        
+        if (pos >= value.getRealSize()) return runtime.getNil();
         int len;
-        if (enc.isSingleByte()) {
-            len = 1;
+        if (is1_9) {
+            Encoding enc = str.getEncoding();
+            len = enc.isSingleByte() ? 1 : StringSupport.length(enc, value.getUnsafeBytes(), value.getBegin() + pos, value.getBegin() + value.getRealSize());
         } else {
-            len = enc.length(value.bytes, value.begin + pos, value.begin + value.realSize);
+            Encoding enc = runtime.getKCode().getEncoding();
+            len = enc.isSingleByte() ? 1 : enc.length(value.getUnsafeBytes(), value.getBegin() + pos, value.getBegin() + value.getRealSize());
         }
-        
-        if (pos + len > value.realSize) len = value.realSize - pos;
+
+        if (pos + len > value.getRealSize()) len = value.getRealSize() - pos;
         lastPos = pos;
         pos += len;
-        
+
         setMatched();
         adjustRegisters();
-        
+
         return extractRange(runtime, lastPos + beg, lastPos + end);
     }
     
@@ -310,7 +322,7 @@ public class RubyStringScanner extends RubyObject {
     public IRubyObject get_byte(ThreadContext context) {
         check();
         clearMatched();
-        if (pos >= str.getByteList().realSize) return getRuntime().getNil();
+        if (pos >= str.getByteList().getRealSize()) return getRuntime().getNil();
         
         lastPos = pos;
         pos++;
@@ -323,9 +335,12 @@ public class RubyStringScanner extends RubyObject {
     
     @JRubyMethod(name = "getbyte")
     public IRubyObject getbyte(ThreadContext context) {
-        context.getRuntime().getWarnings().warning(ID.DEPRECATED_METHOD,
-                "StringScanner#getbyte is obsolete; use #get_byte instead",
-                "StringScanner#getbyte", "#get_byte");
+        Ruby runtime = context.getRuntime();
+        if (runtime.isVerbose()) { 
+            runtime.getWarnings().warning(ID.DEPRECATED_METHOD,
+                    "StringScanner#getbyte is obsolete; use #get_byte instead",
+                    "StringScanner#getbyte", "#get_byte");
+        }
         return get_byte(context);
     }
 
@@ -339,17 +354,20 @@ public class RubyStringScanner extends RubyObject {
         }
 
         ByteList value = str.getByteList();
-        if (pos >= value.realSize) return RubyString.newEmptyString(getRuntime()).infectBy(str);
-        if (pos + len > value.realSize) len = value.realSize - pos;
+        if (pos >= value.getRealSize()) return RubyString.newEmptyString(getRuntime()).infectBy(str);
+        if (pos + len > value.getRealSize()) len = value.getRealSize() - pos;
 
         return extractBegLen(context.getRuntime(), pos, len);
     }
 
     @JRubyMethod(name = "peep", required = 1)
     public IRubyObject peep(ThreadContext context, IRubyObject length) {
-        getRuntime().getWarnings().warning(
-                ID.DEPRECATED_METHOD, "StringScanner#peep is obsolete; use #peek instead",
-                "StringScanner#peep", "#peek");
+        Ruby runtime = context.getRuntime();
+        if (runtime.isVerbose()) {
+            runtime.getWarnings().warning(
+                    ID.DEPRECATED_METHOD, "StringScanner#peep is obsolete; use #peek instead",
+                    "StringScanner#peep", "#peek");
+        }
         return peek(context, length);
     }
     
@@ -372,27 +390,30 @@ public class RubyStringScanner extends RubyObject {
     public IRubyObject bol_p() {
         check();
         ByteList value = str.getByteList();
-        if (pos > value.realSize) return getRuntime().getNil();
+        if (pos > value.getRealSize()) return getRuntime().getNil();
         if (pos == 0) return getRuntime().getTrue();
-        return value.bytes[(value.begin + pos) - 1] == (byte)'\n' ? getRuntime().getTrue() : getRuntime().getFalse();        
+        return value.getUnsafeBytes()[(value.getBegin() + pos) - 1] == (byte)'\n' ? getRuntime().getTrue() : getRuntime().getFalse();
     }
     
     @JRubyMethod(name = "eos?")
     public RubyBoolean eos_p(ThreadContext context) {
         check();
-        return pos >= str.getByteList().realSize ? context.getRuntime().getTrue() : context.getRuntime().getFalse();
+        return pos >= str.getByteList().getRealSize() ? context.getRuntime().getTrue() : context.getRuntime().getFalse();
     }
     
     @JRubyMethod(name = "empty?")
     public RubyBoolean empty_p(ThreadContext context) {
-        getRuntime().getWarnings().warning(ID.DEPRECATED_METHOD, "StringScanner#empty? is obsolete; use #eos? instead", "StringScanner#empty?", "#eos?");
+        Ruby runtime = context.getRuntime();
+        if (runtime.isVerbose()) {
+            runtime.getWarnings().warning(ID.DEPRECATED_METHOD, "StringScanner#empty? is obsolete; use #eos? instead", "StringScanner#empty?", "#eos?");
+        }
         return eos_p(context);
     }
     
     @JRubyMethod(name = "rest?")
     public RubyBoolean rest_p(ThreadContext context) {
         check();
-        return pos >= str.getByteList().realSize ? context.getRuntime().getFalse() : context.getRuntime().getTrue();
+        return pos >= str.getByteList().getRealSize() ? context.getRuntime().getFalse() : context.getRuntime().getTrue();
     }
 
     @JRubyMethod(name = "matched?")
@@ -416,9 +437,12 @@ public class RubyStringScanner extends RubyObject {
     }
 
     @JRubyMethod(name = "matchedsize")
-    public IRubyObject matchedsize() {
-        getRuntime().getWarnings().warning(ID.DEPRECATED_METHOD, "StringScanner#matchedsize is obsolete; use #matched_size instead", 
-                "StringScanner#matchedize", "#matched_size");
+    public IRubyObject matchedsize(ThreadContext context) {
+        Ruby runtime = context.getRuntime();
+        if (runtime.isVerbose()) {
+            runtime.getWarnings().warning(ID.DEPRECATED_METHOD, "StringScanner#matchedsize is obsolete; use #matched_size instead", 
+                    "StringScanner#matchedize", "#matched_size");
+        }
         return matched_size();        
     }
 
@@ -453,28 +477,31 @@ public class RubyStringScanner extends RubyObject {
     public IRubyObject post_match(ThreadContext context) {
         check();
         if (!isMatched()) return context.getRuntime().getNil();
-        return extractRange(context.getRuntime(), lastPos + end, str.getByteList().realSize);
+        return extractRange(context.getRuntime(), lastPos + end, str.getByteList().getRealSize());
     }
     
     @JRubyMethod(name = "rest")
     public IRubyObject rest(ThreadContext context) {
         check();
         ByteList value = str.getByteList();
-        if (pos >= value.realSize) return RubyString.newEmptyString(context.getRuntime()).infectBy(str);
-        return extractRange(context.getRuntime(), pos, value.realSize);
+        if (pos >= value.getRealSize()) return RubyString.newEmptyString(context.getRuntime()).infectBy(str);
+        return extractRange(context.getRuntime(), pos, value.getRealSize());
     }
     
     @JRubyMethod(name = "rest_size")
     public RubyFixnum rest_size() {
         check();
         ByteList value = str.getByteList();
-        if (pos >= value.realSize) return RubyFixnum.zero(getRuntime());
-        return RubyFixnum.newFixnum(getRuntime(), value.realSize - pos);
+        if (pos >= value.getRealSize()) return RubyFixnum.zero(getRuntime());
+        return RubyFixnum.newFixnum(getRuntime(), value.getRealSize() - pos);
     }
 
     @JRubyMethod(name = "restsize")
-    public RubyFixnum restsize() {
-        getRuntime().getWarnings().warning(ID.DEPRECATED_METHOD, "StringScanner#restsize is obsolete; use #rest_size instead", "StringScanner#restsize", "#rest_size");
+    public RubyFixnum restsize(ThreadContext context) {
+        Ruby runtime = context.getRuntime();
+        if (runtime.isVerbose()) {
+            runtime.getWarnings().warning(ID.DEPRECATED_METHOD, "StringScanner#restsize is obsolete; use #rest_size instead", "StringScanner#restsize", "#rest_size");
+        }
         return rest_size();
     }
     
@@ -482,13 +509,13 @@ public class RubyStringScanner extends RubyObject {
     @Override
     public IRubyObject inspect() {
         if (str == null) return inspect("(uninitialized)");
-        if (pos >= str.getByteList().realSize) return inspect("fin");
-        if (pos == 0) return inspect(pos + "/" + str.getByteList().realSize + " @ " + inspect2());
-        return inspect(pos + "/" + str.getByteList().realSize + " " + inspect1() + " @ " + inspect2()); 
+        if (pos >= str.getByteList().getRealSize()) return inspect("fin");
+        if (pos == 0) return inspect(pos + "/" + str.getByteList().getRealSize() + " @ " + inspect2());
+        return inspect(pos + "/" + str.getByteList().getRealSize() + " " + inspect1() + " @ " + inspect2());
     }
     
     private IRubyObject inspect(String msg) {
-        IRubyObject result = getRuntime().newString("#<" + getMetaClass() + " " + msg + ">"); 
+        RubyString result = getRuntime().newString("#<" + getMetaClass() + " " + msg + ">"); 
         if (str != null) result.infectBy(str);
         return result;
     }
@@ -506,8 +533,8 @@ public class RubyStringScanner extends RubyObject {
     }
     
     private IRubyObject inspect2() {
-        if (pos >= str.getByteList().realSize) return RubyString.newEmptyString(getRuntime());
-        int len = str.getByteList().realSize - pos;
+        if (pos >= str.getByteList().getRealSize()) return RubyString.newEmptyString(getRuntime());
+        int len = str.getByteList().getRealSize() - pos;
         if (len > INSPECT_LENGTH) {
             return ((RubyString)str.substr(getRuntime(), pos, INSPECT_LENGTH)).cat("...".getBytes()).inspect();
         } else {
