@@ -32,6 +32,7 @@ import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyClass;
 import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.runtime.Block;
+import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -71,6 +72,9 @@ public class RubyUnboundMethod extends RubyMethod {
         	runtime.defineClass("UnboundMethod", runtime.getMethod(), ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
         runtime.setUnboundMethod(newClass);
 
+        newClass.index = ClassIndex.UNBOUNDMETHOD;
+        newClass.setReifiedClass(RubyUnboundMethod.class);
+
         newClass.defineAnnotatedMethods(RubyUnboundMethod.class);
 
         return newClass;
@@ -124,12 +128,17 @@ public class RubyUnboundMethod extends RubyMethod {
         return super.to_proc(context, unusedBlock);
     }
 
-    @JRubyMethod(name = "name", compat = CompatVersion.RUBY1_9)
+    @JRubyMethod(name = "name", compat = CompatVersion.RUBY1_8)
     public IRubyObject name(ThreadContext context) {
+        return context.getRuntime().newString(methodName);
+    }
+
+    @JRubyMethod(name = "name", compat = CompatVersion.RUBY1_9)
+    public IRubyObject name19(ThreadContext context) {
         return context.getRuntime().newSymbol(methodName);
     }
 
-    @JRubyMethod(name = "owner", compat = CompatVersion.RUBY1_9)
+    @JRubyMethod(name = "owner")
     public IRubyObject owner(ThreadContext context) {
         return implementationModule;
     }
