@@ -2,6 +2,7 @@ package org.jruby.anno;
 
 import com.sun.mirror.apt.*;
 import com.sun.mirror.declaration.*;
+import com.sun.mirror.type.ReferenceType;
 import com.sun.mirror.util.*;
 
 import java.io.ByteArrayOutputStream;
@@ -126,6 +127,7 @@ public class AnnotationBinder implements AnnotationProcessorFactory {
                     Map<String, List<MethodDeclaration>> staticAnnotatedMethods1_9 = new HashMap<String, List<MethodDeclaration>>();
 
                     Set<String> frameAwareMethods = new HashSet<String>();
+                    Set<String> scopeAwareMethods = new HashSet<String>();
 
                     int methodCount = 0;
                     for (MethodDeclaration md : cd.getMethods()) {
@@ -135,6 +137,18 @@ public class AnnotationBinder implements AnnotationProcessorFactory {
                         }
                         methodCount++;
 
+                        // warn if the method raises any exceptions (JRUBY-4494)
+                        if (md.getThrownTypes().size() != 0) {
+                            System.err.print("Method " + cd.toString() + "." + md.toString() + " should not throw exceptions: ");
+                            boolean comma = false;
+                            for (ReferenceType thrownType : md.getThrownTypes()) {
+                                if (comma) System.err.print(", ");
+                                System.err.print(thrownType);
+                                comma = true;
+                            }
+                            System.err.print("\n");
+                        }
+
                         String name = anno.name().length == 0 ? md.getSimpleName() : anno.name()[0];
 
                         List<MethodDeclaration> methodDescs;
@@ -165,9 +179,34 @@ public class AnnotationBinder implements AnnotationProcessorFactory {
 
                         methodDescs.add(md);
 
-                        if (anno.frame() || (anno.reads() != null && anno.reads().length >= 1) || (anno.writes() != null && anno.writes().length >= 1)) {
-                            frameAwareMethods.add(name);
+                        // check for frame field reads or writes
+                        boolean frame = false;
+                        boolean scope = false;
+                        if (anno.frame()) {
+                            frame = true;
                         }
+                        if (anno.reads() != null) for (FrameField read : anno.reads()) {
+                            switch (read) {
+                            case BACKREF:
+                            case LASTLINE:
+                                scope = true;
+                                break;
+                            default:
+                                frame = true;
+                            }
+                        }
+                        if (anno.writes() != null) for (FrameField write : anno.writes()) {
+                            switch (write) {
+                            case BACKREF:
+                            case LASTLINE:
+                                scope = true;
+                                break;
+                            default:
+                                frame = true;
+                            }
+                        }
+                        if (frame) frameAwareMethods.addAll(Arrays.asList(anno.name()));
+                        if (scope) scopeAwareMethods.addAll(Arrays.asList(anno.name()));
                     }
 
                     if (methodCount == 0) {
@@ -208,6 +247,7 @@ public class AnnotationBinder implements AnnotationProcessorFactory {
                     out.println("    }");
 
                     // write out a static initializer for frame names, so it only fires once
+                    out.println("    static {");
                     if (!frameAwareMethods.isEmpty()) {
                         StringBuffer frameMethodsString = new StringBuffer();
                         boolean first = true;
@@ -216,10 +256,19 @@ public class AnnotationBinder implements AnnotationProcessorFactory {
                             first = false;
                             frameMethodsString.append('"').append(name).append('"');
                         }
-                        out.println("    static {");
-                        out.println("        ASTInspector.FRAME_AWARE_METHODS.addAll((List<String>)Arrays.asList(" + frameMethodsString + "));");
-                        out.println("     }");
+                        out.println("        ASTInspector.addFrameAwareMethods(" + frameMethodsString + ");");
                     }
+                    if (!scopeAwareMethods.isEmpty()) {
+                        StringBuffer scopeMethodsString = new StringBuffer();
+                        boolean first = true;
+                        for (String name : scopeAwareMethods) {
+                            if (!first) scopeMethodsString.append(',');
+                            first = false;
+                            scopeMethodsString.append('"').append(name).append('"');
+                        }
+                        out.println("        ASTInspector.addScopeAwareMethods(" + scopeMethodsString + ");");
+                    }
+                    out.println("     }");
 
                     out.println("}");
                     out.close();
@@ -338,16 +387,6 @@ public class AnnotationBinder implements AnnotationProcessorFactory {
                 return qualifiedName;
             }
 
-            private boolean tryClass(String className) {
-                Class tryClass = null;
-                try {
-                    tryClass = Class.forName(className);
-                } catch (ClassNotFoundException cnfe) {
-                }
-
-                return tryClass != null;
-            }
-
             private int calculateActualRequired(int paramsLength, int optional, boolean rest, boolean isStatic, boolean hasContext, boolean hasBlock) {
                 int actualRequired;
                 if (optional == 0 && !rest) {
@@ -397,26 +436,15 @@ public class AnnotationBinder implements AnnotationProcessorFactory {
             }
 
             public void generateMethodAddCalls(MethodDeclaration md, JRubyMethod jrubyMethod) {
-                // TODO: This information
-                if (jrubyMethod.frame()) {
-                    for (String name : jrubyMethod.name()) {
-                        out.println("        ASTInspector.FRAME_AWARE_METHODS.add(\"" + name + "\");");
-                    }
-                }
-                // TODO: compat version
-                //                if(jrubyMethod.compat() == CompatVersion.BOTH ||
-                //                        module.getRuntime().getInstanceConfig().getCompatVersion() == jrubyMethod.compat()) {
-                //RubyModule metaClass = module.metaClass;
-
                 if (jrubyMethod.meta()) {
                     String baseName;
                     if (jrubyMethod.name().length == 0) {
                         baseName = md.getSimpleName();
-                        out.println("        metaClass.addMethod(\"" + baseName + "\", javaMethod);");
+                        out.println("        metaClass.addMethodAtBootTimeOnly(\"" + baseName + "\", javaMethod);");
                     } else {
                         baseName = jrubyMethod.name()[0];
                         for (String name : jrubyMethod.name()) {
-                            out.println("        metaClass.addMethod(\"" + name + "\", javaMethod);");
+                            out.println("        metaClass.addMethodAtBootTimeOnly(\"" + name + "\", javaMethod);");
                         }
                     }
 
@@ -429,11 +457,11 @@ public class AnnotationBinder implements AnnotationProcessorFactory {
                     String baseName;
                     if (jrubyMethod.name().length == 0) {
                         baseName = md.getSimpleName();
-                        out.println("        cls.addMethod(\"" + baseName + "\", javaMethod);");
+                        out.println("        cls.addMethodAtBootTimeOnly(\"" + baseName + "\", javaMethod);");
                     } else {
                         baseName = jrubyMethod.name()[0];
                         for (String name : jrubyMethod.name()) {
-                            out.println("        cls.addMethod(\"" + name + "\", javaMethod);");
+                            out.println("        cls.addMethodAtBootTimeOnly(\"" + name + "\", javaMethod);");
                         }
                     }
 
@@ -451,11 +479,11 @@ public class AnnotationBinder implements AnnotationProcessorFactory {
 
                         if (jrubyMethod.name().length == 0) {
                             baseName = md.getSimpleName();
-                            out.println("        singletonClass.addMethod(\"" + baseName + "\", moduleMethod);");
+                            out.println("        singletonClass.addMethodAtBootTimeOnly(\"" + baseName + "\", moduleMethod);");
                         } else {
                             baseName = jrubyMethod.name()[0];
                             for (String name : jrubyMethod.name()) {
-                                out.println("        singletonClass.addMethod(\"" + name + "\", moduleMethod);");
+                                out.println("        singletonClass.addMethodAtBootTimeOnly(\"" + name + "\", moduleMethod);");
                             }
                         }
 
