@@ -32,7 +32,6 @@ import java.util.List;
 
 import org.jruby.ast.types.INameNode;
 import org.jruby.ast.visitor.NodeVisitor;
-import org.jruby.evaluator.Instruction;
 import org.jruby.lexer.yacc.ISourcePosition;
 
 /**
@@ -41,18 +40,18 @@ import org.jruby.lexer.yacc.ISourcePosition;
  */
 public class ArgumentNode extends Node implements INameNode {
     private String identifier;
-    
-    public ArgumentNode(ISourcePosition position, String identifier) {
-        this(position, NodeType.ARGUMENTNODE, identifier);
-    }
 
-    protected ArgumentNode(ISourcePosition position, NodeType type, String identifier) {
-        super(position, type);
+    public ArgumentNode(ISourcePosition position, String identifier) {
+        super(position);
 
         this.identifier = identifier;
     }
+
+    public NodeType getNodeType() {
+        return NodeType.ARGUMENTNODE;
+    }
     
-    public Instruction accept(NodeVisitor visitor) {
+    public Object accept(NodeVisitor visitor) {
         throw new RuntimeException("ArgumentNode should never be evaluated");
     }
     
