@@ -35,7 +35,6 @@ package org.jruby.ast;
 import org.jruby.Ruby;
 import org.jruby.ast.types.ILiteralNode;
 import org.jruby.ast.visitor.NodeVisitor;
-import org.jruby.evaluator.Instruction;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
@@ -50,13 +49,17 @@ public class ArrayNode extends ListNode implements ILiteralNode {
     private boolean lightweight = false;
     
     public ArrayNode(ISourcePosition position, Node firstNode) {
-        super(position, NodeType.ARRAYNODE, firstNode);
+        super(position, firstNode);
         
         assert firstNode != null : "ArrayNode.first == null";
     }
 
     public ArrayNode(ISourcePosition position) {
-        super(position, NodeType.ARRAYNODE);
+        super(position);
+    }
+
+    public NodeType getNodeType() {
+        return NodeType.ARRAYNODE;
     }
 
     /**
@@ -64,7 +67,7 @@ public class ArrayNode extends ListNode implements ILiteralNode {
      * @param iVisitor the visitor
      **/
     @Override
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitArrayNode(this);
     }
     
