@@ -147,11 +147,6 @@ public class CallNode extends Node implements INameNode, IArgumentNode, BlockAcc
         return Node.createList(receiverNode, argsNode, iterNode);
     }
     
-    @Override
-    public String toString() {
-        return "" + getClass().getName() + ": '" + getName() + "' @ " + getPosition();
-    }
-    
     @Override
     public IRubyObject interpret(Ruby runtime, ThreadContext context, IRubyObject self, Block aBlock) {
         assert false: "No longer called";
