@@ -35,7 +35,6 @@ import java.util.List;
 
 import org.jruby.Ruby;
 import org.jruby.ast.visitor.NodeVisitor;
-import org.jruby.evaluator.Instruction;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
@@ -49,7 +48,7 @@ public class EnsureNode extends Node {
     private final Node ensureNode;
 
     public EnsureNode(ISourcePosition position, Node bodyNode, Node ensureNode) {
-        super(position, NodeType.ENSURENODE);
+        super(position);
         
         assert bodyNode != null : "bodyNode is not null";
         
@@ -57,11 +56,15 @@ public class EnsureNode extends Node {
         this.ensureNode = ensureNode;
     }
 
+    public NodeType getNodeType() {
+        return NodeType.ENSURENODE;
+    }
+
     /**
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
      **/
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitEnsureNode(this);
     }
 
