@@ -6,6 +6,7 @@
 package org.jruby.ast;
 
 import org.jruby.Ruby;
+import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
@@ -30,10 +31,9 @@ public class FCallTwoArgBlockNode extends FCallNode {
 
     @Override
     public IRubyObject interpret(Ruby runtime, ThreadContext context, IRubyObject self, Block aBlock) {
-        Block block = getBlock(context, self);
-        
         return callAdapter.callIter(context, self, self,
                 arg1.interpret(runtime, context, self, aBlock),
-                arg2.interpret(runtime, context, self, aBlock), block);
+                arg2.interpret(runtime, context, self, aBlock),
+                RuntimeHelpers.getBlock(context, self, iterNode));
     }
 }
