@@ -36,7 +36,6 @@ import java.util.List;
 import org.jruby.Ruby;
 import org.jruby.ast.visitor.NodeVisitor;
 import org.jruby.evaluator.ASTInterpreter;
-import org.jruby.evaluator.Instruction;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallSite;
@@ -55,7 +54,7 @@ public class OpAsgnNode extends Node {
     public final CallSite variableAsgnCallAdapter;
 
     public OpAsgnNode(ISourcePosition position, Node receiverNode, Node valueNode, String variableName, String operatorName) {
-        super(position, NodeType.OPASGNNODE);
+        super(position);
         
         assert receiverNode != null : "receiverNode is not null";
         assert valueNode != null : "valueNode is not null";
@@ -67,11 +66,15 @@ public class OpAsgnNode extends Node {
         this.variableAsgnCallAdapter = MethodIndex.getCallSite((variableName + "=").intern());
     }
 
+    public NodeType getNodeType() {
+        return NodeType.OPASGNNODE;
+    }
+
     /**
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
      **/
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitOpAsgnNode(this);
     }
 
