@@ -44,7 +44,7 @@ import org.jruby.runtime.builtin.IRubyObject;
 /**
  * Represents a 'redo'
  */
-public class RedoNode extends Node {
+public class RedoNode extends Node implements NonLocalControlFlowNode {
     public RedoNode(ISourcePosition position) {
         super(position);
     }
@@ -53,6 +53,14 @@ public class RedoNode extends Node {
         return NodeType.REDONODE;
     }
 
+    public Node getValueNode() {
+        return null; // Needed for non local control flow node marker
+    }
+
+    public boolean hasValue() {
+        return false; // Redo never allows a value
+    }
+
     /**
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
