@@ -36,7 +36,6 @@ import java.util.List;
 import org.jruby.Ruby;
 import org.jruby.ast.visitor.NodeVisitor;
 import org.jruby.evaluator.ASTInterpreter;
-import org.jruby.evaluator.Instruction;
 import org.jruby.exceptions.JumpException;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.lexer.yacc.ISourcePosition;
@@ -58,8 +57,12 @@ public class UntilNode extends Node {
         this(position, conditionNode, bodyNode, true);
     }
 
+    public NodeType getNodeType() {
+        return NodeType.UNTILNODE;
+    }
+
     public UntilNode(ISourcePosition position, Node conditionNode, Node bodyNode, boolean evaluateAtStart) {
-        super(position, NodeType.UNTILNODE);
+        super(position);
         
         assert conditionNode != null : "conditionNode is not null";
         assert bodyNode != null : "bodyNode is not null";
@@ -73,7 +76,7 @@ public class UntilNode extends Node {
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
      **/
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitUntilNode(this);
     }
 
