@@ -36,7 +36,6 @@ import java.util.List;
 import org.jruby.Ruby;
 import org.jruby.ast.types.INameNode;
 import org.jruby.ast.visitor.NodeVisitor;
-import org.jruby.evaluator.Instruction;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallSite;
@@ -52,16 +51,20 @@ public class VCallNode extends Node implements INameNode {
     public final CallSite callAdapter;
 
     public VCallNode(ISourcePosition position, String name) {
-        super(position, NodeType.VCALLNODE);
+        super(position);
 
         this.callAdapter = MethodIndex.getVariableCallSite(name);
     }
+
+    public NodeType getNodeType() {
+        return NodeType.VCALLNODE;
+    }
     
     /**
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
      **/
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitVCallNode(this);
     }
 
