@@ -42,6 +42,7 @@ import org.jruby.ast.BackRefNode;
 import org.jruby.ast.BeginNode;
 import org.jruby.ast.BignumNode;
 import org.jruby.ast.BlockArgNode;
+import org.jruby.ast.BlockArg18Node;
 import org.jruby.ast.BlockNode;
 import org.jruby.ast.BlockPassNode;
 import org.jruby.ast.BreakNode;
@@ -65,6 +66,7 @@ import org.jruby.ast.DefinedNode;
 import org.jruby.ast.DefnNode;
 import org.jruby.ast.DefsNode;
 import org.jruby.ast.DotNode;
+import org.jruby.ast.EncodingNode;
 import org.jruby.ast.EnsureNode;
 import org.jruby.ast.EvStrNode;
 import org.jruby.ast.FCallNode;
@@ -80,6 +82,7 @@ import org.jruby.ast.IfNode;
 import org.jruby.ast.InstAsgnNode;
 import org.jruby.ast.InstVarNode;
 import org.jruby.ast.IterNode;
+import org.jruby.ast.LiteralNode;
 import org.jruby.ast.LocalAsgnNode;
 import org.jruby.ast.LocalVarNode;
 import org.jruby.ast.Match2Node;
@@ -148,6 +151,7 @@ public interface NodeVisitor {
     public Object visitBeginNode(BeginNode iVisited);
     public Object visitBignumNode(BignumNode iVisited);
     public Object visitBlockArgNode(BlockArgNode iVisited);
+    public Object visitBlockArg18Node(BlockArg18Node iVisited);
     public Object visitBlockNode(BlockNode iVisited);
     public Object visitBlockPassNode(BlockPassNode iVisited);
     public Object visitBreakNode(BreakNode iVisited);
@@ -171,6 +175,7 @@ public interface NodeVisitor {
     public Object visitDefnNode(DefnNode iVisited);
     public Object visitDefsNode(DefsNode iVisited);
     public Object visitDotNode(DotNode iVisited);
+    public Object visitEncodingNode(EncodingNode iVisited);
     public Object visitEnsureNode(EnsureNode iVisited);
     public Object visitEvStrNode(EvStrNode iVisited);
     public Object visitFCallNode(FCallNode iVisited);
@@ -186,6 +191,7 @@ public interface NodeVisitor {
     public Object visitInstVarNode(InstVarNode iVisited);
     public Object visitIfNode(IfNode iVisited);
     public Object visitIterNode(IterNode iVisited);
+    public Object visitLiteralNode(LiteralNode iVisited);
     public Object visitLocalAsgnNode(LocalAsgnNode iVisited);
     public Object visitLocalVarNode(LocalVarNode iVisited);
     public Object visitMultipleAsgnNode(MultipleAsgnNode iVisited);
