@@ -29,6 +29,7 @@
 
 package org.jruby.compiler;
 
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.Set;
@@ -59,6 +60,7 @@ import org.jruby.ast.ForNode;
 import org.jruby.ast.GlobalAsgnNode;
 import org.jruby.ast.GlobalVarNode;
 import org.jruby.ast.HashNode;
+import org.jruby.ast.Hash19Node;
 import org.jruby.ast.IArgumentNode;
 import org.jruby.ast.IScopingNode;
 import org.jruby.ast.IfNode;
@@ -134,8 +136,16 @@ public class ASTInspector {
     private boolean noFrame;
     
     public static Set<String> FRAME_AWARE_METHODS = Collections.synchronizedSet(new HashSet<String>());
-    private static Set<String> SCOPE_AWARE_METHODS = Collections.synchronizedSet(new HashSet<String>());
+    public static Set<String> SCOPE_AWARE_METHODS = Collections.synchronizedSet(new HashSet<String>());
+
+    public static void addFrameAwareMethods(String... methods) {
+        FRAME_AWARE_METHODS.addAll(Arrays.asList(methods));
+    }
     
+    public static void addScopeAwareMethods(String... methods) {
+        SCOPE_AWARE_METHODS.addAll(Arrays.asList(methods));
+    }
+
     public static Set<String> PRAGMAS = Collections.synchronizedSet(new HashSet<String>());
     
     static {
@@ -239,8 +249,10 @@ public class ASTInspector {
     }
     
     public void inspect(Node node) {
-        // TODO: This code effectively disables all inspection-based optimizations; none of them are 100% safe yet
-        if (!ENABLED) disable();
+        if (!ENABLED || RubyInstanceConfig.FULL_TRACE_ENABLED) {
+            disable();
+            return;
+        }
 
         if (node == null) return;
         
@@ -425,6 +437,8 @@ public class ASTInspector {
         case ENSURENODE:
             disable();
             break;
+        case ENCODINGNODE:
+            break;
         case EVSTRNODE:
             inspect(((EvStrNode)node).getBody());
             break;
@@ -706,6 +720,8 @@ public class ASTInspector {
             inspect(((WhenNode)node).getBodyNode());
             inspect(((WhenNode)node).getExpressionNodes());
             inspect(((WhenNode)node).getNextCase());
+            // Because Regexp#=== sets backref, we have to make this backref-aware
+            setFlag(BACKREF);
             break;
         case WHILENODE:
             WhileNode whileNode = (WhileNode)node;
@@ -765,11 +781,11 @@ public class ASTInspector {
     public boolean hasFrameAwareMethods() {
         return getFlag(
                 FRAME_AWARE | FRAME_BLOCK | FRAME_CLASS | FRAME_NAME | FRAME_SELF | FRAME_VISIBILITY |
-                CLOSURE | EVAL | BACKREF | LASTLINE | ZSUPER | SUPER);
+                CLOSURE | EVAL | ZSUPER | SUPER);
     }
 
     public boolean hasScopeAwareMethods() {
-        return getFlag(SCOPE_AWARE);
+        return getFlag(SCOPE_AWARE | BACKREF | LASTLINE);
     }
 
     public boolean hasBlockArg() {
