@@ -79,6 +79,9 @@ public interface VariableCompiler {
             ArrayCallback optNotGivenAssignment,
             CompilerCallback restAssignment,
             CompilerCallback blockAssignment);
+    public void assignClosureArguments(
+            CompilerCallback masgnCallback,
+            CompilerCallback blockAssignment);
     public int grabTempLocal();
     public void setTempLocal(int index);
     public void getTempLocal(int index);
