@@ -12,7 +12,7 @@ public class ChildScopedBodyCompiler19 extends ChildScopedBodyCompiler {
     }
 
     protected String getSignature() {
-        return StandardASMCompiler.CLOSURE_SIGNATURE19;
+        return StandardASMCompiler.getStaticClosure19Signature(script.getClassname());
     }
 
     public ChainedChildBodyCompiler outline(String methodName) {
@@ -25,7 +25,7 @@ public class ChildScopedBodyCompiler19 extends ChildScopedBodyCompiler {
         }
         // we append an index to ensure two identical method names will not conflict
         methodName = methodName + "_" + script.getAndIncrementMethodIndex();
-        method.invokevirtual(script.getClassname(), methodName, getSignature());
+        method.invokestatic(script.getClassname(), methodName, getSignature());
 
         ChainedChildBodyCompiler19 methodCompiler = new ChainedChildBodyCompiler19(script, methodName, inspector, scope, this);
 
