@@ -18,6 +18,7 @@ import org.jruby.RubyString;
 import org.jruby.RubySymbol;
 import org.jruby.ast.NodeType;
 import org.jruby.ast.executable.AbstractScript;
+import org.jruby.ast.executable.RuntimeCache;
 import org.jruby.compiler.ASTInspector;
 import org.jruby.compiler.CacheCompiler;
 import org.jruby.compiler.CompilerCallback;
@@ -108,7 +109,7 @@ public class InheritedCacheCompiler implements CacheCompiler {
     public void cacheSymbol(BaseBodyCompiler method, String symbol) {
         Integer index = symbolIndices.get(symbol);
         if (index == null) {
-            index = new Integer(inheritedSymbolCount++);
+            index = Integer.valueOf(inheritedSymbolCount++);
             symbolIndices.put(symbol, index);
         }
 
@@ -145,20 +146,20 @@ public class InheritedCacheCompiler implements CacheCompiler {
         Label alreadyCompiled = new Label();
 
         method.loadThis();
-        method.method.getfield(scriptCompiler.getClassname(), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
+        method.method.getfield(scriptCompiler.getClassname(), "runtimeCache", ci(RuntimeCache.class));
         method.method.pushInt(index);
-        method.method.invokevirtual(p(AbstractScript.RuntimeCache.class), "getRegexp", sig(RubyRegexp.class, int.class));
+        method.method.invokevirtual(p(RuntimeCache.class), "getRegexp", sig(RubyRegexp.class, int.class));
         method.method.dup();
 
         method.ifNotNull(alreadyCompiled);
 
         method.method.pop();
         method.loadThis();
-        method.method.getfield(scriptCompiler.getClassname(), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
+        method.method.getfield(scriptCompiler.getClassname(), "runtimeCache", ci(RuntimeCache.class));
         method.method.pushInt(index);
         createStringCallback.call(method);
         method.method.ldc(options);
-        method.method.invokevirtual(p(AbstractScript.RuntimeCache.class), "cacheRegexp", sig(RubyRegexp.class, int.class, RubyString.class, int.class));
+        method.method.invokevirtual(p(RuntimeCache.class), "cacheRegexp", sig(RubyRegexp.class, int.class, RubyString.class, int.class));
 
         method.method.label(alreadyCompiled);
     }
@@ -194,7 +195,7 @@ public class InheritedCacheCompiler implements CacheCompiler {
         } else {
             Integer index = fixnumIndices.get(value);
             if (index == null) {
-                index = new Integer(inheritedFixnumCount++);
+                index =  Integer.valueOf(inheritedFixnumCount++);
                 fixnumIndices.put(value, index);
             }
             
@@ -251,7 +252,7 @@ public class InheritedCacheCompiler implements CacheCompiler {
         String asString = contents.toString();
         Integer index = stringIndices.get(asString);
         if (index == null) {
-            index = new Integer(inheritedStringCount++);
+            index = Integer.valueOf(inheritedStringCount++);
             stringIndices.put(asString, index);
         }
 
@@ -430,187 +431,81 @@ public class InheritedCacheCompiler implements CacheCompiler {
     public void finish() {
         SkinnyMethodAdapter initMethod = scriptCompiler.getInitMethod();
 
-        // generate call sites initialization code
-        int size = callSiteList.size();
-        if (size != 0) {
+        // generate call sites portion of descriptor
+        int callSiteCount = callSiteList.size();
+        int otherCount = scopeCount
+                + inheritedSymbolCount
+                + inheritedFixnumCount
+                + inheritedConstantCount
+                + inheritedRegexpCount
+                + inheritedBigIntegerCount
+                + inheritedVariableReaderCount
+                + inheritedVariableWriterCount
+                + inheritedBlockBodyCount
+                + inheritedBlockCallbackCount
+                + inheritedMethodCount
+                + inheritedStringCount;
+        if (callSiteCount + otherCount != 0) {
             ensureRuntimeCacheInited(initMethod);
-            
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.dup();
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initCallSites", sig(void.class, int.class));
-            initMethod.getfield(p(AbstractScript.RuntimeCache.class), "callSites", ci(CallSite[].class));
-            
-            for (int i = size - 1; i >= 0; i--) {
+
+            StringBuffer descriptor = new StringBuffer(callSiteCount * 5 + 12); // rough guess of total size
+
+            for (int i = 0; i < callSiteCount; i++) {
                 String name = callSiteList.get(i);
                 CallType callType = callTypeList.get(i);
-
-                initMethod.pushInt(i);
+                
+                if (i > 0) descriptor.append('\uFFFF');
+                
                 if (callType.equals(CallType.NORMAL)) {
-                    initMethod.ldc(name);
-                    initMethod.invokestatic(scriptCompiler.getClassname(), "setCallSite", sig(CallSite[].class, params(CallSite[].class, int.class, String.class)));
+                    descriptor.append(name).append("\uFFFFN");
                 } else if (callType.equals(CallType.FUNCTIONAL)) {
-                    initMethod.ldc(name);
-                    initMethod.invokestatic(scriptCompiler.getClassname(), "setFunctionalCallSite", sig(CallSite[].class, params(CallSite[].class, int.class, String.class)));
+                    descriptor.append(name).append("\uFFFFF");
                 } else if (callType.equals(CallType.VARIABLE)) {
-                    initMethod.ldc(name);
-                    initMethod.invokestatic(scriptCompiler.getClassname(), "setVariableCallSite", sig(CallSite[].class, params(CallSite[].class, int.class, String.class)));
+                    descriptor.append(name).append("\uFFFFV");
                 } else if (callType.equals(CallType.SUPER)) {
-                    initMethod.invokestatic(scriptCompiler.getClassname(), "setSuperCallSite", sig(CallSite[].class, params(CallSite[].class, int.class)));
+                    descriptor.append("super").append("\uFFFFS");
                 }
             }
 
-            initMethod.pop();
-        }
-
-        size = scopeCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
+            // generate "others" part of descriptor
+            descriptor.append('\uFFFF');
+            descriptor.append((char)scopeCount);
+            descriptor.append((char)inheritedSymbolCount);
+            descriptor.append((char)inheritedFixnumCount);
+            descriptor.append((char)inheritedConstantCount);
+            descriptor.append((char)inheritedRegexpCount);
+            descriptor.append((char)inheritedBigIntegerCount);
+            descriptor.append((char)inheritedVariableReaderCount);
+            descriptor.append((char)inheritedVariableWriterCount);
+            descriptor.append((char)inheritedBlockBodyCount);
+            descriptor.append((char)inheritedBlockCallbackCount);
+            descriptor.append((char)inheritedMethodCount);
+            descriptor.append((char)inheritedStringCount);
 
+            // init from descriptor
             initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initScopes", sig(void.class, params(int.class)));
-        }
+            initMethod.ldc(descriptor.toString());
+            initMethod.invokevirtual(p(AbstractScript.class), "initFromDescriptor", sig(void.class, String.class));
 
-        // generate symbols initialization code
-        size = inheritedSymbolCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initSymbols", sig(void.class, params(int.class)));
-        }
-
-        // generate fixnums initialization code
-        size = inheritedFixnumCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initFixnums", sig(void.class, params(int.class)));
-        }
-
-        // generate constants initialization code
-        size = inheritedConstantCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initConstants", sig(void.class, params(int.class)));
-        }
-
-        // generate regexps initialization code
-        size = inheritedRegexpCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initRegexps", sig(void.class, params(int.class)));
-        }
-
-        // generate regexps initialization code
-        size = inheritedBigIntegerCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initBigIntegers", sig(void.class, params(int.class)));
-        }
-
-        // generate variable readers initialization code
-        size = inheritedVariableReaderCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initVariableReaders", sig(void.class, params(int.class)));
-        }
-
-        // generate variable writers initialization code
-        size = inheritedVariableWriterCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initVariableWriters", sig(void.class, params(int.class)));
-        }
-
-        // generate block bodies initialization code
-        size = inheritedBlockBodyCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initBlockBodies", sig(void.class, params(int.class)));
-        }
-
-        // generate block bodies initialization code
-        size = inheritedBlockCallbackCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initBlockCallbacks", sig(void.class, params(int.class)));
-        }
-
-        // generate bytelists initialization code
-        size = inheritedStringCount;
-        if (inheritedStringCount > 0) {
-            ensureRuntimeCacheInited(initMethod);
-
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initStrings", sig(ByteList[].class, params(int.class)));
-
-            for (Map.Entry<String, Integer> entry : stringIndices.entrySet()) {
-                initMethod.ldc(entry.getValue());
-                initMethod.ldc(entry.getKey());
-                initMethod.invokestatic(p(AbstractScript.class), "createByteList", sig(ByteList[].class, ByteList[].class, int.class, String.class));
+            if (inheritedStringCount > 0) {
+                // init all strings
+                for (Map.Entry<String, Integer> entry : stringIndices.entrySet()) {
+                    initMethod.aload(0);
+                    initMethod.ldc(entry.getValue());
+                    initMethod.ldc(entry.getKey());
+                    initMethod.invokevirtual(p(AbstractScript.class), "setByteList", sig(void.class, int.class, String.class));
+                }
             }
-            
-            initMethod.pop();
-        }
-
-        // generate method cache initialization code
-        size = inheritedMethodCount;
-        if (size != 0) {
-            ensureRuntimeCacheInited(initMethod);
-            
-            initMethod.aload(0);
-            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
-            initMethod.pushInt(size);
-            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initMethodCache", sig(void.class, params(int.class)));
         }
     }
 
     private void ensureRuntimeCacheInited(SkinnyMethodAdapter initMethod) {
         if (!runtimeCacheInited) {
             initMethod.aload(0);
-            initMethod.newobj(p(AbstractScript.RuntimeCache.class));
+            initMethod.newobj(p(RuntimeCache.class));
             initMethod.dup();
-            initMethod.invokespecial(p(AbstractScript.RuntimeCache.class), "<init>", sig(void.class));
-            initMethod.putfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
+            initMethod.invokespecial(p(RuntimeCache.class), "<init>", sig(void.class));
+            initMethod.putfield(p(AbstractScript.class), "runtimeCache", ci(RuntimeCache.class));
             runtimeCacheInited = true;
         }
     }
