@@ -28,9 +28,11 @@
 
 package org.jruby.compiler.impl;
 
+import org.jruby.Ruby;
 import org.jruby.compiler.CompilerCallback;
 import org.jruby.compiler.NotCompilableException;
 import org.jruby.parser.StaticScope;
+import org.jruby.runtime.Block;
 import org.jruby.runtime.DynamicScope;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.objectweb.asm.Label;
@@ -120,9 +122,14 @@ public class StackBasedVariableCompiler extends AbstractVariableCompiler {
             // temp locals must start after last real local
             tempVariableIndex += scope.getNumberOfVariables();
         }
-        
+
         if (argsCallback != null) {
-            // load args[0] which will be the IRubyObject representing block args
+            // load block
+            methodCompiler.loadRuntime();
+            method.aload(methodCompiler.getClosureIndex());
+            methodCompiler.invokeUtilityMethod("processBlockArgument", sig(IRubyObject.class, params(Ruby.class, Block.class)));
+
+            // load args (the IRubyObject representing incoming normal args)
             method.aload(argsIndex);
             argsCallback.call(methodCompiler);
         }
