@@ -37,6 +37,9 @@ import java.io.PrintWriter;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.net.URL;
+import java.util.ArrayList;
+import java.util.List;
 
 import org.jruby.Ruby;
 import org.jruby.RubyInstanceConfig;
@@ -46,9 +49,12 @@ import org.jruby.compiler.CacheCompiler;
 import org.jruby.compiler.CompilerCallback;
 import org.jruby.compiler.BodyCompiler;
 import org.jruby.compiler.ScriptCompiler;
+import org.jruby.internal.runtime.methods.CallConfiguration;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.parser.StaticScope;
+import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
+import org.jruby.runtime.BlockBody;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import static org.jruby.util.CodegenUtils.*;
@@ -60,6 +66,7 @@ import org.objectweb.asm.ClassWriter;
 import org.objectweb.asm.Label;
 import org.objectweb.asm.MethodVisitor;
 import org.objectweb.asm.Opcodes;
+import org.objectweb.asm.Type;
 import org.objectweb.asm.util.CheckClassAdapter;
 import org.objectweb.asm.util.TraceClassVisitor;
 
@@ -107,7 +114,7 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
     }
 
     public static String getStaticClosureSignature(String classdesc) {
-        return sig(IRubyObject.class, "L" + classdesc + ";", ThreadContext.class, IRubyObject.class, IRubyObject.class);
+        return sig(IRubyObject.class, "L" + classdesc + ";", ThreadContext.class, IRubyObject.class, IRubyObject.class, Block.class);
     }
 
     public static String getStaticClosure19Signature(String classdesc) {
@@ -115,7 +122,7 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
     }
 
     public static String getClosureSignature() {
-        return sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, IRubyObject.class);
+        return sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, IRubyObject.class, Block.class);
     }
 
     public static String getClosure19Signature() {
@@ -156,6 +163,10 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
     public static final Constructor invDynInvCompilerConstructor;
     public static final Method invDynSupportInstaller;
 
+    private List<InvokerDescriptor> invokerDescriptors = new ArrayList<InvokerDescriptor>();
+    private List<BlockCallbackDescriptor> blockCallbackDescriptors = new ArrayList<BlockCallbackDescriptor>();
+    private List<BlockCallbackDescriptor> blockCallback19Descriptors = new ArrayList<BlockCallbackDescriptor>();
+
     static {
         Constructor compilerConstructor = null;
         Method installerMethod = null;
@@ -204,18 +215,56 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
     }
 
     public void writeClass(File destination) throws IOException {
-        writeClass(getClassname(),destination, classWriter);
+        writeClass(getClassname(), destination, classWriter);
+    }
+
+    public void writeInvokers(File destination) throws IOException {
+        for (InvokerDescriptor descriptor : invokerDescriptors) {
+            byte[] invokerBytes = RuntimeHelpers.defOffline(
+                    descriptor.getName(),
+                    descriptor.getClassname(),
+                    descriptor.getInvokerName(),
+                    descriptor.getArity(),
+                    descriptor.getScope(),
+                    descriptor.getCallConfig(),
+                    descriptor.getFile(),
+                    descriptor.getLine());
+
+            CheckClassAdapter.verify(new ClassReader(invokerBytes), false, new PrintWriter(System.err));
+
+            writeClassFile(destination, invokerBytes, descriptor.getInvokerName());
+        }
+
+        for (BlockCallbackDescriptor descriptor : blockCallbackDescriptors) {
+            byte[] callbackBytes = RuntimeHelpers.createBlockCallbackOffline(descriptor.getClassname(), descriptor.getMethod());
+
+            CheckClassAdapter.verify(new ClassReader(callbackBytes), false, new PrintWriter(System.err));
+
+            writeClassFile(destination, callbackBytes, descriptor.getCallbackName());
+        }
+
+        for (BlockCallbackDescriptor descriptor : blockCallback19Descriptors) {
+            byte[] callbackBytes = RuntimeHelpers.createBlockCallback19Offline(descriptor.getClassname(), descriptor.getMethod());
+
+            CheckClassAdapter.verify(new ClassReader(callbackBytes), false, new PrintWriter(System.err));
+
+            writeClassFile(destination, callbackBytes, descriptor.getCallbackName());
+        }
     }
 
     private void writeClass(String classname, File destination, ClassWriter writer) throws IOException {
-        String fullname = classname + ".class";
-        String filename = null;
-        String path = null;
-        
         // verify the class
         byte[] bytecode = writer.toByteArray();
         CheckClassAdapter.verify(new ClassReader(bytecode), false, new PrintWriter(System.err));
-        
+
+        writeClassFile(destination, bytecode, classname);
+    }
+
+    private void writeClassFile(File destination, byte[] bytecode, String classname) throws IOException {
+        String fullname = classname + ".class";
+        String filename = null;
+        String path = null;
+
         if (fullname.lastIndexOf("/") == -1) {
             filename = fullname;
             path = "";
@@ -233,6 +282,101 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
         out.close();
     }
 
+    public static class InvokerDescriptor {
+        private final String name;
+        private final String classname;
+        private final String invokerName;
+        private final Arity arity;
+        private final StaticScope scope;
+        private final CallConfiguration callConfig;
+        private final String file;
+        private final int line;
+        
+        public InvokerDescriptor(String name, String classname, String invokerName, Arity arity, StaticScope scope, CallConfiguration callConfig, String file, int line) {
+            this.name = name;
+            this.classname = classname;
+            this.invokerName = invokerName;
+            this.arity = arity;
+            this.scope = scope;
+            this.callConfig = callConfig;
+            this.file = file;
+            this.line = line;
+        }
+
+        public Arity getArity() {
+            return arity;
+        }
+
+        public CallConfiguration getCallConfig() {
+            return callConfig;
+        }
+
+        public String getClassname() {
+            return classname;
+        }
+
+        public String getFile() {
+            return file;
+        }
+
+        public String getInvokerName() {
+            return invokerName;
+        }
+
+        public int getLine() {
+            return line;
+        }
+
+        public String getName() {
+            return name;
+        }
+
+        public StaticScope getScope() {
+            return scope;
+        }
+    }
+
+    private static class BlockCallbackDescriptor {
+        private final String method;
+        private final String classname;
+        private final String callbackName;
+
+        public BlockCallbackDescriptor(String method, String classname) {
+            this.method = method;
+            this.classname = classname;
+            this.callbackName = classname + "BlockCallback$" + method + "xx1";
+        }
+
+        public String getClassname() {
+            return classname;
+        }
+
+        public String getMethod() {
+            return method;
+        }
+
+        public String getCallbackName() {
+            return callbackName;
+        }
+    }
+
+    public void addInvokerDescriptor(String newMethodName, int methodArity, StaticScope scope, CallConfiguration callConfig, String filename, int line) {
+        String classPath = classname.replaceAll("/", "_");
+        Arity arity = Arity.createArity(methodArity);
+        String invokerName = classPath + "Invoker" + newMethodName + arity;
+        InvokerDescriptor descriptor = new InvokerDescriptor(newMethodName, classname, invokerName, arity, scope, callConfig, filename, line);
+
+        invokerDescriptors.add(descriptor);
+    }
+
+    public void addBlockCallbackDescriptor(String method) {
+        blockCallbackDescriptors.add(new BlockCallbackDescriptor(method, classname));
+    }
+
+    public void addBlockCallback19Descriptor(String method) {
+        blockCallback19Descriptors.add(new BlockCallbackDescriptor(method, classname));
+    }
+
     public String getClassname() {
         return classname;
     }
@@ -279,8 +423,8 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
             sourceNoPath = sourcename;
         }
 
-        
-        classWriter.visitSource(sourceNoPath, null);
+        final File sourceFile = new File(getSourcename());
+        classWriter.visitSource(sourceNoPath, sourceFile.getAbsolutePath());
     }
 
     public void endScript(boolean generateLoad, boolean generateMain) {
@@ -335,6 +479,15 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
             method.dup();
             method.invokespecial(getClassname(), "<init>", sig(Void.TYPE));
 
+            // set filename for the loaded script class (JRUBY-4825)
+            method.dup();
+            method.ldc(Type.getType("L" + getClassname() + ";"));
+            method.invokevirtual(p(Class.class), "getClassLoader", sig(ClassLoader.class));
+            method.ldc(getClassname() + ".class");
+            method.invokevirtual(p(ClassLoader.class), "getResource", sig(URL.class, String.class));
+            method.invokevirtual(p(Object.class), "toString", sig(String.class));
+            method.invokevirtual(p(AbstractScript.class), "setFilename", sig(void.class, String.class));
+
             // instance config for the script run
             method.newobj(p(RubyInstanceConfig.class));
             method.dup();
@@ -403,14 +556,6 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
         initMethod.aload(THIS);
         initMethod.invokespecial(p(AbstractScript.class), "<init>", sig(Void.TYPE));
         
-        cv.visitField(ACC_PRIVATE | ACC_FINAL, "$class", ci(Class.class), null, null);
-        
-        // FIXME: this really ought to be in clinit, but it doesn't matter much
-        initMethod.aload(THIS);
-        initMethod.ldc(c(getClassname()));
-        initMethod.invokestatic(p(Class.class), "forName", sig(Class.class, params(String.class)));
-        initMethod.putfield(getClassname(), "$class", ci(Class.class));
-        
         // JRUBY-3014: make __FILE__ dynamically determined at load time, but
         // we provide a reasonable default here
         initMethod.aload(THIS);
