@@ -27,11 +27,15 @@
 
 package org.jruby.compiler.impl;
 
-import org.jruby.RubyArray;
+import org.jruby.RubyClass;
+import org.jruby.RubyInstanceConfig;
+import org.jruby.RubyModule;
 import org.jruby.compiler.ArgumentsCallback;
+import org.jruby.compiler.BodyCompiler;
 import org.jruby.compiler.CompilerCallback;
 import org.jruby.compiler.InvocationCompiler;
 import org.jruby.compiler.NotCompilableException;
+import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallSite;
 import org.jruby.runtime.CallType;
@@ -62,76 +66,79 @@ public class StandardInvocationCompiler implements InvocationCompiler {
     }
 
     public void invokeAttrAssignMasgn(String name, CompilerCallback receiverCallback, ArgumentsCallback argsCallback) {
-        // value is already on stack, evaluate receiver and args and call helper
+        // value is already on stack, save it for later
+        int temp = methodCompiler.getVariableCompiler().grabTempLocal();
+        methodCompiler.getVariableCompiler().setTempLocal(temp);
+        
+        // receiver first, so we know which call site to use
         receiverCallback.call(methodCompiler);
 
-        String signature;
+        // select appropriate call site
+        method.dup(); // dup receiver
+        methodCompiler.loadSelf(); // load self
+        methodCompiler.getScriptCompiler().getCacheCompiler().cacheCallSite(methodCompiler, name, CallType.NORMAL);
+        methodCompiler.getScriptCompiler().getCacheCompiler().cacheCallSite(methodCompiler, name, CallType.VARIABLE);
+        methodCompiler.invokeUtilityMethod("selectAttrAsgnCallSite", sig(CallSite.class, IRubyObject.class, IRubyObject.class, CallSite.class, CallSite.class));
+
+        String signature = null;
         if (argsCallback == null) {
             signature = sig(IRubyObject.class,
-                    IRubyObject.class /*value*/,
                     IRubyObject.class /*receiver*/,
+                    CallSite.class,
+                    IRubyObject.class /*value*/,
                     ThreadContext.class,
-                    IRubyObject.class, /*self*/
-                    CallSite.class);
+                    IRubyObject.class /*self*/);
         } else {
             switch (argsCallback.getArity()) {
-                case 0:
-                    signature = sig(IRubyObject.class,
-                            IRubyObject.class /*value*/,
-                            IRubyObject.class /*receiver*/,
-                            ThreadContext.class,
-                            IRubyObject.class, /*self*/
-                            CallSite.class);
-                    break;
-                case 1:
-                    argsCallback.call(methodCompiler);
-                    signature = sig(IRubyObject.class,
-                            IRubyObject.class /*value*/,
-                            IRubyObject.class /*receiver*/,
-                            IRubyObject.class /*arg0*/,
-                            ThreadContext.class,
-                            IRubyObject.class, /*self*/
-                            CallSite.class);
-                    break;
-                case 2:
-                    argsCallback.call(methodCompiler);
-                    signature = sig(IRubyObject.class,
-                            IRubyObject.class /*value*/,
-                            IRubyObject.class /*receiver*/,
-                            IRubyObject.class /*arg0*/,
-                            IRubyObject.class /*arg1*/,
-                            ThreadContext.class,
-                            IRubyObject.class, /*self*/
-                            CallSite.class);
-                    break;
-                case 3:
-                    argsCallback.call(methodCompiler);
-                    signature = sig(IRubyObject.class,
-                            IRubyObject.class /*value*/,
-                            IRubyObject.class /*receiver*/,
-                            IRubyObject.class /*arg0*/,
-                            IRubyObject.class /*arg1*/,
-                            IRubyObject.class /*arg2*/,
-                            ThreadContext.class,
-                            IRubyObject.class, /*self*/
-                            CallSite.class);
-                    break;
-                default:
-                    argsCallback.call(methodCompiler);
-                    signature = sig(IRubyObject.class,
-                            IRubyObject.class /*value*/,
-                            IRubyObject.class /*receiver*/,
-                            IRubyObject[].class /*args*/,
-                            ThreadContext.class,
-                            IRubyObject.class, /*self*/
-                            CallSite.class);
-                    break;
+            case 1:
+                argsCallback.call(methodCompiler);
+                signature = sig(IRubyObject.class,
+                        IRubyObject.class, /*receiver*/
+                        CallSite.class,
+                        IRubyObject.class, /*arg0*/
+                        IRubyObject.class, /*value*/
+                        ThreadContext.class,
+                        IRubyObject.class /*self*/);
+                break;
+            case 2:
+                argsCallback.call(methodCompiler);
+                signature = sig(IRubyObject.class,
+                        IRubyObject.class, /*receiver*/
+                        CallSite.class,
+                        IRubyObject.class, /*arg0*/
+                        IRubyObject.class, /*arg1*/
+                        IRubyObject.class, /*value*/
+                        ThreadContext.class,
+                        IRubyObject.class /*self*/);
+                break;
+            case 3:
+                argsCallback.call(methodCompiler);
+                signature = sig(IRubyObject.class,
+                        IRubyObject.class, /*receiver*/
+                        CallSite.class,
+                        IRubyObject.class, /*arg0*/
+                        IRubyObject.class, /*arg1*/
+                        IRubyObject.class, /*arg2*/
+                        IRubyObject.class, /*value*/
+                        ThreadContext.class,
+                        IRubyObject.class /*self*/);
+                break;
+            default:
+                argsCallback.call(methodCompiler);
+                signature = sig(IRubyObject.class,
+                        IRubyObject.class, /*receiver*/
+                        CallSite.class,
+                        IRubyObject[].class, /*args*/
+                        IRubyObject.class, /*value*/
+                        ThreadContext.class,
+                        IRubyObject.class /*self*/);
             }
         }
-        
+
+        methodCompiler.getVariableCompiler().getTempLocal(temp);
+        methodCompiler.getVariableCompiler().releaseTempLocal();
         methodCompiler.loadThreadContext();
         methodCompiler.loadSelf();
-        methodCompiler.getScriptCompiler().getCacheCompiler().cacheCallSite(methodCompiler, name, CallType.NORMAL);
 
         methodCompiler.invokeUtilityMethod("doAttrAsgn", signature);
     }
@@ -370,41 +377,48 @@ public class StandardInvocationCompiler implements InvocationCompiler {
         }
     }
 
-    public void invokeSuper(CompilerCallback argsCallback, CompilerCallback closureArg) {
-        methodCompiler.loadThreadContext();
-        methodCompiler.invokeUtilityMethod("checkSuperDisabledOrOutOfMethod", sig(void.class, ThreadContext.class));
-        
+    public void invokeBinaryFixnumRHS(String name, CompilerCallback receiverCallback, long fixnum) {
+        methodCompiler.getScriptCompiler().getCacheCompiler().cacheCallSite(methodCompiler, name, CallType.NORMAL);
+        methodCompiler.loadThreadContext(); // [adapter, tc]
+
+        // for visibility checking without requiring frame self
+        // TODO: don't bother passing when fcall or vcall, and adjust callsite appropriately
         methodCompiler.loadSelf();
 
-        methodCompiler.loadThreadContext(); // [self, tc]
-        
-        // args
-        if (argsCallback == null) {
-            method.getstatic(p(IRubyObject.class), "NULL_ARRAY", ci(IRubyObject[].class));
-            // block
-            if (closureArg == null) {
-                // no args, no block
-                methodCompiler.loadBlock();
-            } else {
-                // no args, with block
-                closureArg.call(methodCompiler);
-            }
+        if (receiverCallback != null) {
+            receiverCallback.call(methodCompiler);
         } else {
-            argsCallback.call(methodCompiler);
-            // block
-            if (closureArg == null) {
-                // with args, no block
-                methodCompiler.loadBlock();
-            } else {
-                // with args, with block
-                closureArg.call(methodCompiler);
+            methodCompiler.loadSelf();
+        }
+
+        method.ldc(fixnum);
+
+        String signature = sig(IRubyObject.class, params(ThreadContext.class, IRubyObject.class, IRubyObject.class, long.class));
+        String callSiteMethod = "call";
+
+        method.invokevirtual(p(CallSite.class), callSiteMethod, signature);
+    }
+    
+    public void invokeDynamic(String name, CompilerCallback receiverCallback, ArgumentsCallback argsCallback, CallType callType, CompilerCallback closureArg, boolean iterator) {
+        if (RubyInstanceConfig.INLINE_DYNCALL_ENABLED && closureArg == null) {
+            if (callType == CallType.FUNCTIONAL || callType == CallType.VARIABLE) {
+                if (argsCallback == null) {
+                    invokeDynamicSelfNoBlockZero(name);
+                    return;
+                } else if (argsCallback.getArity() >= 1 && argsCallback.getArity() <= 3) {
+                    invokeDynamicSelfNoBlockSpecificArity(name, argsCallback);
+                    return;
+                }
+//            } else if (callType == CallType.NORMAL) {
+//                if (argsCallback == null) {
+//                    invokeDynamicNoBlockZero(name, receiverCallback);
+//                    return;
+//                } else if (argsCallback.getArity() >= 1 && argsCallback.getArity() <= 3) {
+//                    invokeDynamicNoBlockSpecificArity(name, receiverCallback, argsCallback);
+//                    return;
+//                }
             }
         }
-        
-        method.invokeinterface(p(IRubyObject.class), "callSuper", sig(IRubyObject.class, ThreadContext.class, IRubyObject[].class, Block.class));
-    }
-
-    public void invokeDynamic(String name, CompilerCallback receiverCallback, ArgumentsCallback argsCallback, CallType callType, CompilerCallback closureArg, boolean iterator) {
         methodCompiler.getScriptCompiler().getCacheCompiler().cacheCallSite(methodCompiler, name, callType);
 
         methodCompiler.loadThreadContext(); // [adapter, tc]
@@ -418,6 +432,15 @@ public class StandardInvocationCompiler implements InvocationCompiler {
         } else {
             methodCompiler.loadSelf();
         }
+
+        // super uses current block if none given
+        if (callType == CallType.SUPER && closureArg == null) {
+            closureArg = new CompilerCallback() {
+                public void call(BodyCompiler context) {
+                    methodCompiler.loadBlock();
+                }
+            };
+        }
         
         String signature;
         String callSiteMethod = "call";
@@ -477,6 +500,76 @@ public class StandardInvocationCompiler implements InvocationCompiler {
         method.invokevirtual(p(CallSite.class), callSiteMethod, signature);
     }
 
+    public void invokeDynamicSelfNoBlockZero(String name) {
+        methodCompiler.getScriptCompiler().getCacheCompiler().cacheMethod(methodCompiler, name);
+        methodCompiler.loadThreadContext();
+        methodCompiler.loadSelf();
+        method.dup();
+        method.invokeinterface(p(IRubyObject.class), "getMetaClass", sig(RubyClass.class));
+        method.ldc(name);
+        method.invokevirtual(p(DynamicMethod.class), "call", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, RubyModule.class, String.class));
+    }
+
+    public void invokeDynamicSelfNoBlockSpecificArity(String name, ArgumentsCallback argsCallback) {
+        methodCompiler.getScriptCompiler().getCacheCompiler().cacheMethod(methodCompiler, name);
+        methodCompiler.loadThreadContext();
+        methodCompiler.loadSelf();
+        method.dup();
+        method.invokeinterface(p(IRubyObject.class), "getMetaClass", sig(RubyClass.class));
+        method.ldc(name);
+        argsCallback.call(methodCompiler);
+        switch (argsCallback.getArity()) {
+        case 1:
+            method.invokevirtual(p(DynamicMethod.class), "call", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, RubyModule.class, String.class, IRubyObject.class));
+            break;
+        case 2:
+            method.invokevirtual(p(DynamicMethod.class), "call", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, RubyModule.class, String.class, IRubyObject.class, IRubyObject.class));
+            break;
+        case 3:
+            method.invokevirtual(p(DynamicMethod.class), "call", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, RubyModule.class, String.class, IRubyObject.class, IRubyObject.class, IRubyObject.class));
+            break;
+        }
+    }
+
+    public void invokeDynamicNoBlockZero(String name, CompilerCallback receiverCallback) {
+        receiverCallback.call(methodCompiler);
+        int recv = methodCompiler.getVariableCompiler().grabTempLocal();
+        method.astore(recv);
+        methodCompiler.getScriptCompiler().getCacheCompiler().cacheMethod(methodCompiler, name, recv);
+        methodCompiler.loadThreadContext();
+        method.aload(recv);
+        methodCompiler.getVariableCompiler().releaseTempLocal();
+        method.dup();
+        method.invokeinterface(p(IRubyObject.class), "getMetaClass", sig(RubyClass.class));
+        method.ldc(name);
+        method.invokevirtual(p(DynamicMethod.class), "call", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, RubyModule.class, String.class));
+    }
+
+    public void invokeDynamicNoBlockSpecificArity(String name, CompilerCallback receiverCallback, ArgumentsCallback argsCallback) {
+        receiverCallback.call(methodCompiler);
+        int recv = methodCompiler.getVariableCompiler().grabTempLocal();
+        method.astore(recv);
+        methodCompiler.getScriptCompiler().getCacheCompiler().cacheMethod(methodCompiler, name, recv);
+        methodCompiler.loadThreadContext();
+        method.aload(recv);
+        methodCompiler.getVariableCompiler().releaseTempLocal();
+        method.dup();
+        method.invokeinterface(p(IRubyObject.class), "getMetaClass", sig(RubyClass.class));
+        method.ldc(name);
+        argsCallback.call(methodCompiler);
+        switch (argsCallback.getArity()) {
+        case 1:
+            method.invokevirtual(p(DynamicMethod.class), "call", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, RubyModule.class, String.class, IRubyObject.class));
+            break;
+        case 2:
+            method.invokevirtual(p(DynamicMethod.class), "call", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, RubyModule.class, String.class, IRubyObject.class, IRubyObject.class));
+            break;
+        case 3:
+            method.invokevirtual(p(DynamicMethod.class), "call", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, RubyModule.class, String.class, IRubyObject.class, IRubyObject.class, IRubyObject.class));
+            break;
+        }
+    }
+
     public void invokeOpAsgnWithOr(String attrName, String attrAsgnName, CompilerCallback receiverCallback, ArgumentsCallback argsCallback) {
         receiverCallback.call(methodCompiler);
         method.dup();
@@ -573,16 +666,19 @@ public class StandardInvocationCompiler implements InvocationCompiler {
         methodCompiler.loadBlock();
         methodCompiler.loadThreadContext();
 
-        String signature;
         if (argsCallback != null) {
             argsCallback.call(methodCompiler);
-            signature = sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, boolean.class);
         } else {
-            signature = sig(IRubyObject.class, ThreadContext.class, boolean.class);
+            method.aconst_null();
         }
-        method.ldc(unwrap);
 
-        method.invokevirtual(p(Block.class), "yield", signature);
+        if (unwrap) {
+            method.aconst_null();
+            method.aconst_null();
+            method.invokevirtual(p(Block.class), "yieldArray", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, IRubyObject.class, RubyModule.class));
+        } else {
+            method.invokevirtual(p(Block.class), "yield", sig(IRubyObject.class, ThreadContext.class, IRubyObject.class));
+        }
     }
 
     public void yieldSpecific(ArgumentsCallback argsCallback) {
