@@ -24,12 +24,20 @@ public class TwoOperandInstr extends IR_Instr
     public String toString() { return super.toString() + "(" + _arg1 + ", " + _arg2 + ")"; }
 
     public Operand[] getOperands() {
-        return new Operand[] {_arg1, _arg2};
+        // SSS FIXME: Looks like _arg2 can be null for NOT alu instructions -- fix this IR bug.
+        if (_arg2 == null)
+            return new Operand[] {_arg1};
+        else 
+            return new Operand[] {_arg1, _arg2};
     }
 
     public void simplifyOperands(Map<Operand, Operand> valueMap)
     {
         _arg1 = _arg1.getSimplifiedOperand(valueMap);
-        _arg2 = _arg2.getSimplifiedOperand(valueMap);
+        // SSS FIXME: Looks like _arg2 can be null for NOT alu instructions -- fix this IR bug.
+        if (_arg2 == null)
+           ; //System.out.println("Got null arg2 for a 2-operand instruction: " + this.toString());
+        else
+           _arg2 = _arg2.getSimplifiedOperand(valueMap);
     }
 }
