@@ -33,10 +33,8 @@ package org.jruby.evaluator;
 
 import org.jruby.Ruby;
 import org.jruby.RubyArray;
-import org.jruby.RubyBinding;
 import org.jruby.RubyLocalJumpError;
 import org.jruby.RubyModule;
-import org.jruby.RubyProc;
 import org.jruby.RubyString;
 import org.jruby.ast.ArrayNode;
 import org.jruby.ast.BlockPassNode;
@@ -68,8 +66,8 @@ public class ASTInterpreter {
         
         try {
             return node.interpret(runtime, context, self, block);
-        } catch (StackOverflowError sfe) {
-            throw runtime.newSystemStackError("stack level too deep");
+        } catch (StackOverflowError soe) {
+            throw runtime.newSystemStackError("stack level too deep", soe);
         }
     }
     
@@ -83,31 +81,9 @@ public class ASTInterpreter {
      * @param lineNumber is the line number to pretend we are starting from
      * @return An IRubyObject result from the evaluation
      */
-    public static IRubyObject evalWithBinding(ThreadContext context, IRubyObject src, IRubyObject scope, 
-            String file, int lineNumber) {
-        // both of these are ensured by the (very few) callers
-        assert !scope.isNil();
-        //assert file != null;
-
+    public static IRubyObject evalWithBinding(ThreadContext context, IRubyObject src, Binding binding) {
         Ruby runtime = src.getRuntime();
-        String savedFile = context.getFile();
-        int savedLine = context.getLine();
-
-        if (!(scope instanceof RubyBinding)) {
-            if (scope instanceof RubyProc) {
-                scope = ((RubyProc) scope).binding();
-            } else {
-                // bomb out, it's not a binding or a proc
-                throw runtime.newTypeError("wrong argument type " + scope.getMetaClass() + " (expected Proc/Binding)");
-            }
-        }
-
-        Binding binding = ((RubyBinding)scope).getBinding();
         DynamicScope evalScope = binding.getDynamicScope().getEvalScope();
-
-        // If no explicit file passed in we will use the bindings location
-        if (file == null) file = binding.getFrame().getFile();
-        if (lineNumber == -1) lineNumber = binding.getFrame().getLine();
         
         // FIXME:  This determine module is in a strange location and should somehow be in block
         evalScope.getStaticScope().determineModule();
@@ -117,21 +93,17 @@ public class ASTInterpreter {
             // Binding provided for scope, use it
             IRubyObject newSelf = binding.getSelf();
             RubyString source = src.convertToString();
-            Node node = runtime.parseEval(source.getByteList(), file, evalScope, lineNumber);
+            Node node = runtime.parseEval(source.getByteList(), binding.getFile(), evalScope, binding.getLine());
 
             return node.interpret(runtime, context, newSelf, binding.getFrame().getBlock());
         } catch (JumpException.BreakJump bj) {
             throw runtime.newLocalJumpError(RubyLocalJumpError.Reason.BREAK, (IRubyObject)bj.getValue(), "unexpected break");
         } catch (JumpException.RedoJump rj) {
             throw runtime.newLocalJumpError(RubyLocalJumpError.Reason.REDO, (IRubyObject)rj.getValue(), "unexpected redo");
-        } catch (StackOverflowError sfe) {
-            throw runtime.newSystemStackError("stack level too deep");            
+        } catch (StackOverflowError soe) {
+            throw runtime.newSystemStackError("stack level too deep", soe);
         } finally {
             context.postEvalWithBinding(binding, lastFrame);
-
-            // restore position
-            context.setFile(savedFile);
-            context.setLine(savedLine);
         }
     }
 
@@ -177,8 +149,8 @@ public class ASTInterpreter {
             return node.interpret(runtime, context, self, Block.NULL_BLOCK);
         } catch (JumpException.BreakJump bj) {
             throw runtime.newLocalJumpError(RubyLocalJumpError.Reason.BREAK, (IRubyObject)bj.getValue(), "unexpected break");
-        } catch (StackOverflowError sfe) {
-            throw runtime.newSystemStackError("stack level too deep");
+        } catch (StackOverflowError soe) {
+            throw runtime.newSystemStackError("stack level too deep", soe);
         } finally {
             // restore position
             context.setFile(savedFile);
@@ -289,7 +261,10 @@ public class ASTInterpreter {
     public static RubyModule getClassVariableBase(ThreadContext context, Ruby runtime) {
         StaticScope scope = context.getCurrentScope().getStaticScope();
         RubyModule rubyClass = scope.getModule();
-        if (rubyClass.isSingleton() || rubyClass == runtime.getDummy()) {
+        while (rubyClass.isSingleton() || rubyClass == runtime.getDummy()) {
+            // We ran out of scopes to check
+            if (scope == null) return null;
+
             scope = scope.getPreviousCRefScope();
             rubyClass = scope.getModule();
             if (scope.getPreviousCRefScope() == null) {
