@@ -240,6 +240,32 @@ abstract public class AbstractMemory extends RubyObject {
         return context.getRuntime().newFixnum(typeSize);
     }
 
+    /**
+     * Writes a 8 bit signed integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_char" } , required = 1)
+    public IRubyObject write_char(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putByte(0, Util.int8Value(value));
+
+        return this;
+    }
+
+    /**
+     * Writes a 8 bit signed integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_int8", "put_char" } , required = 1)
+    public IRubyObject put_int8(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putByte(0, Util.int8Value(value));
+
+        return this;
+    }
+
     /**
      * Writes a 8 bit signed integer value to the memory area.
      *
@@ -253,6 +279,26 @@ abstract public class AbstractMemory extends RubyObject {
 
         return this;
     }
+    
+    /**
+     * Reads an 8 bit signed integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_char" })
+    public IRubyObject read_char(ThreadContext context) {
+        return Util.newSigned8(context.getRuntime(), getMemoryIO().getByte(0));
+    }
+
+    /**
+     * Reads an 8 bit signed integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_int8", "get_char" })
+    public IRubyObject get_int8(ThreadContext context) {
+        return Util.newSigned8(context.getRuntime(), getMemoryIO().getByte(0));
+    }
 
     /**
      * Reads an 8 bit signed integer value from the memory address.
@@ -264,7 +310,31 @@ abstract public class AbstractMemory extends RubyObject {
     public IRubyObject get_int8(ThreadContext context, IRubyObject offset) {
         return Util.newSigned8(context.getRuntime(), getMemoryIO().getByte(getOffset(offset)));
     }
-    
+
+    /**
+     * Writes a 8 bit unsigned integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_uchar" }, required = 1)
+    public IRubyObject write_uchar(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putByte(0, (byte) Util.uint8Value(value));
+        return this;
+    }
+
+    /**
+     * Writes a 8 bit unsigned integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_uint8", "put_uchar" }, required = 1)
+    public IRubyObject put_uint8(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putByte(0, (byte) Util.uint8Value(value));
+        return this;
+    }
+
     /**
      * Writes a 8 bit unsigned integer value to the memory area.
      *
@@ -278,6 +348,26 @@ abstract public class AbstractMemory extends RubyObject {
         return this;
     }
     
+    /**
+     * Reads an 8 bit unsigned integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_uchar" })
+    public IRubyObject read_uchar(ThreadContext context) {
+        return Util.newUnsigned8(context.getRuntime(), getMemoryIO().getByte(0));
+    }
+
+    /**
+     * Reads an 8 bit unsigned integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_uint8", "get_uchar" })
+    public IRubyObject get_uint8(ThreadContext context) {
+        return Util.newUnsigned8(context.getRuntime(), getMemoryIO().getByte(0));
+    }
+
     /**
      * Reads an 8 bit unsigned integer value from the memory address.
      *
@@ -289,6 +379,32 @@ abstract public class AbstractMemory extends RubyObject {
         return Util.newUnsigned8(context.getRuntime(), getMemoryIO().getByte(getOffset(offset)));
     }
 
+    /**
+     * Writes a 16 bit signed integer value to the memory address.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_short" }, required = 1)
+    public IRubyObject write_short(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putShort(0, Util.int16Value(value));
+
+        return this;
+    }
+
+    /**
+     * Writes a 16 bit signed integer value to the memory address.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_int16", "put_short" }, required = 1)
+    public IRubyObject put_int16(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putShort(0, Util.int16Value(value));
+
+        return this;
+    }
+
     /**
      * Writes a 16 bit signed integer value to the memory address.
      *
@@ -303,6 +419,26 @@ abstract public class AbstractMemory extends RubyObject {
         return this;
     }
 
+    /**
+     * Reads a 16 bit signed integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_short" })
+    public IRubyObject read_short(ThreadContext context) {
+        return Util.newSigned16(context.getRuntime(), getMemoryIO().getShort(0));
+    }
+
+    /**
+     * Reads a 16 bit signed integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_int16", "get_short" })
+    public IRubyObject get_int16(ThreadContext context) {
+        return Util.newSigned16(context.getRuntime(), getMemoryIO().getShort(0));
+    }
+
     /**
      * Reads a 16 bit signed integer value from the memory address.
      *
@@ -314,6 +450,32 @@ abstract public class AbstractMemory extends RubyObject {
         return Util.newSigned16(context.getRuntime(), getMemoryIO().getShort(getOffset(offset)));
     }
     
+    /**
+     * Writes a 16 bit unsigned integer value to the memory address.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_ushort" }, required = 1)
+    public IRubyObject write_ushort(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putShort(0, (short) Util.uint16Value(value));
+
+        return this;
+    }
+
+    /**
+     * Writes a 16 bit unsigned integer value to the memory address.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_uint16", "put_ushort" }, required = 1)
+    public IRubyObject put_uint16(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putShort(0, (short) Util.uint16Value(value));
+
+        return this;
+    }
+
     /**
      * Writes a 16 bit unsigned integer value to the memory address.
      *
@@ -328,6 +490,26 @@ abstract public class AbstractMemory extends RubyObject {
         return this;
     }
 
+    /**
+     * Reads a 16 bit unsigned integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_ushort" })
+    public IRubyObject read_ushort(ThreadContext context) {
+        return Util.newUnsigned16(context.getRuntime(), getMemoryIO().getShort(0));
+    }
+
+    /**
+     * Reads a 16 bit unsigned integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_uint16", "get_ushort" })
+    public IRubyObject get_uint16(ThreadContext context) {
+        return Util.newUnsigned16(context.getRuntime(), getMemoryIO().getShort(0));
+    }
+
     /**
      * Reads a 16 bit unsigned integer value from the memory address.
      *
@@ -339,6 +521,32 @@ abstract public class AbstractMemory extends RubyObject {
         return Util.newUnsigned16(context.getRuntime(), getMemoryIO().getShort(getOffset(offset)));
     }
 
+    /**
+     * Writes a 32 bit signed integer value to the memory address.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_int" })
+    public IRubyObject write_int(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putInt(0, Util.int32Value(value));
+
+        return this;
+    }
+
+    /**
+     * Writes a 32 bit signed integer value to the memory address.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_int32", "put_int" })
+    public IRubyObject put_int32(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putInt(0, Util.int32Value(value));
+
+        return this;
+    }
+
     /**
      * Writes a 32 bit signed integer value to the memory address.
      *
@@ -353,6 +561,26 @@ abstract public class AbstractMemory extends RubyObject {
         return this;
     }
 
+    /**
+     * Reads a 32 bit signed integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_int" })
+    public IRubyObject read_int(ThreadContext context) {
+        return Util.newSigned32(context.getRuntime(), getMemoryIO().getInt(0));
+    }
+
+    /**
+     * Reads a 32 bit signed integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_int32", "get_int" })
+    public IRubyObject get_int32(ThreadContext context) {
+        return Util.newSigned32(context.getRuntime(), getMemoryIO().getInt(0));
+    }
+
     /**
      * Reads a 32 bit signed integer value from the memory address.
      *
@@ -363,7 +591,33 @@ abstract public class AbstractMemory extends RubyObject {
     public IRubyObject get_int32(ThreadContext context, IRubyObject offset) {
         return Util.newSigned32(context.getRuntime(), getMemoryIO().getInt(getOffset(offset)));
     }
-    
+
+    /**
+     * Writes an 32 bit unsigned integer value to the memory address.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_uint" }, required = 1)
+    public IRubyObject write_uint(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putInt(0, (int) Util.uint32Value(value));
+
+        return this;
+    }
+
+    /**
+     * Writes an 32 bit unsigned integer value to the memory address.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_uint32", "put_uint" }, required = 1)
+    public IRubyObject put_uint32(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putInt(0, (int) Util.uint32Value(value));
+
+        return this;
+    }
+
     /**
      * Writes an 32 bit unsigned integer value to the memory address.
      *
@@ -378,6 +632,26 @@ abstract public class AbstractMemory extends RubyObject {
         return this;
     }
 
+    /**
+     * Reads a 32 bit unsigned integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_uint" })
+    public IRubyObject read_uint(ThreadContext context) {
+        return Util.newUnsigned32(context.getRuntime(), getMemoryIO().getInt(0));
+    }
+
+    /**
+     * Reads a 32 bit unsigned integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_uint32", "get_uint" })
+    public IRubyObject get_uint32(ThreadContext context) {
+        return Util.newUnsigned32(context.getRuntime(), getMemoryIO().getInt(0));
+    }
+
     /**
      * Reads a 32 bit unsigned integer value from the memory address.
      *
@@ -388,6 +662,32 @@ abstract public class AbstractMemory extends RubyObject {
     public IRubyObject get_uint32(ThreadContext context, IRubyObject offset) {
         return Util.newUnsigned32(context.getRuntime(), getMemoryIO().getInt(getOffset(offset)));
     }
+    
+    /**
+     * Writes a 64 bit integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_long_long" }, required = 1)
+    public IRubyObject write_long_long(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putLong(0, Util.int64Value(value));
+
+        return this;
+    }
+
+    /**
+     * Writes a 64 bit integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_int64", "put_long_long" }, required = 1)
+    public IRubyObject put_int64(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putLong(0, Util.int64Value(value));
+
+        return this;
+    }
 
     /**
      * Writes a 64 bit integer value to the memory area.
@@ -403,6 +703,26 @@ abstract public class AbstractMemory extends RubyObject {
         return this;
     }
     
+    /**
+     * Reads a 64 bit integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_long_long" })
+    public IRubyObject read_long_long(ThreadContext context) {
+        return Util.newSigned64(context.getRuntime(), getMemoryIO().getLong(0));
+    }
+
+    /**
+     * Reads a 64 bit integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_int64", "get_long_long" })
+    public IRubyObject get_int64(ThreadContext context) {
+        return Util.newSigned64(context.getRuntime(), getMemoryIO().getLong(0));
+    }
+
     /**
      * Reads a 64 bit integer value from the memory address.
      *
@@ -414,6 +734,32 @@ abstract public class AbstractMemory extends RubyObject {
         return Util.newSigned64(context.getRuntime(), getMemoryIO().getLong(getOffset(offset)));
     }
 
+    /**
+     * Writes a 64 bit unsigned integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_ulong_long" }, required = 1)
+    public IRubyObject write_ulong_long(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putLong(0, Util.uint64Value(value));
+
+        return this;
+    }
+
+    /**
+     * Writes a 64 bit unsigned integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_uint64", "put_ulong_long" }, required = 1)
+    public IRubyObject put_uint64(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putLong(0, Util.uint64Value(value));
+
+        return this;
+    }
+
     /**
      * Writes a 64 bit unsigned integer value to the memory area.
      *
@@ -428,6 +774,26 @@ abstract public class AbstractMemory extends RubyObject {
         return this;
     }
 
+    /**
+     * Reads a 64 bit unsigned integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_ulong_long" })
+    public IRubyObject read_ulong_long(ThreadContext context) {
+        return Util.newUnsigned64(context.getRuntime(), getMemoryIO().getLong(0));
+    }
+
+    /**
+     * Reads a 64 bit unsigned integer value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_uint64", "get_ulong_long" })
+    public IRubyObject get_uint64(ThreadContext context) {
+        return Util.newUnsigned64(context.getRuntime(), getMemoryIO().getLong(0));
+    }
+
     /**
      * Reads a 64 bit unsigned integer value from the memory address.
      *
@@ -439,6 +805,31 @@ abstract public class AbstractMemory extends RubyObject {
         return Util.newUnsigned64(context.getRuntime(), getMemoryIO().getLong(getOffset(offset)));
     }
 
+    /**
+     * Writes a C long integer value to the memory area. This version is added
+     * to support the "write_long" alias for the single-arg "put_long".
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = "write_long", required = 1)
+    public IRubyObject write_long(ThreadContext context, IRubyObject value) {
+        return put_long(context, value);
+    }
+
+    /**
+     * Writes a C long integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = "put_long", required = 1)
+    public IRubyObject put_long(ThreadContext context, IRubyObject value) {
+        return Platform.getPlatform().longSize() == 32
+                ? put_int32(context, value)
+                : put_int64(context, value);
+    }
+
     /**
      * Writes a C long integer value to the memory area.
      *
@@ -452,7 +843,28 @@ abstract public class AbstractMemory extends RubyObject {
                 ? put_int32(context, offset, value)
                 : put_int64(context, offset, value);
     }
-    
+
+    /**
+     * Reads a C long integer value from the memory area.
+     *
+     * @return The value read.
+     */
+    @JRubyMethod(name = { "read_long" })
+    public IRubyObject read_long(ThreadContext context) {
+        return get_long(context);
+    }
+
+    /**
+     * Reads a C long integer value from the memory area.
+     *
+     * @return The value read.
+     */
+    @JRubyMethod(name = { "get_long" })
+    public IRubyObject get_long(ThreadContext context) {
+        return Platform.getPlatform().longSize() == 32
+                ? get_int32(context) : get_int64(context);
+    }
+
     /**
      * Reads a C long integer value from the memory area.
      *
@@ -466,6 +878,19 @@ abstract public class AbstractMemory extends RubyObject {
                 : get_int64(context, offset);
     }
     
+    /**
+     * Writes a C long integer value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_ulong", "write_ulong" }, required = 1)
+    public IRubyObject put_ulong(ThreadContext context, IRubyObject value) {
+        return Platform.getPlatform().longSize() == 32
+                ? put_uint32(context, value)
+                : put_uint64(context, value);
+    }
+
     /**
      * Writes a C long integer value to the memory area.
      *
@@ -479,7 +904,29 @@ abstract public class AbstractMemory extends RubyObject {
                 ? put_uint32(context, offset, value)
                 : put_uint64(context, offset, value);
     }
+
     
+    /**
+     * Reads a C unsigned long integer value from the memory area.
+     *
+     * @return The value read.
+     */
+    @JRubyMethod(name = { "read_ulong" })
+    public IRubyObject read_ulong(ThreadContext context) {
+        return get_ulong(context);
+    }
+
+    /**
+     * Reads a C unsigned long integer value from the memory area.
+     *
+     * @return The value read.
+     */
+    @JRubyMethod(name = { "get_ulong", "read_ulong" })
+    public IRubyObject get_ulong(ThreadContext context) {
+        return Platform.getPlatform().longSize() == 32
+                ? get_uint32(context) : get_uint64(context);
+    }
+
     /**
      * Reads a C unsigned long integer value from the memory area.
      *
@@ -492,6 +939,33 @@ abstract public class AbstractMemory extends RubyObject {
                 ? get_uint32(context, offset)
                 : get_uint64(context, offset);
     }
+
+    /**
+     * Writes an 32 bit floating point value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_float" }, required = 1)
+    public IRubyObject write_float(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putFloat(0, Util.floatValue(value));
+
+        return this;
+    }
+
+    /**
+     * Writes an 32 bit floating point value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_float32", "put_float" }, required = 1)
+    public IRubyObject put_float32(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putFloat(0, Util.floatValue(value));
+
+        return this;
+    }
+
     /**
      * Writes an 32 bit floating point value to the memory area.
      *
@@ -506,6 +980,26 @@ abstract public class AbstractMemory extends RubyObject {
         return this;
     }
 
+    /**
+     * Reads a 32 bit floating point value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_float" })
+    public IRubyObject read_float(ThreadContext context) {
+        return RubyFloat.newFloat(context.getRuntime(), getMemoryIO().getFloat(0));
+    }
+
+    /**
+     * Reads a 32 bit floating point value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_float32", "get_float" })
+    public IRubyObject get_float32(ThreadContext context) {
+        return RubyFloat.newFloat(context.getRuntime(), getMemoryIO().getFloat(0));
+    }
+
     /**
      * Reads a 32 bit floating point value from the memory address.
      *
@@ -517,6 +1011,32 @@ abstract public class AbstractMemory extends RubyObject {
         return RubyFloat.newFloat(context.getRuntime(), getMemoryIO().getFloat(getOffset(offset)));
     }
     
+    /**
+     * Writes an 64 bit floating point value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "write_double" }, required = 1)
+    public IRubyObject write_double(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putDouble(0, Util.doubleValue(value));
+
+        return this;
+    }
+
+    /**
+     * Writes an 64 bit floating point value to the memory area.
+     *
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_float64", "put_double" }, required = 1)
+    public IRubyObject put_float64(ThreadContext context, IRubyObject value) {
+        getMemoryIO().putDouble(0, Util.doubleValue(value));
+
+        return this;
+    }
+
     /**
      * Writes an 64 bit floating point value to the memory area.
      *
@@ -531,6 +1051,26 @@ abstract public class AbstractMemory extends RubyObject {
         return this;
     }
 
+    /**
+     * Reads a 64 bit floating point value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "read_double" })
+    public IRubyObject read_double(ThreadContext context) {
+        return RubyFloat.newFloat(context.getRuntime(), getMemoryIO().getDouble(0));
+    }
+
+    /**
+     * Reads a 64 bit floating point value from the memory address.
+     *
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_float64", "get_double" })
+    public IRubyObject get_float64(ThreadContext context) {
+        return RubyFloat.newFloat(context.getRuntime(), getMemoryIO().getDouble(0));
+    }
+
     /**
      * Reads a 64 bit floating point value from the memory address.
      *
@@ -860,7 +1400,7 @@ abstract public class AbstractMemory extends RubyObject {
      */
     @JRubyMethod(name = { "put_array_of_float64", "put_array_of_double" }, required = 2)
     public IRubyObject put_array_of_float64(ThreadContext context, IRubyObject offset, IRubyObject arrParam) {
-        MemoryUtil.putArrayOfFloat32(context.getRuntime(), getMemoryIO(), getOffset(offset), checkArray(arrParam));
+        MemoryUtil.putArrayOfFloat64(context.getRuntime(), getMemoryIO(), getOffset(offset), checkArray(arrParam));
 
         return this;
     }
@@ -878,12 +1418,17 @@ abstract public class AbstractMemory extends RubyObject {
                 : MemoryUtil.getTaintedString(context.getRuntime(), getMemoryIO(), 0);
     }
 
-    @JRubyMethod(name = "get_string", required = 1)
+    @JRubyMethod(name = "get_string")
+    public IRubyObject get_string(ThreadContext context) {
+        return MemoryUtil.getTaintedString(context.getRuntime(), getMemoryIO(), 0);
+    }
+
+    @JRubyMethod(name = "get_string")
     public IRubyObject get_string(ThreadContext context, IRubyObject offArg) {
         return MemoryUtil.getTaintedString(context.getRuntime(), getMemoryIO(), getOffset(offArg));
     }
 
-    @JRubyMethod(name = "get_string", required = 2)
+    @JRubyMethod(name = "get_string")
     public IRubyObject get_string(ThreadContext context, IRubyObject offArg, IRubyObject lenArg) {
         return MemoryUtil.getTaintedString(context.getRuntime(), getMemoryIO(),
                 getOffset(offArg), Util.int32Value(lenArg));
@@ -930,7 +1475,7 @@ abstract public class AbstractMemory extends RubyObject {
     public IRubyObject put_string(ThreadContext context, IRubyObject offArg, IRubyObject strArg) {
         long off = getOffset(offArg);
         ByteList bl = strArg.convertToString().getByteList();
-        getMemoryIO().putZeroTerminatedByteArray(off, bl.unsafeBytes(), bl.begin(), bl.length());
+        getMemoryIO().putZeroTerminatedByteArray(off, bl.getUnsafeBytes(), bl.begin(), bl.length());
         return this;
     }
 
@@ -952,9 +1497,21 @@ abstract public class AbstractMemory extends RubyObject {
         if (len < 0 || len > (bl.length() - idx)) {
             throw context.getRuntime().newRangeError("Invalid length");
         }
-        getMemoryIO().put(off, bl.unsafeBytes(), bl.begin() + idx, len);
+        getMemoryIO().put(off, bl.getUnsafeBytes(), bl.begin() + idx, len);
         return this;
     }
+
+
+    @JRubyMethod(name = { "read_pointer" })
+    public IRubyObject read_pointer(ThreadContext context) {
+        return getPointer(context.getRuntime(), 0);
+    }
+
+    @JRubyMethod(name = { "get_pointer" })
+    public IRubyObject get_pointer(ThreadContext context) {
+        return getPointer(context.getRuntime(), 0);
+    }
+
     @JRubyMethod(name = "get_pointer", required = 1)
     public IRubyObject get_pointer(ThreadContext context, IRubyObject offset) {
         return getPointer(context.getRuntime(), getOffset(offset));
@@ -983,6 +1540,18 @@ abstract public class AbstractMemory extends RubyObject {
         }
     }
 
+    @JRubyMethod(name = { "write_pointer" })
+    public IRubyObject write_pointer(ThreadContext context, IRubyObject value) {
+        putPointer(context, 0, value);
+        return this;
+    }
+
+    @JRubyMethod(name = { "put_pointer" }, required = 1)
+    public IRubyObject put_pointer(ThreadContext context, IRubyObject value) {
+        putPointer(context, 0, value);
+        return this;
+    }
+
     @JRubyMethod(name = "put_pointer", required = 2)
     public IRubyObject put_pointer(ThreadContext context, IRubyObject offset, IRubyObject value) {
         putPointer(context, getOffset(offset), value);
@@ -1032,5 +1601,6 @@ abstract public class AbstractMemory extends RubyObject {
         return slice(context.getRuntime(), RubyNumeric.fix2long(value));
     }
     abstract protected AbstractMemory slice(Ruby runtime, long offset);
+    abstract protected AbstractMemory slice(Ruby runtime, long offset, long size);
     abstract protected Pointer getPointer(Ruby runtime, long offset);
 }
