@@ -7,9 +7,10 @@ import com.kenai.jffi.HeapInvocationBuffer;
 import com.kenai.jffi.InvocationBuffer;
 import com.kenai.jffi.Invoker;
 import com.kenai.jffi.ArrayFlags;
-import org.jruby.Ruby;
+import org.jruby.RubyArray;
 import org.jruby.RubyBoolean;
 import org.jruby.RubyHash;
+import org.jruby.RubyInteger;
 import org.jruby.RubyModule;
 import org.jruby.RubyNumeric;
 import org.jruby.RubyString;
@@ -24,12 +25,14 @@ import org.jruby.ext.ffi.MemoryPointer;
 import org.jruby.ext.ffi.NativeType;
 import org.jruby.ext.ffi.Platform;
 import org.jruby.ext.ffi.Pointer;
+import org.jruby.ext.ffi.Pointer;
 import org.jruby.ext.ffi.Struct;
 import org.jruby.ext.ffi.StructByValue;
 import org.jruby.ext.ffi.StructLayout;
 import org.jruby.ext.ffi.Type;
 import org.jruby.ext.ffi.Util;
 import org.jruby.internal.runtime.methods.DynamicMethod;
+import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
@@ -80,7 +83,8 @@ public final class DefaultMethodFactory {
         // This just applies to buffer/pointer types.
         //
         FastIntMethodFactory fastIntFactory = FastIntMethodFactory.getFactory();
-        boolean canBeFastInt = enums.isNil() && parameterTypes.length <= 3 && fastIntFactory.isFastIntResult(returnType);
+        boolean canBeFastInt = enums.isNil() && parameterTypes.length <= 3 
+                && fastIntFactory.isFastIntResult(returnType) && convention == CallingConvention.DEFAULT;
         for (int i = 0; canBeFastInt && i < parameterTypes.length; ++i) {
             if (!(parameterTypes[i] instanceof Type.Builtin) || marshallers[i].needsInvocationSession()) {
                 canBeFastInt = false;
@@ -187,6 +191,10 @@ public final class DefaultMethodFactory {
                 return Float64Invoker.INSTANCE;
             case STRING:
                 return StringInvoker.INSTANCE;
+
+            case STRPTR:
+                return StrptrInvoker.INSTANCE;
+
             default:
                 throw new IllegalArgumentException("Invalid return type: " + returnType);
         }
@@ -286,6 +294,10 @@ public final class DefaultMethodFactory {
                 return BufferMarshaller.OUT;
             case BUFFER_INOUT:
                 return BufferMarshaller.INOUT;
+
+            case WIN32PTR:
+                return BufferMarshaller.WIN32PTR;
+                
             default:
                 throw new IllegalArgumentException("Invalid parameter type: " + type);
         }
@@ -298,9 +310,9 @@ public final class DefaultMethodFactory {
      * Invokes the native function with no return type, and returns nil to ruby.
      */
     private static final class VoidInvoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
             invoker.invokeInt(function, args);
-            return runtime.getNil();
+            return context.getRuntime().getNil();
         }
         public static final FunctionInvoker INSTANCE = new VoidInvoker();
     }
@@ -310,8 +322,8 @@ public final class DefaultMethodFactory {
      * Returns a Boolean to ruby.
      */
     private static final class BooleanInvoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return runtime.newBoolean(invoker.invokeInt(function, args) != 0);
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return context.getRuntime().newBoolean((invoker.invokeInt(function, args) & 0xff) != 0);
         }
         public static final FunctionInvoker INSTANCE = new BooleanInvoker();
     }
@@ -326,9 +338,9 @@ public final class DefaultMethodFactory {
             this.returnType = returnType;
         }
 
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return returnType.callMethod(runtime.getCurrentContext(), "find",
-                    runtime.newFixnum(invoker.invokeInt(function, args)));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return returnType.callMethod(context, "find",
+                    context.getRuntime().newFixnum(invoker.invokeInt(function, args)));
         }
     }
 
@@ -337,8 +349,8 @@ public final class DefaultMethodFactory {
      * Returns a Fixnum to ruby.
      */
     private static final class Signed8Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return Util.newSigned8(runtime, invoker.invokeInt(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return Util.newSigned8(context.getRuntime(), (byte) invoker.invokeInt(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Signed8Invoker();
     }
@@ -348,8 +360,8 @@ public final class DefaultMethodFactory {
      * Returns a Fixnum to ruby.
      */
     private static final class Unsigned8Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return Util.newUnsigned8(runtime, invoker.invokeInt(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return Util.newUnsigned8(context.getRuntime(), (byte) invoker.invokeInt(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Unsigned8Invoker();
     }
@@ -359,8 +371,8 @@ public final class DefaultMethodFactory {
      * Returns a Fixnum to ruby.
      */
     private static final class Signed16Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return Util.newSigned16(runtime, invoker.invokeInt(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return Util.newSigned16(context.getRuntime(), (short) invoker.invokeInt(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Signed16Invoker();
     }
@@ -370,8 +382,8 @@ public final class DefaultMethodFactory {
      * Returns a Fixnum to ruby.
      */
     private static final class Unsigned16Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return Util.newUnsigned16(runtime, invoker.invokeInt(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return Util.newUnsigned16(context.getRuntime(), (short) invoker.invokeInt(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Unsigned16Invoker();
     }
@@ -380,8 +392,8 @@ public final class DefaultMethodFactory {
      * Returns a Fixnum to ruby.
      */
     private static final class Signed32Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return Util.newSigned32(runtime, invoker.invokeInt(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return Util.newSigned32(context.getRuntime(), invoker.invokeInt(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Signed32Invoker();
     }
@@ -391,8 +403,8 @@ public final class DefaultMethodFactory {
      * Returns a Fixnum to ruby.
      */
     private static final class Unsigned32Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return Util.newUnsigned32(runtime, invoker.invokeInt(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return Util.newUnsigned32(context.getRuntime(), invoker.invokeInt(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Unsigned32Invoker();
     }
@@ -402,8 +414,8 @@ public final class DefaultMethodFactory {
      * Returns a Fixnum to ruby.
      */
     private static final class Signed64Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return Util.newSigned64(runtime, invoker.invokeLong(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return Util.newSigned64(context.getRuntime(), invoker.invokeLong(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Signed64Invoker();
     }
@@ -413,8 +425,8 @@ public final class DefaultMethodFactory {
      * Returns a ruby Fixnum or Bignum.
      */
     private static final class Unsigned64Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return Util.newUnsigned64(runtime, invoker.invokeLong(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return Util.newUnsigned64(context.getRuntime(), invoker.invokeLong(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Unsigned64Invoker();
     }
@@ -424,8 +436,8 @@ public final class DefaultMethodFactory {
      * Returns a Float to ruby.
      */
     private static final class Float32Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return runtime.newFloat(invoker.invokeFloat(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return context.getRuntime().newFloat(invoker.invokeFloat(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Float32Invoker();
     }
@@ -435,8 +447,8 @@ public final class DefaultMethodFactory {
      * Returns a Float to ruby.
      */
     private static final class Float64Invoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return runtime.newFloat(invoker.invokeDouble(function, args));
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return context.getRuntime().newFloat(invoker.invokeDouble(function, args));
         }
         public static final FunctionInvoker INSTANCE = new Float64Invoker();
     }
@@ -446,9 +458,9 @@ public final class DefaultMethodFactory {
      * Returns a {@link MemoryPointer} to ruby.
      */
     private static final class PointerInvoker extends BaseInvoker {
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
             final long address = invoker.invokeAddress(function, args);
-            return new Pointer(runtime, NativeMemoryIO.wrap(runtime, address));
+            return new Pointer(context.getRuntime(), NativeMemoryIO.wrap(context.getRuntime(), address));
         }
         public static final FunctionInvoker INSTANCE = new PointerInvoker();
     }
@@ -458,10 +470,9 @@ public final class DefaultMethodFactory {
      * Returns a {@link RubyString} to ruby.
      */
     private static final class StringInvoker extends BaseInvoker {
-        private static final com.kenai.jffi.MemoryIO IO = com.kenai.jffi.MemoryIO.getInstance();
-
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return FFIUtil.getString(runtime, invoker.invokeAddress(function, args));
+        
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            return FFIUtil.getString(context.getRuntime(), invoker.invokeAddress(function, args));
         }
         public static final FunctionInvoker INSTANCE = new StringInvoker();
     }
@@ -471,15 +482,15 @@ public final class DefaultMethodFactory {
      * Returns a FFI::Struct instance to ruby.
      */
     private static final class StructByValueInvoker extends BaseInvoker {
-        private static final com.kenai.jffi.MemoryIO IO = com.kenai.jffi.MemoryIO.getInstance();
         private final StructByValue info;
 
         public StructByValueInvoker(StructByValue info) {
             this.info = info;
         }
 
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
-            return info.newStruct(runtime, invoker.invokeStruct(function, args), 0);
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            Buffer buf = new Buffer(context.getRuntime(), invoker.invokeStruct(function, args), 0, info.getStructLayout().getSize());
+            return info.getStructClass().newInstance(context, new IRubyObject[] { buf }, Block.NULL_BLOCK);
         }
     }
 
@@ -488,25 +499,38 @@ public final class DefaultMethodFactory {
      * Returns a {@link Invoker} to ruby.
      */
     private static final class CallbackInvoker extends BaseInvoker {
-        private static final com.kenai.jffi.MemoryIO IO = com.kenai.jffi.MemoryIO.getInstance();
         private final Type returnType;
         private final Type[] parameterTypes;
+        private final CallingConvention convention;
 
         public CallbackInvoker(CallbackInfo cbInfo) {
             this.returnType = cbInfo.getReturnType();
             this.parameterTypes = cbInfo.getParameterTypes();
+            this.convention = cbInfo.isStdcall() ? CallingConvention.STDCALL : CallingConvention.DEFAULT;
         }
         
 
-        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
             long address = invoker.invokeAddress(function, args);
             if (address == 0) {
-                return runtime.getNil();
+                return context.getRuntime().getNil();
             }
-            return new JFFIInvoker(runtime, address, returnType, parameterTypes);
+            return new JFFIInvoker(context.getRuntime(), address, returnType, parameterTypes, convention);
         }
     }
 
+    private static final class StrptrInvoker extends BaseInvoker {
+        public static final FunctionInvoker INSTANCE = new StrptrInvoker();
+
+        public final IRubyObject invoke(ThreadContext context, Function function, HeapInvocationBuffer args) {
+            final long address = invoker.invokeAddress(function, args);
+            return RubyArray.newArray(context.getRuntime(),
+                    FFIUtil.getString(context.getRuntime(), address),
+                    new Pointer(context.getRuntime(), NativeMemoryIO.wrap(context.getRuntime(), address)));
+        }
+
+    }
+
     /*------------------------------------------------------------------------*/
     static abstract class BaseMarshaller implements ParameterMarshaller {
         public boolean needsInvocationSession() {
@@ -544,13 +568,13 @@ public final class DefaultMethodFactory {
      */
     static final class BooleanMarshaller extends BaseMarshaller {
         public final void marshal(ThreadContext context, InvocationBuffer buffer, IRubyObject parameter) {
-            buffer.putInt(parameter.isTrue() ? 1 : 0);
+            buffer.putByte(parameter.isTrue() ? 1 : 0);
         }
         public void marshal(Invocation invocation, InvocationBuffer buffer, IRubyObject parameter) {
             if (!(parameter instanceof RubyBoolean)) {
                 throw invocation.getThreadContext().getRuntime().newTypeError("wrong argument type.  Expected true or false");
             }
-            buffer.putInt(parameter.isTrue() ? 1 : 0);
+            buffer.putByte(parameter.isTrue() ? 1 : 0);
         }
         public static final ParameterMarshaller INSTANCE = new BooleanMarshaller();
     }
@@ -692,9 +716,20 @@ public final class DefaultMethodFactory {
         static final ParameterMarshaller IN = new BufferMarshaller(ArrayFlags.IN);
         static final ParameterMarshaller OUT = new BufferMarshaller(ArrayFlags.OUT);
         static final ParameterMarshaller INOUT = new BufferMarshaller(ArrayFlags.IN | ArrayFlags.OUT);
+
+        /** A special case for implementation of the Win32API 'P' type */
+        static final ParameterMarshaller WIN32PTR = new BufferMarshaller(ArrayFlags.IN | ArrayFlags.OUT, true);
+
         private final int flags;
+        private final boolean acceptIntegerValues;
+
         public BufferMarshaller(int flags) {
+            this(flags, false);
+        }
+
+        public BufferMarshaller(int flags, boolean acceptIntegerValues) {
             this.flags = flags;
+            this.acceptIntegerValues = acceptIntegerValues;
         }
         private static final int bufferFlags(Buffer buffer) {
             int f = buffer.getInOutFlags();
@@ -733,7 +768,7 @@ public final class DefaultMethodFactory {
                 buffer.putAddress(0L);
             } else if (parameter instanceof RubyString) {
                 ByteList bl = ((RubyString) parameter).getByteList();
-                buffer.putArray(bl.unsafeBytes(), bl.begin(), bl.length(), flags | ArrayFlags.NULTERMINATE);
+                buffer.putArray(bl.getUnsafeBytes(), bl.begin(), bl.length(), flags | ArrayFlags.NULTERMINATE);
 
             } else if (parameter.respondsTo("to_ptr")) {
                 final int MAXRECURSE = 4;
@@ -753,6 +788,11 @@ public final class DefaultMethodFactory {
                     }
                     break;
                 }
+
+            } else if (parameter instanceof RubyInteger && acceptIntegerValues) {
+                
+                buffer.putAddress(((RubyInteger) parameter).getLongValue());
+
             } else {
                 throw context.getRuntime().newArgumentError("Invalid buffer/pointer parameter");
             }
@@ -771,7 +811,7 @@ public final class DefaultMethodFactory {
             if (parameter instanceof RubyString) {
                 Util.checkStringSafety(context.getRuntime(), parameter);
                 ByteList bl = ((RubyString) parameter).getByteList();
-                buffer.putArray(bl.unsafeBytes(), bl.begin(), bl.length(),
+                buffer.putArray(bl.getUnsafeBytes(), bl.begin(), bl.length(),
                         ArrayFlags.IN | ArrayFlags.NULTERMINATE);
             } else if (parameter.isNil()) {
                 buffer.putAddress(0);
@@ -802,27 +842,24 @@ public final class DefaultMethodFactory {
                         + parameter.getMetaClass().getName() + " (expected instance of FFI::Struct)");
             }
 
-            IRubyObject memory = ((Struct) parameter).getMemory();
-            if (!(memory instanceof AbstractMemory)) {
-                throw context.getRuntime().newTypeError("wrong struct memory type "
-                        + memory.getMetaClass().getName());
-            }
-
-            if (((AbstractMemory) memory).getSize() < layout.getSize()) {
+            final AbstractMemory memory = ((Struct) parameter).getMemory();
+            if (memory.getSize() < layout.getSize()) {
                 throw context.getRuntime().newArgumentError("struct memory too small for parameter");
             }
 
-            MemoryIO io = ((AbstractMemory) memory).getMemoryIO();
+            final MemoryIO io = memory.getMemoryIO();
             if (io instanceof DirectMemoryIO) {
                 if (io.isNull()) {
                     throw context.getRuntime().newRuntimeError("Cannot use a NULL pointer as a struct by value argument");
                 }
                 buffer.putStruct(((DirectMemoryIO) io).getAddress());
+
             } else if (io instanceof ArrayMemoryIO) {
                 ArrayMemoryIO aio = (ArrayMemoryIO) io;
                 buffer.putStruct(aio.array(), aio.arrayOffset());
+
             } else {
-                throw context.getRuntime().newRuntimeError("Invalid struct memory");
+                throw context.getRuntime().newRuntimeError("invalid struct memory");
             }
         }
 
