@@ -26,14 +26,14 @@ final class DefaultMethodTwoArg extends DefaultMethod {
             try {
                 m1.marshal(invocation, buffer, arg1);
                 m2.marshal(invocation, buffer, arg2);
-                return functionInvoker.invoke(context.getRuntime(), function, buffer);
+                return functionInvoker.invoke(context, function, buffer);
             } finally {
                 invocation.finish();
             }
         } else {
             m1.marshal(context, buffer, arg1);
             m2.marshal(context, buffer, arg2);
-            return functionInvoker.invoke(context.getRuntime(), function, buffer);
+            return functionInvoker.invoke(context, function, buffer);
         }
     }
 }
