@@ -27,45 +27,55 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby.ext.socket;
 
-import com.kenai.constantine.platform.SocketLevel;
-import com.kenai.constantine.platform.SocketOption;
-import static com.kenai.constantine.platform.Sock.*;
-import static com.kenai.constantine.platform.IPProto.*;
-import static com.kenai.constantine.platform.TCP.*;
+import static com.kenai.constantine.platform.IPProto.IPPROTO_TCP;
+import static com.kenai.constantine.platform.Sock.SOCK_DGRAM;
+import static com.kenai.constantine.platform.Sock.SOCK_STREAM;
+import static com.kenai.constantine.platform.TCP.TCP_NODELAY;
+
 import java.io.EOFException;
 import java.io.IOException;
-import java.nio.channels.Channel;
-import java.nio.channels.SocketChannel;
-import java.nio.channels.ServerSocketChannel;
-import java.nio.channels.DatagramChannel;
-import java.net.Socket;
-import java.net.ServerSocket;
 import java.net.DatagramSocket;
-import java.net.SocketAddress;
 import java.net.InetSocketAddress;
+import java.net.ServerSocket;
+import java.net.Socket;
+import java.net.SocketAddress;
+import java.nio.channels.Channel;
+import java.nio.channels.DatagramChannel;
+import java.nio.channels.ServerSocketChannel;
+import java.nio.channels.SocketChannel;
+
 import org.jruby.CompatVersion;
-import org.jruby.util.io.OpenFile;
 import org.jruby.Ruby;
+import org.jruby.RubyArray;
 import org.jruby.RubyBoolean;
 import org.jruby.RubyClass;
-import org.jruby.RubyNumeric;
+import org.jruby.RubyFixnum;
 import org.jruby.RubyIO;
+import org.jruby.RubyNumeric;
 import org.jruby.RubyString;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.util.ByteList;
+import org.jruby.util.Pack;
 import org.jruby.util.io.BadDescriptorException;
+import org.jruby.util.io.ChannelDescriptor;
 import org.jruby.util.io.ChannelStream;
 import org.jruby.util.io.ModeFlags;
-import org.jruby.util.io.ChannelDescriptor;
+import org.jruby.util.io.OpenFile;
+
+import com.kenai.constantine.platform.SocketLevel;
+import com.kenai.constantine.platform.SocketOption;
 
 /**
  * @author <a href="mailto:ola.bini@ki.se">Ola Bini</a>
  */
 @JRubyClass(name="BasicSocket", parent="IO")
 public class RubyBasicSocket extends RubyIO {
+    private static final ByteList FORMAT_SMALL_I = new ByteList(ByteList.plain("i"));
+
     private static ObjectAllocator BASICSOCKET_ALLOCATOR = new ObjectAllocator() {
         public IRubyObject allocate(Ruby runtime, RubyClass klass) {
             return new RubyBasicSocket(runtime, klass);
@@ -107,29 +117,24 @@ public class RubyBasicSocket extends RubyIO {
         if (context.getRuntime().getSafeLevel() >= 4 && isTaint()) {
             throw context.getRuntime().newSecurityError("Insecure: can't close");
         }
-        
+
+        if (!openFile.isWritable()) {
+            return context.getRuntime().getNil();
+        }
+
         if (openFile.getPipeStream() == null && openFile.isReadable()) {
             throw context.getRuntime().newIOError("closing non-duplex IO for writing");
         }
-        
+
         if (!openFile.isReadable()) {
             close();
         } else {
-            Channel socketChannel = openFile.getMainStream().getDescriptor().getChannel();
-            if (socketChannel instanceof SocketChannel
-                    || socketChannel instanceof DatagramChannel) {
-                try {
-                    asSocket().shutdownOutput();
-                } catch (IOException e) {
-                    throw context.getRuntime().newIOError(e.getMessage());
-                }
-            }
-            openFile.setPipeStream(null);
-            openFile.setMode(openFile.getMode() & ~OpenFile.WRITABLE);
+            // shutdown write
+            shutdownInternal(context, 1);
         }
         return context.getRuntime().getNil();
     }
-    
+
     @Override
     public IRubyObject close_read(ThreadContext context) {
         Ruby runtime = context.getRuntime();
@@ -137,23 +142,15 @@ public class RubyBasicSocket extends RubyIO {
             throw runtime.newSecurityError("Insecure: can't close");
         }
 
+        if (!openFile.isOpen()) {
+            throw context.getRuntime().newIOError("not opened for reading");
+        }
+
         if (!openFile.isWritable()) {
             close();
         } else {
-            if(openFile.getPipeStream() != null) {
-                Channel socketChannel = openFile.getMainStream().getDescriptor().getChannel();
-                if (socketChannel instanceof SocketChannel
-                    || socketChannel instanceof DatagramChannel) {
-                    try {
-                        asSocket().shutdownInput();
-                    } catch (IOException e) {
-                        throw runtime.newIOError(e.getMessage());
-                    }
-                }
-                openFile.setMainStream(openFile.getPipeStream());
-                openFile.setPipeStream(null);
-                openFile.setMode(openFile.getMode() & ~OpenFile.READABLE);
-            }
+            // shutdown read
+            shutdownInternal(context, 0);
         }
         return runtime.getNil();
     }
@@ -171,21 +168,24 @@ public class RubyBasicSocket extends RubyIO {
     public IRubyObject recv(ThreadContext context, IRubyObject[] args) {
         OpenFile openFile = getOpenFileChecked();
         try {
+            context.getThread().beforeBlockingCall();
             return RubyString.newString(context.getRuntime(), openFile.getMainStream().read(RubyNumeric.fix2int(args[0])));
         } catch (BadDescriptorException e) {
             throw context.getRuntime().newErrnoEBADFError();
         } catch (EOFException e) {
             // recv returns nil on EOF
             return context.getRuntime().getNil();
-    	} catch (IOException e) {
+        } catch (IOException e) {
             // All errors to sysread should be SystemCallErrors, but on a closed stream
             // Ruby returns an IOError.  Java throws same exception for all errors so
             // we resort to this hack...
             if ("Socket not open".equals(e.getMessage())) {
 	            throw context.getRuntime().newIOError(e.getMessage());
             }
-    	    throw context.getRuntime().newSystemCallError(e.getMessage());
-    	}
+            throw context.getRuntime().newSystemCallError(e.getMessage());
+        } finally {
+            context.getThread().afterBlockingCall();
+        }
     }
 
     protected InetSocketAddress getLocalSocket() {
@@ -262,10 +262,15 @@ public class RubyBasicSocket extends RubyIO {
             asSocket().setTcpNoDelay(asBoolean(val));
         }
     }
+
     private void setReuseAddr(IRubyObject val) throws IOException {
         Channel socketChannel = openFile.getMainStream().getDescriptor().getChannel();
-        if(socketChannel instanceof ServerSocketChannel) {
+        if (socketChannel instanceof ServerSocketChannel) {
             asServerSocket().setReuseAddress(asBoolean(val));
+        } else if (socketChannel instanceof SocketChannel) {
+            asSocket().setReuseAddress(asBoolean(val));
+        } else if (socketChannel instanceof DatagramChannel) {
+            asDatagramSocket().setReuseAddress(asBoolean(val));
         }
     }
 
@@ -292,11 +297,15 @@ public class RubyBasicSocket extends RubyIO {
     }
 
     private void setSndBuf(IRubyObject val) throws IOException {
-        Channel socketChannel = openFile.getMainStream().getDescriptor().getChannel();
-        if(socketChannel instanceof SocketChannel) {
-            asSocket().setSendBufferSize(asNumber(val));
-        } else if(socketChannel instanceof DatagramChannel) {
-            asDatagramSocket().setSendBufferSize(asNumber(val));
+        try {
+            Channel socketChannel = openFile.getMainStream().getDescriptor().getChannel();
+            if(socketChannel instanceof SocketChannel) {
+                asSocket().setSendBufferSize(asNumber(val));
+            } else if(socketChannel instanceof DatagramChannel) {
+                asDatagramSocket().setSendBufferSize(asNumber(val));
+            }
+        } catch (IllegalArgumentException iae) {
+            throw getRuntime().newErrnoEINVALError(iae.getMessage());
         }
     }
 
@@ -324,25 +333,29 @@ public class RubyBasicSocket extends RubyIO {
     }
 
     private int asNumber(IRubyObject val) {
-        if(val instanceof RubyNumeric) {
+        if (val instanceof RubyNumeric) {
             return RubyNumeric.fix2int(val);
-        } else {
+        } else if (val instanceof RubyBoolean) {
+            return val.isTrue() ? 1 : 0;
+        }
+        else {
             return stringAsNumber(val);
         }
     }
 
     private int stringAsNumber(IRubyObject val) {
-        String str = val.convertToString().toString();
-        int res = 0;
-        res += (str.charAt(0)<<24);
-        res += (str.charAt(1)<<16);
-        res += (str.charAt(2)<<8);
-        res += (str.charAt(3));
-        return res;
+        ByteList str = val.convertToString().getByteList();
+        IRubyObject res = Pack.unpack(getRuntime(), str, FORMAT_SMALL_I).entry(0);
+        
+        if (res.isNil()) {
+            throw getRuntime().newErrnoEINVALError();
+        }
+
+        return RubyNumeric.fix2int(res);
     }
 
     protected boolean asBoolean(IRubyObject val) {
-        if(val instanceof RubyString) {
+        if (val instanceof RubyString) {
             return stringAsNumber(val) != 0;
         } else if(val instanceof RubyNumeric) {
             return RubyNumeric.fix2int(val) != 0;
@@ -360,9 +373,16 @@ public class RubyBasicSocket extends RubyIO {
 
     private IRubyObject getLinger(Ruby runtime) throws IOException {
         Channel socketChannel = openFile.getMainStream().getDescriptor().getChannel();
-        return number(runtime,
-                      (socketChannel instanceof SocketChannel) ? asSocket().getSoLinger() : 0
-                      );
+
+        int linger = 0;
+        if (socketChannel instanceof SocketChannel) {
+            linger = asSocket().getSoLinger();
+            if (linger < 0) {
+                linger = 0;
+            }
+        }
+
+        return number(runtime, linger);
     }
 
     private IRubyObject getOOBInline(Ruby runtime) throws IOException {
@@ -391,9 +411,17 @@ public class RubyBasicSocket extends RubyIO {
 
     private IRubyObject getReuseAddr(Ruby runtime) throws IOException {
         Channel socketChannel = openFile.getMainStream().getDescriptor().getChannel();
-        return trueFalse(runtime,
-                         (socketChannel instanceof ServerSocketChannel) ? asServerSocket().getReuseAddress() : false
-                         );
+
+        boolean reuse = false;
+        if (socketChannel instanceof ServerSocketChannel) {
+            reuse = asServerSocket().getReuseAddress();
+        } else if (socketChannel instanceof SocketChannel) {
+            reuse = asSocket().getReuseAddress();
+        } else if (socketChannel instanceof DatagramChannel) {
+            reuse = asDatagramSocket().getReuseAddress();
+        }
+
+        return trueFalse(runtime, reuse);
     }
 
     private IRubyObject getTimeout(Ruby runtime) throws IOException {
@@ -423,15 +451,14 @@ public class RubyBasicSocket extends RubyIO {
     }
 
     private IRubyObject trueFalse(Ruby runtime, boolean val) {
-        return runtime.newString( val ? " \u0000\u0000\u0000" : "\u0000\u0000\u0000\u0000" );
+        return number(runtime, val ? 1 : 0);
     }
 
-    private IRubyObject number(Ruby runtime, long s) {
-        StringBuilder result = new StringBuilder();
-        result.append((char) ((s>>24) &0xff)).append((char) ((s>>16) &0xff));
-        result.append((char) ((s >> 8) & 0xff)).append((char) (s & 0xff));
-        return runtime.newString(result.toString());
+    private static IRubyObject number(Ruby runtime, int s) {
+        RubyArray array = runtime.newArray(runtime.newFixnum(s));
+        return Pack.pack(runtime, array, FORMAT_SMALL_I);
     }
+
     @Deprecated
     public IRubyObject getsockopt(IRubyObject lev, IRubyObject optname) {
         return getsockopt(getRuntime().getCurrentContext(), lev, optname);
@@ -569,7 +596,7 @@ public class RubyBasicSocket extends RubyIO {
     public IRubyObject getsockname(ThreadContext context) {
         SocketAddress sock = getLocalSocket();
         if(null == sock) {
-            throw context.getRuntime().newIOError("Not Supported");
+            return RubySocket.pack_sockaddr_in(context, null, 0, "0.0.0.0");
         }
         return context.getRuntime().newString(sock.toString());
     }
@@ -591,18 +618,57 @@ public class RubyBasicSocket extends RubyIO {
         if (context.getRuntime().getSafeLevel() >= 4 && tainted_p(context).isFalse()) {
             throw context.getRuntime().newSecurityError("Insecure: can't shutdown socket");
         }
-        
+
         int how = 2;
         if (args.length > 0) {
             how = RubyNumeric.fix2int(args[0]);
         }
-        if (how < 0 || 2 < how) {
+        return shutdownInternal(context, how);
+    }
+
+    private IRubyObject shutdownInternal(ThreadContext context, int how) {
+        Channel socketChannel;
+        switch (how) {
+        case 0:
+            socketChannel = openFile.getMainStream().getDescriptor().getChannel();
+            try {
+                if (socketChannel instanceof SocketChannel
+                        || socketChannel instanceof DatagramChannel) {
+                    asSocket().shutdownInput();
+                } else if (socketChannel instanceof Shutdownable) {
+                    ((Shutdownable)socketChannel).shutdownInput();
+                }
+            } catch (IOException e) {
+                throw context.getRuntime().newIOError(e.getMessage());
+            }
+            if(openFile.getPipeStream() != null) {
+                openFile.setMainStream(openFile.getPipeStream());
+                openFile.setPipeStream(null);
+            }
+            openFile.setMode(openFile.getMode() & ~OpenFile.READABLE);
+            return RubyFixnum.zero(context.getRuntime());
+        case 1:
+            socketChannel = openFile.getMainStream().getDescriptor().getChannel();
+            try {
+                if (socketChannel instanceof SocketChannel
+                        || socketChannel instanceof DatagramChannel) {
+                    asSocket().shutdownOutput();
+                } else if (socketChannel instanceof Shutdownable) {
+                    ((Shutdownable)socketChannel).shutdownOutput();
+                }
+            } catch (IOException e) {
+                throw context.getRuntime().newIOError(e.getMessage());
+            }
+            openFile.setPipeStream(null);
+            openFile.setMode(openFile.getMode() & ~OpenFile.WRITABLE);
+            return RubyFixnum.zero(context.getRuntime());
+        case 2:
+            shutdownInternal(context, 0);
+            shutdownInternal(context, 1);
+            return RubyFixnum.zero(context.getRuntime());
+        default:
             throw context.getRuntime().newArgumentError("`how' should be either 0, 1, 2");
         }
-        if (how != 2) {
-            throw context.getRuntime().newNotImplementedError("Shutdown currently only works with how=2");
-        }
-        return close();
     }
 
     protected boolean doNotReverseLookup(ThreadContext context) {
