@@ -31,13 +31,13 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby.internal.runtime;
 
-import edu.emory.mathcs.backport.java.util.concurrent.ConcurrentHashMap;
-import java.util.HashMap;
-import java.util.Iterator;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.Map;
+import java.util.Set;
 
 import org.jruby.Ruby;
 import org.jruby.RubyProc;
+import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.runtime.IAccessor;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -48,7 +48,7 @@ import org.jruby.runtime.builtin.IRubyObject;
  */
 public class GlobalVariables {
     private Ruby runtime;
-    private Map globalVariables = new ConcurrentHashMap();
+    private Map<String, GlobalVariable> globalVariables = new ConcurrentHashMap<String, GlobalVariable>();
 
     public GlobalVariables(Ruby runtime) {
         this.runtime = runtime;
@@ -97,7 +97,7 @@ public class GlobalVariables {
         GlobalVariable variable = (GlobalVariable)globalVariables.get(name);
 
         if (variable != null && oldVariable != variable && variable.isTracing()) {
-            throw new RaiseException(runtime, runtime.getClass("RuntimeError"), "can't alias in tracer", false);
+            throw new RaiseException(runtime, runtime.getRuntimeError(), "can't alias in tracer", false);
         }
 
         globalVariables.put(name, oldVariable);
@@ -108,10 +108,11 @@ public class GlobalVariables {
 	    assert name.startsWith("$");
 	
 	    GlobalVariable variable = (GlobalVariable)globalVariables.get(name);
-	    if (variable != null) {
-	        return variable.getAccessor().getValue();
+	    if (variable != null) return variable.getAccessor().getValue();
+
+	    if (runtime.isVerbose()) {
+	        runtime.getWarnings().warning(ID.GLOBAL_NOT_INITIALIZED, "global variable `" + name + "' not initialized", name);
 	    }
-		runtime.getWarnings().warning("global variable `" + name + "' not initialized");
 		return runtime.getNil();
 	}
 
@@ -128,7 +129,11 @@ public class GlobalVariables {
         variable.trace(value);
         return result;
     }
-    
+
+    public IRubyObject clear(String name) {
+        return set(name, runtime.getNil());
+    }
+
     public void setTraceVar(String name, RubyProc proc) {
         assert name != null;
         assert name.startsWith("$");
@@ -158,8 +163,8 @@ public class GlobalVariables {
         }
     }
 
-    public Iterator getNames() {
-        return globalVariables.keySet().iterator();
+    public Set<String> getNames() {
+        return globalVariables.keySet();
     }
 
     private GlobalVariable createIfNotDefined(String name) {
@@ -170,4 +175,14 @@ public class GlobalVariables {
         }
         return variable;
     }
+
+    private IRubyObject defaultSeparator;
+
+    public IRubyObject getDefaultSeparator() {
+        return defaultSeparator;
+    }
+
+    public void setDefaultSeparator(IRubyObject defaultSeparator) {
+        this.defaultSeparator = defaultSeparator;    
+    }
 }
