@@ -31,7 +31,6 @@ package org.jruby.internal.runtime.methods;
 
 import org.jruby.RubyModule;
 import org.jruby.RubyUnboundMethod;
-import org.jruby.internal.runtime.JumpTarget;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
@@ -41,7 +40,7 @@ import org.jruby.runtime.builtin.IRubyObject;
  * 
  * @author jpetersen
  */
-public class MethodMethod extends DynamicMethod implements JumpTarget {
+public class MethodMethod extends DynamicMethod {
     private RubyUnboundMethod method;
 
     /**
