@@ -32,7 +32,6 @@ package org.jruby.internal.runtime.methods;
 
 import org.jruby.RubyModule;
 import org.jruby.RubyProc;
-import org.jruby.internal.runtime.JumpTarget;
 import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
@@ -43,7 +42,7 @@ import org.jruby.runtime.builtin.IRubyObject;
  * 
  * @author jpetersen
  */
-public class ProcMethod extends DynamicMethod implements JumpTarget {
+public class ProcMethod extends DynamicMethod {
     private RubyProc proc;
 
     /**
@@ -60,8 +59,7 @@ public class ProcMethod extends DynamicMethod implements JumpTarget {
      * @see org.jruby.runtime.ICallable#call(Ruby, IRubyObject, String, IRubyObject[], boolean)
      */
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule klazz, String name, IRubyObject[] args, Block block) {
-        // FIXME: Not sure if we should pass block or not if 1.9
-        return proc.call(context, args, self, Block.NULL_BLOCK);
+        return proc.call(context, args, self, block);
     }
     
     public DynamicMethod dup() {
