@@ -47,7 +47,7 @@ public class SimpleCallbackMethod extends DynamicMethod {
     private Callback callback;
 
     public SimpleCallbackMethod(RubyModule implementationClass, Callback callback, Visibility visibility) {
-        super(implementationClass, visibility, CallConfiguration.NO_FRAME_NO_SCOPE);
+        super(implementationClass, visibility, CallConfiguration.FrameNoneScopeNone);
         this.callback = callback;
     }
 
