@@ -3,7 +3,6 @@ package org.jruby.java.invokers;
 import java.lang.reflect.Field;
 import org.jruby.RubyModule;
 import org.jruby.java.proxies.JavaProxy;
-import org.jruby.javasupport.JavaUtil;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
@@ -20,7 +19,9 @@ public class InstanceFieldSetter extends FieldMethodOne {
             Object newValue = arg.toJava(field.getType());
             field.set(proxy.getObject(), newValue);
         } catch (IllegalAccessException iae) {
-            throw context.getRuntime().newTypeError("illegal access setting variable: " + iae.getMessage());
+            throw context.getRuntime().newSecurityError(iae.getMessage());
+        } catch (IllegalArgumentException iae) {
+            throw context.getRuntime().newTypeError(iae.getMessage());
         }
         return arg;
     }
