@@ -460,7 +460,7 @@ public class Java implements Library {
                 // JRUBY-1000, fail early when attempting to subclass a final Java class;
                 // solved here by adding an exception-throwing "inherited"
                 if (Modifier.isFinal(c.getModifiers())) {
-                    proxyClass.getMetaClass().addMethod("inherited", new org.jruby.internal.runtime.methods.JavaMethod() {
+                    proxyClass.getMetaClass().addMethod("inherited", new org.jruby.internal.runtime.methods.JavaMethod(proxyClass, Visibility.PUBLIC) {
                         @Override
                         public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
                             throw context.getRuntime().newTypeError("can not extend final Java class: " + c.getCanonicalName());
@@ -1181,6 +1181,7 @@ public class Java implements Library {
             }
         }
         clazz.setReifiedClass(proxyImplClass);
+        clazz.setRubyClassAllocator(proxyImplClass);
 
         return proxyImplClass;
     }
