@@ -59,11 +59,15 @@ import org.jruby.javasupport.proxy.JavaProxyClass;
 import org.jruby.javasupport.proxy.JavaProxyMethod;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.util.CodegenUtils;
 
 @JRubyClass(name="Java::JavaMethod")
 public class JavaMethod extends JavaCallable {
     private final static boolean USE_HANDLES = RubyInstanceConfig.USE_GENERATED_HANDLES;
+    private final static boolean HANDLE_DEBUG = false;
     private final Method method;
+    private final Class boxedReturnType;
+    private final boolean isFinal;
     private final Handle handle;
     private final JavaUtil.JavaConverter returnConverter;
 
@@ -88,10 +92,58 @@ public class JavaMethod extends JavaCallable {
     public JavaMethod(Ruby runtime, Method method) {
         super(runtime, runtime.getJavaSupport().getJavaMethodClass(), method.getParameterTypes());
         this.method = method;
+        this.isFinal = Modifier.isFinal(method.getModifiers());
+        if (method.getReturnType().isPrimitive() && method.getReturnType() != void.class) {
+            this.boxedReturnType = CodegenUtils.getBoxType(method.getReturnType());
+        } else {
+            this.boxedReturnType = method.getReturnType();
+        }
 
         boolean methodIsPublic = Modifier.isPublic(method.getModifiers());
         boolean classIsPublic = Modifier.isPublic(method.getDeclaringClass().getModifiers());
 
+        // try to find a "totally" public version of the method using superclasses and interfaces
+        if (methodIsPublic && !classIsPublic) {
+            if (HANDLE_DEBUG) System.out.println("Method " + method + " is not on a public class, searching for a better one");
+            Method newMethod = method;
+            Class newClass = method.getDeclaringClass();
+
+            OUTER: while (newClass != null) {
+                // try class
+                try {
+                    if (HANDLE_DEBUG) System.out.println("Trying to find " + method + " on " + newClass);
+                    newMethod = newClass.getMethod(method.getName(), method.getParameterTypes());
+
+                    // got it; break if this class is public
+                    if (Modifier.isPublic(newMethod.getDeclaringClass().getModifiers())) {
+                        break;
+                    }
+                } catch (NoSuchMethodException nsme) {
+                }
+
+                // try interfaces
+                for (Class ifc : newClass.getInterfaces()) {
+                    try {
+                        if (HANDLE_DEBUG) System.out.println("Trying to find " + method + " on " + ifc);
+                        newMethod = ifc.getMethod(method.getName(), method.getParameterTypes());
+                        break OUTER;
+                    } catch (NoSuchMethodException nsme) {
+                    }
+                }
+
+                // go to superclass
+                newClass = newClass.getSuperclass();
+                newMethod = null;
+            }
+            
+            if (newMethod != null) {
+                if (HANDLE_DEBUG) System.out.println("Found a better method target: " + newMethod);
+                method = newMethod;
+                methodIsPublic = Modifier.isPublic(method.getModifiers());
+                classIsPublic = Modifier.isPublic(method.getDeclaringClass().getModifiers());
+            }
+        }
+
         // prepare a faster handle if handles are enabled and the method and class are public
         Handle tmpHandle = null;
         try {
@@ -109,14 +161,28 @@ public class JavaMethod extends JavaCallable {
         } catch (ClassNotFoundException cnfe) {
             tmpHandle = null;
         }
+
+        if (tmpHandle == null) {
+            if (HANDLE_DEBUG) System.out.println("did not use handle for " + method);
+        }
+        
         handle = tmpHandle;
 
         // Special classes like Collections.EMPTY_LIST are inner classes that are private but 
         // implement public interfaces.  Their methods are all public methods for the public 
-        // interface.  Let these public methods execute via setAccessible(true). 
-        if (methodIsPublic &&
-            !Modifier.isPublic(method.getDeclaringClass().getModifiers())) {
-            accessibleObject().setAccessible(true);
+        // interface.  Let these public methods execute via setAccessible(true).
+        if (JavaClass.CAN_SET_ACCESSIBLE) {
+            // we should be able to setAccessible ok...
+            try {
+                if (methodIsPublic &&
+                    !Modifier.isPublic(method.getDeclaringClass().getModifiers())) {
+                    accessibleObject().setAccessible(true);
+                }
+            } catch (SecurityException se) {
+                // we shouldn't get here if JavaClass.CAN_SET_ACCESSIBLE is doing
+                // what it should, so we warn.
+               runtime.getWarnings().warn("failed to setAccessible: " + accessibleObject() + ", exception follows: " + se.getMessage());
+            }
         }
         
         returnConverter = JavaUtil.getJavaConverter(method.getReturnType());
@@ -225,27 +291,31 @@ public class JavaMethod extends JavaCallable {
             return invokeWithExceptionHandling(method, null, arguments);
         }
 
-        Object javaInvokee = JavaUtil.unwrapJavaObject(getRuntime(), invokee, "invokee not a java object").getValue();
+        Object javaInvokee = null;
 
-        if (! method.getDeclaringClass().isInstance(javaInvokee)) {
-            throw getRuntime().newTypeError("invokee not instance of method's class (" +
-                                              "got" + javaInvokee.getClass().getName() + " wanted " +
-                                              method.getDeclaringClass().getName() + ")");
-        }
-        
-        //
-        // this test really means, that this is a ruby-defined subclass of a java class
-        //
-        if (javaInvokee instanceof InternalJavaProxy &&
-                // don't bother to check if final method, it won't
-                // be there (not generated, can't be!)
-                !Modifier.isFinal(method.getModifiers())) {
-            JavaProxyClass jpc = ((InternalJavaProxy) javaInvokee)
-                    .___getProxyClass();
-            JavaProxyMethod jpm;
-            if ((jpm = jpc.getMethod(method.getName(), parameterTypes)) != null &&
-                    jpm.hasSuperImplementation()) {
-                return invokeWithExceptionHandling(jpm.getSuperMethod(), javaInvokee, arguments);
+        if (!isStatic()) {
+            javaInvokee = JavaUtil.unwrapJavaValue(getRuntime(), invokee, "invokee not a java object");
+
+            if (! method.getDeclaringClass().isInstance(javaInvokee)) {
+                throw getRuntime().newTypeError("invokee not instance of method's class (" +
+                                                  "got" + javaInvokee.getClass().getName() + " wanted " +
+                                                  method.getDeclaringClass().getName() + ")");
+            }
+
+            //
+            // this test really means, that this is a ruby-defined subclass of a java class
+            //
+            if (javaInvokee instanceof InternalJavaProxy &&
+                    // don't bother to check if final method, it won't
+                    // be there (not generated, can't be!)
+                    !Modifier.isFinal(method.getModifiers())) {
+                JavaProxyClass jpc = ((InternalJavaProxy) javaInvokee)
+                        .___getProxyClass();
+                JavaProxyMethod jpm;
+                if ((jpm = jpc.getMethod(method.getName(), parameterTypes)) != null &&
+                        jpm.hasSuperImplementation()) {
+                    return invokeWithExceptionHandling(jpm.getSuperMethod(), javaInvokee, arguments);
+                }
             }
         }
         return invokeWithExceptionHandling(method, javaInvokee, arguments);
@@ -287,8 +357,9 @@ public class JavaMethod extends JavaCallable {
     }
 
     public IRubyObject invokeDirect(Object javaInvokee) {
+        assert method.getDeclaringClass().isInstance(javaInvokee);
+
         checkArity(0);
-        checkInstanceof(javaInvokee);
 
         if (mightBeProxy(javaInvokee)) {
             return tryProxyInvocation(javaInvokee);
@@ -298,8 +369,9 @@ public class JavaMethod extends JavaCallable {
     }
 
     public IRubyObject invokeDirect(Object javaInvokee, Object arg0) {
+        assert method.getDeclaringClass().isInstance(javaInvokee);
+
         checkArity(1);
-        checkInstanceof(javaInvokee);
 
         if (mightBeProxy(javaInvokee)) {
             return tryProxyInvocation(javaInvokee, arg0);
@@ -309,8 +381,9 @@ public class JavaMethod extends JavaCallable {
     }
 
     public IRubyObject invokeDirect(Object javaInvokee, Object arg0, Object arg1) {
+        assert method.getDeclaringClass().isInstance(javaInvokee);
+
         checkArity(2);
-        checkInstanceof(javaInvokee);
 
         if (mightBeProxy(javaInvokee)) {
             return tryProxyInvocation(javaInvokee, arg0, arg1);
@@ -320,8 +393,9 @@ public class JavaMethod extends JavaCallable {
     }
 
     public IRubyObject invokeDirect(Object javaInvokee, Object arg0, Object arg1, Object arg2) {
+        assert method.getDeclaringClass().isInstance(javaInvokee);
+
         checkArity(3);
-        checkInstanceof(javaInvokee);
 
         if (mightBeProxy(javaInvokee)) {
             return tryProxyInvocation(javaInvokee, arg0, arg1, arg2);
@@ -331,8 +405,9 @@ public class JavaMethod extends JavaCallable {
     }
 
     public IRubyObject invokeDirect(Object javaInvokee, Object arg0, Object arg1, Object arg2, Object arg3) {
+        assert method.getDeclaringClass().isInstance(javaInvokee);
+
         checkArity(4);
-        checkInstanceof(javaInvokee);
 
         if (mightBeProxy(javaInvokee)) {
             return tryProxyInvocation(javaInvokee, arg0, arg1, arg2, arg3);
@@ -514,7 +589,7 @@ public class JavaMethod extends JavaCallable {
     }
 
     private IRubyObject convertReturn(Object result) {
-        if (result != null && result.getClass() != method.getReturnType()) {
+        if (result != null && result.getClass() != boxedReturnType) {
             // actual type does not exactly match method return type, re-get converter
             // FIXME: when the only autoconversions are primitives, this won't be needed
             return JavaUtil.convertJavaToUsableRubyObject(getRuntime(), result);
@@ -601,7 +676,7 @@ public class JavaMethod extends JavaCallable {
 
     private boolean mightBeProxy(Object javaInvokee) {
         // this test really means, that this is a ruby-defined subclass of a java class
-        return javaInvokee instanceof InternalJavaProxy && !Modifier.isFinal(method.getModifiers());
+        return javaInvokee instanceof InternalJavaProxy && !isFinal;
     }
 
     private IRubyObject tryProxyInvocation(Object javaInvokee, Object... args) {
@@ -668,7 +743,7 @@ public class JavaMethod extends JavaCallable {
         return runtime.newNameError("java method not found: " + target.getName() + "." + prettyName, simpleName);
     }
 
-    public static RaiseException newArgSizeMismatchError(Ruby runtime, int typeCount, int argCount) {
-        return runtime.newArgumentError("arg type count (" + typeCount + ") != arg count (" + argCount + ")");
+    public static RaiseException newArgSizeMismatchError(Ruby runtime, Class ... argTypes) {
+        return runtime.newArgumentError("argument count mismatch for method signature " + CodegenUtils.prettyParams(argTypes));
     }
 }
