@@ -46,10 +46,12 @@ import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import org.jruby.Ruby;
+import org.jruby.RubyBasicObject;
 import org.jruby.RubyBigDecimal;
 import org.jruby.RubyBignum;
 import org.jruby.RubyBoolean;
 import org.jruby.RubyClass;
+import org.jruby.RubyEncoding;
 import org.jruby.RubyFixnum;
 import org.jruby.RubyFloat;
 import org.jruby.RubyModule;
@@ -61,6 +63,8 @@ import org.jruby.RubyString;
 import org.jruby.RubyTime;
 import org.jruby.internal.runtime.methods.CallConfiguration;
 import org.jruby.internal.runtime.methods.DynamicMethod;
+import org.jruby.java.proxies.JavaProxy;
+import org.jruby.java.proxies.RubyObjectHolderProxy;
 import org.jruby.javasupport.proxy.InternalJavaProxy;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Block;
@@ -101,6 +105,26 @@ public class JavaUtil {
         return convertJavaToUsableRubyObjectWithConverter(runtime, object, getJavaConverter(javaClass));
     }
 
+    public static IRubyObject convertJavaToRuby(Ruby runtime, int i) {
+        return runtime.newFixnum(i);
+    }
+
+    public static IRubyObject convertJavaToRuby(Ruby runtime, long l) {
+        return runtime.newFixnum(l);
+    }
+
+    public static IRubyObject convertJavaToRuby(Ruby runtime, float f) {
+        return runtime.newFloat(f);
+    }
+
+    public static IRubyObject convertJavaToRuby(Ruby runtime, double d) {
+        return runtime.newFloat(d);
+    }
+
+    public static IRubyObject convertJavaToRuby(Ruby runtime, boolean b) {
+        return runtime.newBoolean(b);
+    }
+
     /**
      * Returns a usable RubyObject; for types that are not converted to Ruby native
      * types, a Java proxy will be returned.
@@ -134,7 +158,8 @@ public class JavaUtil {
 
     public static IRubyObject convertJavaArrayElementToRuby(Ruby runtime, JavaConverter converter, Object array, int i) {
         if (converter == null || converter == JAVA_DEFAULT_CONVERTER) {
-            return convertJavaToUsableRubyObject(runtime, ((Object[])array)[i]);
+            IRubyObject x = convertJavaToUsableRubyObject(runtime, ((Object[])array)[i]);
+            return x;
         }
         return converter.get(runtime, array, i);
     }
@@ -172,10 +197,15 @@ public class JavaUtil {
     }
 
     public static Object convertProcToInterface(ThreadContext context, RubyObject rubyObject, Class target) {
+        return convertProcToInterface(context, (RubyBasicObject)rubyObject, target);
+    }
+
+    public static Object convertProcToInterface(ThreadContext context, RubyBasicObject rubyObject, Class target) {
         Ruby runtime = context.getRuntime();
-        IRubyObject javaInterfaceModule = Java.get_interface_module(runtime, JavaClass.get(runtime, target));
+        RubyModule javaInterfaceModule = (RubyModule)Java.get_interface_module(runtime, JavaClass.get(runtime, target));
         if (!((RubyModule) javaInterfaceModule).isInstance(rubyObject)) {
-            rubyObject.extend(new IRubyObject[]{javaInterfaceModule});
+            javaInterfaceModule.callMethod(context, "extend_object", rubyObject);
+            javaInterfaceModule.callMethod(context, "extended", rubyObject);
         }
 
         if (rubyObject instanceof RubyProc) {
@@ -211,20 +241,6 @@ public class JavaUtil {
         return jo.getValue();
     }
 
-    public static Object coerceJavaObjectToType(ThreadContext context, Object javaObject, Class target) {
-        if (javaObject != null && isDuckTypeConvertable(javaObject.getClass(), target)) {
-            RubyObject rubyObject = (RubyObject) javaObject;
-            if (!rubyObject.respondsTo("java_object")) {
-                return convertProcToInterface(context, rubyObject, target);
-            }
-
-            // can't be converted any more, return it
-            return javaObject;
-        } else {
-            return javaObject;
-        }
-    }
-
     public static NumericConverter getNumericConverter(Class target) {
         NumericConverter converter = NUMERIC_CONVERTERS.get(target);
         if (converter == null) {
@@ -233,72 +249,21 @@ public class JavaUtil {
         return converter;
     }
 
-    // FIXME: This doesn't actually support anything but String
-    public static Object coerceStringToType(RubyString string, Class target) {
-        try {
-            ByteList bytes = string.getByteList();
-
-            // 1.9 support for encodings
-            if (string.getRuntime().is1_9()) {
-                return new String(bytes.unsafeBytes(), bytes.begin(), bytes.length(), string.getEncoding().toString());
-            }
-
-            return new String(bytes.unsafeBytes(), bytes.begin(), bytes.length(), "UTF8");
-        } catch (UnsupportedEncodingException uee) {
-            return string.toString();
-        }
-    }
-
-    public static Object coerceOtherToType(ThreadContext context, IRubyObject arg, Class target) {
-        Ruby runtime = context.getRuntime();
-
-        if (isDuckTypeConvertable(arg.getClass(), target)) {
-            RubyObject rubyObject = (RubyObject) arg;
-            if (!rubyObject.respondsTo("java_object")) {
-                return convertProcToInterface(context, rubyObject, target);
-            }
-        } else if (arg.respondsTo("to_java_object")) {
-            Object javaObject = arg.callMethod(context, "to_java_object");
-            if (javaObject instanceof JavaObject) {
-                runtime.getJavaSupport().getObjectProxyCache().put(((JavaObject) javaObject).getValue(), arg);
-                javaObject = ((JavaObject)javaObject).getValue();
-            }
-            return javaObject;
-        }
-
-        // it's either as converted as we can make it via above logic or it's
-        // not one of the types we convert, so just pass it out as-is without wrapping
-        return arg;
-    }
-
     public static boolean isJavaObject(IRubyObject candidate) {
-        return candidate.dataGetStruct() instanceof JavaObject;
+        return candidate instanceof JavaProxy || candidate.dataGetStruct() instanceof JavaObject;
     }
 
     public static Object unwrapJavaObject(IRubyObject object) {
+        if (object instanceof JavaProxy) {
+            return ((JavaProxy)object).getObject();
+        }
         return ((JavaObject)object.dataGetStruct()).getValue();
     }
 
-    public static JavaObject unwrapJavaObject(Ruby runtime, IRubyObject convertee, String errorMessage) {
-        IRubyObject obj = convertee;
-        if(!(obj instanceof JavaObject)) {
-            if (obj.dataGetStruct() != null && (obj.dataGetStruct() instanceof JavaObject)) {
-                obj = (JavaObject)obj.dataGetStruct();
-            } else {
-                throw runtime.newTypeError(errorMessage);
-            }
-        }
-        return (JavaObject)obj;
-    }
-
     public static Object unwrapJavaValue(Ruby runtime, IRubyObject obj, String errorMessage) {
-        if(obj instanceof JavaMethod) {
-            return ((JavaMethod)obj).getValue();
-        } else if(obj instanceof JavaConstructor) {
-            return ((JavaConstructor)obj).getValue();
-        } else if(obj instanceof JavaField) {
-            return ((JavaField)obj).getValue();
-        } else if(obj instanceof JavaObject) {
+        if (obj instanceof JavaProxy) {
+            return ((JavaProxy)obj).getObject();
+        } else if (obj instanceof JavaObject) {
             return ((JavaObject)obj).getValue();
         } else if(obj.dataGetStruct() != null && (obj.dataGetStruct() instanceof IRubyObject)) {
             return unwrapJavaValue(runtime, ((IRubyObject)obj.dataGetStruct()), errorMessage);
@@ -424,6 +389,10 @@ public class JavaUtil {
             return (IRubyObject) object;
         }
 
+        if (object instanceof RubyObjectHolderProxy) {
+            return ((RubyObjectHolderProxy) object).__ruby_object();
+        }
+
         if (object instanceof InternalJavaProxy) {
             InternalJavaProxy internalJavaProxy = (InternalJavaProxy) object;
             IRubyObject orig = internalJavaProxy.___getInvocationHandler().getOrig();
@@ -758,13 +727,7 @@ public class JavaUtil {
         }
     };
     private static boolean isDoubleFloatable(double value) {
-        return
-                value == 0.0 || // 0.0 is ok
-                value >= Float.MIN_VALUE && value <= Float.MAX_VALUE || // float range
-                value >= -Float.MAX_VALUE && value <= -Float.MIN_VALUE || // inverted float range?
-                Double.isNaN(value) || // NaN
-                value == Double.POSITIVE_INFINITY || // +infinity
-                value == Double.NEGATIVE_INFINITY; // -infinity
+        return true;
     }
     private static boolean isLongByteable(long value) {
         return value >= Byte.MIN_VALUE && value <= 0xFF;
@@ -824,11 +787,6 @@ public class JavaUtil {
             if(rubyObject == null) {
                 throw new RuntimeException("java_object returned null for " + origObject.getType().getName());
             }
-        } else if (rubyObject.respondsTo("to_java_object")) {
-            rubyObject = rubyObject.callMethod(context, "to_java_object");
-            if(rubyObject == null) {
-                throw new RuntimeException("to_java_object returned null for " + origObject.getType().getName());
-            }
         }
 
         if (rubyObject instanceof JavaObject) {
@@ -859,17 +817,13 @@ public class JavaUtil {
         if (javaClass.isPrimitive()) {
             String s = ((RubyString)TypeConverter.convertToType(rubyObject, rubyObject.getRuntime().getString(), "to_s", true)).getUnicodeValue();
             if (s.length() > 0) {
-                return new Character(s.charAt(0));
+                return Character.valueOf(s.charAt(0));
             }
-            return new Character('\0');
+            return Character.valueOf('\0');
         } else if (javaClass == String.class) {
             RubyString rubyString = (RubyString) rubyObject.callMethod(context, "to_s");
             ByteList bytes = rubyString.getByteList();
-            try {
-                return new String(bytes.unsafeBytes(), bytes.begin(), bytes.length(), "UTF8");
-            } catch (UnsupportedEncodingException uee) {
-                return new String(bytes.unsafeBytes(), bytes.begin(), bytes.length());
-            }
+            return RubyEncoding.decodeUTF8(bytes.getUnsafeBytes(), bytes.begin(), bytes.length());
         } else if (javaClass == ByteList.class) {
             return rubyObject.convertToString().getByteList();
         } else if (javaClass == BigInteger.class) {
@@ -960,10 +914,10 @@ public class JavaUtil {
     public static final RubyConverter RUBY_BYTE_CONVERTER = new RubyConverter() {
         public Object convert(ThreadContext context, IRubyObject rubyObject) {
             if (rubyObject.respondsTo("to_i")) {
-                return new Byte((byte) ((RubyNumeric) rubyObject.callMethod(
+                return Byte.valueOf((byte) ((RubyNumeric) rubyObject.callMethod(
                         context, "to_i")).getLongValue());
             }
-            return new Byte((byte) 0);
+            return Byte.valueOf((byte) 0);
         }
     };
 
@@ -971,10 +925,10 @@ public class JavaUtil {
     public static final RubyConverter RUBY_SHORT_CONVERTER = new RubyConverter() {
         public Object convert(ThreadContext context, IRubyObject rubyObject) {
             if (rubyObject.respondsTo("to_i")) {
-                return new Short((short) ((RubyNumeric) rubyObject.callMethod(
+                return Short.valueOf((short) ((RubyNumeric) rubyObject.callMethod(
                         context, "to_i")).getLongValue());
             }
-            return new Short((short) 0);
+            return Short.valueOf((short) 0);
         }
     };
 
@@ -982,10 +936,10 @@ public class JavaUtil {
     public static final RubyConverter RUBY_CHAR_CONVERTER = new RubyConverter() {
         public Object convert(ThreadContext context, IRubyObject rubyObject) {
             if (rubyObject.respondsTo("to_i")) {
-                return new Character((char) ((RubyNumeric) rubyObject.callMethod(
+                return Character.valueOf((char) ((RubyNumeric) rubyObject.callMethod(
                         context, "to_i")).getLongValue());
             }
-            return new Character((char) 0);
+            return Character.valueOf((char) 0);
         }
     };
 
@@ -993,10 +947,10 @@ public class JavaUtil {
     public static final RubyConverter RUBY_INTEGER_CONVERTER = new RubyConverter() {
         public Object convert(ThreadContext context, IRubyObject rubyObject) {
             if (rubyObject.respondsTo("to_i")) {
-                return new Integer((int) ((RubyNumeric) rubyObject.callMethod(
+                return Integer.valueOf((int) ((RubyNumeric) rubyObject.callMethod(
                         context, "to_i")).getLongValue());
             }
-            return new Integer(0);
+            return Integer.valueOf(0);
         }
     };
 
@@ -1004,10 +958,10 @@ public class JavaUtil {
     public static final RubyConverter RUBY_LONG_CONVERTER = new RubyConverter() {
         public Object convert(ThreadContext context, IRubyObject rubyObject) {
             if (rubyObject.respondsTo("to_i")) {
-                return new Long(((RubyNumeric) rubyObject.callMethod(
+                return Long.valueOf(((RubyNumeric) rubyObject.callMethod(
                         context, "to_i")).getLongValue());
             }
-            return new Long(0);
+            return Long.valueOf(0);
         }
     };
 
@@ -1052,31 +1006,6 @@ public class JavaUtil {
         RUBY_CONVERTERS.put(Double.TYPE, RUBY_DOUBLE_CONVERTER);
     }
 
-    @Deprecated
-    public static IRubyObject convertJavaToRuby(Ruby runtime, int i) {
-        return runtime.newFixnum(i);
-    }
-
-    @Deprecated
-    public static IRubyObject convertJavaToRuby(Ruby runtime, long l) {
-        return runtime.newFixnum(l);
-    }
-
-    @Deprecated
-    public static IRubyObject convertJavaToRuby(Ruby runtime, float f) {
-        return runtime.newFloat(f);
-    }
-
-    @Deprecated
-    public static IRubyObject convertJavaToRuby(Ruby runtime, double d) {
-        return runtime.newFloat(d);
-    }
-
-    @Deprecated
-    public static IRubyObject convertJavaToRuby(Ruby runtime, boolean b) {
-        return runtime.newBoolean(b);
-    }
-
     @Deprecated
     public static IRubyObject convertJavaToRuby(Ruby runtime, JavaConverter converter, Object object) {
         if (converter == null || converter == JAVA_DEFAULT_CONVERTER) {
@@ -1255,7 +1184,7 @@ public class JavaUtil {
             javaObject = null;
             break;
         case ClassIndex.FIXNUM:
-            javaObject = new Long(((RubyFixnum) object).getLongValue());
+            javaObject = Long.valueOf(((RubyFixnum) object).getLongValue());
             break;
         case ClassIndex.BIGNUM:
             javaObject = ((RubyBignum) object).getValue();
@@ -1264,12 +1193,8 @@ public class JavaUtil {
             javaObject = new Double(((RubyFloat) object).getValue());
             break;
         case ClassIndex.STRING:
-            try {
-                ByteList bytes = ((RubyString) object).getByteList();
-                javaObject = new String(bytes.unsafeBytes(), bytes.begin(), bytes.length(), "UTF8");
-            } catch (UnsupportedEncodingException uee) {
-                javaObject = object.toString();
-            }
+            ByteList bytes = ((RubyString) object).getByteList();
+            javaObject = RubyEncoding.decodeUTF8(bytes.getUnsafeBytes(), bytes.begin(), bytes.length());
             break;
         case ClassIndex.TRUE:
             javaObject = Boolean.TRUE;
@@ -1312,19 +1237,19 @@ public class JavaUtil {
         if (argument instanceof Number) {
             final Number number = (Number) argument;
             if (type == Long.class) {
-                return new Long(number.longValue());
+                return Long.valueOf(number.longValue());
             } else if (type == Integer.class) {
-                return new Integer(number.intValue());
+                return Integer.valueOf(number.intValue());
             } else if (type == Byte.class) {
-                return new Byte(number.byteValue());
+                return Byte.valueOf(number.byteValue());
             } else if (type == Character.class) {
-                return new Character((char) number.intValue());
+                return Character.valueOf((char) number.intValue());
             } else if (type == Double.class) {
                 return new Double(number.doubleValue());
             } else if (type == Float.class) {
                 return new Float(number.floatValue());
             } else if (type == Short.class) {
-                return new Short(number.shortValue());
+                return Short.valueOf(number.shortValue());
             }
         }
         if (isDuckTypeConvertable(argument.getClass(), parameterType)) {
@@ -1346,4 +1271,64 @@ public class JavaUtil {
         }
         return object;
     }
+
+    // FIXME: This doesn't actually support anything but String
+    @Deprecated
+    public static Object coerceStringToType(RubyString string, Class target) {
+        try {
+            ByteList bytes = string.getByteList();
+
+            // 1.9 support for encodings
+            // TODO: Fix charset use for JRUBY-4553
+            if (string.getRuntime().is1_9()) {
+                return new String(bytes.getUnsafeBytes(), bytes.begin(), bytes.length(), string.getEncoding().toString());
+            }
+
+            return RubyEncoding.decodeUTF8(bytes.getUnsafeBytes(), bytes.begin(), bytes.length());
+        } catch (UnsupportedEncodingException uee) {
+            return string.toString();
+        }
+    }
+
+    @Deprecated
+    public static Object coerceOtherToType(ThreadContext context, IRubyObject arg, Class target) {
+        if (isDuckTypeConvertable(arg.getClass(), target)) {
+            RubyObject rubyObject = (RubyObject) arg;
+            if (!rubyObject.respondsTo("java_object")) {
+                return convertProcToInterface(context, rubyObject, target);
+            }
+        }
+
+        // it's either as converted as we can make it via above logic or it's
+        // not one of the types we convert, so just pass it out as-is without wrapping
+        return arg;
+    }
+
+    @Deprecated
+    public static Object coerceJavaObjectToType(ThreadContext context, Object javaObject, Class target) {
+        if (javaObject != null && isDuckTypeConvertable(javaObject.getClass(), target)) {
+            RubyObject rubyObject = (RubyObject) javaObject;
+            if (!rubyObject.respondsTo("java_object")) {
+                return convertProcToInterface(context, rubyObject, target);
+            }
+
+            // can't be converted any more, return it
+            return javaObject;
+        } else {
+            return javaObject;
+        }
+    }
+
+    @Deprecated
+    public static JavaObject unwrapJavaObject(Ruby runtime, IRubyObject convertee, String errorMessage) {
+        IRubyObject obj = convertee;
+        if(!(obj instanceof JavaObject)) {
+            if (obj.dataGetStruct() != null && (obj.dataGetStruct() instanceof JavaObject)) {
+                obj = (JavaObject)obj.dataGetStruct();
+            } else {
+                throw runtime.newTypeError(errorMessage);
+            }
+        }
+        return (JavaObject)obj;
+    }
 }
