@@ -17,19 +17,24 @@ import org.jruby.RubyProc;
 import org.jruby.RubyRegexp;
 import org.jruby.RubyString;
 import org.jruby.RubySymbol;
+import org.jruby.ast.DSymbolNode;
 import org.jruby.ast.IterNode;
+import org.jruby.ast.LiteralNode;
 import org.jruby.ast.Node;
 import org.jruby.ast.util.ArgsUtil;
 import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.evaluator.ASTInterpreter;
 import org.jruby.exceptions.JumpException;
 import org.jruby.exceptions.RaiseException;
+import org.jruby.exceptions.Unrescuable;
 import org.jruby.internal.runtime.methods.CallConfiguration;
 import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.internal.runtime.methods.UndefinedMethod;
 import org.jruby.internal.runtime.methods.WrapperMethod;
 import org.jruby.javasupport.JavaClass;
 import org.jruby.javasupport.JavaUtil;
+import org.jruby.lexer.yacc.ISourcePosition;
+import org.jruby.lexer.yacc.SimpleSourcePosition;
 import org.jruby.parser.BlockStaticScope;
 import org.jruby.parser.LocalStaticScope;
 import org.jruby.parser.StaticScope;
@@ -49,11 +54,13 @@ import org.jruby.runtime.DynamicScope;
 import org.jruby.runtime.Interpreted19Block;
 import org.jruby.runtime.InterpretedBlock;
 import org.jruby.runtime.MethodFactory;
+import org.jruby.runtime.RubyEvent;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
 import org.jruby.util.TypeConverter;
+import org.jruby.util.unsafe.UnsafeFactory;
 
 /**
  * Helper methods which are called by the compiler.  Note: These will show no consumers, but
@@ -153,6 +160,18 @@ public class RuntimeHelpers {
 
         return factory.getBlockCallback19(closureMethod, scriptObject);
     }
+
+    public static byte[] createBlockCallbackOffline(String classPath, String closureMethod) {
+        MethodFactory factory = MethodFactory.createFactory(RuntimeHelpers.class.getClassLoader());
+
+        return factory.getBlockCallbackOffline(closureMethod, classPath);
+    }
+
+    public static byte[] createBlockCallback19Offline(String classPath, String closureMethod) {
+        MethodFactory factory = MethodFactory.createFactory(RuntimeHelpers.class.getClassLoader());
+
+        return factory.getBlockCallback19Offline(closureMethod, classPath);
+    }
     
     public static BlockBody createCompiledBlockBody(ThreadContext context, Object scriptObject, String closureMethod, int arity, 
             String[] staticScopeNames, boolean hasMultipleArgsHead, int argsNodeType, boolean light) {
@@ -232,7 +251,7 @@ public class RuntimeHelpers {
     }
     
     public static IRubyObject def(ThreadContext context, IRubyObject self, Object scriptObject, String name, String javaName, String[] scopeNames,
-            int arity, int required, int optional, int rest, CallConfiguration callConfig) {
+            int arity, int required, int optional, int rest, String filename, int line, CallConfiguration callConfig) {
         Class compiledClass = scriptObject.getClass();
         Ruby runtime = context.getRuntime();
         
@@ -244,7 +263,10 @@ public class RuntimeHelpers {
         StaticScope scope = creatScopeForClass(context, scopeNames, required, optional, rest);
         
         MethodFactory factory = MethodFactory.createFactory(compiledClass.getClassLoader());
-        DynamicMethod method = constructNormalMethod(name, visibility, factory, containingClass, javaName, arity, scope, scriptObject, callConfig);
+        DynamicMethod method = constructNormalMethod(
+                factory, javaName,
+                name, containingClass, new SimpleSourcePosition(filename, line), arity, scope, visibility, scriptObject,
+                callConfig);
         
         addInstanceMethod(containingClass, name, method, visibility,context, runtime);
         
@@ -252,7 +274,7 @@ public class RuntimeHelpers {
     }
     
     public static IRubyObject defs(ThreadContext context, IRubyObject self, IRubyObject receiver, Object scriptObject, String name, String javaName, String[] scopeNames,
-            int arity, int required, int optional, int rest, CallConfiguration callConfig) {
+            int arity, int required, int optional, int rest, String filename, int line, CallConfiguration callConfig) {
         Class compiledClass = scriptObject.getClass();
         Ruby runtime = context.getRuntime();
 
@@ -261,7 +283,7 @@ public class RuntimeHelpers {
         StaticScope scope = creatScopeForClass(context, scopeNames, required, optional, rest);
         
         MethodFactory factory = MethodFactory.createFactory(compiledClass.getClassLoader());
-        DynamicMethod method = constructSingletonMethod(factory, rubyClass, javaName, arity, scope,scriptObject, callConfig);
+        DynamicMethod method = constructSingletonMethod( factory, javaName, rubyClass, new SimpleSourcePosition(filename, line), arity, scope,scriptObject, callConfig);
         
         rubyClass.addMethod(name, method);
         
@@ -269,6 +291,13 @@ public class RuntimeHelpers {
         
         return runtime.getNil();
     }
+
+    public static byte[] defOffline(String name, String classPath, String invokerName, Arity arity, StaticScope scope, CallConfiguration callConfig, String filename, int line) {
+        MethodFactory factory = MethodFactory.createFactory(RuntimeHelpers.class.getClassLoader());
+        byte[] methodBytes = factory.getCompiledMethodOffline(name, classPath, invokerName, arity, scope, callConfig, filename, line);
+
+        return methodBytes;
+    }
     
     public static RubyClass getSingletonClass(Ruby runtime, IRubyObject receiver) {
         if (receiver instanceof RubyFixnum || receiver instanceof RubySymbol) {
@@ -346,6 +375,20 @@ public class RuntimeHelpers {
         return new MethodMissingMethod(methodMissing);
     }
 
+    public static DynamicMethod selectMethodMissing(RubyClass selfClass, Visibility visibility, String name, CallType callType) {
+        Ruby runtime = selfClass.getClassRuntime();
+
+        if (name.equals("method_missing")) {
+            return selectInternalMM(runtime, visibility, callType);
+        }
+
+        DynamicMethod methodMissing = selfClass.searchMethod("method_missing");
+        if (methodMissing.isUndefined() || methodMissing == runtime.getDefaultMethodMissing()) {
+            return selectInternalMM(runtime, visibility, callType);
+        }
+        return new MethodMissingMethod(methodMissing);
+    }
+
     private static class MethodMissingMethod extends DynamicMethod {
         private DynamicMethod delegate;
 
@@ -449,6 +492,10 @@ public class RuntimeHelpers {
         return asClass.finvoke(context, self, name, arg0, arg1, arg2, block);
     }
 
+    public static IRubyObject invokeChecked(ThreadContext context, IRubyObject self, String name) {
+        return self.getMetaClass().finvokeChecked(context, self, name);
+    }
+
     /**
      * The protocol for super method invocation is a bit complicated
      * in Ruby. In real terms it involves first finding the real
@@ -851,6 +898,10 @@ public class RuntimeHelpers {
     }
 
     public static IRubyObject isExceptionHandled(RubyException currentException, IRubyObject exception, ThreadContext context) {
+        return isExceptionHandled((IRubyObject)currentException, exception, context);
+    }
+
+    public static IRubyObject isExceptionHandled(IRubyObject currentException, IRubyObject exception, ThreadContext context) {
         Ruby runtime = context.getRuntime();
         if (!runtime.getModule().isInstance(exception)) {
             throw runtime.newTypeError("class or module required for rescue clause");
@@ -872,13 +923,21 @@ public class RuntimeHelpers {
         return isExceptionHandled(currentException, exception1, exception2, context);
     }
 
-    private static boolean checkJavaException(Throwable currentThrowable, IRubyObject throwable) {
-        if (throwable instanceof RubyClass) {
-            RubyClass rubyClass = (RubyClass)throwable;
+    private static boolean checkJavaException(Throwable throwable, IRubyObject catchable, ThreadContext context) {
+        if (context.getRuntime().getException().op_ge(catchable).isTrue()
+                || context.getRuntime().getObject() == catchable) {
+            if (throwable instanceof RaiseException) {
+                return isExceptionHandled(((RaiseException)throwable).getException(), catchable, context).isTrue();
+            }
+            // let Ruby exceptions decide if they handle it
+            return isExceptionHandled(JavaUtil.convertJavaToUsableRubyObject(context.getRuntime(), throwable), catchable, context).isTrue();
+        }
+        if (catchable instanceof RubyClass && catchable.getInstanceVariables().hasInstanceVariable("@java_class")) {
+            RubyClass rubyClass = (RubyClass)catchable;
             JavaClass javaClass = (JavaClass)rubyClass.fastGetInstanceVariable("@java_class");
             if (javaClass != null) {
                 Class cls = javaClass.javaClass();
-                if (cls.isInstance(currentThrowable)) {
+                if (cls.isInstance(throwable)) {
                     return true;
                 }
             }
@@ -887,11 +946,15 @@ public class RuntimeHelpers {
     }
     
     public static IRubyObject isJavaExceptionHandled(Throwable currentThrowable, IRubyObject[] throwables, ThreadContext context) {
+        if (currentThrowable instanceof Unrescuable) {
+            UnsafeFactory.getUnsafe().throwException(currentThrowable);
+        }
+
         if (currentThrowable instanceof RaiseException) {
             return isExceptionHandled(((RaiseException)currentThrowable).getException(), throwables, context);
         } else {
             for (int i = 0; i < throwables.length; i++) {
-                if (checkJavaException(currentThrowable, throwables[0])) {
+                if (checkJavaException(currentThrowable, throwables[0], context)) {
                     return context.getRuntime().getTrue();
                 }
             }
@@ -901,10 +964,14 @@ public class RuntimeHelpers {
     }
 
     public static IRubyObject isJavaExceptionHandled(Throwable currentThrowable, IRubyObject throwable, ThreadContext context) {
+        if (currentThrowable instanceof Unrescuable) {
+            UnsafeFactory.getUnsafe().throwException(currentThrowable);
+        }
+
         if (currentThrowable instanceof RaiseException) {
             return isExceptionHandled(((RaiseException)currentThrowable).getException(), throwable, context);
         } else {
-            if (checkJavaException(currentThrowable, throwable)) {
+            if (checkJavaException(currentThrowable, throwable, context)) {
                 return context.getRuntime().getTrue();
             }
 
@@ -913,13 +980,17 @@ public class RuntimeHelpers {
     }
 
     public static IRubyObject isJavaExceptionHandled(Throwable currentThrowable, IRubyObject throwable0, IRubyObject throwable1, ThreadContext context) {
+        if (currentThrowable instanceof Unrescuable) {
+            UnsafeFactory.getUnsafe().throwException(currentThrowable);
+        }
+
         if (currentThrowable instanceof RaiseException) {
             return isExceptionHandled(((RaiseException)currentThrowable).getException(), throwable0, throwable1, context);
         } else {
-            if (checkJavaException(currentThrowable, throwable0)) {
+            if (checkJavaException(currentThrowable, throwable0, context)) {
                 return context.getRuntime().getTrue();
             }
-            if (checkJavaException(currentThrowable, throwable1)) {
+            if (checkJavaException(currentThrowable, throwable1, context)) {
                 return context.getRuntime().getTrue();
             }
 
@@ -928,16 +999,20 @@ public class RuntimeHelpers {
     }
 
     public static IRubyObject isJavaExceptionHandled(Throwable currentThrowable, IRubyObject throwable0, IRubyObject throwable1, IRubyObject throwable2, ThreadContext context) {
+        if (currentThrowable instanceof Unrescuable) {
+            UnsafeFactory.getUnsafe().throwException(currentThrowable);
+        }
+        
         if (currentThrowable instanceof RaiseException) {
             return isExceptionHandled(((RaiseException)currentThrowable).getException(), throwable0, throwable1, throwable2, context);
         } else {
-            if (checkJavaException(currentThrowable, throwable0)) {
+            if (checkJavaException(currentThrowable, throwable0, context)) {
                 return context.getRuntime().getTrue();
             }
-            if (checkJavaException(currentThrowable, throwable1)) {
+            if (checkJavaException(currentThrowable, throwable1, context)) {
                 return context.getRuntime().getTrue();
             }
-            if (checkJavaException(currentThrowable, throwable2)) {
+            if (checkJavaException(currentThrowable, throwable2, context)) {
                 return context.getRuntime().getTrue();
             }
 
@@ -1192,13 +1267,33 @@ public class RuntimeHelpers {
         hash.fastASetCheckString19(runtime, key3, value3);
         return hash;
     }
+
+    public static IRubyObject undefMethod(ThreadContext context, Object nameArg) {
+        RubyModule module = context.getRubyClass();
+
+        String name = (nameArg instanceof String) ?
+            (String) nameArg : nameArg.toString();
+
+        if (module == null) {
+            throw context.getRuntime().newTypeError("No class to undef method '" + name + "'.");
+        }
+
+        module.undef(context, name);
+
+        return context.getRuntime().getNil();
+    }
     
-    public static IRubyObject defineAlias(ThreadContext context, String newName, String oldName) {
+    public static IRubyObject defineAlias(ThreadContext context, Object newNameArg, Object oldNameArg) {
         Ruby runtime = context.getRuntime();
         RubyModule module = context.getRubyClass();
    
         if (module == null) throw runtime.newTypeError("no class to make alias");
-   
+
+        String newName = (newNameArg instanceof String) ?
+            (String) newNameArg : newNameArg.toString();
+        String oldName = (oldNameArg instanceof String) ? 
+            (String) oldNameArg : oldNameArg.toString();
+
         module.defineAlias(newName, oldName);
         module.callMethod(context, "method_added", runtime.newSymbol(newName));
    
@@ -1349,15 +1444,19 @@ public class RuntimeHelpers {
     }
 
     public static RubyArray arrayValue(IRubyObject value) {
+        Ruby runtime = value.getRuntime();
+        return arrayValue(runtime.getCurrentContext(), runtime, value);
+    }
+
+    public static RubyArray arrayValue(ThreadContext context, Ruby runtime, IRubyObject value) {
         IRubyObject tmp = value.checkArrayType();
 
         if (tmp.isNil()) {
-            // Object#to_a is obsolete.  We match Ruby's hack until to_a goes away.  Then we can 
+            // Object#to_a is obsolete.  We match Ruby's hack until to_a goes away.  Then we can
             // remove this hack too.
-            Ruby runtime = value.getRuntime();
-            
-            if (value.getMetaClass().searchMethod("to_a").getImplementationClass() != runtime.getKernel()) {
-                value = value.callMethod(runtime.getCurrentContext(), "to_a");
+
+            if (value.respondsTo("to_a") && value.getMetaClass().searchMethod("to_a").getImplementationClass() != runtime.getKernel()) {
+                value = value.callMethod(context, "to_a");
                 if (!(value instanceof RubyArray)) throw runtime.newTypeError("`to_a' did not return Array");
                 return (RubyArray)value;
             } else {
@@ -1387,6 +1486,14 @@ public class RuntimeHelpers {
         return array.getLength() == 1 ? array.first() : array;
     }
 
+    public static IRubyObject aValueSplat19(IRubyObject value) {
+        if (!(value instanceof RubyArray)) {
+            return value.getRuntime().getNil();
+        }
+
+        return (RubyArray) value;
+    }
+
     public static RubyArray splatValue(IRubyObject value) {
         if (value.isNil()) {
             return value.getRuntime().newArray(value);
@@ -1395,6 +1502,14 @@ public class RuntimeHelpers {
         return arrayValue(value);
     }
 
+    public static RubyArray splatValue19(IRubyObject value) {
+        if (value.isNil()) {
+            return value.getRuntime().newEmptyArray();
+        }
+
+        return arrayValue(value);
+    }
+
     public static void addInstanceMethod(RubyModule containingClass, String name, DynamicMethod method, Visibility visibility, ThreadContext context, Ruby runtime) {
         containingClass.addMethod(name, method);
 
@@ -1424,7 +1539,7 @@ public class RuntimeHelpers {
         receiver.callMethod(context, "singleton_method_added", name);
     }
 
-    private static DynamicMethod constructNormalMethod(String name, Visibility visibility, MethodFactory factory, RubyModule containingClass, String javaName, int arity, StaticScope scope, Object scriptObject, CallConfiguration callConfig) {
+    private static DynamicMethod constructNormalMethod( MethodFactory factory, String javaName, String name, RubyModule containingClass, ISourcePosition position, int arity, StaticScope scope, Visibility visibility, Object scriptObject, CallConfiguration callConfig) {
         DynamicMethod method;
 
         if (name.equals("initialize") || name.equals("initialize_copy") || visibility == Visibility.MODULE_FUNCTION) {
@@ -1432,19 +1547,19 @@ public class RuntimeHelpers {
         }
         
         if (RubyInstanceConfig.LAZYHANDLES_COMPILE) {
-            method = factory.getCompiledMethodLazily(containingClass, javaName, Arity.createArity(arity), visibility, scope, scriptObject, callConfig);
+            method = factory.getCompiledMethodLazily(containingClass, javaName, Arity.createArity(arity), visibility, scope, scriptObject, callConfig, position);
         } else {
-            method = factory.getCompiledMethod(containingClass, javaName, Arity.createArity(arity), visibility, scope, scriptObject, callConfig);
+            method = factory.getCompiledMethod(containingClass, javaName, Arity.createArity(arity), visibility, scope, scriptObject, callConfig, position);
         }
 
         return method;
     }
 
-    private static DynamicMethod constructSingletonMethod(MethodFactory factory, RubyClass rubyClass, String javaName, int arity, StaticScope scope, Object scriptObject, CallConfiguration callConfig) {
+    private static DynamicMethod constructSingletonMethod(MethodFactory factory, String javaName, RubyClass rubyClass, ISourcePosition position, int arity, StaticScope scope, Object scriptObject, CallConfiguration callConfig) {
         if (RubyInstanceConfig.LAZYHANDLES_COMPILE) {
-            return factory.getCompiledMethodLazily(rubyClass, javaName, Arity.createArity(arity), Visibility.PUBLIC, scope, scriptObject, callConfig);
+            return factory.getCompiledMethodLazily(rubyClass, javaName, Arity.createArity(arity), Visibility.PUBLIC, scope, scriptObject, callConfig, position);
         } else {
-            return factory.getCompiledMethod(rubyClass, javaName, Arity.createArity(arity), Visibility.PUBLIC, scope, scriptObject, callConfig);
+            return factory.getCompiledMethod(rubyClass, javaName, Arity.createArity(arity), Visibility.PUBLIC, scope, scriptObject, callConfig, position);
         }
     }
 
@@ -1643,9 +1758,93 @@ public class RuntimeHelpers {
      * @param b
      * @return
      */
-    public static IRubyObject rbEqual(ThreadContext context, IRubyObject a, IRubyObject b) {
+    public static RubyBoolean rbEqual(ThreadContext context, IRubyObject a, IRubyObject b) {
         Ruby runtime = context.getRuntime();
         if (a == b) return runtime.getTrue();
-        return a.callMethod(context, "==", b);
+        IRubyObject res = a.callMethod(context, "==", b);
+        return runtime.newBoolean(res.isTrue());
+    }
+
+    public static void traceLine(ThreadContext context) {
+        String name = context.getFrameName();
+        RubyModule type = context.getFrameKlazz();
+        context.getRuntime().callEventHooks(context, RubyEvent.LINE, context.getFile(), context.getLine(), name, type);
+    }
+
+    public static void traceClass(ThreadContext context) {
+        String name = context.getFrameName();
+        RubyModule type = context.getFrameKlazz();
+        context.getRuntime().callEventHooks(context, RubyEvent.CLASS, context.getFile(), context.getLine(), name, type);
+    }
+
+    public static void traceEnd(ThreadContext context) {
+        String name = context.getFrameName();
+        RubyModule type = context.getFrameKlazz();
+        context.getRuntime().callEventHooks(context, RubyEvent.END, context.getFile(), context.getLine(), name, type);
+    }
+
+    /**
+     * Some of this code looks scary.  All names for an alias or undef is a
+     * fitem in 1.8/1.9 grammars.  This means it is guaranteed to be either
+     * a LiteralNode of a DSymbolNode.  Nothing else is possible.  Also
+     * Interpreting a DSymbolNode will always yield a RubySymbol.
+     */
+    public static String interpretAliasUndefName(Node nameNode, Ruby runtime,
+            ThreadContext context, IRubyObject self, Block aBlock) {
+        String name;
+
+        if (nameNode instanceof LiteralNode) {
+            name = ((LiteralNode) nameNode).getName();
+        } else {
+            assert nameNode instanceof DSymbolNode: "Alias or Undef not literal or dsym";
+            name = ((RubySymbol) nameNode.interpret(runtime, context, self, aBlock)).asJavaString();
+        }
+
+        return name;
+    }
+
+    /**
+     * Used by the compiler to simplify arg checking in variable-arity paths
+     *
+     * @param context thread context
+     * @param args arguments array
+     * @param min minimum required
+     * @param max maximum allowed
+     */
+    public static void checkArgumentCount(ThreadContext context, IRubyObject[] args, int min, int max) {
+        checkArgumentCount(context, args.length, min, max);
+    }
+
+    /**
+     * Used by the compiler to simplify arg checking in variable-arity paths
+     *
+     * @param context thread context
+     * @param args arguments array
+     * @param req required number
+     */
+    public static void checkArgumentCount(ThreadContext context, IRubyObject[] args, int req) {
+        checkArgumentCount(context, args.length, req, req);
+    }
+    
+    public static void checkArgumentCount(ThreadContext context, int length, int min, int max) {
+        int expected = 0;
+        if (length < min) {
+            expected = min;
+        } else if (max > -1 && length > max) {
+            expected = max;
+        } else {
+            return;
+        }
+        throw context.getRuntime().newArgumentError(length, expected);
+    }
+
+    public static boolean isModuleAndHasConstant(IRubyObject left, String name) {
+        return left instanceof RubyModule && ((RubyModule) left).fastGetConstantFromNoConstMissing(name) != null;
+    }
+
+    public static String getDefinedConstantOrBoundMethod(IRubyObject left, String name) {
+        if (isModuleAndHasConstant(left, name)) return "constant";
+        if (left.getMetaClass().isMethodBound(name, true)) return "method";
+        return null;
     }
 }
