@@ -43,7 +43,6 @@ import java.util.HashMap;
 import org.jcodings.Encoding;
 import org.jruby.ast.BackRefNode;
 import org.jruby.ast.BignumNode;
-import org.jruby.ast.CommentNode;
 import org.jruby.ast.FixnumNode;
 import org.jruby.ast.FloatNode;
 import org.jruby.ast.NthRefNode;
@@ -107,6 +106,7 @@ public class RubyYaccLexer {
         map.put("BEGIN", Keyword.LBEGIN);
         map.put("while", Keyword.WHILE);
         map.put("alias", Keyword.ALIAS);
+        map.put("__ENCODING__", Keyword.__ENCODING__);
     }
 
     private Encoding encoding;
@@ -172,7 +172,8 @@ public class RubyYaccLexer {
         LEND ("END", Tokens.klEND, Tokens.klEND, LexState.EXPR_END),
         LBEGIN ("BEGIN", Tokens.klBEGIN, Tokens.klBEGIN, LexState.EXPR_END),
         WHILE ("while", Tokens.kWHILE, Tokens.kWHILE_MOD, LexState.EXPR_BEG),
-        ALIAS ("alias", Tokens.kALIAS, Tokens.kALIAS, LexState.EXPR_FNAME);
+        ALIAS ("alias", Tokens.kALIAS, Tokens.kALIAS, LexState.EXPR_FNAME),
+        __ENCODING__("__ENCODING__", Tokens.k__ENCODING__, Tokens.k__ENCODING__, LexState.EXPR_END);
         
         public final String name;
         public final int id0;
@@ -223,9 +224,6 @@ public class RubyYaccLexer {
     private StackState cmdArgumentState = new StackState();
     private StrTerm lex_strterm;
     public boolean commandStart;
-    
-    // Whether we're processing comments
-    private boolean doComments;
 
     // Give a name to a value.  Enebo: This should be used more.
     static final int EOF = -1;
@@ -316,10 +314,6 @@ public class RubyYaccLexer {
     public Object value() {
         return yaccValue;
     }
-
-    public ISourcePositionFactory getPositionFactory() {
-        return src.getPositionFactory();
-    }
     
     /**
      * Get position information for Token/Node that follows node represented by startPosition 
@@ -329,12 +323,16 @@ public class RubyYaccLexer {
      * @param inclusive include previous node into position information of current node
      * @return a new position
      */
-    public ISourcePosition getPosition(ISourcePosition startPosition, boolean inclusive) {
-    	return src.getPosition(startPosition, inclusive); 
+    public ISourcePosition getPosition(ISourcePosition startPosition) {
+    	return src.getPosition(startPosition); 
     }
     
     public ISourcePosition getPosition() {
-        return src.getPosition(null, false);
+        return src.getPosition();
+    }
+
+    public String getCurrentLine() {
+        return src.getCurrentLine();
     }
 
     /**
@@ -346,9 +344,6 @@ public class RubyYaccLexer {
      */
     public void setParserSupport(ParserSupport parserSupport) {
         this.parserSupport = parserSupport;
-        if (parserSupport.getConfiguration() != null) {
-            this.doComments = parserSupport.getConfiguration().hasExtraPositionInformation();
-        }
     }
 
     public void setEncoding(Encoding encoding) {
@@ -491,11 +486,11 @@ public class RubyYaccLexer {
             shortHand = false;
             begin = src.read();
             if (Character.isLetterOrDigit(begin) /* no mb || ismbchar(term)*/) {
-                throw new SyntaxException(PID.STRING_UNKNOWN_TYPE, getPosition(), "unknown type of %string");
+                throw new SyntaxException(PID.STRING_UNKNOWN_TYPE, getPosition(), getCurrentLine(), "unknown type of %string");
             }
         }
         if (c == EOF || begin == EOF) {
-            throw new SyntaxException(PID.STRING_HITS_EOF, getPosition(), "unterminated quoted string meets end of file");
+            throw new SyntaxException(PID.STRING_HITS_EOF, getPosition(), getCurrentLine(), "unterminated quoted string meets end of file");
         }
         
         // Figure end-char.  '\0' is special to indicate begin=end and that no nesting?
@@ -551,7 +546,7 @@ public class RubyYaccLexer {
             return Tokens.tSYMBEG;
 
         default:
-            throw new SyntaxException(PID.STRING_UNKNOWN_TYPE, getPosition(), 
+            throw new SyntaxException(PID.STRING_UNKNOWN_TYPE, getPosition(), getCurrentLine(),
                     "Unknown type of %string. Expected 'Q', 'q', 'w', 'x', 'r' or any non letter character, but found '" + c + "'.");
         }
     }
@@ -582,7 +577,8 @@ public class RubyYaccLexer {
                 markerValue.append(c);
             }
             if (c == EOF) {
-                throw new SyntaxException(PID.STRING_MARKER_MISSING, getPosition(), "unterminated here document identifier");
+                throw new SyntaxException(PID.STRING_MARKER_MISSING, getPosition(), 
+                        getCurrentLine(), "unterminated here document identifier");
             }	
         } else {
             if (!isIdentifierChar(c)) {
@@ -651,44 +647,16 @@ public class RubyYaccLexer {
     }
 
     /**
-     * Read a comment up to end of line.  When found each comment will get stored away into
-     * the parser result so that any interested party can use them as they seem fit.  One idea
-     * is that IDE authors can do distance based heuristics to associate these comments to the
-     * AST node they think they belong to.
+     * Read a comment up to end of line.
      * 
      * @param c last character read from lexer source
      * @return newline or eof value 
      */
     protected int readComment(int c) throws IOException {
-        if (doComments) {
-            return readCommentLong(c);
-        }
-        
         return src.skipUntil('\n');
         
     }
     
-    private int readCommentLong(int c) throws IOException {
-        ISourcePosition startPosition = src.getPosition();
-        tokenBuffer.setLength(0);
-        tokenBuffer.append((char) c);
-
-        // FIXME: Consider making a better LexerSource.readLine
-        while ((c = src.read()) != '\n') {
-            if (c == EOF) break;
-
-            tokenBuffer.append((char) c);
-        }
-        src.unread(c);
-        
-        // Store away each comment to parser result so IDEs can do whatever they want with them.
-        ISourcePosition position = startPosition.union(getPosition());
-        
-        parserSupport.getResult().addComment(new CommentNode(position, tokenBuffer.toString()));
-        
-        return c;
-    }
-    
     /*
      * Not normally used, but is left in here since it can be useful in debugging
      * grammar and lexing problems.
@@ -858,11 +826,12 @@ public class RubyYaccLexer {
         commandStart = false;
 
         loop: for(;;) {
-            c = src.read();            
+            c = src.read();
             switch(c) {
-            case '\004':		/* ^D */
-            case '\032':		/* ^Z */
-            case EOF:			/* end of script. */
+            case '\000': /* NUL */
+            case '\004': /* ^D */
+            case '\032': /* ^Z */
+            case EOF:	 /* end of script. */
                 return EOF;
            
                 /* white spaces */
@@ -930,10 +899,6 @@ public class RubyYaccLexer {
                 // documentation nodes
                 if (src.wasBeginOfLine()) {
                     if (src.matchMarker(BEGIN_DOC_MARKER, false, false)) {
-                        if (doComments) {
-                            tokenBuffer.setLength(0);
-                            tokenBuffer.append(BEGIN_DOC_MARKER);
-                        }
                         c = src.read();
                         
                         if (Character.isWhitespace(c)) {
@@ -941,30 +906,24 @@ public class RubyYaccLexer {
                             src.unread(c);
                             for (;;) {
                                 c = src.read();
-                                if (doComments) tokenBuffer.append((char) c);
 
                                 // If a line is followed by a blank line put
                                 // it back.
                                 while (c == '\n') {
                                     c = src.read();
-                                    if (doComments) tokenBuffer.append((char) c);
                                 }
                                 if (c == EOF) {
-                                    throw new SyntaxException(PID.STRING_HITS_EOF, getPosition(), "embedded document meets end of file");
+                                    throw new SyntaxException(PID.STRING_HITS_EOF, getPosition(),
+                                            getCurrentLine(), "embedded document meets end of file");
                                 }
                                 if (c != '=') continue;
                                 if (src.wasBeginOfLine() && src.matchMarker(END_DOC_MARKER, false, false)) {
-                                    if (doComments) tokenBuffer.append(END_DOC_MARKER);
                                     ByteList list = src.readLineBytes();
-                                    if (doComments) tokenBuffer.append(list);
                                     src.unread('\n');
                                     break;
                                 }
                             }
 
-                            if (doComments) {
-                                parserSupport.getResult().addComment(new CommentNode(getPosition(), tokenBuffer.toString()));
-                            }
                             continue;
                         }
 						src.unread(c);
@@ -1160,9 +1119,11 @@ public class RubyYaccLexer {
         
         if (Character.isDigit(c)) {
             if (tokenBuffer.length() == 1) {
-                throw new SyntaxException(PID.IVAR_BAD_NAME, getPosition(), "`@" + c + "' is not allowed as an instance variable name");
+                throw new SyntaxException(PID.IVAR_BAD_NAME, getPosition(), getCurrentLine(),
+                        "`@" + c + "' is not allowed as an instance variable name");
             }
-            throw new SyntaxException(PID.CVAR_BAD_NAME, getPosition(), "`@@" + c + "' is not allowed as a class variable name");
+            throw new SyntaxException(PID.CVAR_BAD_NAME, getPosition(), getCurrentLine(),
+                    "`@@" + c + "' is not allowed as a class variable name");
         }
         
         if (!isIdentifierChar(c)) {
@@ -1412,7 +1373,8 @@ public class RubyYaccLexer {
         
         src.unread(c);
         if (Character.isDigit(c)) {
-            throw new SyntaxException(PID.FLOAT_MISSING_ZERO, getPosition(), "no .<digit> floating literal anymore; put 0 before dot"); 
+            throw new SyntaxException(PID.FLOAT_MISSING_ZERO, getPosition(), getCurrentLine(),
+                    "no .<digit> floating literal anymore; put 0 before dot");
         }
         
         lex_state = LexState.EXPR_DOT;
@@ -1528,7 +1490,7 @@ public class RubyYaccLexer {
             /* See if it is a reserved word.  */
             //Keyword keyword = Keyword.getKeyword(tempVal, tempVal.length());
             Keyword keyword = getKeyword(tempVal);
-            if (keyword != null) {
+            if (keyword != null && (keyword != Keyword.__ENCODING__ || !isOneEight)) {
                 // enum lex_state
                 LexState state = lex_state;
 
@@ -1538,6 +1500,8 @@ public class RubyYaccLexer {
                 } else {
                     yaccValue = new Token(tempVal, getPosition());
                     if (keyword.id0 == Tokens.kDO) {
+                        commandStart = true;
+                        
                         if (!isOneEight && leftParenBegin > 0 && leftParenBegin == parenNest) {
                             leftParenBegin = 0;
                             parenNest--;
@@ -1826,7 +1790,8 @@ public class RubyYaccLexer {
         }
         
         c = src.read();
-        if (c == EOF) throw new SyntaxException(PID.INCOMPLETE_CHAR_SYNTAX, getPosition(), "incomplete character syntax");
+        if (c == EOF) throw new SyntaxException(PID.INCOMPLETE_CHAR_SYNTAX, getPosition(), 
+                getCurrentLine(), "incomplete character syntax");
 
         if (Character.isWhitespace(c)){
             if (!isARG()) {
@@ -2046,9 +2011,11 @@ public class RubyYaccLexer {
                     src.unread(c);
 
                     if (tokenBuffer.length() == startLen) {
-                        throw new SyntaxException(PID.BAD_HEX_NUMBER, getPosition(), "Hexadecimal number without hex-digits.");
+                        throw new SyntaxException(PID.BAD_HEX_NUMBER, getPosition(), 
+                                getCurrentLine(), "Hexadecimal number without hex-digits.");
                     } else if (nondigit != '\0') {
-                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(), "Trailing '_' in number.");
+                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(),
+                                getCurrentLine(), "Trailing '_' in number.");
                     }
                     yaccValue = getInteger(tokenBuffer.toString(), 16);
                     return Tokens.tINTEGER;
@@ -2071,9 +2038,11 @@ public class RubyYaccLexer {
                     src.unread(c);
 
                     if (tokenBuffer.length() == startLen) {
-                        throw new SyntaxException(PID.EMPTY_BINARY_NUMBER, getPosition(), "Binary number without digits.");
+                        throw new SyntaxException(PID.EMPTY_BINARY_NUMBER, getPosition(),
+                                getCurrentLine(), "Binary number without digits.");
                     } else if (nondigit != '\0') {
-                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(), "Trailing '_' in number.");
+                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(),
+                                getCurrentLine(), "Trailing '_' in number.");
                     }
                     yaccValue = getInteger(tokenBuffer.toString(), 2);
                     return Tokens.tINTEGER;
@@ -2096,13 +2065,16 @@ public class RubyYaccLexer {
                     src.unread(c);
 
                     if (tokenBuffer.length() == startLen) {
-                        throw new SyntaxException(PID.EMPTY_BINARY_NUMBER, getPosition(), "Binary number without digits.");
+                        throw new SyntaxException(PID.EMPTY_BINARY_NUMBER, getPosition(), 
+                                getCurrentLine(), "Binary number without digits.");
                     } else if (nondigit != '\0') {
-                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(), "Trailing '_' in number.");
+                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(),
+                                getCurrentLine(), "Trailing '_' in number.");
                     }
                     yaccValue = getInteger(tokenBuffer.toString(), 10);
                     return Tokens.tINTEGER;
                 case 'o':
+                case 'O':
                     c = src.read();
                 case '0': case '1': case '2': case '3': case '4': //Octal
                 case '5': case '6': case '7': case '_': 
@@ -2122,7 +2094,8 @@ public class RubyYaccLexer {
                         src.unread(c);
 
                         if (nondigit != '\0') {
-                            throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(), "Trailing '_' in number.");
+                            throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, 
+                                    getPosition(), getCurrentLine(), "Trailing '_' in number.");
                         }
 
                         yaccValue = getInteger(tokenBuffer.toString(), 8);
@@ -2130,7 +2103,8 @@ public class RubyYaccLexer {
                     }
                 case '8' :
                 case '9' :
-                    throw new SyntaxException(PID.BAD_OCTAL_DIGIT, getPosition(), "Illegal octal digit.");
+                    throw new SyntaxException(PID.BAD_OCTAL_DIGIT, getPosition(),
+                            getCurrentLine(), "Illegal octal digit.");
                 case '.' :
                 case 'e' :
                 case 'E' :
@@ -2164,7 +2138,8 @@ public class RubyYaccLexer {
                 case '.' :
                     if (nondigit != '\0') {
                         src.unread(c);
-                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(), "Trailing '_' in number.");
+                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(),
+                                getCurrentLine(), "Trailing '_' in number.");
                     } else if (seen_point || seen_e) {
                         src.unread(c);
                         return getNumberToken(tokenBuffer.toString(), true, nondigit);
@@ -2191,7 +2166,8 @@ public class RubyYaccLexer {
                 case 'e' :
                 case 'E' :
                     if (nondigit != '\0') {
-                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(), "Trailing '_' in number.");
+                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(),
+                                getCurrentLine(), "Trailing '_' in number.");
                     } else if (seen_e) {
                         src.unread(c);
                         return getNumberToken(tokenBuffer.toString(), true, nondigit);
@@ -2210,7 +2186,8 @@ public class RubyYaccLexer {
                     break;
                 case '_' : //  '_' in number just ignored
                     if (nondigit != '\0') {
-                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(), "Trailing '_' in number.");
+                        throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(),
+                                getCurrentLine(), "Trailing '_' in number.");
                     }
                     nondigit = c;
                     break;
@@ -2223,7 +2200,8 @@ public class RubyYaccLexer {
 
     private int getNumberToken(String number, boolean isFloat, int nondigit) {
         if (nondigit != '\0') {
-            throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(), "Trailing '_' in number.");
+            throw new SyntaxException(PID.TRAILING_UNDERSCORE_IN_NUMBER, getPosition(),
+                    getCurrentLine(), "Trailing '_' in number.");
         } else if (isFloat) {
             return getFloatToken(number);
         }
@@ -2275,7 +2253,8 @@ public class RubyYaccLexer {
                 
                 // No hex value after the 'x'.
                 if (i == 0) {
-                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(), "Invalid escape character syntax");
+                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(),
+                            getCurrentLine(), "Invalid escape character syntax");
                 }
                 return hexValue;
             case 'b' : // backspace
@@ -2284,16 +2263,19 @@ public class RubyYaccLexer {
                 return ' ';
             case 'M' :
                 if ((c = src.read()) != '-') {
-                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(), "Invalid escape character syntax");
+                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(), 
+                            getCurrentLine(), "Invalid escape character syntax");
                 } else if ((c = src.read()) == '\\') {
                     return (char) (readEscape() | 0x80);
                 } else if (c == EOF) {
-                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(), "Invalid escape character syntax");
+                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(),
+                            getCurrentLine(), "Invalid escape character syntax");
                 } 
                 return (char) ((c & 0xff) | 0x80);
             case 'C' :
                 if ((c = src.read()) != '-') {
-                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(), "Invalid escape character syntax");
+                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(),
+                            getCurrentLine(), "Invalid escape character syntax");
                 }
             case 'c' :
                 if ((c = src.read()) == '\\') {
@@ -2301,11 +2283,13 @@ public class RubyYaccLexer {
                 } else if (c == '?') {
                     return '\u0177';
                 } else if (c == EOF) {
-                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(), "Invalid escape character syntax");
+                    throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(),
+                            getCurrentLine(), "Invalid escape character syntax");
                 }
                 return (char) (c & 0x9f);
             case EOF :
-                throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(), "Invalid escape character syntax");
+                throw new SyntaxException(PID.INVALID_ESCAPE_SYNTAX, getPosition(),
+                        getCurrentLine(), "Invalid escape character syntax");
             default :
                 return c;
         }
