@@ -103,16 +103,21 @@ public class ThreadLibrary implements Library {
 
         @JRubyMethod(name = "locked?")
         public synchronized RubyBoolean locked_p(ThreadContext context) {
-            return context.getRuntime().newBoolean(owner != null);
+            return context.getRuntime().newBoolean(isLocked());
+        }
+
+        // should be called from properly synchronized context
+        private boolean isLocked() {
+            return owner != null && owner.isAlive();
         }
 
         @JRubyMethod
-        public RubyBoolean try_lock(ThreadContext context) throws InterruptedException {
+        public RubyBoolean try_lock(ThreadContext context) {
             //if (Thread.interrupted()) {
             //    throw new InterruptedException();
             //}
             synchronized (this) {
-                if ( owner != null ) {
+                if (isLocked()) {
                     return context.getRuntime().getFalse();
                 }
                 lock(context);
@@ -121,7 +126,7 @@ public class ThreadLibrary implements Library {
         }
 
         @JRubyMethod(frame = true)
-        public IRubyObject lock(ThreadContext context) throws InterruptedException {
+        public IRubyObject lock(ThreadContext context) {
             //if (Thread.interrupted()) {
             //    throw new InterruptedException();
             //}
@@ -132,15 +137,15 @@ public class ThreadLibrary implements Library {
                         if (owner == context.getThread()) {
                             throw context.getRuntime().newThreadError("Mutex relocking by same thread");
                         }
-                        while ( owner != null ) {
+                        while (isLocked()) {
                             wait();
                         }
                         owner = context.getThread();
                     } catch (InterruptedException ex) {
-                        if ( owner == null ) {
+                        if (!isLocked()) {
                             notify();
                         }
-                        throw ex;
+                        throw context.getRuntime().newConcurrencyError(ex.getLocalizedMessage());
                     }
                 }
             } finally {
@@ -149,19 +154,42 @@ public class ThreadLibrary implements Library {
             return this;
         }
 
-        @JRubyMethod
-        public synchronized RubyBoolean unlock(ThreadContext context) {
-            if ( owner != null ) {
-                owner = null;
-                notify();
-                return context.getRuntime().getTrue();
-            } else {
-                return context.getRuntime().getFalse();
+        @JRubyMethod(name = "unlock", compat = CompatVersion.RUBY1_8)
+        public synchronized IRubyObject unlock(ThreadContext context) {
+            Ruby runtime = context.getRuntime();
+
+            if (!isLocked()) throw runtime.newThreadError("Mutex is not locked");
+
+            if (owner != context.getThread()) {
+                throw runtime.newThreadError("Mutex is not owned by calling thread");
             }
+
+            // FIXME: 1.8 throws nil when the unlock is not waking.  I don't
+            // think we can know this?
+            owner = null;
+            notify();
+            return this;
+        }
+
+        @JRubyMethod(name = "unlock", compat = CompatVersion.RUBY1_9)
+        public synchronized IRubyObject unlock19(ThreadContext context) {
+            Ruby runtime = context.getRuntime();
+
+            if (!isLocked()) {
+                throw runtime.newThreadError("Attempt to unlock a mutex which is not locked");
+            }
+
+            if (owner != context.getThread()) {
+                throw runtime.newThreadError("Attempt to unlock a mutex which is locked by another thread");
+            }
+
+            owner = null;
+            notify();
+            return this;
         }
 
         @JRubyMethod
-        public IRubyObject synchronize(ThreadContext context, Block block) throws InterruptedException {
+        public IRubyObject synchronize(ThreadContext context, Block block) {
             try {
                 lock(context);
                 return block.yield(context, null);
@@ -195,7 +223,7 @@ public class ThreadLibrary implements Library {
         }
 
         @JRubyMethod(name = "wait", required = 1, optional = 1)
-        public IRubyObject wait_ruby(ThreadContext context, IRubyObject args[]) throws InterruptedException {
+        public IRubyObject wait_ruby(ThreadContext context, IRubyObject args[]) {
             if ( args.length < 1 ) {
                 throw context.getRuntime().newArgumentError(args.length, 1);
             }
@@ -214,7 +242,7 @@ public class ThreadLibrary implements Library {
             }
 
             if (Thread.interrupted()) {
-                throw new InterruptedException();
+                throw context.getRuntime().newConcurrencyError("thread interrupted");
             }
             boolean success = false;
             try {
@@ -222,6 +250,8 @@ public class ThreadLibrary implements Library {
                     mutex.unlock(context);
                     try {
                         success = context.getThread().wait_timeout(this, timeout);
+                    } catch (InterruptedException ie) {
+                        throw context.getRuntime().newConcurrencyError(ie.getLocalizedMessage());
                     } finally {
                         // An interrupt or timeout may have caused us to miss
                         // a notify that we consumed, so do another notify in
