@@ -29,10 +29,7 @@ public interface ConfigMBean {
     public boolean isSplit();
     public boolean isVerbose();
     public boolean isDebug();
-    public boolean isYARVEnabled();
     public String getInputFieldSeparator();
-    public boolean isRubiniusEnabled();
-    public boolean isYARVCompileEnabled();
     public String getKCode();
     public String getRecordSeparator();
     public int getSafeLevel();
