@@ -36,6 +36,7 @@
 package org.jruby.parser;
 
 import org.jruby.CompatVersion;
+import org.jruby.ast.AliasNode;
 import org.jruby.ast.AndNode;
 import org.jruby.ast.ArgsPreOneArgNode;
 import org.jruby.ast.ArgsPreTwoArgNode;
@@ -53,10 +54,11 @@ import org.jruby.ast.AttrAssignTwoArgNode;
 import org.jruby.ast.BackRefNode;
 import org.jruby.ast.BeginNode;
 import org.jruby.ast.BignumNode;
+import org.jruby.ast.BinaryOperatorNode;
+import org.jruby.ast.BlockArg18Node;
 import org.jruby.ast.BlockArgNode;
 import org.jruby.ast.BlockNode;
 import org.jruby.ast.BlockPassNode;
-import org.jruby.ast.BreakNode;
 import org.jruby.ast.CallManyArgsBlockNode;
 import org.jruby.ast.CallManyArgsBlockPassNode;
 import org.jruby.ast.CallManyArgsNode;
@@ -150,6 +152,7 @@ import org.jruby.ast.StrNode;
 import org.jruby.ast.SuperNode;
 import org.jruby.ast.SymbolNode;
 import org.jruby.ast.TrueNode;
+import org.jruby.ast.UndefNode;
 import org.jruby.ast.WhenNode;
 import org.jruby.ast.WhenOneArgNode;
 import org.jruby.ast.YieldNode;
@@ -247,7 +250,7 @@ public class ParserSupport {
         case LOCALASGNNODE:
             return currentScope.declare(node.getPosition(), ((INameNode) node).getName());
         case CONSTDECLNODE: // CONSTANT
-            return currentScope.declare(node.getPosition(), ((INameNode) node).getName());
+            return new ConstNode(node.getPosition(), ((INameNode) node).getName());
         case INSTASGNNODE: // INSTANCE VARIABLE
             return new InstVarNode(node.getPosition(), ((INameNode) node).getName());
         case CLASSVARDECLNODE:
@@ -515,7 +518,7 @@ public class ParserSupport {
             } else if (node instanceof ArrayNode && ((ArrayNode)node).size() == 1) {
                 node = ((ArrayNode)node).get(0);
             } else if (node instanceof SplatNode) {
-                node = new SValueNode(position, node);
+                node = newSValueNode(position, node);
             }
         }
         
@@ -557,17 +560,6 @@ public class ParserSupport {
         }
     }
 
-    /**
-     * Does this node represent an expression?
-     * @param node to be checked
-     * @return true if an expression, false otherwise
-     */
-    public void checkExpression(Node node) {
-        if (warnings.isVerbose() && !isExpression(node)) {
-            warnings.warning(ID.VOID_VALUE_EXPRESSION, node.getPosition(), "void value expression");
-        }
-    }
-
     private Node compactNewlines(Node head) {
         while (head instanceof NewlineNode) {
             Node nextNode = ((NewlineNode) head).getNextNode();
@@ -579,34 +571,48 @@ public class ParserSupport {
         }
         return head;
     }
-    
-    private boolean isExpression(Node node) {
-        do {
-            if (node == null) return true;
-            
+
+    // logical equivalent to value_expr in MRI
+    public boolean checkExpression(Node node) {
+        boolean conditional = false;
+
+        while (node != null) {
             switch (node.getNodeType()) {
-            case BEGINNODE:
-                node = ((BeginNode) node).getBodyNode();
-                break;
+            case DEFNNODE: case DEFSNODE:
+                warnings.warning(ID.VOID_VALUE_EXPRESSION, node.getPosition(),
+                        "void value expression");
+                return false;
+            case RETURNNODE: case BREAKNODE: case NEXTNODE: case REDONODE:
+            case RETRYNODE:
+                if (!conditional) {
+                    throw new SyntaxException(PID.VOID_VALUE_EXPRESSION,
+                            node.getPosition(), lexer.getCurrentLine(),
+                            "void value expression");
+                }
+                return false;
             case BLOCKNODE:
                 node = ((BlockNode) node).getLast();
                 break;
-            case BREAKNODE:
-                node = ((BreakNode) node).getValueNode();
+            case BEGINNODE:
+                node = ((BeginNode) node).getBodyNode();
                 break;
-            case CLASSNODE: case DEFNNODE: case DEFSNODE: case MODULENODE: case NEXTNODE: 
-            case REDONODE: case RETRYNODE: case RETURNNODE: case UNTILNODE: case WHILENODE:
-                return false;
             case IFNODE:
-                return isExpression(((IfNode) node).getThenBody()) &&
-                  isExpression(((IfNode) node).getElseBody());
+                if (!checkExpression(((IfNode) node).getThenBody())) return false;
+                node = ((IfNode) node).getElseBody();
+                break;
+            case ANDNODE: case ORNODE:
+                conditional = true;
+                node = ((BinaryOperatorNode) node).getSecondNode();
+                break;
             case NEWLINENODE:
                 node = ((NewlineNode) node).getNextNode();
                 break;
             default: // Node
                 return true;
             }
-        } while (true);
+        }
+
+        return true;
     }
     
     /**
@@ -797,6 +803,10 @@ public class ParserSupport {
 
         return node;
     }
+
+    public SValueNode newSValueNode(ISourcePosition position, Node node) {
+        return new SValueNode(position, node);
+    }
     
     public SplatNode newSplatNode(ISourcePosition position, Node node) {
         return new SplatNode(position, makeNullNil(node));
@@ -1283,6 +1293,18 @@ public class ParserSupport {
         
         return new EvStrNode(position, head);
     }
+
+    public IterNode new_iter(ISourcePosition position, Node vars, 
+            StaticScope scope, Node body) {
+        if (vars != null && vars instanceof BlockPassNode) {
+            BlockPassNode blockPass = (BlockPassNode) vars;
+
+            return new IterNode(position, blockPass.getArgsNode(), blockPass,
+                    scope, body);
+        }
+        
+        return new IterNode(position, vars, null, scope, body);
+    }
     
     public Node new_yield(ISourcePosition position, Node node) {
         boolean state = true;
@@ -1385,4 +1407,140 @@ public class ParserSupport {
         }
         return new ArgsNode(position, pre, optional, rest, post, block);
     }
+
+    public Node newAlias(ISourcePosition position, Node newNode, Node oldNode) {
+        return new AliasNode(position, newNode, oldNode);
+    }
+
+    public Node newUndef(ISourcePosition position, Node nameNode) {
+        return new UndefNode(position, nameNode);
+    }
+
+    public BlockArg18Node newBlockArg18(ISourcePosition position, Node blockValue, Node args) {
+        return new BlockArg18Node(position, blockValue, args);
+    }
+
+    public BlockArgNode newBlockArg(ISourcePosition position, Token nameToken) {
+        String identifier = (String) nameToken.getValue();
+
+        if (getCurrentScope().getLocalScope().isDefined(identifier) >= 0) {
+            throw new SyntaxException(PID.BAD_IDENTIFIER, position, lexer.getCurrentLine(),
+                    "duplicate block argument name");
+        }
+
+        return new BlockArgNode(position, getCurrentScope().getLocalScope().addVariable(identifier), identifier);
+    }
+
+    /**
+     * generate parsing error
+     */
+    public void yyerror(String message) {
+        throw new SyntaxException(PID.GRAMMAR_ERROR, lexer.getPosition(), lexer.getCurrentLine(), message);
+    }
+
+    /**
+     * generate parsing error
+     * @param message text to be displayed.
+     * @param expected list of acceptable tokens, if available.
+     */
+    public void yyerror(String message, String[] expected, String found) {
+        String text = message + ", unexpected " + found + "\n";
+        throw new SyntaxException(PID.GRAMMAR_ERROR, lexer.getPosition(), lexer.getCurrentLine(), text, found);
+    }
+
+    public ISourcePosition getPosition(ISourcePositionHolder start) {
+        return start != null ? lexer.getPosition(start.getPosition()) : lexer.getPosition();
+    }
+
+    public void warn(ID id, ISourcePosition position, String message, Object... data) {
+        warnings.warn(id, position, message, data);
+    }
+
+    public void warning(ID id, ISourcePosition position, String message, Object... data) {
+        if (warnings.isVerbose()) {
+            warnings.warning(id, position, message, data);
+        }
+    }
+
+    // ENEBO: Totally weird naming (in MRI is not allocated and is a local var name) [1.9]
+    public boolean is_local_id(Token identifier) {
+        String name = (String) identifier.getValue();
+
+        return getCurrentScope().getLocalScope().isDefined(name) < 0;
+    }
+
+    // 1.9
+    public ListNode list_append(Node list, Node item) {
+        if (list == null) return new ArrayNode(item.getPosition(), item);
+        if (!(list instanceof ListNode)) return new ArrayNode(list.getPosition(), list).add(item);
+
+        return ((ListNode) list).add(item);
+    }
+
+    // 1.9
+    public Node new_bv(Token identifier) {
+        if (!is_local_id(identifier)) {
+            getterIdentifierError(identifier.getPosition(), (String) identifier.getValue());
+        }
+        shadowing_lvar(identifier);
+        arg_var(identifier);
+
+        return null;
+    }
+
+    // 1.9
+    public int arg_var(Token identifier) {
+        return getCurrentScope().addVariableThisScope((String) identifier.getValue());
+    }
+
+    // 1.9
+    public void shadowing_lvar(Token identifier) {
+        String name = (String) identifier.getValue();
+
+        if (getCurrentScope().isDefined(name) > 0) {
+            if (warnings.isVerbose()) warnings.warning(ID.STATEMENT_NOT_REACHED, identifier.getPosition(), "shadowing outer local variable - " + name);
+        }
+    }
+
+    // 1.9
+    public ListNode list_concat(Node first, Node second) {
+        if (first instanceof ListNode) {
+            if (second instanceof ListNode) {
+                return ((ListNode) first).addAll((ListNode) second);
+            } else {
+                return ((ListNode) first).addAll(second);
+            }
+        }
+
+        return new ArrayNode(first.getPosition(), first).add(second);
+    }
+
+    // 1.9
+    /**
+     * If node is a splat and it is splatting a literal array then return the literal array.
+     * Otherwise return null.  This allows grammar to not splat into a Ruby Array if splatting
+     * a literal array.
+     */
+    public Node splat_array(Node node) {
+        if (node instanceof SplatNode) node = ((SplatNode) node).getValue();
+        if (node instanceof ArrayNode) return node;
+        return null;
+    }
+
+    // 1.9
+    public Node arg_append(Node node1, Node node2) {
+        if (node1 == null) return new ArrayNode(node2.getPosition(), node2);
+        if (node1 instanceof ListNode) return ((ListNode) node1).add(node2);
+        if (node1 instanceof BlockPassNode) return arg_append(((BlockPassNode) node1).getBodyNode(), node2);
+        if (node1 instanceof ArgsPushNode) {
+            ArgsPushNode pushNode = (ArgsPushNode) node1;
+            Node body = pushNode.getSecondNode();
+
+            return new ArgsCatNode(pushNode.getPosition(), pushNode.getFirstNode(),
+                    new ArrayNode(body.getPosition(), body).add(node2));
+        }
+
+        return new ArgsPushNode(position(node1, node2), node1, node2);
+    }
+
 }
