@@ -33,6 +33,7 @@
 package org.jruby.runtime;
 
 import org.jruby.RubyModule;
+import org.jruby.parser.StaticScope;
 import org.jruby.runtime.builtin.IRubyObject;
 
 /**
@@ -115,6 +116,15 @@ public class Binding {
         return dynamicScope;
     }
 
+    private DynamicScope dummyScope;
+
+    public DynamicScope getDummyScope(StaticScope staticScope) {
+        if (dummyScope == null || dummyScope.getStaticScope() != staticScope) {
+            return dummyScope = DynamicScope.newDummyScope(staticScope, dynamicScope);
+        }
+        return dummyScope;
+    }
+
     /**
      * Gets the frame.
      * 
@@ -147,4 +157,19 @@ public class Binding {
     public void setLine(int line) {
         this.line = line;
     }
+
+    public boolean equals(Object other) {
+        if(this == other) {
+            return true;
+        }
+
+        if(!(other instanceof Binding)) {
+            return false;
+        }
+
+        Binding bOther = (Binding)other;
+
+        return this.self == bOther.self &&
+            this.dynamicScope == bOther.dynamicScope;
+    }
 }
