@@ -30,6 +30,7 @@ package org.jruby.runtime;
 
 import java.util.List;
 import org.jruby.Ruby;
+import org.jruby.RubyInstanceConfig;
 import org.jruby.RubyModule;
 import org.jruby.anno.JavaMethodDescriptor;
 import org.jruby.internal.runtime.methods.CallConfiguration;
@@ -37,6 +38,7 @@ import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.internal.runtime.methods.ReflectionMethodFactory;
 import org.jruby.internal.runtime.methods.InvocationMethodFactory;
 import org.jruby.internal.runtime.methods.DumpingInvocationMethodFactory;
+import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.parser.StaticScope;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.SafePropertyAccessor;
@@ -59,6 +61,7 @@ public abstract class MethodFactory {
      * This interface should be implemented by code calling any batched methods
      * on this MethodFactory.
      */
+    @Deprecated
     public interface MethodDefiningCallback {
         public void define(RubyModule targetMetaClass, JavaMethodDescriptor desc, DynamicMethod dynamicMethod);
     }
@@ -94,13 +97,36 @@ public abstract class MethodFactory {
      * @param scope The methods static scoping information.
      * @param scriptObject An instace of the target compiled method class.
      * @param callConfig The call configuration to use for this method.
+     * @param position The position to use when generating traceable handles.
      * @return A new method handle for the target compiled method.
      */
     public abstract DynamicMethod getCompiledMethod(
             RubyModule implementationClass, String method, 
             Arity arity, Visibility visibility, StaticScope scope, 
-            Object scriptObject, CallConfiguration callConfig);
-    
+            Object scriptObject, CallConfiguration callConfig, ISourcePosition position);
+
+    /**
+     * Like getCompiledMethod, but produces the actual bytes for the compiled
+     * method handle rather than loading and constructing it. This can be used
+     * to generate all the handles ahead of time, as when doing a full system
+     * precompile.
+     *
+     * @param method The name of the method
+     * @param classPath The path-like (with / instead of .) name of the class
+     * @param invokerPath The path-line name of the invoker to generate
+     * @param arity The Arity of the method
+     * @param scope The methods static scoping information.
+     * @param callConfig The call configuration to use for this method.
+     * @param position The position to use when generating traceable handles.
+     * @return
+     */
+    public byte[] getCompiledMethodOffline(
+            String method, String classPath, String invokerPath,
+            Arity arity, StaticScope scope,
+            CallConfiguration callConfig, String filename, int line) {
+        return null;
+    }
+
     /**
      * Like getCompiledMethod, but postpones any heavy lifting involved in
      * creating the method until first invocation. This helps reduce the cost
@@ -120,7 +146,7 @@ public abstract class MethodFactory {
     public abstract DynamicMethod getCompiledMethodLazily(
             RubyModule implementationClass, String method, 
             Arity arity, Visibility visibility, StaticScope scope, 
-            Object scriptObject, CallConfiguration callConfig);
+            Object scriptObject, CallConfiguration callConfig, ISourcePosition position);
     
     /**
      * Based on a list of annotated Java methods, generate a method handle using
@@ -148,6 +174,50 @@ public abstract class MethodFactory {
      * @return A method handle for the target object.
      */
     public abstract DynamicMethod getAnnotatedMethod(RubyModule implementationClass, JavaMethodDescriptor desc);
+    
+    /**
+     * Get a CompiledBlockCallback for the specified block
+     *
+     * @param method The name of the method
+     * @param scriptObject The object in which the method can be found
+     * @return A new CompiledBlockCallback for the method
+     */
+    public abstract CompiledBlockCallback getBlockCallback(String method, Object scriptObject);
+
+    /**
+     * Get a CompiledBlockCallback for the specified block
+     *
+     * @param method The name of the method
+     * @param scriptObject The object in which the method can be found
+     * @return A new CompiledBlockCallback for the method
+     */
+    public abstract CompiledBlockCallback19 getBlockCallback19(String method, Object scriptObject);
+
+    /**
+     * Get a CompiledBlockCallback for the specified block, returning the bytes
+     * but not loading the class. This is used for offline generation of the
+     * callback class file.
+     *
+     * @param method The name of the method
+     * @param classPath The /-based name of the class containing the method
+     * @return The bytes of the class
+     */
+    public byte[] getBlockCallbackOffline(String method, String classPath) {
+        return null;
+    }
+
+    /**
+     * Get a CompiledBlockCallback for the specified block, returning the bytes
+     * but not loading the class. This is used for offline generation of the
+     * callback class file. This version generates a 1.9-compatible callback.
+     *
+     * @param method The name of the method
+     * @param classPath The /-based name of the class containing the method
+     * @return The bytes of the class
+     */
+    public byte[] getBlockCallback19Offline(String method, String classPath) {
+        return null;
+    }
 
     /**
      * Use the reflection-based factory.
@@ -169,9 +239,7 @@ public abstract class MethodFactory {
         if (Ruby.isSecurityRestricted()) {
             reflection_ = true;
         } else {
-            if (SafePropertyAccessor.getProperty("jruby.reflection") != null && SafePropertyAccessor.getBoolean("jruby.reflection")) {
-                reflection_ = true;
-            }
+            reflection_ = RubyInstanceConfig.REFLECTED_HANDLES;
             if (SafePropertyAccessor.getProperty("jruby.dump_invocations") != null) {
                 dumping_ = true;
                 dumpingPath_ = SafePropertyAccessor.getProperty("jruby.dump_invocations");
