@@ -43,8 +43,11 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import org.jruby.RubyInstanceConfig;
+import org.jruby.runtime.callsite.ArefCallSite;
+import org.jruby.runtime.callsite.AsetCallSite;
 import org.jruby.runtime.callsite.FunctionalCachingCallSite;
 import org.jruby.runtime.callsite.RespondToCallSite;
+import org.jruby.runtime.callsite.SuperCallSite;
 import org.jruby.runtime.callsite.VariableCachingCallSite;
 
 /**
@@ -120,7 +123,7 @@ public class MethodIndex {
         Integer index = NUMBERS.get(methodName);
         
         if (index == null) {
-            index = new Integer(NAMES.size());
+            index = Integer.valueOf(NAMES.size());
             NUMBERS.put(methodName, index);
             NAMES.add(methodName);
         }
@@ -132,29 +135,33 @@ public class MethodIndex {
         // fast and safe respond_to? call site logic
         if (name.equals("respond_to?")) return new RespondToCallSite();
         
-        if (!RubyInstanceConfig.FASTOPS_COMPILE_ENABLED) {
-            return new NormalCachingCallSite(name);
-        } else {
-            if (name.equals("+")) {
-                return new PlusCallSite();
-            } else if (name.equals("-")) {
-                return new MinusCallSite();
-            } else if (name.equals("*")) {
-                return new MulCallSite();
-            } else if (name.equals("/")) {
-                return new DivCallSite();
-            } else if (name.equals("<")) {
-                return new LtCallSite();
-            } else if (name.equals("<=")) {
-                return new LeCallSite();
-            } else if (name.equals(">")) {
-                return new GtCallSite();
-            } else if (name.equals(">=")) {
-                return new GeCallSite();
-            } else {
-                return new NormalCachingCallSite(name);
-            }
+        if (RubyInstanceConfig.FASTOPS_COMPILE_ENABLED) return getFastOpsCallSite(name);
+
+        return new NormalCachingCallSite(name);
+    }
+
+    public synchronized static CallSite getFastOpsCallSite(String name) {
+        if (name.equals("+")) {
+            return new PlusCallSite();
+        } else if (name.equals("-")) {
+            return new MinusCallSite();
+        } else if (name.equals("*")) {
+            return new MulCallSite();
+        } else if (name.equals("<")) {
+            return new LtCallSite();
+        } else if (name.equals("<=")) {
+            return new LeCallSite();
+        } else if (name.equals(">")) {
+            return new GtCallSite();
+        } else if (name.equals(">=")) {
+            return new GeCallSite();
+        } else if (name.equals("[]")) {
+            return new ArefCallSite();
+        } else if (name.equals("[]=")) {
+            return new AsetCallSite();
         }
+
+        return new NormalCachingCallSite(name);
     }
     
     public synchronized static CallSite getFunctionalCallSite(String name) {
@@ -164,4 +171,8 @@ public class MethodIndex {
     public synchronized static CallSite getVariableCallSite(String name) {
         return new VariableCachingCallSite(name);
     }
+
+    public synchronized static CallSite getSuperCallSite() {
+        return new SuperCallSite();
+    }
 }
