@@ -79,9 +79,4 @@ public interface InternalVariables {
      * @return the value of the remove variable, if present; else null
      */
     Object removeInternalVariable(String name);
-
-    /**
-     * @return only internal variables (NOT ivar/cvar/constant)
-     */
-    List<Variable<Object>> getInternalVariableList();
 }
