@@ -15,6 +15,10 @@ public class CacheEntry {
     }
 
     public final boolean typeOk(RubyClass incomingType) {
-        return token == incomingType.getCacheToken();
+        return typeOk(this, incomingType);
+    }
+
+    public static final boolean typeOk(CacheEntry entry, RubyClass incomingType) {
+        return entry.token == incomingType.getCacheToken();
     }
 }
