@@ -16,9 +16,8 @@ import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
 public abstract class CachingCallSite extends CallSite {
-    protected volatile CacheEntry cache = CacheEntry.NULL_CACHE;
+    protected CacheEntry cache = CacheEntry.NULL_CACHE;
     public static volatile int totalCallSites;
-    public static volatile int failedCallSites;
 
     public CachingCallSite(String methodName, CallType callType) {
         super(methodName, callType);
@@ -63,7 +62,7 @@ public abstract class CachingCallSite extends CallSite {
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject... args) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName, args);
         }
         return cacheAndCall(caller, selfType, args, context, self);
@@ -72,7 +71,7 @@ public abstract class CachingCallSite extends CallSite {
     private IRubyObject callBlock(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject[] args, Block block) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName, args, block);
         }
         return cacheAndCall(caller, selfType, block, args, context, self);
@@ -103,7 +102,7 @@ public abstract class CachingCallSite extends CallSite {
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName);
         }
         return cacheAndCall(caller, selfType, context, self);
@@ -112,7 +111,7 @@ public abstract class CachingCallSite extends CallSite {
     private IRubyObject callBlock(ThreadContext context, IRubyObject caller, IRubyObject self, Block block) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName, block);
         }
         return cacheAndCall(caller, selfType, block, context, self);
@@ -143,7 +142,7 @@ public abstract class CachingCallSite extends CallSite {
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName, arg1);
         }
         return cacheAndCall(caller, selfType, context, self, arg1);
@@ -152,7 +151,7 @@ public abstract class CachingCallSite extends CallSite {
     private IRubyObject callBlock(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, Block block) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName, arg1, block);
         }
         return cacheAndCall(caller, selfType, block, context, self, arg1);
@@ -183,7 +182,7 @@ public abstract class CachingCallSite extends CallSite {
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, IRubyObject arg2) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName, arg1, arg2);
         }
         return cacheAndCall(caller, selfType, context, self, arg1, arg2);
@@ -192,7 +191,7 @@ public abstract class CachingCallSite extends CallSite {
     private IRubyObject callBlock(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, IRubyObject arg2, Block block) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName, arg1, arg2, block);
         }
         return cacheAndCall(caller, selfType, block, context, self, arg1, arg2);
@@ -223,7 +222,7 @@ public abstract class CachingCallSite extends CallSite {
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName, arg1, arg2, arg3);
         }
         return cacheAndCall(caller, selfType, context, self, arg1, arg2, arg3);
@@ -232,7 +231,7 @@ public abstract class CachingCallSite extends CallSite {
     private IRubyObject callBlock(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3, Block block) {
         RubyClass selfType = pollAndGetClass(context, self);
         CacheEntry myCache = cache;
-        if (myCache.typeOk(selfType)) {
+        if (CacheEntry.typeOk(myCache, selfType)) {
             return myCache.method.call(context, self, selfType, methodName, arg1, arg2, arg3, block);
         }
         return cacheAndCall(caller, selfType, block, context, self, arg1, arg2, arg3);
@@ -360,50 +359,50 @@ public abstract class CachingCallSite extends CallSite {
         return method.call(context, self, selfType, methodName, arg1, arg2, arg3, block);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject[] args) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject[] args) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, args, Block.NULL_BLOCK);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, Block.NULL_BLOCK);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, Block block) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, Block block) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, block);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, arg, Block.NULL_BLOCK);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject[] args, Block block) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject[] args, Block block) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, args, block);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, Block block) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, Block block) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, arg0, block);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, IRubyObject arg1) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, IRubyObject arg1) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, arg0, arg1, Block.NULL_BLOCK);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, IRubyObject arg1, Block block) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, IRubyObject arg1, Block block) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, arg0, arg1, block);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, arg0, arg1, arg2, Block.NULL_BLOCK);
     }
 
-    private IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Block block) {
+    protected IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, DynamicMethod method, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Block block) {
         return RuntimeHelpers.selectMethodMissing(context, self, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, arg0, arg1, arg2, block);
     }
 
     protected abstract boolean methodMissing(DynamicMethod method, IRubyObject caller);
 
     private static RubyClass pollAndGetClass(ThreadContext context, IRubyObject self) {
-        context.callThreadPoll();
+        ThreadContext.callThreadPoll(context);
         RubyClass selfType = self.getMetaClass();
         return selfType;
     }
