@@ -10,6 +10,13 @@ public class PlusCallSite extends NormalCachingCallSite {
         super("+");
     }
 
+    public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, long fixnum) {
+        if (self instanceof RubyFixnum) {
+            return ((RubyFixnum) self).op_plus(context, fixnum);
+        }
+        return super.call(context, caller, self, fixnum);
+    }
+
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg) {
         if (self instanceof RubyFixnum) {
