@@ -34,8 +34,12 @@ import java.util.Enumeration;
 import java.util.List;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
+import java.util.zip.ZipEntry;
+import org.jruby.RubyEncoding;
+import org.jruby.RubyFile;
 
 import org.jruby.ext.posix.JavaSecuredFile;
+import org.jruby.platform.Platform;
 
 /**
  * This class exists as a counterpart to the dir.c file in 
@@ -45,7 +49,7 @@ import org.jruby.ext.posix.JavaSecuredFile;
  * @author <a href="mailto:ola.bini@ki.se">Ola Bini</a>
  */
 public class Dir {
-    public final static boolean DOSISH = System.getProperty("os.name").indexOf("Windows") != -1;
+    public final static boolean DOSISH = Platform.IS_WINDOWS;
     public final static boolean CASEFOLD_FILESYSTEM = DOSISH;
 
     public final static int FNM_NOESCAPE = 0x01;
@@ -137,10 +141,8 @@ public class Dir {
                 s++;
                 break;
             case '\\':
-                if(escape &&
-                   (!DOSISH ||
-                    (pat < pend && "*?[]\\".indexOf((char)bytes[pat]) != -1))) {
-                    if(pat >= pend) {
+                if (escape) {
+                    if (pat >= pend) {
                         c = '\\';
                     } else {
                         c = bytes[pat++];
@@ -315,7 +317,7 @@ public class Dir {
         int index;
 
         public GlobPattern(ByteList bytelist, int flags) {
-            this(bytelist.bytes, bytelist.begin,  bytelist.begin + bytelist.realSize, flags);
+            this(bytelist.getUnsafeBytes(), bytelist.getBegin(),  bytelist.getBegin() + bytelist.getRealSize(), flags);
         }
         
         public GlobPattern(byte[] bytes, int index, int end, int flags) {
@@ -438,7 +440,7 @@ public class Dir {
             buf.append(pattern.bytes, pattern.begin, lbrace - pattern.begin);
             buf.append(pattern.bytes, middleRegionIndex, i - middleRegionIndex);
             buf.append(pattern.bytes, rbrace + 1, pattern.end - (rbrace + 1));
-            int status = push_braces(cwd, result, new GlobPattern(buf.bytes, buf.begin, buf.realSize, pattern.flags));
+            int status = push_braces(cwd, result, new GlobPattern(buf.getUnsafeBytes(), buf.getBegin(), buf.getRealSize(),pattern.flags));
             if(status != 0) return status;
         }
         
@@ -565,7 +567,7 @@ public class Dir {
 
             if (bytes[begin] == '/' || (DOSISH && begin+2<end && bytes[begin+1] == ':' && isdirsep(bytes[begin+2]))) {
                 if (new JavaSecuredFile(newStringFromUTF8(bytes, begin, end - begin)).exists()) {
-                    status = func.call(bytes, begin, end, arg);
+                    status = func.call(bytes, begin, end - begin, arg);
                 }
             } else if (isJarFilePath(bytes, begin, end)) {
                 int ix = end;
@@ -577,13 +579,13 @@ public class Dir {
                 }
 
                 st = new JavaSecuredFile(newStringFromUTF8(bytes, begin+5, ix-5));
-                String jar = newStringFromUTF8(bytes, begin+ix+1, end-(ix+1));
                 try {
+                    String jar = newStringFromUTF8(bytes, begin+ix+1, end-(ix+1));
                     JarFile jf = new JarFile(st);
                     
                     if (jar.startsWith("/")) jar = jar.substring(1);
-                    if (jf.getEntry(jar + "/") != null) jar = jar + "/";
-                    if (jf.getEntry(jar) != null) {
+                    ZipEntry entry = RubyFile.getDirOrFileEntry(jf, jar);
+                    if (entry != null) {
                         status = func.call(bytes, begin, end, arg);
                     }
                 } catch(Exception e) {}
@@ -644,7 +646,7 @@ public class Dir {
                             buf.length(0);
                             buf.append(base);
                             buf.append(bytes, (base.length > 0 ? m : m + 1), end - (base.length > 0 ? m : m + 1));
-                            status = glob_helper(cwd, buf.bytes, buf.begin, buf.realSize, n, flags, func, arg);
+                            status = glob_helper(cwd, buf.getUnsafeBytes(), buf.getBegin(), buf.getRealSize(), n, flags, func, arg);
                             if(status != 0) {
                                 break finalize;
                             }
@@ -666,17 +668,17 @@ public class Dir {
                                 buf.append(base);
                                 buf.append( BASE(base) ? SLASH : EMPTY );
                                 buf.append(getBytesInUTF8(dirp[i]));
-                                if (buf.bytes[0] == '/' || (DOSISH && 2<buf.realSize && buf.bytes[1] == ':' && isdirsep(buf.bytes[2]))) {
-                                    st = new JavaSecuredFile(newStringFromUTF8(buf.bytes, buf.begin, buf.realSize));
+                                if (buf.getUnsafeBytes()[0] == '/' || (DOSISH && 2<buf.getRealSize() && buf.getUnsafeBytes()[1] == ':' && isdirsep(buf.getUnsafeBytes()[2]))) {
+                                    st = new JavaSecuredFile(newStringFromUTF8(buf.getUnsafeBytes(), buf.getBegin(), buf.getRealSize()));
                                 } else {
-                                    st = new JavaSecuredFile(cwd, newStringFromUTF8(buf.bytes, buf.begin, buf.realSize));
+                                    st = new JavaSecuredFile(cwd, newStringFromUTF8(buf.getUnsafeBytes(), buf.getBegin(), buf.getRealSize()));
                                 }
                                 if(st.isDirectory() && !".".equals(dirp[i]) && !"..".equals(dirp[i])) {
-                                    int t = buf.realSize;
+                                    int t = buf.getRealSize();
                                     buf.append(SLASH);
                                     buf.append(DOUBLE_STAR);
                                     buf.append(bytes, m, end - m);
-                                    status = glob_helper(cwd, buf.bytes, buf.begin, buf.realSize, t, flags, func, arg);
+                                    status = glob_helper(cwd, buf.getUnsafeBytes(), buf.getBegin(), buf.getRealSize(), t, flags, func, arg);
                                     if(status != 0) {
                                         break;
                                     }
@@ -690,7 +692,7 @@ public class Dir {
                                 buf.append( BASE(base) ? SLASH : EMPTY );
                                 buf.append(getBytesInUTF8(dirp[i]));
                                 if(m == -1) {
-                                    status = func.call(buf.bytes,0,buf.realSize,arg);
+                                    status = func.call(buf.getUnsafeBytes(),0, buf.getRealSize(),arg);
                                     if(status != 0) {
                                         break;
                                     }
@@ -731,11 +733,11 @@ public class Dir {
                                     buf.append(bs, 0, len);
 
                                     if(je.isDirectory()) {
-                                        int t = buf.realSize;
+                                        int t = buf.getRealSize();
                                         buf.append(SLASH);
                                         buf.append(DOUBLE_STAR);
                                         buf.append(bytes, m, end - m);
-                                        status = glob_helper(cwd, buf.bytes, buf.begin, buf.realSize, t, flags, func, arg);
+                                        status = glob_helper(cwd, buf.getUnsafeBytes(), buf.getBegin(), buf.getRealSize(), t, flags, func, arg);
                                         if(status != 0) {
                                             break;
                                         }
@@ -749,7 +751,7 @@ public class Dir {
                                     buf.append( BASE(base) ? SLASH : EMPTY );
                                     buf.append(bs, 0, len);
                                     if(m == -1) {
-                                        status = func.call(buf.bytes,0,buf.realSize,arg);
+                                        status = func.call(buf.getUnsafeBytes(),0, buf.getRealSize(),arg);
                                         if(status != 0) {
                                             break;
                                         }
@@ -766,18 +768,18 @@ public class Dir {
                 if (link.size() > 0) {
                     for (ByteList b : link) {
                         if (status == 0) {
-                            if(b.bytes[0] == '/'  || (DOSISH && 2<b.realSize && b.bytes[1] == ':' && isdirsep(b.bytes[2]))) {
-                                st = new JavaSecuredFile(newStringFromUTF8(b.bytes, 0, b.realSize));
+                            if(b.getUnsafeBytes()[0] == '/'  || (DOSISH && 2<b.getRealSize() && b.getUnsafeBytes()[1] == ':' && isdirsep(b.getUnsafeBytes()[2]))) {
+                                st = new JavaSecuredFile(newStringFromUTF8(b.getUnsafeBytes(), 0, b.getRealSize()));
                             } else {
-                                st = new JavaSecuredFile(cwd, newStringFromUTF8(b.bytes, 0, b.realSize));
+                                st = new JavaSecuredFile(cwd, newStringFromUTF8(b.getUnsafeBytes(), 0, b.getRealSize()));
                             }
 
                             if(st.isDirectory()) {
-                                int len = b.realSize;
+                                int len = b.getRealSize();
                                 buf.length(0);
                                 buf.append(b);
                                 buf.append(bytes, m, end - m);
-                                status = glob_helper(cwd,buf.bytes,0,buf.realSize,len,flags,func,arg);
+                                status = glob_helper(cwd, buf.getUnsafeBytes(),0, buf.getRealSize(),len,flags,func,arg);
                             }
                         }
                     }
@@ -790,26 +792,14 @@ public class Dir {
     }
 
     private static byte[] getBytesInUTF8(String s) {
-        try {
-            return s.getBytes("UTF-8");
-        } catch (java.io.UnsupportedEncodingException ex) {
-            return s.getBytes(); // NOT REACHED HERE
-        }
+        return RubyEncoding.encodeUTF8(s);
     }
 
     private static String newStringFromUTF8(byte[] buf, int offset, int len) {
-        try {
-            return new String(buf, offset, len, "UTF-8");
-        } catch (java.io.UnsupportedEncodingException ex) {
-            return new String(buf, offset, len); // NOT REACHED HERE
-        }
+        return RubyEncoding.decodeUTF8(buf, offset, len);
     }
 
     private static String newStringFromUTF8(byte[] buf) {
-        try {
-            return new String(buf, "UTF-8");
-        } catch (java.io.UnsupportedEncodingException ex) {
-            return new String(buf); // NOT REACHED HERE
-        }
+        return RubyEncoding.decodeUTF8(buf);
     }
 }
